@@ -233,7 +233,7 @@ HRESULT WINAPI ADsGetLastError(LPDWORD perror, LPWSTR errorbuf, DWORD errorbufle
  */
 LPVOID WINAPI AllocADsMem(DWORD cb)
 {
-    return HeapAlloc(GetProcessHeap(), 0, cb);
+    return malloc(cb);
 }
 
 /*****************************************************
@@ -241,7 +241,8 @@ LPVOID WINAPI AllocADsMem(DWORD cb)
  */
 BOOL WINAPI FreeADsMem(LPVOID pMem)
 {
-    return HeapFree(GetProcessHeap(), 0, pMem);
+    free(pMem);
+    return TRUE;
 }
 
 /*****************************************************
@@ -249,7 +250,7 @@ BOOL WINAPI FreeADsMem(LPVOID pMem)
  */
 LPVOID WINAPI ReallocADsMem(LPVOID pOldMem, DWORD cbOld, DWORD cbNew)
 {
-    return HeapReAlloc(GetProcessHeap(), 0, pOldMem, cbNew);
+    return realloc(pOldMem, cbNew);
 }
 
 /*****************************************************
@@ -257,18 +258,8 @@ LPVOID WINAPI ReallocADsMem(LPVOID pOldMem, DWORD cbOld, DWORD cbNew)
  */
 LPWSTR WINAPI AllocADsStr(LPWSTR pStr)
 {
-    LPWSTR ret;
-    SIZE_T len;
-
     TRACE("(%p)\n", pStr);
-
-    if (!pStr) return NULL;
-
-    len = (wcslen(pStr) + 1) * sizeof(WCHAR);
-    ret = AllocADsMem(len);
-    if (ret) memcpy(ret, pStr, len);
-
-    return ret;
+    return wcsdup(pStr);
 }
 
 /*****************************************************
