@@ -99,28 +99,28 @@ static void version_test(void)
     major = minor = 0;
     hr = pGetVersionFromFile("kernel32.dll", &major, &minor, FALSE);
     ok (hr == S_OK, "GetVersionFromFileEx(kernel32.dll) failed, returned "
-        "0x%08x\n", hr);
-    trace("kernel32.dll Language ID: 0x%08x, Codepage ID: 0x%08x\n",
+        "0x%08lx\n", hr);
+    trace("kernel32.dll Language ID: 0x%08lx, Codepage ID: 0x%08lx\n",
            major, minor);
 
     major = minor = 0;
     hr = pGetVersionFromFile("kernel32.dll", &major, &minor, TRUE);
     ok (hr == S_OK, "GetVersionFromFileEx(kernel32.dll) failed, returned "
-        "0x%08x\n", hr);
+        "0x%08lx\n", hr);
     trace("kernel32.dll version: %d.%d.%d.%d\n", HIWORD(major), LOWORD(major),
           HIWORD(minor), LOWORD(minor));
 
     major = minor = 0;
     hr = pGetVersionFromFile("advpack.dll", &major, &minor, FALSE);
     ok (hr == S_OK, "GetVersionFromFileEx(advpack.dll) failed, returned "
-        "0x%08x\n", hr);
-    trace("advpack.dll Language ID: 0x%08x, Codepage ID: 0x%08x\n",
+        "0x%08lx\n", hr);
+    trace("advpack.dll Language ID: 0x%08lx, Codepage ID: 0x%08lx\n",
            major, minor);
 
     major = minor = 0;
     hr = pGetVersionFromFile("advpack.dll", &major, &minor, TRUE);
     ok (hr == S_OK, "GetVersionFromFileEx(advpack.dll) failed, returned "
-        "0x%08x\n", hr);
+        "0x%08lx\n", hr);
     trace("advpack.dll version: %d.%d.%d.%d\n", HIWORD(major), LOWORD(major),
           HIWORD(minor), LOWORD(minor));
 }
@@ -278,7 +278,7 @@ static void translateinfstring_test(void)
     }
     ok(hr == S_OK, "Expected S_OK, got 0x%08x\n", (UINT)hr);
     ok(!strcmp(buffer, TEST_STRING2), "Expected %s, got %s\n", TEST_STRING2, buffer);
-    ok(dwSize == 25, "Expected size 25, got %d\n", dwSize);
+    ok(dwSize == 25, "Expected size 25, got %ld\n", dwSize);
 
     buffer[0] = 0;
     /* try other nonexistent section */
@@ -304,7 +304,7 @@ static void translateinfstring_test(void)
     if(hr == ERROR_SUCCESS)
     {
         ok(!strcmp(buffer, APP_PATH), "Expected '%s', got '%s'\n", APP_PATH, buffer);
-        ok(dwSize == APP_PATH_LEN, "Expected size %d, got %d\n", APP_PATH_LEN, dwSize);
+        ok(dwSize == APP_PATH_LEN, "Expected size %ld, got %ld\n", APP_PATH_LEN, dwSize);
     }
 
     buffer[0] = 0;
@@ -315,7 +315,7 @@ static void translateinfstring_test(void)
     todo_wine
     {
         ok(!strcmp(buffer, TEST_STRING2), "Expected %s, got %s\n", TEST_STRING2, buffer);
-        ok(dwSize == 25, "Expected size 25, got %d\n", dwSize);
+        ok(dwSize == 25, "Expected size 25, got %ld\n", dwSize);
     }
 
     DeleteFileA("c:\\a.inf");
@@ -342,31 +342,31 @@ static void translateinfstringex_test(void)
 
     /* try a NULL filename */
     hr = pOpenINFEngine(NULL, "Options.NTx86", 0, &hinf, NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* try an empty filename */
     hr = pOpenINFEngine("", "Options.NTx86", 0, &hinf, NULL);
     ok(hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND) /* NT+ */ ||
        hr == HRESULT_FROM_WIN32(E_UNEXPECTED) /* 9x */,
-        "Expected HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND or E_UNEXPECTED), got %08x\n", hr);
+        "Expected HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND or E_UNEXPECTED), got %08lx\n", hr);
 
     /* try a NULL hinf */
     hr = pOpenINFEngine(inf_file, "Options.NTx86", 0, NULL, NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* open the INF without the Install section specified */
     hr = pOpenINFEngine(inf_file, NULL, 0, &hinf, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
 
     /* try a NULL hinf */
     hr = pTranslateInfStringEx(NULL, inf_file, "Options.NTx86", "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* try a NULL filename */
     hr = pTranslateInfStringEx(hinf, NULL, "Options.NTx86", "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* try an empty filename */
     memset(buffer, 'a', 25);
@@ -374,32 +374,32 @@ static void translateinfstringex_test(void)
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, "", "Options.NTx86", "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     todo_wine
     {
         ok(!strcmp(buffer, TEST_STRING2), "Expected %s, got %s\n", TEST_STRING2, buffer);
-        ok(size == 25, "Expected size 25, got %d\n", size);
+        ok(size == 25, "Expected size 25, got %ld\n", size);
     }
 
     /* try a NULL translate section */
     hr = pTranslateInfStringEx(hinf, inf_file, NULL, "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* try an empty translate section */
     hr = pTranslateInfStringEx(hinf, inf_file, "", "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == SPAPI_E_LINE_NOT_FOUND, "Expected SPAPI_E_LINE_NOT_FOUND, got %08x\n", hr);
+    ok(hr == SPAPI_E_LINE_NOT_FOUND, "Expected SPAPI_E_LINE_NOT_FOUND, got %08lx\n", hr);
 
     /* try a NULL translate key */
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", NULL,
                               buffer, size, &size, NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* try an empty translate key */
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "",
                               buffer, size, &size, NULL);
-    ok(hr == SPAPI_E_LINE_NOT_FOUND, "Expected SPAPI_E_LINE_NOT_FOUND, got %08x\n", hr);
+    ok(hr == SPAPI_E_LINE_NOT_FOUND, "Expected SPAPI_E_LINE_NOT_FOUND, got %08lx\n", hr);
 
     /* successfully translate the string */
     memset(buffer, 'a', 25);
@@ -407,20 +407,20 @@ static void translateinfstringex_test(void)
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     todo_wine
     {
         ok(!strcmp(buffer, TEST_STRING2), "Expected %s, got %s\n", TEST_STRING2, buffer);
-        ok(size == 25, "Expected size 25, got %d\n", size);
+        ok(size == 25, "Expected size 25, got %ld\n", size);
     }
 
     /* try a NULL hinf */
     hr = pCloseINFEngine(NULL);
-    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
+    ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08lx\n", hr);
 
     /* successfully close the hinf */
     hr = pCloseINFEngine(hinf);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
 
     /* open the inf with the install section */
     hr = pOpenINFEngine(inf_file, "section", 0, &hinf, NULL);
@@ -430,7 +430,7 @@ static void translateinfstringex_test(void)
         DeleteFileA(inf_file);
         return;
     }
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
 
     /* translate the string with the install section specified */
     memset(buffer, 'a', APP_PATH_LEN);
@@ -438,9 +438,9 @@ static void translateinfstringex_test(void)
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "InstallDir",
                               buffer, size, &size, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(!strcmp(buffer, APP_PATH), "Expected %s, got %s\n", APP_PATH, buffer);
-    ok(size == APP_PATH_LEN, "Expected size %d, got %d\n", APP_PATH_LEN, size);
+    ok(size == APP_PATH_LEN, "Expected size %ld, got %ld\n", APP_PATH_LEN, size);
 
     /* Single quote test (Note size includes null on return from call) */
     memset(buffer, 'a', APP_PATH_LEN);
@@ -448,10 +448,10 @@ static void translateinfstringex_test(void)
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result1",
                               buffer, size, &size, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(!lstrcmpiA(buffer, PROG_FILES_ROOT),
            "Expected %s, got %s\n", PROG_FILES_ROOT, buffer);
-    ok(size == strlen(PROG_FILES_ROOT)+1, "Expected size %d, got %d\n",
+    ok(size == strlen(PROG_FILES_ROOT)+1, "Expected size %d, got %ld\n",
            lstrlenA(PROG_FILES_ROOT)+1, size);
 
     memset(buffer, 'a', APP_PATH_LEN);
@@ -459,10 +459,10 @@ static void translateinfstringex_test(void)
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result2",
                               buffer, size, &size, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(!lstrcmpiA(buffer, PROG_FILES_ROOT),
            "Expected %s, got %s\n", PROG_FILES_ROOT, buffer);
-    ok(size == strlen(PROG_FILES_ROOT)+1, "Expected size %d, got %d\n",
+    ok(size == strlen(PROG_FILES_ROOT)+1, "Expected size %d, got %ld\n",
            lstrlenA(PROG_FILES_ROOT)+1, size);
 
     {
@@ -475,16 +475,16 @@ static void translateinfstringex_test(void)
         size = MAX_PATH;
         hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result3",
                                   buffer, size, &size, NULL);
-        ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+        ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
         ok(!lstrcmpiA(buffer, drive),
                "Expected %s, got %s\n", drive, buffer);
-        ok(size == strlen(drive)+1, "Expected size %d, got %d\n",
+        ok(size == strlen(drive)+1, "Expected size %d, got %ld\n",
                lstrlenA(drive)+1, size);
     }
 
     /* close the INF again */
     hr = pCloseINFEngine(hinf);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
 
     DeleteFileA(inf_file);
 
@@ -525,7 +525,7 @@ static void translateinfstringex_test(void)
 
     /* open the inf with the install section */
     hr = pOpenINFEngine(inf_file, "section", 0, &hinf, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
 
     /* Single quote test (Note size includes null on return from call) */
     memset(buffer, 'a', APP_PATH_LEN);
@@ -533,15 +533,15 @@ static void translateinfstringex_test(void)
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result2",
                               buffer, size, &size, NULL);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(!lstrcmpiA(buffer, PROG_FILES_ROOT),
            "Expected %s, got %s\n", PROG_FILES_ROOT, buffer);
-    ok(size == strlen(PROG_FILES_ROOT)+1, "Expected size %d, got %d\n",
+    ok(size == strlen(PROG_FILES_ROOT)+1, "Expected size %d, got %ld\n",
            lstrlenA(PROG_FILES_ROOT)+1, size);
 
     /* close the INF again */
     hr = pCloseINFEngine(hinf);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
 
     DeleteFileA(inf_file);
 }
@@ -588,14 +588,14 @@ static void setperusersecvalues_test(void)
         /* This crashes on systems with IE7 */
         hr = pSetPerUserSecValues(NULL);
         todo_wine
-        ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
+        ok(hr == S_OK, "Expected S_OK, got %ld\n", hr);
         ok(!OPEN_GUID_KEY(), "Expected guid key to not exist\n");
     }
 
     /* at the very least, szGUID must be valid */
     peruser.szGUID[0] = '\0';
     hr = pSetPerUserSecValues(&peruser);
-    ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %ld\n", hr);
     ok(!OPEN_GUID_KEY(), "Expected guid key to not exist\n");
 
     /* set initial values */
@@ -606,7 +606,7 @@ static void setperusersecvalues_test(void)
         skip("SetPerUserSecValues is broken\n");
         return;
     }
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(OPEN_GUID_KEY(), "Expected guid key to exist\n");
     ok(check_reg_str(guid, NULL, "displayname"), "Expected displayname\n");
     ok(check_reg_str(guid, "ComponentID", "compid"), "Expected compid\n");
@@ -623,7 +623,7 @@ static void setperusersecvalues_test(void)
     /* raise the version, but bRollback is FALSE, so vals not saved */
     lstrcpyA(peruser.szVersion, "2,1,1,1");
     hr = pSetPerUserSecValues(&peruser);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(check_reg_str(guid, NULL, "displayname"), "Expected displayname\n");
     ok(check_reg_str(guid, "ComponentID", "compid"), "Expected compid\n");
     ok(check_reg_str(guid, "Locale", "locale"), "Expected locale\n");
@@ -640,7 +640,7 @@ static void setperusersecvalues_test(void)
     peruser.bRollback = TRUE;
     lstrcpyA(peruser.szVersion, "3,1,1,1");
     hr = pSetPerUserSecValues(&peruser);
-    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08lx\n", hr);
     ok(check_reg_str(guid, NULL, "displayname"), "Expected displayname\n");
     ok(check_reg_str(guid, "ComponentID", "compid"), "Expected compid\n");
     ok(check_reg_str(guid, "Locale", "locale"), "Expected locale\n");
