@@ -39,9 +39,9 @@ typedef struct
     UCHAR buf[64];
 } SHA256_CTX;
 
-void sha256_init(SHA256_CTX *ctx) DECLSPEC_HIDDEN;
-void sha256_update(SHA256_CTX *ctx, const UCHAR *buffer, ULONG len) DECLSPEC_HIDDEN;
-void sha256_finalize(SHA256_CTX *ctx, UCHAR *buffer) DECLSPEC_HIDDEN;
+void sha256_init(SHA256_CTX *ctx);
+void sha256_update(SHA256_CTX *ctx, const UCHAR *buffer, ULONG len);
+void sha256_finalize(SHA256_CTX *ctx, UCHAR *buffer);
 
 typedef struct
 {
@@ -50,22 +50,22 @@ typedef struct
   UCHAR buf[128];
 } SHA512_CTX;
 
-void sha512_init(SHA512_CTX *ctx) DECLSPEC_HIDDEN;
-void sha512_update(SHA512_CTX *ctx, const UCHAR *buffer, ULONG len) DECLSPEC_HIDDEN;
-void sha512_finalize(SHA512_CTX *ctx, UCHAR *buffer) DECLSPEC_HIDDEN;
+void sha512_init(SHA512_CTX *ctx);
+void sha512_update(SHA512_CTX *ctx, const UCHAR *buffer, ULONG len);
+void sha512_finalize(SHA512_CTX *ctx, UCHAR *buffer);
 
-void sha384_init(SHA512_CTX *ctx) DECLSPEC_HIDDEN;
+void sha384_init(SHA512_CTX *ctx);
 #define sha384_update sha512_update
-void sha384_finalize(SHA512_CTX *ctx, UCHAR *buffer) DECLSPEC_HIDDEN;
+void sha384_finalize(SHA512_CTX *ctx, UCHAR *buffer);
 
 typedef struct {
     unsigned char chksum[16], X[48], buf[16];
     unsigned long curlen;
 } MD2_CTX;
 
-void md2_init(MD2_CTX *ctx) DECLSPEC_HIDDEN;
-void md2_update(MD2_CTX *ctx, const unsigned char *buf, ULONG len) DECLSPEC_HIDDEN;
-void md2_finalize(MD2_CTX *ctx, unsigned char *hash) DECLSPEC_HIDDEN;
+void md2_init(MD2_CTX *ctx);
+void md2_update(MD2_CTX *ctx, const unsigned char *buf, ULONG len);
+void md2_finalize(MD2_CTX *ctx, unsigned char *hash);
 
 /* Definitions from advapi32 */
 typedef struct tagMD4_CTX {
@@ -117,6 +117,7 @@ enum alg_id
     /* cipher */
     ALG_ID_3DES,
     ALG_ID_AES,
+    ALG_ID_RC4,
 
     /* hash */
     ALG_ID_SHA256,
@@ -131,6 +132,7 @@ enum alg_id
     ALG_ID_RSA,
 
     /* secret agreement */
+    ALG_ID_DH,
     ALG_ID_ECDH_P256,
     ALG_ID_ECDH_P384,
 
@@ -196,6 +198,8 @@ struct key
 struct secret
 {
     struct object hdr;
+    struct key *privkey;
+    struct key *pubkey;
 };
 
 struct key_symmetric_set_auth_data_params
@@ -252,8 +256,8 @@ struct key_asymmetric_encrypt_params
 
 struct key_asymmetric_duplicate_params
 {
-    struct key  *key_orig;
-    struct key  *key_copy;
+    struct key *key_orig;
+    struct key *key_copy;
 };
 
 struct key_asymmetric_sign_params
@@ -299,6 +303,15 @@ struct key_asymmetric_import_params
     ULONG        len;
 };
 
+struct key_asymmetric_derive_key_params
+{
+    struct key *privkey;
+    struct key *pubkey;
+    UCHAR      *output;
+    ULONG       output_len;
+    ULONG      *ret_len;
+};
+
 enum key_funcs
 {
     unix_process_attach,
@@ -318,6 +331,8 @@ enum key_funcs
     unix_key_asymmetric_destroy,
     unix_key_asymmetric_export,
     unix_key_asymmetric_import,
+    unix_key_asymmetric_derive_key,
+    unix_funcs_count,
 };
 
 #endif /* __BCRYPT_INTERNAL_H */
