@@ -138,14 +138,12 @@ typedef struct
 #define BUTTON_SPACINGX         3
 #define BUTTON_SPACINGY         3
 #define FLAT_BTN_SPACINGX       8
-#define DEFAULT_MIN_TAB_WIDTH   54
 #define DEFAULT_PADDING_X       6
 #define EXTRA_ICON_PADDING      3
+#define MIN_CHAR_LENGTH         6
 
 #define TAB_GetInfoPtr(hwnd) ((TAB_INFO *)GetWindowLongPtrW(hwnd,0))
 
-#define GET_DEFAULT_MIN_TAB_WIDTH(infoPtr) (DEFAULT_MIN_TAB_WIDTH - (DEFAULT_PADDING_X - (infoPtr)->uHItemPadding) * 2)
-
 /******************************************************************************
  * Hot-tracking timer constants
  */
@@ -203,9 +201,9 @@ static void
 TAB_DumpItemExternalT(const TCITEMW *pti, UINT iItem, BOOL isW)
 {
     if (TRACE_ON(tab)) {
-	TRACE("external tab %d, mask=0x%08x, dwState=0x%08x, dwStateMask=0x%08x, cchTextMax=0x%08x\n",
+	TRACE("external tab %d, mask=0x%08x, dwState %#lx, dwStateMask %#lx, cchTextMax=0x%08x\n",
 	      iItem, pti->mask, pti->dwState, pti->dwStateMask, pti->cchTextMax);
-	TRACE("external tab %d,   iImage=%d, lParam=0x%08lx, pszTextW=%s\n",
+	TRACE("external tab %d, iImage=%d, lParam %Ix, pszTextW=%s\n",
 	      iItem, pti->iImage, pti->lParam, isW ? debugstr_w(pti->pszText) : debugstr_a((LPSTR)pti->pszText));
     }
 }
@@ -216,10 +214,8 @@ TAB_DumpItemInternal(const TAB_INFO *infoPtr, UINT iItem)
     if (TRACE_ON(tab)) {
 	TAB_ITEM *ti = TAB_GetItem(infoPtr, iItem);
 
-	TRACE("tab %d, dwState=0x%08x, pszText=%s, iImage=%d\n",
-	      iItem, ti->dwState, debugstr_w(ti->pszText), ti->iImage);
-	TRACE("tab %d, rect.left=%d, rect.top(row)=%d\n",
-	      iItem, ti->rect.left, ti->rect.top);
+        TRACE("tab %d, dwState %#lx, pszText %s, iImage %d\n", iItem, ti->dwState, debugstr_w(ti->pszText), ti->iImage);
+        TRACE("tab %d, rect.left=%ld, rect.top(row)=%ld\n", iItem, ti->rect.left, ti->rect.top);
     }
 }
 
@@ -273,6 +269,7 @@ static inline LRESULT TAB_SetCurSel (TAB_INFO *infoPtr, INT iItem)
 
       TAB_EnsureSelectionVisible(infoPtr);
       TAB_InvalidateTabArea(infoPtr);
+      NotifyWinEvent(EVENT_OBJECT_SELECTION, infoPtr->hwnd, OBJID_CLIENT, infoPtr->iSelected + 1);
   }
 
   return prevItem;
@@ -288,6 +285,8 @@ static LRESULT TAB_SetCurFocus (TAB_INFO *infoPtr, INT iItem)
           infoPtr->iSelected = -1;
           TAB_SendSimpleNotify(infoPtr, TCN_SELCHANGE);
           TAB_InvalidateTabArea(infoPtr);
+          if (!(infoPtr->dwStyle & TCS_BUTTONS))
+            NotifyWinEvent(EVENT_OBJECT_SELECTION, infoPtr->hwnd, OBJID_CLIENT, 0);
       }
   }
   else if (iItem < infoPtr->uNumItem) {
@@ -308,6 +307,7 @@ static LRESULT TAB_SetCurFocus (TAB_INFO *infoPtr, INT iItem)
             InvalidateRect(infoPtr->hwnd, &r, FALSE);
 
         TAB_SendSimpleNotify(infoPtr, TCN_FOCUSCHANGE);
+        NotifyWinEvent(EVENT_OBJECT_FOCUS, infoPtr->hwnd, OBJID_CLIENT, iItem + 1);
       }
     } else {
       INT oldFocus = infoPtr->uFocus;
@@ -322,6 +322,7 @@ static LRESULT TAB_SetCurFocus (TAB_INFO *infoPtr, INT iItem)
             infoPtr->iSelected = iItem;
           TAB_EnsureSelectionVisible(infoPtr);
           TAB_InvalidateTabArea(infoPtr);
+          NotifyWinEvent(EVENT_OBJECT_SELECTION, infoPtr->hwnd, OBJID_CLIENT, iItem + 1);
         }
       }
     }
@@ -929,8 +930,7 @@ static LRESULT TAB_AdjustRect(const TAB_INFO *infoPtr, WPARAM fLarger, LPRECT pr
 {
     LONG *iRightBottom, *iLeftTop;
 
-    TRACE ("hwnd=%p fLarger=%ld (%s)\n", infoPtr->hwnd, fLarger,
-           wine_dbgstr_rect(prc));
+    TRACE("hwnd %p, fLarger %Id, (%s)\n", infoPtr->hwnd, fLarger, wine_dbgstr_rect(prc));
 
     if (!prc) return -1;
 
@@ -1099,8 +1099,10 @@ static void TAB_SetupScrolling(
  * It also uses the current font to determine the height of the tab row and
  * it checks if all the tabs fit in the client area of the window. If they
  * don't, a scrolling control is added.
+ *
+ * Returns the default minimum tab width
  */
-static void TAB_SetItemBounds (TAB_INFO *infoPtr)
+static INT TAB_SetItemBounds (TAB_INFO *infoPtr)
 {
   TEXTMETRICW fontMetrics;
   UINT        curItem;
@@ -1113,6 +1115,8 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
   RECT*       rcItem;
   INT         iIndex;
   INT         icon_width = 0;
+  INT         default_min_tab_width;
+  TEXTMETRICW text_metrics;
 
   /*
    * We need to get text information so we need a DC and we need to select
@@ -1142,6 +1146,9 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
   /* Now use hPadding and vPadding */
   infoPtr->uHItemPadding = infoPtr->uHItemPadding_s;
   infoPtr->uVItemPadding = infoPtr->uVItemPadding_s;
+
+  GetTextMetricsW(hdc, &text_metrics);
+  default_min_tab_width = text_metrics.tmAveCharWidth * MIN_CHAR_LENGTH + infoPtr->uHItemPadding * 2;
   
   /* The leftmost item will be "0" aligned */
   curItemLeftPos = 0;
@@ -1173,11 +1180,10 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
 	                 ((infoPtr->dwStyle & TCS_BUTTONS) ? 2 : 1) *
                           infoPtr->uVItemPadding;
 
-    TRACE("tabH=%d, tmH=%d, iconh=%d\n",
-	  infoPtr->tabHeight, fontMetrics.tmHeight, icon_height);
+    TRACE("tabH=%d, tmH %ld, iconh %d\n", infoPtr->tabHeight, fontMetrics.tmHeight, icon_height);
   }
 
-  TRACE("client right=%d\n", clientRect.right);
+  TRACE("client right %ld\n", clientRect.right);
 
   /* Get the icon width */
   if (infoPtr->himl)
@@ -1209,13 +1215,13 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
     {
       /* If no text use minimum tab width including padding. */
       if (infoPtr->tabMinWidth < 0)
-        curr->rect.right = curr->rect.left + GET_DEFAULT_MIN_TAB_WIDTH(infoPtr);
+        curr->rect.right = curr->rect.left + default_min_tab_width;
       else
       {
         curr->rect.right = curr->rect.left + infoPtr->tabMinWidth;
 
         /* Add extra padding if icon is present */
-        if (infoPtr->himl && infoPtr->tabMinWidth > 0 && infoPtr->tabMinWidth < DEFAULT_MIN_TAB_WIDTH
+        if (infoPtr->himl && infoPtr->tabMinWidth > 0 && infoPtr->tabMinWidth < MIN_CHAR_LENGTH * text_metrics.tmAveCharWidth + DEFAULT_PADDING_X * 2
             && infoPtr->uHItemPadding > 1)
           curr->rect.right += EXTRA_ICON_PADDING * (infoPtr->uHItemPadding-1);
       }
@@ -1231,7 +1237,7 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
       tabwidth = size.cx + icon_width + 2 * infoPtr->uHItemPadding;
 
       if (infoPtr->tabMinWidth < 0)
-        tabwidth = max(tabwidth, GET_DEFAULT_MIN_TAB_WIDTH(infoPtr));
+        tabwidth = max(tabwidth, default_min_tab_width);
       else
         tabwidth = max(tabwidth, infoPtr->tabMinWidth);
 
@@ -1331,7 +1337,7 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
           curr->rect.right -= curr->rect.left;
           curr->rect.left = 0;
 
-          TRACE("r=%d, cl=%d, cl.r=%d, iCount=%d, iRow=%d, uNumRows=%d, remTab=%d, tabPerRow=%d\n",
+          TRACE("r=%ld, cl=%d, cl.r=%ld, iCount=%d, iRow=%d, uNumRows=%d, remTab=%d, tabPerRow=%d\n",
 	      curr->rect.right, curItemLeftPos, clientRect.right,
 	      iCount, iRow, infoPtr->uNumRows, remTab, tabPerRow);
 
@@ -1460,6 +1466,8 @@ static void TAB_SetItemBounds (TAB_INFO *infoPtr)
   /* Cleanup */
   SelectObject (hdc, hOldFont);
   ReleaseDC (infoPtr->hwnd, hdc);
+
+  return default_min_tab_width;
 }
 
 
@@ -1796,8 +1804,8 @@ TAB_DrawItemInterior(const TAB_INFO *infoPtr, HDC hdc, INT iItem, RECT *drawRect
 	
       if (center_offset_v < 0)
         center_offset_v = 0;
-	
-      TRACE("for <%s>, c_o_h=%d, c_o_v=%d, draw=(%s), textlen=%d\n",
+
+      TRACE("for <%s>, c_o_h=%d, c_o_v=%d, draw=(%s), textlen=%ld\n",
 	  debugstr_w(item->pszText), center_offset_h, center_offset_v,
           wine_dbgstr_rect(drawRect), (rcText.right-rcText.left));
 
@@ -1823,8 +1831,7 @@ TAB_DrawItemInterior(const TAB_INFO *infoPtr, HDC hdc, INT iItem, RECT *drawRect
         drawRect->left += cx + infoPtr->uHItemPadding;
       }
 
-      TRACE("drawing image=%d, left=%d, top=%d\n",
-	    item->iImage, rcImage.left, rcImage.top-1);
+      TRACE("drawing image %d, left %ld, top %ld\n", item->iImage, rcImage.left, rcImage.top-1);
       ImageList_Draw
         (
         infoPtr->himl,
@@ -1914,7 +1921,7 @@ TAB_DrawItemInterior(const TAB_INFO *infoPtr, HDC hdc, INT iItem, RECT *drawRect
     }
     else
     {
-      TRACE("for <%s>, c_o_h=%d, c_o_v=%d, draw=(%s), textlen=%d\n",
+      TRACE("for <%s>, c_o_h=%d, c_o_v=%d, draw=(%s), textlen=%ld\n",
 	  debugstr_w(item->pszText), center_offset_h, center_offset_v,
           wine_dbgstr_rect(drawRect), (rcText.right-rcText.left));
       if (item->pszText)
@@ -2320,9 +2327,14 @@ static void TAB_DrawBorder(const TAB_INFO *infoPtr, HDC hdc)
   TRACE("border=(%s)\n", wine_dbgstr_rect(&rect));
 
   if (theme)
+  {
+      DrawThemeParentBackground(infoPtr->hwnd, hdc, &rect);
       DrawThemeBackground (theme, hdc, TABP_PANE, 0, &rect, NULL);
+  }
   else
+  {
       DrawEdge(hdc, &rect, EDGE_RAISED, BF_SOFT|BF_RECT);
+  }
 }
 
 /******************************************************************************
@@ -2635,6 +2647,9 @@ TAB_InsertItemT (TAB_INFO *infoPtr, INT iItem, const TCITEMW *pti, BOOL bUnicode
   else
     InvalidateRect(infoPtr->hwnd, NULL, TRUE);
 
+  /* The last item is always the "new" MSAA object. */
+  NotifyWinEvent(EVENT_OBJECT_CREATE, infoPtr->hwnd, OBJID_CLIENT, infoPtr->uNumItem);
+
   TRACE("[%p]: added item %d %s\n",
         infoPtr->hwnd, iItem, debugstr_w(item->pszText));
 
@@ -2654,19 +2669,20 @@ TAB_SetItemSize (TAB_INFO *infoPtr, INT cx, INT cy)
   lResult = MAKELONG(infoPtr->tabWidth, infoPtr->tabHeight);
 
   /* UNDOCUMENTED: If requested Width or Height is 0 this means that program wants to use auto size. */
-  if (infoPtr->dwStyle & TCS_FIXEDWIDTH && (infoPtr->tabWidth != cx))
+  if (infoPtr->tabWidth != cx)
   {
     infoPtr->tabWidth = cx;
     bNeedPaint = TRUE;
   }
 
-  if (infoPtr->tabHeight != cy)
+  if (infoPtr->tabHeight != cy && cy != 0)
   {
-    if ((infoPtr->fHeightSet = (cy != 0)))
-      infoPtr->tabHeight = cy;
-
+    infoPtr->tabHeight = cy;
     bNeedPaint = TRUE;
   }
+
+  infoPtr->fHeightSet = (cy != 0);
+
   TRACE("was h=%d,w=%d, now h=%d,w=%d\n",
        HIWORD(lResult), LOWORD(lResult),
        infoPtr->tabHeight, infoPtr->tabWidth);
@@ -2682,17 +2698,14 @@ TAB_SetItemSize (TAB_INFO *infoPtr, INT cx, INT cy)
 
 static inline LRESULT TAB_SetMinTabWidth (TAB_INFO *infoPtr, INT cx)
 {
-  INT oldcx = 0;
+  INT default_min_tab_width;
+  INT prevMinWidth = infoPtr->tabMinWidth;
+  infoPtr->tabMinWidth = cx;
 
   TRACE("(%p,%d)\n", infoPtr, cx);
 
-  if (infoPtr->tabMinWidth < 0)
-    oldcx = DEFAULT_MIN_TAB_WIDTH;
-  else
-    oldcx = infoPtr->tabMinWidth;
-  infoPtr->tabMinWidth = cx;
-  TAB_SetItemBounds(infoPtr);
-  return oldcx;
+  default_min_tab_width = TAB_SetItemBounds(infoPtr);
+  return (prevMinWidth < 0) ? default_min_tab_width : prevMinWidth;
 }
 
 static inline LRESULT 
@@ -2861,6 +2874,9 @@ static LRESULT TAB_DeleteItem (TAB_INFO *infoPtr, INT iItem)
     /* reposition and repaint tabs */
     TAB_SetItemBounds(infoPtr);
 
+    /* The last item is always the destroyed MSAA object */
+    NotifyWinEvent(EVENT_OBJECT_DESTROY, infoPtr->hwnd, OBJID_CLIENT, infoPtr->uNumItem + 1);
+
     return TRUE;
 }
 
@@ -3052,8 +3068,7 @@ static LRESULT TAB_Create (HWND hwnd, LPARAM lParam)
                         infoPtr->uVItemPadding;
 
   /* Initialize the width of a tab. */
-  if (infoPtr->dwStyle & TCS_FIXEDWIDTH)
-    infoPtr->tabWidth = GetDeviceCaps(hdc, LOGPIXELSX);
+  infoPtr->tabWidth = GetDeviceCaps(hdc, LOGPIXELSX);
 
   infoPtr->tabMinWidth = -1;
 
@@ -3240,15 +3255,13 @@ TAB_DeselectAll (TAB_INFO *infoPtr, BOOL excludesel)
 static INT TAB_StyleChanged(TAB_INFO *infoPtr, WPARAM wStyleType,
                             const STYLESTRUCT *lpss)
 {
-    TRACE("(styletype=%lx, styleOld=0x%08x, styleNew=0x%08x)\n",
-          wStyleType, lpss->styleOld, lpss->styleNew);
+    TRACE("style type %Ix, styleOld %#lx, styleNew %#lx\n", wStyleType, lpss->styleOld, lpss->styleNew);
 
     if (wStyleType != GWL_STYLE) return 0;
 
     infoPtr->dwStyle = lpss->styleNew;
 
     TAB_SetItemBounds (infoPtr);
-    InvalidateRect(infoPtr->hwnd, NULL, TRUE);
 
     return 0;
 }
@@ -3258,7 +3271,8 @@ TAB_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
     TAB_INFO *infoPtr = TAB_GetInfoPtr(hwnd);
 
-    TRACE("hwnd=%p msg=%x wParam=%lx lParam=%lx\n", hwnd, uMsg, wParam, lParam);
+    TRACE("hwnd %p, msg %x, wParam %Ix, lParam %Ix\n", hwnd, uMsg, wParam, lParam);
+
     if (!infoPtr && (uMsg != WM_CREATE))
       return DefWindowProcW (hwnd, uMsg, wParam, lParam);
 
@@ -3425,8 +3439,7 @@ TAB_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 
     default:
       if (uMsg >= WM_USER && uMsg < WM_APP && !COMCTL32_IsReflectedMessage(uMsg))
-	WARN("unknown msg %04x wp=%08lx lp=%08lx\n",
-	     uMsg, wParam, lParam);
+          WARN("unknown msg %04x wp %Ix, lp %Ix\n", uMsg, wParam, lParam);
       break;
     }
     return DefWindowProcW(hwnd, uMsg, wParam, lParam);
