@@ -27,14 +27,16 @@
 #include "v6util.h"
 #include "msg.h"
 
-#define EDITBOX_SEQ_INDEX  0
-#define NUM_MSG_SEQUENCES  1
+enum message_seq_index
+{
+    EDITBOX_SEQ_INDEX = 0,
+    PARENT_SEQ_INDEX,
+    NUM_MSG_SEQUENCES,
+};
 
 #define EDITBOX_ID         0
 #define COMBO_ID           1995
 
-#define expect(expected, got) ok(got == expected, "Expected %d, got %d\n", expected, got)
-
 #define expect_rect(r, _left, _top, _right, _bottom) ok(r.left == _left && r.top == _top && \
     r.bottom == _bottom && r.right == _right, "Invalid rect %s vs (%d,%d)-(%d,%d)\n", \
     wine_dbgstr_rect(&r), _left, _top, _right, _bottom);
@@ -61,7 +63,7 @@ static void get_combobox_info(HWND hwnd, COMBOBOXINFO *info)
 
     info->cbSize = sizeof(*info);
     ret = GetComboBoxInfo(hwnd, info);
-    ok(ret, "Failed to get combobox info structure, error %d\n", GetLastError());
+    ok(ret, "Failed to get combobox info structure, error %ld\n", GetLastError());
 }
 
 static HWND createComboEx(DWORD style) {
@@ -159,26 +161,26 @@ static void test_comboex(void)
                 *out_of_range_item = "Out of Range Item";
 
     /* Allocate space for result */
-    textBuffer = heap_alloc(MAX_CHARS);
+    textBuffer = malloc(MAX_CHARS);
 
     /* Basic comboboxex test */
     myHwnd = createComboEx(WS_BORDER | WS_VISIBLE | WS_CHILD | CBS_DROPDOWN);
 
     /* Add items onto the end of the combobox */
     res = addItem(myHwnd, -1, first_item);
-    ok(res == 0, "Adding simple item failed (%d)\n", res);
+    ok(res == 0, "Adding simple item failed (%ld)\n", res);
     res = addItem(myHwnd, -1, second_item);
-    ok(res == 1, "Adding simple item failed (%d)\n", res);
+    ok(res == 1, "Adding simple item failed (%ld)\n", res);
     res = addItem(myHwnd, 2, third_item);
-    ok(res == 2, "Adding simple item failed (%d)\n", res);
+    ok(res == 2, "Adding simple item failed (%ld)\n", res);
     res = addItem(myHwnd, 1, middle_item);
-    ok(res == 1, "Inserting simple item failed (%d)\n", res);
+    ok(res == 1, "Inserting simple item failed (%ld)\n", res);
 
     /* Add an item completely out of range */
     res = addItem(myHwnd, 99, out_of_range_item);
-    ok(res == -1, "Adding using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == -1, "Adding using out of range index worked unexpectedly (%ld)\n", res);
     res = addItem(myHwnd, 5, out_of_range_item);
-    ok(res == -1, "Adding using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == -1, "Adding using out of range index worked unexpectedly (%ld)\n", res);
     /* Removed: Causes traps on Windows XP
        res = addItem(myHwnd, -2, "Out Of Range Item");
        ok(res == -1, "Adding out of range worked unexpectedly (%ld)\n", res);
@@ -186,66 +188,66 @@ static void test_comboex(void)
 
     /* Get an item completely out of range */ 
     res = getItem(myHwnd, 99, &cbexItem); 
-    ok(res == 0, "Getting item using out of range index worked unexpectedly (%d, %s)\n", res, cbexItem.pszText);
+    ok(res == 0, "Getting item using out of range index worked unexpectedly (%ld, %s)\n", res, cbexItem.pszText);
     res = getItem(myHwnd, 4, &cbexItem); 
-    ok(res == 0, "Getting item using out of range index worked unexpectedly (%d, %s)\n", res, cbexItem.pszText);
+    ok(res == 0, "Getting item using out of range index worked unexpectedly (%ld, %s)\n", res, cbexItem.pszText);
     res = getItem(myHwnd, -2, &cbexItem); 
-    ok(res == 0, "Getting item using out of range index worked unexpectedly (%d, %s)\n", res, cbexItem.pszText);
+    ok(res == 0, "Getting item using out of range index worked unexpectedly (%ld, %s)\n", res, cbexItem.pszText);
 
     /* Get an item in range */ 
     res = getItem(myHwnd, 0, &cbexItem); 
-    ok(res != 0, "Getting item using valid index failed unexpectedly (%d)\n", res);
+    ok(res != 0, "Getting item using valid index failed unexpectedly (%ld)\n", res);
     ok(strcmp(first_item, cbexItem.pszText) == 0, "Getting item returned wrong string (%s)\n", cbexItem.pszText);
 
     res = getItem(myHwnd, 1, &cbexItem); 
-    ok(res != 0, "Getting item using valid index failed unexpectedly (%d)\n", res);
+    ok(res != 0, "Getting item using valid index failed unexpectedly (%ld)\n", res);
     ok(strcmp(middle_item, cbexItem.pszText) == 0, "Getting item returned wrong string (%s)\n", cbexItem.pszText);
 
     res = getItem(myHwnd, 2, &cbexItem); 
-    ok(res != 0, "Getting item using valid index failed unexpectedly (%d)\n", res);
+    ok(res != 0, "Getting item using valid index failed unexpectedly (%ld)\n", res);
     ok(strcmp(second_item, cbexItem.pszText) == 0, "Getting item returned wrong string (%s)\n", cbexItem.pszText);
 
     res = getItem(myHwnd, 3, &cbexItem); 
-    ok(res != 0, "Getting item using valid index failed unexpectedly (%d)\n", res);
+    ok(res != 0, "Getting item using valid index failed unexpectedly (%ld)\n", res);
     ok(strcmp(third_item, cbexItem.pszText) == 0, "Getting item returned wrong string (%s)\n", cbexItem.pszText);
 
     /* Set an item completely out of range */ 
     res = setItem(myHwnd, 99, replacement_item); 
-    ok(res == 0, "Setting item using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == 0, "Setting item using out of range index worked unexpectedly (%ld)\n", res);
     res = setItem(myHwnd, 4, replacement_item); 
-    ok(res == 0, "Setting item using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == 0, "Setting item using out of range index worked unexpectedly (%ld)\n", res);
     res = setItem(myHwnd, -2, replacement_item); 
-    ok(res == 0, "Setting item using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == 0, "Setting item using out of range index worked unexpectedly (%ld)\n", res);
 
     /* Set an item in range */ 
     res = setItem(myHwnd, 0, replacement_item);
-    ok(res != 0, "Setting first item failed (%d)\n", res);
+    ok(res != 0, "Setting first item failed (%ld)\n", res);
     res = setItem(myHwnd, 3, replacement_item);
-    ok(res != 0, "Setting last item failed (%d)\n", res);
+    ok(res != 0, "Setting last item failed (%ld)\n", res);
 
     /* Remove items completely out of range (4 items in control at this point) */
     res = delItem(myHwnd, -1);
-    ok(res == CB_ERR, "Deleting using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == CB_ERR, "Deleting using out of range index worked unexpectedly (%ld)\n", res);
     res = delItem(myHwnd, 4);
-    ok(res == CB_ERR, "Deleting using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == CB_ERR, "Deleting using out of range index worked unexpectedly (%ld)\n", res);
 
     /* Remove items in range (4 items in control at this point) */
     res = delItem(myHwnd, 3);
-    ok(res == 3, "Deleting using out of range index failed (%d)\n", res);
+    ok(res == 3, "Deleting using out of range index failed (%ld)\n", res);
     res = delItem(myHwnd, 0);
-    ok(res == 2, "Deleting using out of range index failed (%d)\n", res);
+    ok(res == 2, "Deleting using out of range index failed (%ld)\n", res);
     res = delItem(myHwnd, 0);
-    ok(res == 1, "Deleting using out of range index failed (%d)\n", res);
+    ok(res == 1, "Deleting using out of range index failed (%ld)\n", res);
     res = delItem(myHwnd, 0);
-    ok(res == 0, "Deleting using out of range index failed (%d)\n", res);
+    ok(res == 0, "Deleting using out of range index failed (%ld)\n", res);
 
     /* Remove from an empty box */
     res = delItem(myHwnd, 0);
-    ok(res == CB_ERR, "Deleting using out of range index worked unexpectedly (%d)\n", res);
+    ok(res == CB_ERR, "Deleting using out of range index worked unexpectedly (%ld)\n", res);
 
 
     /* Cleanup */
-    heap_free(textBuffer);
+    free(textBuffer);
     DestroyWindow(myHwnd);
 }
 
@@ -292,7 +294,7 @@ static void test_comboex_WM_LBUTTONDOWN(void)
     x = cbInfo.rcButton.left + (cbInfo.rcButton.right-cbInfo.rcButton.left)/2;
     y = cbInfo.rcButton.top + (cbInfo.rcButton.bottom-cbInfo.rcButton.top)/2;
     result = SendMessageA(hCombo, WM_LBUTTONDOWN, 0, MAKELPARAM(x, y));
-    ok(result, "WM_LBUTTONDOWN was not processed. LastError=%d\n",
+    ok(result, "WM_LBUTTONDOWN was not processed. LastError=%ld\n",
        GetLastError());
     ok(GetFocus() == hCombo ||
        broken(GetFocus() != hCombo), /* win98 */
@@ -304,7 +306,7 @@ static void test_comboex_WM_LBUTTONDOWN(void)
     ok(idx == 0, "For TopIndex expected %d, got %d\n", 0, idx);
 
     result = SendMessageA(hCombo, WM_LBUTTONUP, 0, MAKELPARAM(x, y));
-    ok(result, "WM_LBUTTONUP was not processed. LastError=%d\n",
+    ok(result, "WM_LBUTTONUP was not processed. LastError=%ld\n",
        GetLastError());
     ok(GetFocus() == hCombo ||
        broken(GetFocus() != hCombo), /* win98 */
@@ -317,7 +319,7 @@ static void test_comboex_WM_LBUTTONDOWN(void)
     x = rect.left + (rect.right-rect.left)/2;
     y = item_height/2 + item_height*4;
     result = SendMessageA(hList, WM_MOUSEMOVE, 0, MAKELPARAM(x, y));
-    ok(!result, "WM_MOUSEMOVE was not processed. LastError=%d\n",
+    ok(!result, "WM_MOUSEMOVE was not processed. LastError=%ld\n",
        GetLastError());
     ok(GetFocus() == hCombo ||
        broken(GetFocus() != hCombo), /* win98 */
@@ -325,7 +327,7 @@ static void test_comboex_WM_LBUTTONDOWN(void)
        GetFocus());
 
     result = SendMessageA(hList, WM_LBUTTONDOWN, 0, MAKELPARAM(x, y));
-    ok(!result, "WM_LBUTTONDOWN was not processed. LastError=%d\n",
+    ok(!result, "WM_LBUTTONDOWN was not processed. LastError=%ld\n",
        GetLastError());
     ok(GetFocus() == hCombo ||
        broken(GetFocus() != hCombo), /* win98 */
@@ -335,7 +337,7 @@ static void test_comboex_WM_LBUTTONDOWN(void)
        "The dropdown list should still be visible.\n");
 
     result = SendMessageA(hList, WM_LBUTTONUP, 0, MAKELPARAM(x, y));
-    ok(!result, "WM_LBUTTONUP was not processed. LastError=%d\n",
+    ok(!result, "WM_LBUTTONUP was not processed. LastError=%ld\n",
        GetLastError());
     todo_wine ok(GetFocus() == hEdit ||
        broken(GetFocus() == hCombo), /* win98 */
@@ -442,10 +444,29 @@ static void test_comboex_WM_WINDOWPOSCHANGING(void)
     ok(ret, "DestroyWindow failed\n");
 }
 
-static LRESULT ComboExTestOnNotify(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
+struct di_context
 {
+    unsigned int mask;
+    BOOL set_CBEIF_DI_SETITEM;
+};
+
+static struct di_context di_context;
+
+static LRESULT ComboExTestOnNotify(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
+{
+    struct message msg;
     NMHDR *hdr = (NMHDR*)lParam;
-    switch(hdr->code){
+
+    msg.message = message;
+    msg.flags = sent|wparam|lparam;
+    msg.wParam = wParam;
+    msg.lParam = lParam;
+    if (hdr) msg.id = hdr->code;
+
+    add_message(sequences, PARENT_SEQ_INDEX, &msg);
+
+    switch (hdr->code)
+    {
     case CBEN_ENDEDITA:
         {
             NMCBEENDEDITA *edit_info = (NMCBEENDEDITA*)hdr;
@@ -462,6 +483,42 @@ static LRESULT ComboExTestOnNotify(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lP
             }
             break;
         }
+    case CBEN_GETDISPINFOA:
+    case CBEN_GETDISPINFOW:
+        {
+            NMCOMBOBOXEXA *item = (NMCOMBOBOXEXA *)hdr;
+
+            di_context.mask = item->ceItem.mask;
+
+            if (item->ceItem.mask & CBEIF_IMAGE)
+            {
+                ok(item->ceItem.iImage == I_IMAGECALLBACK, "Unexpected iImage %d.\n", item->ceItem.iImage);
+                item->ceItem.iImage = 123;
+            }
+
+            if (item->ceItem.mask & CBEIF_TEXT)
+            {
+                ok(item->ceItem.pszText && item->ceItem.pszText != LPSTR_TEXTCALLBACKA,
+                        "Unexpected pszText %p.\n", item->ceItem.pszText);
+                ok(item->ceItem.cchTextMax == 0, "Unexpected cchTextMax %d.\n", item->ceItem.cchTextMax);
+            }
+
+            if (item->ceItem.mask & CBEIF_SELECTEDIMAGE)
+                ok(item->ceItem.iSelectedImage == I_IMAGECALLBACK, "Unexpected iSelectedImage %d.\n",
+                        item->ceItem.iSelectedImage);
+
+            if (item->ceItem.mask & CBEIF_OVERLAY)
+                ok(item->ceItem.iOverlay == I_IMAGECALLBACK, "Unexpected iOverlay %d.\n",
+                        item->ceItem.iOverlay);
+
+            if (item->ceItem.mask & CBEIF_INDENT)
+                ok(item->ceItem.iIndent == 0, "Unexpected iIndent %d.\n", item->ceItem.iIndent);
+
+            if (di_context.set_CBEIF_DI_SETITEM)
+                item->ceItem.mask |= CBEIF_DI_SETITEM;
+
+            break;
+        }
     }
     return 0;
 }
@@ -572,7 +629,7 @@ static void test_comboex_get_set_item(void)
     char textA[] = "test";
     HWND hComboEx;
     COMBOBOXEXITEMA item;
-    BOOL ret;
+    DWORD ret;
 
     hComboEx = createComboEx(WS_BORDER | WS_VISIBLE | WS_CHILD | CBS_DROPDOWN);
 
@@ -585,7 +642,7 @@ static void test_comboex_get_set_item(void)
     item.pszText = textA;
     item.iItem = -1;
     ret = SendMessageA(hComboEx, CBEM_SETITEMA, 0, (LPARAM)&item);
-    expect(TRUE, ret);
+    ok(ret == 1, "Unexpected return value %ld.\n", ret);
 
     ok_sequence(sequences, EDITBOX_SEQ_INDEX, test_setitem_edit_seq, "set item data for edit", FALSE);
 
@@ -594,17 +651,17 @@ static void test_comboex_get_set_item(void)
     item.iItem = -1;
     item.lParam = 0xdeadbeef;
     ret = SendMessageA(hComboEx, CBEM_GETITEMA, 0, (LPARAM)&item);
-    expect(TRUE, ret);
-    ok(item.lParam == 0, "Expected zero, got %lx\n", item.lParam);
+    ok(ret == 1, "Unexpected return value %ld.\n", ret);
+    ok(item.lParam == 0, "Expected zero, got %Ix\n", item.lParam);
 
     item.lParam = 0x1abe11ed;
     ret = SendMessageA(hComboEx, CBEM_SETITEMA, 0, (LPARAM)&item);
-    expect(TRUE, ret);
+    ok(ret == 1, "Unexpected return value %ld.\n", ret);
 
     item.lParam = 0;
     ret = SendMessageA(hComboEx, CBEM_GETITEMA, 0, (LPARAM)&item);
-    expect(TRUE, ret);
-    ok(item.lParam == 0x1abe11ed, "Expected 0x1abe11ed, got %lx\n", item.lParam);
+    ok(ret == 1, "Unexpected return value %ld.\n", ret);
+    ok(item.lParam == 0x1abe11ed, "Expected 0x1abe11ed, got %Ix\n", item.lParam);
 
     DestroyWindow(hComboEx);
 }
@@ -658,7 +715,7 @@ static void test_combo_setitemheight(DWORD style)
     font_height = get_font_height(hFont);
     SendMessageA(hCombo, CB_SETITEMHEIGHT, -1, font_height / 2);
     height = SendMessageA(hCombo, CB_GETITEMHEIGHT, -1, 0);
-todo_wine
+    todo_wine
     ok(height == font_height / 2, "Unexpected item height %d, expected %d.\n", height, font_height / 2);
 
     SetWindowPos(hCombo, NULL, 10, 10, 150, 5 * font_height, SWP_SHOWWINDOW);
@@ -874,29 +931,29 @@ static void test_combo_changesize(DWORD style)
     /* first make it slightly smaller */
     MoveWindow( hCombo, 10, 10, clwidth - 2, clheight - 2, TRUE);
     GetClientRect( hCombo, &rc);
-    ok( rc.right - rc.left == clwidth - 2, "clientrect width is %d vs %d\n",
+    ok( rc.right - rc.left == clwidth - 2, "clientrect width is %ld vs %d\n",
             rc.right - rc.left, clwidth - 2);
-    ok( rc.bottom - rc.top == clheight, "clientrect height is %d vs %d\n",
+    ok( rc.bottom - rc.top == clheight, "clientrect height is %ld vs %d\n",
                 rc.bottom - rc.top, clheight);
     SendMessageA(hCombo, CB_GETDROPPEDCONTROLRECT, 0, (LPARAM)&rc);
-    ok( rc.right - rc.left == clwidth - 2, "drop-down rect width is %d vs %d\n",
+    ok( rc.right - rc.left == clwidth - 2, "drop-down rect width is %ld vs %d\n",
             rc.right - rc.left, clwidth - 2);
-    ok( rc.bottom - rc.top == ddheight, "drop-down rect height is %d vs %d\n",
+    ok( rc.bottom - rc.top == ddheight, "drop-down rect height is %ld vs %d\n",
             rc.bottom - rc.top, ddheight);
-    ok( rc.right - rc.left == ddwidth -2, "drop-down rect width is %d vs %d\n",
+    ok( rc.right - rc.left == ddwidth -2, "drop-down rect width is %ld vs %d\n",
             rc.right - rc.left, ddwidth - 2);
     /* new cx, cy is slightly bigger than the initial values */
     MoveWindow( hCombo, 10, 10, clwidth + 2, clheight + 2, TRUE);
     GetClientRect( hCombo, &rc);
-    ok( rc.right - rc.left == clwidth + 2, "clientrect width is %d vs %d\n",
+    ok( rc.right - rc.left == clwidth + 2, "clientrect width is %ld vs %d\n",
             rc.right - rc.left, clwidth + 2);
-    ok( rc.bottom - rc.top == clheight, "clientrect height is %d vs %d\n",
+    ok( rc.bottom - rc.top == clheight, "clientrect height is %ld vs %d\n",
             rc.bottom - rc.top, clheight);
     SendMessageA(hCombo, CB_GETDROPPEDCONTROLRECT, 0, (LPARAM)&rc);
-    ok( rc.right - rc.left == clwidth + 2, "drop-down rect width is %d vs %d\n",
+    ok( rc.right - rc.left == clwidth + 2, "drop-down rect width is %ld vs %d\n",
             rc.right - rc.left, clwidth + 2);
     todo_wine {
-        ok( rc.bottom - rc.top == clheight + 2, "drop-down rect height is %d vs %d\n",
+        ok( rc.bottom - rc.top == clheight + 2, "drop-down rect height is %ld vs %d\n",
                 rc.bottom - rc.top, clheight + 2);
     }
 
@@ -1136,8 +1193,8 @@ static void test_combo_listbox_styles(DWORD cb_style)
 
     style = GetWindowLongW( info.hwndList, GWL_STYLE );
     exstyle = GetWindowLongW( info.hwndList, GWL_EXSTYLE );
-    ok(style == expect_style, "%08x: got %08x\n", cb_style, style);
-    ok(exstyle == expect_exstyle, "%08x: got %08x\n", cb_style, exstyle);
+    ok(style == expect_style, "%08lx: got %08lx\n", cb_style, style);
+    ok(exstyle == expect_exstyle, "%08lx: got %08lx\n", cb_style, exstyle);
 
     if (cb_style != CBS_SIMPLE)
         expect_exstyle |= WS_EX_TOPMOST;
@@ -1145,14 +1202,14 @@ static void test_combo_listbox_styles(DWORD cb_style)
     SendMessageW(combo, CB_SHOWDROPDOWN, TRUE, 0 );
     style = GetWindowLongW( info.hwndList, GWL_STYLE );
     exstyle = GetWindowLongW( info.hwndList, GWL_EXSTYLE );
-    ok(style == (expect_style | WS_VISIBLE), "%08x: got %08x\n", cb_style, style);
-    ok(exstyle == expect_exstyle, "%08x: got %08x\n", cb_style, exstyle);
+    ok(style == (expect_style | WS_VISIBLE), "%08lx: got %08lx\n", cb_style, style);
+    ok(exstyle == expect_exstyle, "%08lx: got %08lx\n", cb_style, exstyle);
 
     SendMessageW(combo, CB_SHOWDROPDOWN, FALSE, 0 );
     style = GetWindowLongW( info.hwndList, GWL_STYLE );
     exstyle = GetWindowLongW( info.hwndList, GWL_EXSTYLE );
-    ok(style == expect_style, "%08x: got %08x\n", cb_style, style);
-    ok(exstyle == expect_exstyle, "%08x: got %08x\n", cb_style, exstyle);
+    ok(style == expect_style, "%08lx: got %08lx\n", cb_style, style);
+    ok(exstyle == expect_exstyle, "%08lx: got %08lx\n", cb_style, exstyle);
 
     DestroyWindow(combo);
 }
@@ -1215,6 +1272,7 @@ static void test_combo_dropdown_size(DWORD style)
         RECT rect_list_client;
         int min_visible_expected;
 
+        winetest_push_context("Test %d", test);
         hCombo = CreateWindowA(WC_COMBOBOXA, "Combo", CBS_DROPDOWN | WS_VISIBLE | WS_CHILD | style, 5, 5, 100,
                 info_test->height_combo, hMainWnd, (HMENU)COMBO_ID, NULL, 0);
 
@@ -1240,8 +1298,8 @@ static void test_combo_dropdown_size(DWORD style)
             ret = SendMessageA(hCombo, CB_SETMINVISIBLE, min_visible_expected, 0);
             ok(ret, "Failed to set visible limit.\n");
             min_visible_actual = SendMessageA(hCombo, CB_GETMINVISIBLE, 0, 0);
-            ok(min_visible_expected == min_visible_actual, "test %d: unexpected number of items %d.\n",
-                    test, min_visible_actual);
+            ok(min_visible_expected == min_visible_actual, "unexpected number of items %d.\n",
+                    min_visible_actual);
         }
 
         ret = SendMessageA(hCombo, CB_SHOWDROPDOWN, TRUE,0);
@@ -1276,18 +1334,19 @@ static void test_combo_dropdown_size(DWORD style)
             if (expected_height_list < 0)
                 expected_height_list = 0;
 
-            ok(expected_height_list == height_list, "Test %d, expected list height to be %d, got %d\n",
-                    test, expected_height_list, height_list);
+            ok(expected_height_list == height_list, "expected list height to be %d, got %d\n",
+                    expected_height_list, height_list);
         }
         else
         {
             expected_height_list = min(info_test->num_items, min_visible_expected) * height_item;
 
-            ok(expected_height_list == height_list, "Test %d, expected list height to be %d, got %d\n",
-                    test, expected_height_list, height_list);
+            ok(expected_height_list == height_list, "expected list height to be %d, got %d\n",
+                    expected_height_list, height_list);
         }
 
         DestroyWindow(hCombo);
+        winetest_pop_context();
     }
 }
 
@@ -1365,6 +1424,96 @@ static void test_combo_ctlcolor(void)
     DestroyWindow(combo);
 }
 
+static const struct message getdisp_parent_seq[] =
+{
+    { WM_NOTIFY, sent|id, 0, 0, CBEN_GETDISPINFOA },
+    { 0 }
+};
+
+static const struct message empty_seq[] =
+{
+    { 0 }
+};
+
+static void test_comboex_CBEN_GETDISPINFO(void)
+{
+    static const unsigned int test_masks[] =
+    {
+        CBEIF_TEXT,
+        CBEIF_IMAGE,
+        CBEIF_INDENT,
+        CBEIF_OVERLAY,
+        CBEIF_SELECTEDIMAGE,
+        CBEIF_IMAGE | CBEIF_INDENT,
+    };
+    COMBOBOXEXITEMA item;
+    unsigned int i;
+    HWND combo;
+    DWORD res;
+
+    combo = createComboEx(WS_BORDER | WS_VISIBLE | WS_CHILD | CBS_DROPDOWN);
+    ok(!!combo, "Failed to create control window.\n");
+
+    /* All possible callback fields. */
+    memset(&item, 0, sizeof(item));
+    item.mask = CBEIF_TEXT | CBEIF_IMAGE | CBEIF_INDENT | CBEIF_OVERLAY | CBEIF_SELECTEDIMAGE;
+    item.pszText = LPSTR_TEXTCALLBACKA;
+    item.iImage = I_IMAGECALLBACK;
+    item.iSelectedImage = I_IMAGECALLBACK;
+    item.iOverlay = I_IMAGECALLBACK;
+    item.iIndent = I_INDENTCALLBACK;
+
+    res = SendMessageA(combo, CBEM_INSERTITEMA, 0, (LPARAM)&item);
+    ok(!res, "Unexpected return value %lu.\n", res);
+
+    for (i = 0; i < ARRAY_SIZE(test_masks); ++i)
+    {
+        flush_sequences(sequences, NUM_MSG_SEQUENCES);
+
+        memset(&item, 0, sizeof(item));
+        item.mask = test_masks[i];
+        res = SendMessageA(combo, CBEM_GETITEMA, 0, (LPARAM)&item);
+        ok(res == 1, "Unexpected return value %lu.\n", res);
+
+        ok_sequence(sequences, PARENT_SEQ_INDEX, getdisp_parent_seq, "Get disp mask seq", TRUE);
+    }
+
+    di_context.set_CBEIF_DI_SETITEM = TRUE;
+
+    flush_sequences(sequences, NUM_MSG_SEQUENCES);
+
+    memset(&item, 0, sizeof(item));
+    item.mask = CBEIF_IMAGE;
+    di_context.mask = 0;
+    res = SendMessageA(combo, CBEM_GETITEMA, 0, (LPARAM)&item);
+    ok(res == 1, "Unexpected return value %lu.\n", res);
+    todo_wine
+    ok(di_context.mask == CBEIF_IMAGE, "Unexpected mask %#x.\n", di_context.mask);
+
+    ok_sequence(sequences, PARENT_SEQ_INDEX, getdisp_parent_seq, "Get disp DI_SETITEM seq", TRUE);
+
+    flush_sequences(sequences, NUM_MSG_SEQUENCES);
+
+    memset(&item, 0, sizeof(item));
+    item.mask = CBEIF_IMAGE;
+    res = SendMessageA(combo, CBEM_GETITEMA, 0, (LPARAM)&item);
+    ok(res == 1, "Unexpected return value %lu.\n", res);
+    ok_sequence(sequences, PARENT_SEQ_INDEX, empty_seq, "Get disp after DI_SETITEM seq", FALSE);
+
+    /* Request two fields, one was set. */
+    memset(&item, 0, sizeof(item));
+    item.mask = CBEIF_IMAGE | CBEIF_INDENT;
+    di_context.mask = 0;
+    res = SendMessageA(combo, CBEM_GETITEMA, 0, (LPARAM)&item);
+    ok(res == 1, "Unexpected return value %lu.\n", res);
+    todo_wine
+    ok(di_context.mask == CBEIF_INDENT, "Unexpected mask %#x.\n", di_context.mask);
+
+    di_context.set_CBEIF_DI_SETITEM = FALSE;
+
+    DestroyWindow(combo);
+}
+
 START_TEST(combo)
 {
     ULONG_PTR ctx_cookie;
@@ -1384,6 +1533,7 @@ START_TEST(combo)
     test_comboex_WM_WINDOWPOSCHANGING();
     test_comboex_subclass();
     test_comboex_get_set_item();
+    test_comboex_CBEN_GETDISPINFO();
 
     if (!load_v6_module(&ctx_cookie, &hCtx))
     {
@@ -1391,6 +1541,12 @@ START_TEST(combo)
         return;
     }
 
+    test_comboex();
+    test_comboex_CB_GETLBTEXT();
+    test_comboex_WM_WINDOWPOSCHANGING();
+    test_comboex_get_set_item();
+    test_comboex_CBEN_GETDISPINFO();
+
     /* ComboBox control tests. */
     test_combo_WS_VSCROLL();
     test_combo_setfont(CBS_DROPDOWN);
