@@ -18,7 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 #include <stdarg.h>
-#define NONAMELESSUNION
+
 #include "windef.h"
 #include "winbase.h"
 #include "wincrypt.h"
@@ -1248,12 +1248,12 @@ static BOOL CRYPT_FormatAltNameEntry(DWORD dwFormatStrType, DWORD indentLevel,
     {
     case CERT_ALT_NAME_RFC822_NAME:
         LoadStringW(hInstance, IDS_ALT_NAME_RFC822_NAME, buf, ARRAY_SIZE(buf));
-        bytesNeeded += lstrlenW(entry->u.pwszRfc822Name) * sizeof(WCHAR);
+        bytesNeeded += lstrlenW(entry->pwszRfc822Name) * sizeof(WCHAR);
         ret = TRUE;
         break;
     case CERT_ALT_NAME_DNS_NAME:
         LoadStringW(hInstance, IDS_ALT_NAME_DNS_NAME, buf, ARRAY_SIZE(buf));
-        bytesNeeded += lstrlenW(entry->u.pwszDNSName) * sizeof(WCHAR);
+        bytesNeeded += lstrlenW(entry->pwszDNSName) * sizeof(WCHAR);
         ret = TRUE;
         break;
     case CERT_ALT_NAME_DIRECTORY_NAME:
@@ -1263,7 +1263,7 @@ static BOOL CRYPT_FormatAltNameEntry(DWORD dwFormatStrType, DWORD indentLevel,
         if (dwFormatStrType & CRYPT_FORMAT_STR_MULTI_LINE)
             strType |= CERT_NAME_STR_CRLF_FLAG;
         directoryNameLen = cert_name_to_str_with_indent(X509_ASN_ENCODING,
-         indentLevel + 1, &entry->u.DirectoryName, strType, NULL, 0);
+         indentLevel + 1, &entry->DirectoryName, strType, NULL, 0);
         LoadStringW(hInstance, IDS_ALT_NAME_DIRECTORY_NAME, buf, ARRAY_SIZE(buf));
         bytesNeeded += (directoryNameLen - 1) * sizeof(WCHAR);
         if (dwFormatStrType & CRYPT_FORMAT_STR_MULTI_LINE)
@@ -1275,45 +1275,45 @@ static BOOL CRYPT_FormatAltNameEntry(DWORD dwFormatStrType, DWORD indentLevel,
     }
     case CERT_ALT_NAME_URL:
         LoadStringW(hInstance, IDS_ALT_NAME_URL, buf, ARRAY_SIZE(buf));
-        bytesNeeded += lstrlenW(entry->u.pwszURL) * sizeof(WCHAR);
+        bytesNeeded += lstrlenW(entry->pwszURL) * sizeof(WCHAR);
         ret = TRUE;
         break;
     case CERT_ALT_NAME_IP_ADDRESS:
     {
         LoadStringW(hInstance, IDS_ALT_NAME_IP_ADDRESS, buf, ARRAY_SIZE(buf));
-        if (entry->u.IPAddress.cbData == 8)
+        if (entry->IPAddress.cbData == 8)
         {
             if (dwFormatStrType & CRYPT_FORMAT_STR_MULTI_LINE)
             {
                 LoadStringW(hInstance, IDS_ALT_NAME_MASK, mask, ARRAY_SIZE(mask));
                 bytesNeeded += lstrlenW(mask) * sizeof(WCHAR);
                 swprintf(ipAddrBuf, ARRAY_SIZE(ipAddrBuf), L"%d.%d.%d.%d",
-                 entry->u.IPAddress.pbData[0],
-                 entry->u.IPAddress.pbData[1],
-                 entry->u.IPAddress.pbData[2],
-                 entry->u.IPAddress.pbData[3]);
+                 entry->IPAddress.pbData[0],
+                 entry->IPAddress.pbData[1],
+                 entry->IPAddress.pbData[2],
+                 entry->IPAddress.pbData[3]);
                 bytesNeeded += lstrlenW(ipAddrBuf) * sizeof(WCHAR);
                 /* indent again, for the mask line */
                 bytesNeeded += indentLevel * lstrlenW(indent) * sizeof(WCHAR);
                 swprintf(maskBuf, ARRAY_SIZE(maskBuf), L"%d.%d.%d.%d",
-                 entry->u.IPAddress.pbData[4],
-                 entry->u.IPAddress.pbData[5],
-                 entry->u.IPAddress.pbData[6],
-                 entry->u.IPAddress.pbData[7]);
+                 entry->IPAddress.pbData[4],
+                 entry->IPAddress.pbData[5],
+                 entry->IPAddress.pbData[6],
+                 entry->IPAddress.pbData[7]);
                 bytesNeeded += lstrlenW(maskBuf) * sizeof(WCHAR);
                 bytesNeeded += lstrlenW(crlf) * sizeof(WCHAR);
             }
             else
             {
                 swprintf(ipAddrBuf, ARRAY_SIZE(ipAddrBuf), L"%d.%d.%d.%d/%d.%d.%d.%d",
-                 entry->u.IPAddress.pbData[0],
-                 entry->u.IPAddress.pbData[1],
-                 entry->u.IPAddress.pbData[2],
-                 entry->u.IPAddress.pbData[3],
-                 entry->u.IPAddress.pbData[4],
-                 entry->u.IPAddress.pbData[5],
-                 entry->u.IPAddress.pbData[6],
-                 entry->u.IPAddress.pbData[7]);
+                 entry->IPAddress.pbData[0],
+                 entry->IPAddress.pbData[1],
+                 entry->IPAddress.pbData[2],
+                 entry->IPAddress.pbData[3],
+                 entry->IPAddress.pbData[4],
+                 entry->IPAddress.pbData[5],
+                 entry->IPAddress.pbData[6],
+                 entry->IPAddress.pbData[7]);
                 bytesNeeded += (lstrlenW(ipAddrBuf) + 1) * sizeof(WCHAR);
             }
             ret = TRUE;
@@ -1321,7 +1321,7 @@ static BOOL CRYPT_FormatAltNameEntry(DWORD dwFormatStrType, DWORD indentLevel,
         else
         {
             FIXME("unknown IP address format (%ld bytes)\n",
-             entry->u.IPAddress.cbData);
+             entry->IPAddress.cbData);
             ret = FALSE;
         }
         break;
@@ -1361,7 +1361,7 @@ static BOOL CRYPT_FormatAltNameEntry(DWORD dwFormatStrType, DWORD indentLevel,
             case CERT_ALT_NAME_RFC822_NAME:
             case CERT_ALT_NAME_DNS_NAME:
             case CERT_ALT_NAME_URL:
-                lstrcpyW(str, entry->u.pwszURL);
+                lstrcpyW(str, entry->pwszURL);
                 break;
             case CERT_ALT_NAME_DIRECTORY_NAME:
                 if (dwFormatStrType & CRYPT_FORMAT_STR_MULTI_LINE)
@@ -1372,7 +1372,7 @@ static BOOL CRYPT_FormatAltNameEntry(DWORD dwFormatStrType, DWORD indentLevel,
                 else
                     *str++ = '=';
                 cert_name_to_str_with_indent(X509_ASN_ENCODING,
-                 indentLevel + 1, &entry->u.DirectoryName, strType, str,
+                 indentLevel + 1, &entry->DirectoryName, strType, str,
                  bytesNeeded / sizeof(WCHAR));
                 break;
             case CERT_ALT_NAME_IP_ADDRESS:
@@ -2039,7 +2039,7 @@ static BOOL WINAPI CRYPT_FormatCRLDistPoints(DWORD dwCertEncodingType,
                  * RDN.)
                  */
                 ret = CRYPT_FormatAltNameInfo(dwFormatStrType, 3,
-                 &distPoint->DistPointName.u.FullName, NULL, &size);
+                 &distPoint->DistPointName.FullName, NULL, &size);
                 if (ret)
                     bytesNeeded += size - sizeof(WCHAR);
                 haveAnEntry = TRUE;
@@ -2158,7 +2158,7 @@ static BOOL WINAPI CRYPT_FormatCRLDistPoints(DWORD dwCertEncodingType,
                         lstrcpyW(str, nameSep);
                         str += lstrlenW(nameSep);
                         ret = CRYPT_FormatAltNameInfo(dwFormatStrType, 3,
-                         &distPoint->DistPointName.u.FullName, str,
+                         &distPoint->DistPointName.FullName, str,
                          &altNameSize);
                         if (ret)
                             str += altNameSize / sizeof(WCHAR) - 1;
