@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <stdlib.h>
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
@@ -28,7 +27,6 @@
 #include "crypt32_private.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(crypt);
 
@@ -50,22 +48,22 @@ static HCRYPTPROV import_key( cert_store_data_t data, DWORD flags )
         acquire_flags &= ~CRYPT_NEWKEYSET;
         if (!CryptAcquireContextW( &prov, NULL, MS_ENHANCED_PROV_W, PROV_RSA_FULL, acquire_flags ))
         {
-            WARN( "CryptAcquireContextW failed %08x\n", GetLastError() );
+            WARN( "CryptAcquireContextW failed %08lx\n", GetLastError() );
             return 0;
         }
     }
 
-    params.buf = key = malloc( size );
+    params.buf = key = CryptMemAlloc( size );
     if (CRYPT32_CALL( import_store_key, &params ) ||
         !CryptImportKey( prov, key, size, 0, flags & CRYPT_EXPORTABLE, &cryptkey ))
     {
-        WARN( "CryptImportKey failed %08x\n", GetLastError() );
+        WARN( "CryptImportKey failed %08lx\n", GetLastError() );
         CryptReleaseContext( prov, 0 );
-        free( key );
+        CryptMemFree( key );
         return 0;
     }
     CryptDestroyKey( cryptkey );
-    free( key );
+    CryptMemFree( key );
     return prov;
 }
 
@@ -156,7 +154,7 @@ HCERTSTORE WINAPI PFXImportCertStore( CRYPT_DATA_BLOB *pfx, const WCHAR *passwor
     }
     if (flags & ~(CRYPT_EXPORTABLE|CRYPT_USER_KEYSET|CRYPT_MACHINE_KEYSET|PKCS12_NO_PERSIST_KEY))
     {
-        FIXME( "flags %08x not supported\n", flags );
+        FIXME( "flags %08lx not supported\n", flags );
         return NULL;
     }
     if (CRYPT32_CALL( open_cert_store, &open_params )) return NULL;
@@ -166,7 +164,7 @@ HCERTSTORE WINAPI PFXImportCertStore( CRYPT_DATA_BLOB *pfx, const WCHAR *passwor
 
     if (!(store = CertOpenStore( CERT_STORE_PROV_MEMORY, 0, 0, 0, NULL )))
     {
-        WARN( "CertOpenStore failed %08x\n", GetLastError() );
+        WARN( "CertOpenStore failed %08lx\n", GetLastError() );
         goto error;
     }
 
@@ -177,33 +175,33 @@ HCERTSTORE WINAPI PFXImportCertStore( CRYPT_DATA_BLOB *pfx, const WCHAR *passwor
         struct import_store_cert_params import_params = { data, i, NULL, &size };
 
         if (CRYPT32_CALL( import_store_cert, &import_params ) != STATUS_BUFFER_TOO_SMALL) break;
-        import_params.buf = cert = malloc( size );
+        import_params.buf = cert = CryptMemAlloc( size );
         if (!CRYPT32_CALL( import_store_cert, &import_params ))
             ctx = CertCreateContext( CERT_STORE_CERTIFICATE_CONTEXT, X509_ASN_ENCODING, cert, size, 0, NULL );
-        free( cert );
+        CryptMemFree( cert );
         if (!ctx)
         {
-            WARN( "CertCreateContext failed %08x\n", GetLastError() );
+            WARN( "CertCreateContext failed %08lx\n", GetLastError() );
             goto error;
         }
         if (flags & PKCS12_NO_PERSIST_KEY)
         {
             if (!set_key_context( ctx, prov ))
             {
-                WARN( "failed to set context property %08x\n", GetLastError() );
+                WARN( "failed to set context property %08lx\n", GetLastError() );
                 CertFreeCertificateContext( ctx );
                 goto error;
             }
         }
         else if (!set_key_prov_info( ctx, prov ))
         {
-            WARN( "failed to set provider info property %08x\n", GetLastError() );
+            WARN( "failed to set provider info property %08lx\n", GetLastError() );
             CertFreeCertificateContext( ctx );
             goto error;
         }
         if (!CertAddCertificateContextToStore( store, ctx, CERT_STORE_ADD_ALWAYS, NULL ))
         {
-            WARN( "CertAddCertificateContextToStore failed %08x\n", GetLastError() );
+            WARN( "CertAddCertificateContextToStore failed %08lx\n", GetLastError() );
             CertFreeCertificateContext( ctx );
             goto error;
         }
@@ -224,7 +222,7 @@ error:
 
 BOOL WINAPI PFXVerifyPassword( CRYPT_DATA_BLOB *pfx, const WCHAR *password, DWORD flags )
 {
-    FIXME( "(%p, %p, %08x): stub\n", pfx, password, flags );
+    FIXME( "(%p, %p, %08lx): stub\n", pfx, password, flags );
     return FALSE;
 }
 
@@ -236,6 +234,6 @@ BOOL WINAPI PFXExportCertStore( HCERTSTORE store, CRYPT_DATA_BLOB *pfx, const WC
 BOOL WINAPI PFXExportCertStoreEx( HCERTSTORE store, CRYPT_DATA_BLOB *pfx, const WCHAR *password, void *reserved,
                                   DWORD flags )
 {
-    FIXME( "(%p, %p, %p, %p, %08x): stub\n", store, pfx, password, reserved, flags );
+    FIXME( "(%p, %p, %p, %p, %08lx): stub\n", store, pfx, password, reserved, flags );
     return FALSE;
 }
