@@ -266,7 +266,7 @@ static void STDMETHODCALLTYPE d3d9_vertexbuffer_wined3d_object_destroyed(void *p
     if (buffer->draw_buffer)
         wined3d_buffer_decref(buffer->wined3d_buffer);
     d3d9_resource_cleanup(&buffer->resource);
-    heap_free(buffer);
+    free(buffer);
 }
 
 static const struct wined3d_parent_ops d3d9_vertexbuffer_wined3d_parent_ops =
@@ -312,6 +312,9 @@ HRESULT vertexbuffer_init(struct d3d9_vertexbuffer *buffer, struct d3d9_device *
     desc.misc_flags = 0;
     desc.structure_byte_stride = 0;
 
+    if (!device->d3d_parent->extended)
+        desc.usage |= WINED3DUSAGE_VIDMEM_ACCOUNTING;
+
     if (desc.access & WINED3D_RESOURCE_ACCESS_GPU)
     {
         desc.bind_flags = WINED3D_BIND_VERTEX_BUFFER;
@@ -583,7 +586,7 @@ static void STDMETHODCALLTYPE d3d9_indexbuffer_wined3d_object_destroyed(void *pa
     struct d3d9_indexbuffer *buffer = parent;
 
     d3d9_resource_cleanup(&buffer->resource);
-    heap_free(buffer);
+    free(buffer);
 }
 
 static const struct wined3d_parent_ops d3d9_indexbuffer_wined3d_parent_ops =
@@ -620,6 +623,9 @@ HRESULT indexbuffer_init(struct d3d9_indexbuffer *buffer, struct d3d9_device *de
     desc.misc_flags = 0;
     desc.structure_byte_stride = 0;
 
+    if (!device->d3d_parent->extended)
+        desc.usage |= WINED3DUSAGE_VIDMEM_ACCOUNTING;
+
     if (desc.access & WINED3D_RESOURCE_ACCESS_GPU)
         desc.bind_flags = WINED3D_BIND_INDEX_BUFFER;
 
