@@ -26,15 +26,12 @@
 #include <assert.h>
 #include <stdarg.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #define COBJMACROS
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
 #include "winuser.h"
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 #include "d3d9.h"
 #include "wine/wined3d.h"
@@ -50,15 +47,15 @@
 #define D3DFMT_RESZ MAKEFOURCC('R','E','S','Z')
 #define D3D9_RESZ_CODE 0x7fa05000
 
-extern const struct wined3d_parent_ops d3d9_null_wined3d_parent_ops DECLSPEC_HIDDEN;
+extern const struct wined3d_parent_ops d3d9_null_wined3d_parent_ops;
 
-HRESULT vdecl_convert_fvf(DWORD FVF, D3DVERTEXELEMENT9 **ppVertexElements) DECLSPEC_HIDDEN;
-D3DFORMAT d3dformat_from_wined3dformat(enum wined3d_format_id format) DECLSPEC_HIDDEN;
-BOOL is_gdi_compat_wined3dformat(enum wined3d_format_id format) DECLSPEC_HIDDEN;
-enum wined3d_format_id wined3dformat_from_d3dformat(D3DFORMAT format) DECLSPEC_HIDDEN;
-unsigned int wined3dmapflags_from_d3dmapflags(unsigned int flags, unsigned int usage) DECLSPEC_HIDDEN;
+HRESULT vdecl_convert_fvf(DWORD FVF, D3DVERTEXELEMENT9 **ppVertexElements);
+D3DFORMAT d3dformat_from_wined3dformat(enum wined3d_format_id format);
+BOOL is_gdi_compat_wined3dformat(enum wined3d_format_id format);
+enum wined3d_format_id wined3dformat_from_d3dformat(D3DFORMAT format);
+unsigned int wined3dmapflags_from_d3dmapflags(unsigned int flags, unsigned int usage);
 void present_parameters_from_wined3d_swapchain_desc(D3DPRESENT_PARAMETERS *present_parameters,
-        const struct wined3d_swapchain_desc *swapchain_desc, DWORD presentation_interval) DECLSPEC_HIDDEN;
+        const struct wined3d_swapchain_desc *swapchain_desc, DWORD presentation_interval);
 
 struct d3d9
 {
@@ -71,8 +68,8 @@ struct d3d9
 };
 
 void d3d9_caps_from_wined3dcaps(const struct d3d9 *d3d9, unsigned int adapter_ordinal,
-        D3DCAPS9 *caps, const struct wined3d_caps *wined3d_caps) DECLSPEC_HIDDEN;
-BOOL d3d9_init(struct d3d9 *d3d9, BOOL extended) DECLSPEC_HIDDEN;
+        D3DCAPS9 *caps, const struct wined3d_caps *wined3d_caps);
+BOOL d3d9_init(struct d3d9 *d3d9, BOOL extended);
 
 struct fvf_declaration
 {
@@ -124,7 +121,7 @@ struct d3d9_device
 
 HRESULT device_init(struct d3d9_device *device, struct d3d9 *parent, struct wined3d *wined3d,
         UINT adapter, D3DDEVTYPE device_type, HWND focus_window, DWORD flags,
-        D3DPRESENT_PARAMETERS *parameters, D3DDISPLAYMODEEX *mode) DECLSPEC_HIDDEN;
+        D3DPRESENT_PARAMETERS *parameters, D3DDISPLAYMODEEX *mode);
 
 struct d3d9_resource
 {
@@ -132,13 +129,13 @@ struct d3d9_resource
     struct wined3d_private_store private_store;
 };
 
-void d3d9_resource_cleanup(struct d3d9_resource *resource) DECLSPEC_HIDDEN;
-HRESULT d3d9_resource_free_private_data(struct d3d9_resource *resource, const GUID *guid) DECLSPEC_HIDDEN;
+void d3d9_resource_cleanup(struct d3d9_resource *resource);
+HRESULT d3d9_resource_free_private_data(struct d3d9_resource *resource, const GUID *guid);
 HRESULT d3d9_resource_get_private_data(struct d3d9_resource *resource, const GUID *guid,
-        void *data, DWORD *data_size) DECLSPEC_HIDDEN;
-void d3d9_resource_init(struct d3d9_resource *resource) DECLSPEC_HIDDEN;
+        void *data, DWORD *data_size);
+void d3d9_resource_init(struct d3d9_resource *resource);
 HRESULT d3d9_resource_set_private_data(struct d3d9_resource *resource, const GUID *guid,
-        const void *data, DWORD data_size, DWORD flags) DECLSPEC_HIDDEN;
+        const void *data, DWORD data_size, DWORD flags);
 
 struct d3d9_volume
 {
@@ -150,7 +147,7 @@ struct d3d9_volume
 };
 
 void volume_init(struct d3d9_volume *volume, struct wined3d_texture *wined3d_texture,
-        unsigned int sub_resource_idx, const struct wined3d_parent_ops **parent_ops) DECLSPEC_HIDDEN;
+        unsigned int sub_resource_idx, const struct wined3d_parent_ops **parent_ops);
 
 struct d3d9_swapchain
 {
@@ -163,7 +160,7 @@ struct d3d9_swapchain
 };
 
 HRESULT d3d9_swapchain_create(struct d3d9_device *device, struct wined3d_swapchain_desc *desc,
-        unsigned int swap_interval, struct d3d9_swapchain **swapchain) DECLSPEC_HIDDEN;
+        unsigned int swap_interval, struct d3d9_swapchain **swapchain);
 
 struct d3d9_surface
 {
@@ -176,15 +173,16 @@ struct d3d9_surface
     IDirect3DDevice9Ex *parent_device;
     IUnknown *container;
     struct d3d9_texture *texture;
+    struct wined3d_swapchain *swapchain;
 };
 
-struct wined3d_rendertarget_view *d3d9_surface_acquire_rendertarget_view(struct d3d9_surface *surface) DECLSPEC_HIDDEN;
-struct d3d9_device *d3d9_surface_get_device(const struct d3d9_surface *surface) DECLSPEC_HIDDEN;
+struct wined3d_rendertarget_view *d3d9_surface_acquire_rendertarget_view(struct d3d9_surface *surface);
+struct d3d9_surface *d3d9_surface_create(struct wined3d_texture *wined3d_texture,
+        unsigned int sub_resource_idx, IUnknown *container);
+struct d3d9_device *d3d9_surface_get_device(const struct d3d9_surface *surface);
 void d3d9_surface_release_rendertarget_view(struct d3d9_surface *surface,
-        struct wined3d_rendertarget_view *rtv) DECLSPEC_HIDDEN;
-void surface_init(struct d3d9_surface *surface, struct wined3d_texture *wined3d_texture,
-        unsigned int sub_resource_idx, const struct wined3d_parent_ops **parent_ops) DECLSPEC_HIDDEN;
-struct d3d9_surface *unsafe_impl_from_IDirect3DSurface9(IDirect3DSurface9 *iface) DECLSPEC_HIDDEN;
+        struct wined3d_rendertarget_view *rtv);
+struct d3d9_surface *unsafe_impl_from_IDirect3DSurface9(IDirect3DSurface9 *iface);
 
 struct d3d9_vertexbuffer
 {
@@ -197,8 +195,8 @@ struct d3d9_vertexbuffer
 };
 
 HRESULT vertexbuffer_init(struct d3d9_vertexbuffer *buffer, struct d3d9_device *device,
-        UINT size, UINT usage, DWORD fvf, D3DPOOL pool) DECLSPEC_HIDDEN;
-struct d3d9_vertexbuffer *unsafe_impl_from_IDirect3DVertexBuffer9(IDirect3DVertexBuffer9 *iface) DECLSPEC_HIDDEN;
+        UINT size, UINT usage, DWORD fvf, D3DPOOL pool);
+struct d3d9_vertexbuffer *unsafe_impl_from_IDirect3DVertexBuffer9(IDirect3DVertexBuffer9 *iface);
 
 struct d3d9_indexbuffer
 {
@@ -212,8 +210,8 @@ struct d3d9_indexbuffer
 };
 
 HRESULT indexbuffer_init(struct d3d9_indexbuffer *buffer, struct d3d9_device *device,
-        UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-struct d3d9_indexbuffer *unsafe_impl_from_IDirect3DIndexBuffer9(IDirect3DIndexBuffer9 *iface) DECLSPEC_HIDDEN;
+        UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+struct d3d9_indexbuffer *unsafe_impl_from_IDirect3DIndexBuffer9(IDirect3DIndexBuffer9 *iface);
 
 struct d3d9_texture
 {
@@ -224,7 +222,6 @@ struct d3d9_texture
     struct list rtv_list;
     DWORD usage;
     BOOL flags;
-    struct wined3d_shader_resource_view *wined3d_srv;
     D3DTEXTUREFILTERTYPE autogen_filter_type;
 };
 
@@ -240,9 +237,9 @@ static inline struct wined3d_texture *d3d9_texture_get_draw_texture(struct d3d9_
     return texture->draw_texture ? texture->draw_texture : texture->wined3d_texture;
 }
 
-struct d3d9_texture *unsafe_impl_from_IDirect3DBaseTexture9(IDirect3DBaseTexture9 *iface) DECLSPEC_HIDDEN;
-void d3d9_texture_flag_auto_gen_mipmap(struct d3d9_texture *texture) DECLSPEC_HIDDEN;
-void d3d9_texture_gen_auto_mipmap(struct d3d9_texture *texture) DECLSPEC_HIDDEN;
+struct d3d9_texture *unsafe_impl_from_IDirect3DBaseTexture9(IDirect3DBaseTexture9 *iface);
+void d3d9_texture_flag_auto_gen_mipmap(struct d3d9_texture *texture);
+void d3d9_texture_gen_auto_mipmap(struct d3d9_texture *texture);
 
 struct d3d9_stateblock
 {
@@ -253,7 +250,7 @@ struct d3d9_stateblock
 };
 
 HRESULT stateblock_init(struct d3d9_stateblock *stateblock, struct d3d9_device *device,
-        D3DSTATEBLOCKTYPE type, struct wined3d_stateblock *wined3d_stateblock) DECLSPEC_HIDDEN;
+        D3DSTATEBLOCKTYPE type, struct wined3d_stateblock *wined3d_stateblock);
 
 struct d3d9_vertex_declaration
 {
@@ -268,9 +265,9 @@ struct d3d9_vertex_declaration
 };
 
 HRESULT d3d9_vertex_declaration_create(struct d3d9_device *device,
-        const D3DVERTEXELEMENT9 *elements, struct d3d9_vertex_declaration **declaration) DECLSPEC_HIDDEN;
+        const D3DVERTEXELEMENT9 *elements, struct d3d9_vertex_declaration **declaration);
 struct d3d9_vertex_declaration *unsafe_impl_from_IDirect3DVertexDeclaration9(
-        IDirect3DVertexDeclaration9 *iface) DECLSPEC_HIDDEN;
+        IDirect3DVertexDeclaration9 *iface);
 
 struct d3d9_vertexshader
 {
@@ -281,8 +278,8 @@ struct d3d9_vertexshader
 };
 
 HRESULT vertexshader_init(struct d3d9_vertexshader *shader,
-        struct d3d9_device *device, const DWORD *byte_code) DECLSPEC_HIDDEN;
-struct d3d9_vertexshader *unsafe_impl_from_IDirect3DVertexShader9(IDirect3DVertexShader9 *iface) DECLSPEC_HIDDEN;
+        struct d3d9_device *device, const DWORD *byte_code);
+struct d3d9_vertexshader *unsafe_impl_from_IDirect3DVertexShader9(IDirect3DVertexShader9 *iface);
 
 struct d3d9_pixelshader
 {
@@ -293,8 +290,8 @@ struct d3d9_pixelshader
 };
 
 HRESULT pixelshader_init(struct d3d9_pixelshader *shader,
-        struct d3d9_device *device, const DWORD *byte_code) DECLSPEC_HIDDEN;
-struct d3d9_pixelshader *unsafe_impl_from_IDirect3DPixelShader9(IDirect3DPixelShader9 *iface) DECLSPEC_HIDDEN;
+        struct d3d9_device *device, const DWORD *byte_code);
+struct d3d9_pixelshader *unsafe_impl_from_IDirect3DPixelShader9(IDirect3DPixelShader9 *iface);
 
 struct d3d9_query
 {
@@ -305,7 +302,7 @@ struct d3d9_query
     DWORD data_size;
 };
 
-HRESULT query_init(struct d3d9_query *query, struct d3d9_device *device, D3DQUERYTYPE type) DECLSPEC_HIDDEN;
+HRESULT query_init(struct d3d9_query *query, struct d3d9_device *device, D3DQUERYTYPE type);
 
 static inline struct d3d9_device *impl_from_IDirect3DDevice9Ex(IDirect3DDevice9Ex *iface)
 {
