@@ -70,7 +70,7 @@ HRESULT WINAPI D3DX11GetImageInfoFromFileW(const WCHAR *filename, ID3DX11ThreadP
 HRESULT WINAPI D3DX11GetImageInfoFromMemory(const void *src_data, SIZE_T src_data_size, ID3DX11ThreadPump *pump,
         D3DX11_IMAGE_INFO *img_info, HRESULT *hresult)
 {
-    FIXME("src_data %p, src_data_size %lu, pump %p, img_info %p, hresult %p stub!\n",
+    FIXME("src_data %p, src_data_size %Iu, pump %p, img_info %p, hresult %p stub!\n",
             src_data, src_data_size, pump, img_info, hresult);
 
     return E_NOTIMPL;
