@@ -26,6 +26,8 @@
 #include <string.h>
 #include <assert.h>
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "dbghelp_private.h"
 #include "image_private.h"
 #include "winternl.h"
@@ -65,6 +67,11 @@ static void pe_unmap_full(struct image_file_map* fmap)
     }
 }
 
+/* as we store either IMAGE_OPTIONAL_HEADER(32|64) inside pe_file_map,
+ * this helper will read to any field 'field' inside such an header
+ */
+#define PE_FROM_OPTHDR(fmap, field) (((fmap)->addr_size == 32) ? ((fmap)->u.pe.opt.header32. field) : ((fmap)->u.pe.opt.header64. field))
+
 /******************************************************************
  *		pe_map_section
  *
@@ -75,14 +82,14 @@ static const char* pe_map_section(struct image_section_map* ism)
     void*       mapping;
     struct pe_file_map* fmap = &ism->fmap->u.pe;
 
-    if (ism->sidx >= 0 && ism->sidx < fmap->ntheader.FileHeader.NumberOfSections &&
+    if (ism->sidx >= 0 && ism->sidx < fmap->file_header.NumberOfSections &&
         fmap->sect[ism->sidx].mapped == IMAGE_NO_MAP)
     {
         IMAGE_NT_HEADERS*       nth;
 
         if (fmap->sect[ism->sidx].shdr.Misc.VirtualSize > fmap->sect[ism->sidx].shdr.SizeOfRawData)
         {
-            FIXME("Section %ld: virtual (0x%x) > raw (0x%x) size - not supported\n",
+            FIXME("Section %Id: virtual (0x%lx) > raw (0x%lx) size - not supported\n",
                   ism->sidx, fmap->sect[ism->sidx].shdr.Misc.VirtualSize,
                   fmap->sect[ism->sidx].shdr.SizeOfRawData);
             return IMAGE_NO_MAP;
@@ -115,7 +122,7 @@ static BOOL pe_find_section(struct image_file_map* fmap, const char* name,
     unsigned                    i;
     char                        tmp[IMAGE_SIZEOF_SHORT_NAME + 1];
 
-    for (i = 0; i < fmap->u.pe.ntheader.FileHeader.NumberOfSections; i++)
+    for (i = 0; i < fmap->u.pe.file_header.NumberOfSections; i++)
     {
         sectname = (const char*)fmap->u.pe.sect[i].shdr.Name;
         /* long section names start with a '/' (at least on MinGW32) */
@@ -147,7 +154,7 @@ static BOOL pe_find_section(struct image_file_map* fmap, const char* name,
  */
 static void pe_unmap_section(struct image_section_map* ism)
 {
-    if (ism->sidx >= 0 && ism->sidx < ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections &&
+    if (ism->sidx >= 0 && ism->sidx < ism->fmap->u.pe.file_header.NumberOfSections &&
         ism->fmap->u.pe.sect[ism->sidx].mapped != IMAGE_NO_MAP)
     {
         pe_unmap_full(ism->fmap);
@@ -162,7 +169,7 @@ static void pe_unmap_section(struct image_section_map* ism)
  */
 static DWORD_PTR pe_get_map_rva(const struct image_section_map* ism)
 {
-    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections)
+    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.file_header.NumberOfSections)
         return 0;
     return ism->fmap->u.pe.sect[ism->sidx].shdr.VirtualAddress;
 }
@@ -174,7 +181,7 @@ static DWORD_PTR pe_get_map_rva(const struct image_section_map* ism)
  */
 static unsigned pe_get_map_size(const struct image_section_map* ism)
 {
-    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections)
+    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.file_header.NumberOfSections)
         return 0;
     return ism->fmap->u.pe.sect[ism->sidx].shdr.Misc.VirtualSize;
 }
@@ -190,7 +197,7 @@ static void pe_unmap_file(struct image_file_map* fmap)
     {
         struct image_section_map  ism;
         ism.fmap = fmap;
-        for (ism.sidx = 0; ism.sidx < fmap->u.pe.ntheader.FileHeader.NumberOfSections; ism.sidx++)
+        for (ism.sidx = 0; ism.sidx < fmap->u.pe.file_header.NumberOfSections; ism.sidx++)
         {
             pe_unmap_section(&ism);
         }
@@ -236,11 +243,14 @@ static BOOL pe_is_valid_pointer_table(const IMAGE_NT_HEADERS* nthdr, const void*
  *
  * Maps an PE file into memory (and checks it's a real PE file)
  */
-BOOL pe_map_file(HANDLE file, struct image_file_map* fmap, enum module_type mt)
+BOOL pe_map_file(HANDLE file, struct image_file_map* fmap)
 {
-    void*       mapping;
+    void*                       mapping;
+    IMAGE_NT_HEADERS*           nthdr;
+    IMAGE_SECTION_HEADER*       section;
+    unsigned                    i;
 
-    fmap->modtype = mt;
+    fmap->modtype = DMT_PE;
     fmap->ops = &pe_file_map_ops;
     fmap->alternate = NULL;
     fmap->u.pe.hMap = CreateFileMappingW(file, NULL, PAGE_READONLY, 0, 0, NULL);
@@ -249,65 +259,61 @@ BOOL pe_map_file(HANDLE file, struct image_file_map* fmap, enum module_type mt)
     fmap->u.pe.full_map = NULL;
     if (!(mapping = pe_map_full(fmap, NULL))) goto error;
 
-    switch (mt)
+    if (!(nthdr = RtlImageNtHeader(mapping))) goto error;
+    memcpy(&fmap->u.pe.file_header, &nthdr->FileHeader, sizeof(fmap->u.pe.file_header));
+    switch (nthdr->OptionalHeader.Magic)
     {
-    case DMT_PE:
-        {
-            IMAGE_NT_HEADERS*       nthdr;
-            IMAGE_SECTION_HEADER*   section;
-            unsigned                i;
-
-            if (!(nthdr = RtlImageNtHeader(mapping))) goto error;
-            memcpy(&fmap->u.pe.ntheader, nthdr, sizeof(fmap->u.pe.ntheader));
-            switch (nthdr->OptionalHeader.Magic)
-            {
-            case 0x10b: fmap->addr_size = 32; break;
-            case 0x20b: fmap->addr_size = 64; break;
-            default: return FALSE;
-            }
-
-            fmap->u.pe.builtin = !memcmp((const IMAGE_DOS_HEADER*)mapping + 1, builtin_signature, sizeof(builtin_signature));
-            section = (IMAGE_SECTION_HEADER*)
-                ((char*)&nthdr->OptionalHeader + nthdr->FileHeader.SizeOfOptionalHeader);
-            fmap->u.pe.sect = HeapAlloc(GetProcessHeap(), 0,
-                                        nthdr->FileHeader.NumberOfSections * sizeof(fmap->u.pe.sect[0]));
-            if (!fmap->u.pe.sect) goto error;
-            for (i = 0; i < nthdr->FileHeader.NumberOfSections; i++)
-            {
-                memcpy(&fmap->u.pe.sect[i].shdr, section + i, sizeof(IMAGE_SECTION_HEADER));
-                fmap->u.pe.sect[i].mapped = IMAGE_NO_MAP;
-            }
-            if (nthdr->FileHeader.PointerToSymbolTable && nthdr->FileHeader.NumberOfSymbols)
-            {
-                LARGE_INTEGER li;
-
-                if (GetFileSizeEx(file, &li) && pe_is_valid_pointer_table(nthdr, mapping, li.QuadPart))
-                {
-                    /* FIXME ugly: should rather map the relevant content instead of copying it */
-                    const char* src = (const char*)mapping +
-                        nthdr->FileHeader.PointerToSymbolTable +
-                        nthdr->FileHeader.NumberOfSymbols * sizeof(IMAGE_SYMBOL);
-                    char* dst;
-                    DWORD sz = *(DWORD*)src;
-
-                    if ((dst = HeapAlloc(GetProcessHeap(), 0, sz)))
-                        memcpy(dst, src, sz);
-                    fmap->u.pe.strtable = dst;
-                }
-                else
-                {
-                    WARN("Bad coff table... wipping out\n");
-                    /* we have bad information here, wipe it out */
-                    fmap->u.pe.ntheader.FileHeader.PointerToSymbolTable = 0;
-                    fmap->u.pe.ntheader.FileHeader.NumberOfSymbols = 0;
-                    fmap->u.pe.strtable = NULL;
-                }
-            }
-            else fmap->u.pe.strtable = NULL;
-        }
+    case IMAGE_NT_OPTIONAL_HDR32_MAGIC:
+        fmap->addr_size = 32;
+        memcpy(&fmap->u.pe.opt.header32, &nthdr->OptionalHeader, sizeof(fmap->u.pe.opt.header32));
         break;
-    default: assert(0); goto error;
+    case IMAGE_NT_OPTIONAL_HDR64_MAGIC:
+        if (sizeof(void*) == 4) return FALSE;
+        fmap->addr_size = 64;
+        memcpy(&fmap->u.pe.opt.header64, &nthdr->OptionalHeader, sizeof(fmap->u.pe.opt.header64));
+        break;
+    default:
+        return FALSE;
     }
+
+    fmap->u.pe.builtin = !memcmp((const IMAGE_DOS_HEADER*)mapping + 1, builtin_signature, sizeof(builtin_signature));
+    section = IMAGE_FIRST_SECTION( nthdr );
+    fmap->u.pe.sect = HeapAlloc(GetProcessHeap(), 0,
+                                nthdr->FileHeader.NumberOfSections * sizeof(fmap->u.pe.sect[0]));
+    if (!fmap->u.pe.sect) goto error;
+    for (i = 0; i < nthdr->FileHeader.NumberOfSections; i++)
+    {
+        memcpy(&fmap->u.pe.sect[i].shdr, section + i, sizeof(IMAGE_SECTION_HEADER));
+        fmap->u.pe.sect[i].mapped = IMAGE_NO_MAP;
+    }
+    if (nthdr->FileHeader.PointerToSymbolTable && nthdr->FileHeader.NumberOfSymbols)
+    {
+        LARGE_INTEGER li;
+
+        if (GetFileSizeEx(file, &li) && pe_is_valid_pointer_table(nthdr, mapping, li.QuadPart))
+        {
+            /* FIXME ugly: should rather map the relevant content instead of copying it */
+            const char* src = (const char*)mapping +
+                nthdr->FileHeader.PointerToSymbolTable +
+                nthdr->FileHeader.NumberOfSymbols * sizeof(IMAGE_SYMBOL);
+            char* dst;
+            DWORD sz = *(DWORD*)src;
+
+            if ((dst = HeapAlloc(GetProcessHeap(), 0, sz)))
+                memcpy(dst, src, sz);
+            fmap->u.pe.strtable = dst;
+        }
+        else
+        {
+            WARN("Bad coff table... wipping out\n");
+            /* we have bad information here, wipe it out */
+            fmap->u.pe.file_header.PointerToSymbolTable = 0;
+            fmap->u.pe.file_header.NumberOfSymbols = 0;
+            fmap->u.pe.strtable = NULL;
+        }
+    }
+    else fmap->u.pe.strtable = NULL;
+
     pe_unmap_full(fmap);
 
     return TRUE;
@@ -362,16 +368,16 @@ static BOOL pe_locate_with_coff_symbol_table(struct module* module)
     struct symt_data*   sym;
     const char*         mapping;
 
-    numsym = fmap->u.pe.ntheader.FileHeader.NumberOfSymbols;
-    if (!fmap->u.pe.ntheader.FileHeader.PointerToSymbolTable || !numsym)
+    numsym = fmap->u.pe.file_header.NumberOfSymbols;
+    if (!fmap->u.pe.file_header.PointerToSymbolTable || !numsym)
         return TRUE;
     if (!(mapping = pe_map_full(fmap, NULL))) return FALSE;
-    isym = (const IMAGE_SYMBOL*)(mapping + fmap->u.pe.ntheader.FileHeader.PointerToSymbolTable);
+    isym = (const IMAGE_SYMBOL*)(mapping + fmap->u.pe.file_header.PointerToSymbolTable);
 
     for (i = 0; i < numsym; i+= naux, isym += naux)
     {
         if (isym->StorageClass == IMAGE_SYM_CLASS_EXTERNAL &&
-            isym->SectionNumber > 0 && isym->SectionNumber <= fmap->u.pe.ntheader.FileHeader.NumberOfSections)
+            isym->SectionNumber > 0 && isym->SectionNumber <= fmap->u.pe.file_header.NumberOfSections)
         {
             if (isym->N.Name.Short)
             {
@@ -389,11 +395,11 @@ static BOOL pe_locate_with_coff_symbol_table(struct module* module)
                     sym->u.var.kind == loc_absolute &&
                     !strcmp(sym->hash_elt.name, name))
                 {
-                    TRACE("Changing absolute address for %d.%s: %lx -> %s\n",
+                    TRACE("Changing absolute address for %d.%s: %Ix -> %I64x\n",
                           isym->SectionNumber, name, sym->u.var.offset,
-                          wine_dbgstr_longlong(module->module.BaseOfImage +
-                                               fmap->u.pe.sect[isym->SectionNumber - 1].shdr.VirtualAddress +
-                                               isym->Value));
+                          module->module.BaseOfImage +
+                          fmap->u.pe.sect[isym->SectionNumber - 1].shdr.VirtualAddress +
+                          isym->Value);
                     sym->u.var.offset = module->module.BaseOfImage +
                         fmap->u.pe.sect[isym->SectionNumber - 1].shdr.VirtualAddress + isym->Value;
                     break;
@@ -424,11 +430,11 @@ static BOOL pe_load_coff_symbol_table(struct module* module)
     const IMAGE_SECTION_HEADER* sect;
     const char*         mapping;
 
-    numsym = fmap->u.pe.ntheader.FileHeader.NumberOfSymbols;
-    if (!fmap->u.pe.ntheader.FileHeader.PointerToSymbolTable || !numsym)
+    numsym = fmap->u.pe.file_header.NumberOfSymbols;
+    if (!fmap->u.pe.file_header.PointerToSymbolTable || !numsym)
         return TRUE;
     if (!(mapping = pe_map_full(fmap, NULL))) return FALSE;
-    isym = (const IMAGE_SYMBOL*)((const char*)mapping + fmap->u.pe.ntheader.FileHeader.PointerToSymbolTable);
+    isym = (const IMAGE_SYMBOL*)(mapping + fmap->u.pe.file_header.PointerToSymbolTable);
     /* FIXME: no way to get strtable size */
     strtable = (const char*)&isym[numsym];
     sect = IMAGE_FIRST_SECTION(RtlImageNtHeader((HMODULE)mapping));
@@ -441,7 +447,7 @@ static BOOL pe_load_coff_symbol_table(struct module* module)
             compiland = NULL;
         }
         if (isym->StorageClass == IMAGE_SYM_CLASS_EXTERNAL &&
-            isym->SectionNumber > 0 && isym->SectionNumber <= fmap->u.pe.ntheader.FileHeader.NumberOfSections)
+            isym->SectionNumber > 0 && isym->SectionNumber <= fmap->u.pe.file_header.NumberOfSections)
         {
             if (isym->N.Name.Short)
             {
@@ -452,8 +458,7 @@ static BOOL pe_load_coff_symbol_table(struct module* module)
             if (name[0] == '_') name++;
 
             if (!compiland && lastfilename)
-                compiland = symt_new_compiland(module, 0,
-                                               source_new(module, NULL, lastfilename));
+                compiland = symt_new_compiland(module, source_new(module, NULL, lastfilename));
 
             if (!(dbghelp_options & SYMOPT_NO_PUBLICS))
                 symt_new_public(module, compiland, name, FALSE,
@@ -496,7 +501,7 @@ static BOOL pe_load_stabs(const struct process* pcs, struct module* module)
         if (stab != IMAGE_NO_MAP && stabstr != IMAGE_NO_MAP)
         {
             ret = stabs_parse(module,
-                              module->module.BaseOfImage - fmap->u.pe.ntheader.OptionalHeader.ImageBase,
+                              module->module.BaseOfImage - PE_FROM_OPTHDR(fmap, ImageBase),
                               stab, image_get_map_size(&sect_stabs) / sizeof(struct stab_nlist), sizeof(struct stab_nlist),
                               stabstr, image_get_map_size(&sect_stabstr),
                               NULL, NULL);
@@ -522,7 +527,7 @@ static BOOL pe_load_dwarf(struct module* module)
     BOOL                        ret;
 
     ret = dwarf2_parse(module,
-                       module->module.BaseOfImage - fmap->u.pe.ntheader.OptionalHeader.ImageBase,
+                       module->module.BaseOfImage - PE_FROM_OPTHDR(fmap, ImageBase),
                        NULL, /* FIXME: some thunks to deal with ? */
                        fmap);
     TRACE("%s the DWARF debug info\n", ret ? "successfully loaded" : "failed to load");
@@ -545,7 +550,7 @@ static BOOL pe_load_dbg_file(const struct process* pcs, struct module* module,
 
     TRACE("Processing DBG file %s\n", debugstr_a(dbg_name));
 
-    if (path_find_symbol_file(pcs, module, dbg_name, DMT_DBG, NULL, timestamp, 0, tmp, &module->module.DbgUnmatched) &&
+    if (path_find_symbol_file(pcs, module, dbg_name, FALSE, NULL, timestamp, 0, tmp, &module->module.DbgUnmatched) &&
         (hFile = CreateFileW(tmp, GENERIC_READ, FILE_SHARE_READ, NULL,
                              OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) != INVALID_HANDLE_VALUE &&
         ((hMap = CreateFileMappingW(hFile, NULL, PAGE_READONLY, 0, 0, NULL)) != 0) &&
@@ -607,7 +612,7 @@ static BOOL pe_load_msc_debug_info(const struct process* pcs, struct module* mod
             misc->DataType != IMAGE_DEBUG_MISC_EXENAME)
         {
             ERR("-Debug info stripped, but no .DBG file in module %s\n",
-                debugstr_w(module->module.ModuleName));
+                debugstr_w(module->modulename));
         }
         else
         {
@@ -616,9 +621,8 @@ static BOOL pe_load_msc_debug_info(const struct process* pcs, struct module* mod
     }
     else
     {
-        const IMAGE_SECTION_HEADER *sectp = (const IMAGE_SECTION_HEADER*)((const char*)&nth->OptionalHeader + nth->FileHeader.SizeOfOptionalHeader);
         /* Debug info is embedded into PE module */
-        ret = pe_load_debug_directory(pcs, module, mapping, sectp,
+        ret = pe_load_debug_directory(pcs, module, mapping, IMAGE_FIRST_SECTION( nth ),
                                       nth->FileHeader.NumberOfSections, dbg, nDbg);
     }
 done:
@@ -634,7 +638,7 @@ static BOOL pe_load_export_debug_info(const struct process* pcs, struct module*
     struct image_file_map*              fmap = &module->format_info[DFI_PE]->u.pe_info->fmap;
     unsigned int 		        i;
     const IMAGE_EXPORT_DIRECTORY* 	exports;
-    DWORD			        base = module->module.BaseOfImage;
+    DWORD_PTR			        base = module->module.BaseOfImage;
     DWORD                               size;
     IMAGE_NT_HEADERS*                   nth;
     void*                               mapping;
@@ -654,10 +658,8 @@ static BOOL pe_load_export_debug_info(const struct process* pcs, struct module*
 #if 0
     /* FIXME: we'd better store addresses linked to sections rather than 
        absolute values */
-    IMAGE_SECTION_HEADER*       section;
+    IMAGE_SECTION_HEADER *section = IMAGE_FIRST_SECTION( nth );
     /* Add start of sections */
-    section = (IMAGE_SECTION_HEADER*)
-        ((char*)&nth->OptionalHeader + nth->FileHeader.SizeOfOptionalHeader);
     for (i = 0; i < nth->FileHeader.NumberOfSections; i++, section++) 
     {
 	symt_new_public(module, NULL, section->Name, FALSE,
@@ -670,7 +672,7 @@ static BOOL pe_load_export_debug_info(const struct process* pcs, struct module*
                                                 IMAGE_DIRECTORY_ENTRY_EXPORT, &size)))
     {
         const WORD*             ordinals = NULL;
-        const DWORD_PTR*	functions = NULL;
+        const DWORD*	        functions = NULL;
         const DWORD*		names = NULL;
         unsigned int		j;
         char			buffer[16];
@@ -697,8 +699,8 @@ static BOOL pe_load_export_debug_info(const struct process* pcs, struct module*
                 for (j = 0; j < exports->NumberOfNames; j++)
                     if ((ordinals[j] == i) && names[j]) break;
                 if (j < exports->NumberOfNames) continue;
-                snprintf(buffer, sizeof(buffer), "%d", i + exports->Base);
-                symt_new_public(module, NULL, buffer, FALSE, base + (DWORD)functions[i], 1);
+                snprintf(buffer, sizeof(buffer), "%ld", i + exports->Base);
+                symt_new_public(module, NULL, buffer, FALSE, base + functions[i], 1);
             }
         }
     }
@@ -733,7 +735,7 @@ BOOL pe_load_debug_info(const struct process* pcs, struct module* module)
     /* FIXME shouldn't we check that? if (!module_get_debug(pcs, module)) */
     if (pe_load_export_debug_info(pcs, module) && !ret)
         ret = TRUE;
-
+    if (!ret) module->module.SymType = SymNone;
     return ret;
 }
 
@@ -746,13 +748,10 @@ struct builtin_search
 static BOOL search_builtin_pe(void *param, HANDLE handle, const WCHAR *path)
 {
     struct builtin_search *search = param;
-    size_t size;
 
-    if (!pe_map_file(handle, &search->fmap, DMT_PE)) return FALSE;
+    if (!pe_map_file(handle, &search->fmap)) return FALSE;
 
-    size = (lstrlenW(path) + 1) * sizeof(WCHAR);
-    if ((search->path = heap_alloc(size)))
-        memcpy(search->path, path, size);
+    search->path = wcsdup(path);
     return TRUE;
 }
 
@@ -767,62 +766,86 @@ struct module* pe_load_native_module(struct process* pcs, const WCHAR* name,
     BOOL                        opened = FALSE;
     struct module_format*       modfmt;
     WCHAR                       loaded_name[MAX_PATH];
+    WCHAR*                      real_path = NULL;
 
     loaded_name[0] = '\0';
     if (!hFile)
     {
         assert(name);
 
-        if ((hFile = FindExecutableImageExW(name, pcs->search_path, loaded_name, NULL, NULL)) == NULL)
+        if ((hFile = FindExecutableImageExW(name, pcs->search_path, loaded_name, NULL, NULL)) == NULL &&
+            (hFile = FindExecutableImageExW(name, L".", loaded_name, NULL, NULL)) == NULL)
             return NULL;
         opened = TRUE;
     }
-    else if (name) lstrcpyW(loaded_name, name);
-    else if (dbghelp_options & SYMOPT_DEFERRED_LOADS)
-        FIXME("Trouble ahead (no module name passed in deferred mode)\n");
-    if (!(modfmt = HeapAlloc(GetProcessHeap(), 0, sizeof(struct module_format) + sizeof(struct pe_module_info))))
-        return NULL;
-    modfmt->u.pe_info = (struct pe_module_info*)(modfmt + 1);
-    if (pe_map_file(hFile, &modfmt->u.pe_info->fmap, DMT_PE))
+    else
     {
-        struct builtin_search builtin = { NULL };
-        if (modfmt->u.pe_info->fmap.u.pe.builtin && search_dll_path(pcs, loaded_name, search_builtin_pe, &builtin))
-        {
-            TRACE("reloaded %s from %s\n", debugstr_w(loaded_name), debugstr_w(builtin.path));
-            image_unmap_file(&modfmt->u.pe_info->fmap);
-            modfmt->u.pe_info->fmap = builtin.fmap;
-        }
-        if (!base) base = modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.ImageBase;
-        if (!size) size = modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.SizeOfImage;
+        ULONG sz = sizeof(OBJECT_NAME_INFORMATION) + MAX_PATH * sizeof(WCHAR), needed;
+        OBJECT_NAME_INFORMATION *obj_name;
+        NTSTATUS nts;
 
-        module = module_new(pcs, loaded_name, DMT_PE, FALSE, base, size,
-                            modfmt->u.pe_info->fmap.u.pe.ntheader.FileHeader.TimeDateStamp,
-                            modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.CheckSum);
-        if (module)
+        obj_name = RtlAllocateHeap(GetProcessHeap(), 0, sz);
+        if (obj_name)
         {
-            module->real_path = builtin.path;
-            modfmt->module = module;
-            modfmt->remove = pe_module_remove;
-            modfmt->loc_compute = NULL;
-
-            module->format_info[DFI_PE] = modfmt;
-            if (dbghelp_options & SYMOPT_DEFERRED_LOADS)
-                module->module.SymType = SymDeferred;
-            else
-                pe_load_debug_info(pcs, module);
-            module->reloc_delta = base - modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.ImageBase;
-        }
-        else
-        {
-            ERR("could not load the module '%s'\n", debugstr_w(loaded_name));
-            heap_free(builtin.path);
-            image_unmap_file(&modfmt->u.pe_info->fmap);
+            nts = NtQueryObject(hFile, ObjectNameInformation, obj_name, sz, &needed);
+            if (nts == STATUS_BUFFER_OVERFLOW)
+            {
+                sz = needed;
+                obj_name = RtlReAllocateHeap(GetProcessHeap(), 0, obj_name, sz);
+                nts = NtQueryObject(hFile, ObjectNameInformation, obj_name, sz, &needed);
+            }
+            if (!nts)
+            {
+                obj_name->Name.Buffer[obj_name->Name.Length / sizeof(WCHAR)] = L'\0';
+                real_path = wcsdup(obj_name->Name.Buffer);
+            }
+            RtlFreeHeap(GetProcessHeap(), 0, obj_name);
         }
+        if (name) lstrcpyW(loaded_name, name);
+    }
+
+    if ((modfmt = HeapAlloc(GetProcessHeap(), 0, sizeof(struct module_format) + sizeof(struct pe_module_info))))
+    {
+        modfmt->u.pe_info = (struct pe_module_info*)(modfmt + 1);
+        if (pe_map_file(hFile, &modfmt->u.pe_info->fmap))
+        {
+            struct builtin_search builtin = { NULL };
+            if (opened && modfmt->u.pe_info->fmap.u.pe.builtin &&
+                search_dll_path(pcs, loaded_name, modfmt->u.pe_info->fmap.u.pe.file_header.Machine, search_builtin_pe, &builtin))
+            {
+                TRACE("reloaded %s from %s\n", debugstr_w(loaded_name), debugstr_w(builtin.path));
+                image_unmap_file(&modfmt->u.pe_info->fmap);
+                modfmt->u.pe_info->fmap = builtin.fmap;
+                real_path = builtin.path;
+            }
+            if (!base) base = PE_FROM_OPTHDR(&modfmt->u.pe_info->fmap, ImageBase);
+            if (!size) size = PE_FROM_OPTHDR(&modfmt->u.pe_info->fmap, SizeOfImage);
+
+            module = module_new(pcs, loaded_name, DMT_PE, modfmt->u.pe_info->fmap.u.pe.builtin, FALSE,
+                                base, size,
+                                modfmt->u.pe_info->fmap.u.pe.file_header.TimeDateStamp,
+                                PE_FROM_OPTHDR(&modfmt->u.pe_info->fmap, CheckSum),
+                                modfmt->u.pe_info->fmap.u.pe.file_header.Machine);
+            if (module)
+            {
+                module->real_path = real_path ? pool_wcsdup(&module->pool, real_path) : NULL;
+                modfmt->module = module;
+                modfmt->remove = pe_module_remove;
+                modfmt->loc_compute = NULL;
+                module->format_info[DFI_PE] = modfmt;
+                module->reloc_delta = base - PE_FROM_OPTHDR(&modfmt->u.pe_info->fmap, ImageBase);
+            }
+            else
+            {
+                ERR("could not load the module '%s'\n", debugstr_w(loaded_name));
+                image_unmap_file(&modfmt->u.pe_info->fmap);
+            }
+        }
+        if (!module) HeapFree(GetProcessHeap(), 0, modfmt);
     }
-    if (!module) HeapFree(GetProcessHeap(), 0, modfmt);
 
     if (opened) CloseHandle(hFile);
-
+    free(real_path);
     return module;
 }
 
@@ -830,15 +853,27 @@ struct module* pe_load_native_module(struct process* pcs, const WCHAR* name,
  *		pe_load_nt_header
  *
  */
-BOOL pe_load_nt_header(HANDLE hProc, DWORD64 base, IMAGE_NT_HEADERS* nth)
+BOOL pe_load_nt_header(HANDLE hProc, DWORD64 base, IMAGE_NT_HEADERS* nth, BOOL* is_builtin)
 {
     IMAGE_DOS_HEADER    dos;
 
-    return ReadProcessMemory(hProc, (char*)(DWORD_PTR)base, &dos, sizeof(dos), NULL) &&
-        dos.e_magic == IMAGE_DOS_SIGNATURE &&
-        ReadProcessMemory(hProc, (char*)(DWORD_PTR)(base + dos.e_lfanew),
-                          nth, sizeof(*nth), NULL) &&
-        nth->Signature == IMAGE_NT_SIGNATURE;
+    if (!ReadProcessMemory(hProc, (char*)(DWORD_PTR)base, &dos, sizeof(dos), NULL) ||
+        dos.e_magic != IMAGE_DOS_SIGNATURE ||
+        !ReadProcessMemory(hProc, (char*)(DWORD_PTR)(base + dos.e_lfanew),
+                           nth, sizeof(*nth), NULL) ||
+        nth->Signature != IMAGE_NT_SIGNATURE)
+        return FALSE;
+    if (is_builtin)
+    {
+        if (dos.e_lfanew >= sizeof(dos) + sizeof(builtin_signature))
+        {
+            char sig[sizeof(builtin_signature)];
+            *is_builtin = ReadProcessMemory(hProc, (char*)(DWORD_PTR)base + sizeof(dos), sig, sizeof(sig), NULL) &&
+                !memcmp(sig, builtin_signature, sizeof(builtin_signature));
+        }
+        else *is_builtin = FALSE;
+    }
+    return TRUE;
 }
 
 /******************************************************************
@@ -853,13 +888,15 @@ struct module* pe_load_builtin_module(struct process* pcs, const WCHAR* name,
     if (base && pcs->dbg_hdr_addr)
     {
         IMAGE_NT_HEADERS    nth;
+        BOOL is_builtin;
 
-        if (pe_load_nt_header(pcs->handle, base, &nth))
+        if (pe_load_nt_header(pcs->handle, base, &nth, &is_builtin))
         {
             if (!size) size = nth.OptionalHeader.SizeOfImage;
-            module = module_new(pcs, name, DMT_PE, FALSE, base, size,
+            module = module_new(pcs, name, DMT_PE, is_builtin, FALSE, base, size,
                                 nth.FileHeader.TimeDateStamp,
-                                nth.OptionalHeader.CheckSum);
+                                nth.OptionalHeader.CheckSum,
+                                nth.FileHeader.Machine);
         }
     }
     return module;
@@ -886,7 +923,7 @@ struct module* pe_load_builtin_module(struct process* pcs, const WCHAR* name,
 PVOID WINAPI ImageDirectoryEntryToDataEx( PVOID base, BOOLEAN image, USHORT dir, PULONG size, PIMAGE_SECTION_HEADER *section )
 {
     const IMAGE_NT_HEADERS *nt;
-    DWORD addr;
+    DWORD_PTR addr;
 
     *size = 0;
     if (section) *section = NULL;
@@ -925,3 +962,30 @@ PVOID WINAPI ImageDirectoryEntryToData( PVOID base, BOOLEAN image, USHORT dir, P
 {
     return ImageDirectoryEntryToDataEx( base, image, dir, size, NULL );
 }
+
+DWORD pe_get_file_indexinfo(void* image, DWORD size, SYMSRV_INDEX_INFOW* info)
+{
+    const IMAGE_NT_HEADERS* nthdr;
+    const IMAGE_DEBUG_DIRECTORY* dbg;
+    ULONG dirsize;
+
+    if (!(nthdr = RtlImageNtHeader(image))) return ERROR_BAD_FORMAT;
+
+    dbg = RtlImageDirectoryEntryToData(image, FALSE, IMAGE_DIRECTORY_ENTRY_DEBUG, &dirsize);
+    if (!dbg || dirsize < sizeof(dbg)) return ERROR_BAD_EXE_FORMAT;
+
+    /* fill in information from NT header */
+    info->timestamp = nthdr->FileHeader.TimeDateStamp;
+    info->stripped = (nthdr->FileHeader.Characteristics & IMAGE_FILE_DEBUG_STRIPPED) != 0;
+    if (nthdr->OptionalHeader.Magic == IMAGE_NT_OPTIONAL_HDR64_MAGIC)
+    {
+        const IMAGE_NT_HEADERS64* nthdr64 = (const IMAGE_NT_HEADERS64*)nthdr;
+        info->size = nthdr64->OptionalHeader.SizeOfImage;
+    }
+    else if (nthdr->OptionalHeader.Magic == IMAGE_NT_OPTIONAL_HDR32_MAGIC)
+    {
+        const IMAGE_NT_HEADERS32* nthdr32 = (const IMAGE_NT_HEADERS32*)nthdr;
+        info->size = nthdr32->OptionalHeader.SizeOfImage;
+    }
+    return msc_get_file_indexinfo(image, dbg, dirsize / sizeof(*dbg), info);
+}
