@@ -66,7 +66,7 @@ static ULONG WINAPI d3d_vertex_buffer7_AddRef(IDirect3DVertexBuffer7 *iface)
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
     ULONG ref = InterlockedIncrement(&buffer->ref);
 
-    TRACE("%p increasing refcount to %u.\n", buffer, ref);
+    TRACE("%p increasing refcount to %lu.\n", buffer, ref);
 
     return ref;
 }
@@ -76,7 +76,7 @@ static ULONG WINAPI d3d_vertex_buffer7_Release(IDirect3DVertexBuffer7 *iface)
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
     ULONG ref = InterlockedDecrement(&buffer->ref);
 
-    TRACE("%p decreasing refcount to %u.\n", buffer, ref);
+    TRACE("%p decreasing refcount to %lu.\n", buffer, ref);
 
     if (!ref)
     {
@@ -95,7 +95,7 @@ static ULONG WINAPI d3d_vertex_buffer7_Release(IDirect3DVertexBuffer7 *iface)
         if (buffer->version == 7)
             IDirectDraw7_Release(&buffer->ddraw->IDirectDraw7_iface);
 
-        heap_free(buffer);
+        free(buffer);
     }
 
     return ref;
@@ -111,7 +111,7 @@ static HRESULT d3d_vertex_buffer_create_wined3d_buffer(struct d3d_vertex_buffer
     struct wined3d_buffer_desc desc;
 
     desc.byte_width = buffer->size;
-    desc.usage = WINED3DUSAGE_STATICDECL;
+    desc.usage = WINED3DUSAGE_STATICDECL | WINED3DUSAGE_VIDMEM_ACCOUNTING;
     if (dynamic)
         desc.usage |= WINED3DUSAGE_DYNAMIC;
     desc.bind_flags = WINED3D_BIND_VERTEX_BUFFER;
@@ -154,15 +154,19 @@ static HRESULT WINAPI d3d_vertex_buffer7_Lock(IDirect3DVertexBuffer7 *iface,
     struct wined3d_map_desc wined3d_map_desc;
     HRESULT hr;
 
-    TRACE("iface %p, flags %#x, data %p, data_size %p.\n", iface, flags, data, data_size);
+    TRACE("iface %p, flags %#lx, data %p, data_size %p.\n", iface, flags, data, data_size);
 
     if (buffer->version != 7)
         flags &= ~(DDLOCK_NOOVERWRITE | DDLOCK_DISCARDCONTENTS);
 
+    if (buffer->discarded)
+        flags &= ~DDLOCK_DISCARDCONTENTS;
+
     if (!(flags & DDLOCK_WAIT))
         flags |= DDLOCK_DONOTWAIT;
     if (flags & DDLOCK_DISCARDCONTENTS)
     {
+        buffer->discarded = true;
         if (!buffer->dynamic)
         {
             struct wined3d_buffer *new_buffer;
@@ -253,7 +257,7 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVertices(IDirect3DVertexBuffer7
     const struct wined3d_stateblock_state *state;
     HRESULT hr;
 
-    TRACE("iface %p, vertex_op %#x, dst_idx %u, count %u, src_buffer %p, src_idx %u, device %p, flags %#x.\n",
+    TRACE("iface %p, vertex_op %#lx, dst_idx %lu, count %lu, src_buffer %p, src_idx %lu, device %p, flags %#lx.\n",
             iface, vertex_op, dst_idx, count, src_buffer, src_idx, device, flags);
 
     /* Vertex operations:
@@ -365,11 +369,11 @@ static HRESULT WINAPI d3d_vertex_buffer7_Optimize(IDirect3DVertexBuffer7 *iface,
     struct d3d_vertex_buffer *buffer = impl_from_IDirect3DVertexBuffer7(iface);
     static BOOL hide = FALSE;
 
-    TRACE("iface %p, device %p, flags %#x.\n", iface, device, flags);
+    TRACE("iface %p, device %p, flags %#lx.\n", iface, device, flags);
 
     if (!hide)
     {
-        FIXME("iface %p, device %p, flags %#x stub!\n", iface, device, flags);
+        FIXME("iface %p, device %p, flags %#lx stub!\n", iface, device, flags);
         hide = TRUE;
     }
 
@@ -409,7 +413,7 @@ static HRESULT WINAPI d3d_vertex_buffer7_ProcessVerticesStrided(IDirect3DVertexB
         DWORD vertex_op, DWORD dst_idx, DWORD count, D3DDRAWPRIMITIVESTRIDEDDATA *data,
         DWORD fvf, IDirect3DDevice7 *device, DWORD flags)
 {
-    FIXME("iface %p, vertex_op %#x, dst_idx %u, count %u, data %p, fvf %#x, device %p, flags %#x stub!\n",
+    FIXME("iface %p, vertex_op %#lx, dst_idx %lu, count %lu, data %p, fvf %#lx, device %p, flags %#lx stub!\n",
             iface, vertex_op, dst_idx, count, data, fvf, device, flags);
 
     return DD_OK;
@@ -439,12 +443,12 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
     HRESULT hr = D3D_OK;
 
     TRACE("Vertex buffer description:\n");
-    TRACE("    dwSize %u\n", desc->dwSize);
-    TRACE("    dwCaps %#x\n", desc->dwCaps);
-    TRACE("    FVF %#x\n", desc->dwFVF);
-    TRACE("    dwNumVertices %u\n", desc->dwNumVertices);
+    TRACE("    dwSize %lu\n", desc->dwSize);
+    TRACE("    dwCaps %#lx\n", desc->dwCaps);
+    TRACE("    FVF %#lx\n", desc->dwFVF);
+    TRACE("    dwNumVertices %lu\n", desc->dwNumVertices);
 
-    if (!(buffer = heap_alloc_zero(sizeof(*buffer))))
+    if (!(buffer = calloc(1, sizeof(*buffer))))
         return DDERR_OUTOFMEMORY;
 
     buffer->IDirect3DVertexBuffer7_iface.lpVtbl = &d3d_vertex_buffer7_vtbl;
@@ -461,7 +465,7 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
 
     if (FAILED(hr = d3d_vertex_buffer_create_wined3d_buffer(buffer, FALSE, &buffer->wined3d_buffer)))
     {
-        WARN("Failed to create wined3d vertex buffer, hr %#x.\n", hr);
+        WARN("Failed to create wined3d vertex buffer, hr %#lx.\n", hr);
         if (hr == WINED3DERR_INVALIDCALL)
             hr = DDERR_INVALIDPARAMS;
         goto end;
@@ -469,7 +473,7 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
 
     if (!(buffer->wined3d_declaration = ddraw_find_decl(ddraw, desc->dwFVF)))
     {
-        ERR("Failed to find vertex declaration for fvf %#x.\n", desc->dwFVF);
+        ERR("Failed to find vertex declaration for fvf %#lx.\n", desc->dwFVF);
         wined3d_buffer_decref(buffer->wined3d_buffer);
         hr = DDERR_INVALIDPARAMS;
         goto end;
@@ -481,7 +485,7 @@ end:
     if (hr == D3D_OK)
         *vertex_buf = buffer;
     else
-        heap_free(buffer);
+        free(buffer);
 
     return hr;
 }
