@@ -28,7 +28,6 @@
 
 #include "wine/mscvpdb.h"
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 #include "diasymreader_private.h"
 
@@ -107,7 +106,7 @@ static ULONG WINAPI SymDocumentWriter_Release(ISymUnmanagedDocumentWriter *iface
 
     if (ref == 0)
     {
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -187,7 +186,7 @@ static ULONG WINAPI SymWriter_Release(ISymUnmanagedWriter5 *iface)
     {
         This->lock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->lock);
-        heap_free(This);
+        free(This);
     }
 
     return ref;
@@ -205,7 +204,7 @@ static HRESULT WINAPI SymWriter_DefineDocument(ISymUnmanagedWriter5 *iface, cons
     if (!pRetVal)
         return E_POINTER;
 
-    result = heap_alloc(sizeof(*result));
+    result = malloc(sizeof(*result));
     if (!result)
         return E_OUTOFMEMORY;
 
@@ -596,7 +595,7 @@ HRESULT SymWriter_CreateInstance(REFIID iid, void **ppv)
     SymWriter *This;
     HRESULT hr;
 
-    This = heap_alloc(sizeof(*This));
+    This = malloc(sizeof(*This));
     if (!This)
         return E_OUTOFMEMORY;
 
