@@ -60,7 +60,7 @@ static const char * dumpCooperativeLevel(DWORD level)
         LE(DSSCL_WRITEPRIMARY);
     }
 #undef LE
-    return wine_dbg_sprintf("Unknown(%08x)", level);
+    return wine_dbg_sprintf("Unknown(%08lx)", level);
 }
 
 static void _dump_DSCAPS(DWORD xmask) {
@@ -128,7 +128,7 @@ static HRESULT DirectSoundDevice_Create(DirectSoundDevice ** ppDevice)
     TRACE("(%p)\n", ppDevice);
 
     /* Allocate memory */
-    device = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(DirectSoundDevice));
+    device = calloc(1, sizeof(DirectSoundDevice));
     if (device == NULL) {
         WARN("out of memory\n");
         return DSERR_OUTOFMEMORY;
@@ -162,10 +162,10 @@ static HRESULT DirectSoundDevice_Create(DirectSoundDevice ** ppDevice)
     device->guid = GUID_NULL;
 
     /* Set default wave format (may need it for waveOutOpen) */
-    device->primary_pwfx = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(WAVEFORMATEXTENSIBLE));
+    device->primary_pwfx = calloc(1, sizeof(WAVEFORMATEXTENSIBLE));
     if (!device->primary_pwfx) {
         WARN("out of memory\n");
-        HeapFree(GetProcessHeap(),0,device);
+        free(device);
         return DSERR_OUTOFMEMORY;
     }
 
@@ -190,7 +190,7 @@ static HRESULT DirectSoundDevice_Create(DirectSoundDevice ** ppDevice)
 static ULONG DirectSoundDevice_AddRef(DirectSoundDevice * device)
 {
     ULONG ref = InterlockedIncrement(&(device->ref));
-    TRACE("(%p) ref %d\n", device, ref);
+    TRACE("(%p) ref %ld\n", device, ref);
     return ref;
 }
 
@@ -198,7 +198,7 @@ static ULONG DirectSoundDevice_Release(DirectSoundDevice * device)
 {
     HRESULT hr;
     ULONG ref = InterlockedDecrement(&(device->ref));
-    TRACE("(%p) ref %d\n", device, ref);
+    TRACE("(%p) ref %ld\n", device, ref);
     if (!ref) {
         int i;
 
@@ -234,13 +234,13 @@ static ULONG DirectSoundDevice_Release(DirectSoundDevice * device)
         if(device->mmdevice)
             IMMDevice_Release(device->mmdevice);
         CloseHandle(device->sleepev);
-        HeapFree(GetProcessHeap(), 0, device->tmp_buffer);
-        HeapFree(GetProcessHeap(), 0, device->cp_buffer);
-        HeapFree(GetProcessHeap(), 0, device->buffer);
+        free(device->tmp_buffer);
+        free(device->cp_buffer);
+        free(device->buffer);
         device->mixlock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&device->mixlock);
-        HeapFree(GetProcessHeap(),0,device);
         TRACE("(%p) released\n", device);
+        free(device);
     }
     return ref;
 }
@@ -288,9 +288,10 @@ static HRESULT DirectSoundDevice_Initialize(DirectSoundDevice ** ppDevice, LPCGU
             IsEqualGUID(lpcGUID, &DSDEVID_DefaultVoiceCapture))
         return DSERR_NODRIVER;
 
-    if (GetDeviceID(lpcGUID, &devGUID) != DS_OK) {
+    hr = GetDeviceID(lpcGUID, &devGUID);
+    if (FAILED(hr)) {
         WARN("invalid parameter: lpcGUID\n");
-        return DSERR_INVALIDPARAM;
+        return hr;
     }
 
     hr = get_mmdevice(eRender, &devGUID, &mmdevice);
@@ -325,10 +326,10 @@ static HRESULT DirectSoundDevice_Initialize(DirectSoundDevice ** ppDevice, LPCGU
     hr = DSOUND_ReopenDevice(device, FALSE);
     if (FAILED(hr))
     {
-        HeapFree(GetProcessHeap(), 0, device);
+        free(device);
         LeaveCriticalSection(&DSOUND_renderers_lock);
         IMMDevice_Release(mmdevice);
-        WARN("DSOUND_ReopenDevice failed: %08x\n", hr);
+        WARN("DSOUND_ReopenDevice failed: %08lx\n", hr);
         return hr;
     }
 
@@ -426,11 +427,11 @@ static HRESULT DirectSoundDevice_CreateSoundBuffer(
     *ppdsb = NULL;
 
     if (TRACE_ON(dsound)) {
-        TRACE("(structsize=%d)\n",dsbd->dwSize);
-        TRACE("(flags=0x%08x:\n",dsbd->dwFlags);
+        TRACE("(structsize=%ld)\n",dsbd->dwSize);
+        TRACE("(flags=0x%08lx:\n",dsbd->dwFlags);
         _dump_DSBCAPS(dsbd->dwFlags);
         TRACE(")\n");
-        TRACE("(bufferbytes=%d)\n",dsbd->dwBufferBytes);
+        TRACE("(bufferbytes=%ld)\n",dsbd->dwBufferBytes);
         TRACE("(lpwfxFormat=%p)\n",dsbd->lpwfxFormat);
     }
 
@@ -526,8 +527,8 @@ static HRESULT DirectSoundDevice_CreateSoundBuffer(
             }
         }
 
-        TRACE("(formattag=0x%04x,chans=%d,samplerate=%d,"
-              "bytespersec=%d,blockalign=%d,bitspersamp=%d,cbSize=%d)\n",
+        TRACE("(formattag=0x%04x,chans=%d,samplerate=%ld,"
+              "bytespersec=%ld,blockalign=%d,bitspersamp=%d,cbSize=%d)\n",
               dsbd->lpwfxFormat->wFormatTag, dsbd->lpwfxFormat->nChannels,
               dsbd->lpwfxFormat->nSamplesPerSec,
               dsbd->lpwfxFormat->nAvgBytesPerSec,
@@ -611,10 +612,7 @@ HRESULT DirectSoundDevice_AddBuffer(
 
     AcquireSRWLockExclusive(&device->buffer_list_lock);
 
-    if (device->buffers)
-        newbuffers = HeapReAlloc(GetProcessHeap(),0,device->buffers,sizeof(IDirectSoundBufferImpl*)*(device->nrofbuffers+1));
-    else
-        newbuffers = HeapAlloc(GetProcessHeap(),0,sizeof(IDirectSoundBufferImpl*)*(device->nrofbuffers+1));
+    newbuffers = realloc(device->buffers, sizeof(IDirectSoundBufferImpl*) * (device->nrofbuffers + 1));
 
     if (newbuffers) {
         device->buffers = newbuffers;
@@ -645,7 +643,7 @@ void DirectSoundDevice_RemoveBuffer(DirectSoundDevice * device, IDirectSoundBuff
 
     if (device->nrofbuffers == 1) {
         assert(device->buffers[0] == pDSB);
-        HeapFree(GetProcessHeap(), 0, device->buffers);
+        free(device->buffers);
         device->buffers = NULL;
     } else {
         for (i = 0; i < device->nrofbuffers; i++) {
@@ -670,8 +668,8 @@ static void directsound_destroy(IDirectSoundImpl *This)
 {
     if (This->device)
         DirectSoundDevice_Release(This->device);
-    HeapFree(GetProcessHeap(),0,This);
     TRACE("(%p) released\n", This);
+    free(This);
 }
 
 static inline IDirectSoundImpl *impl_from_IUnknown(IUnknown *iface)
@@ -710,7 +708,7 @@ static ULONG WINAPI IUnknownImpl_AddRef(IUnknown *iface)
     IDirectSoundImpl *This = impl_from_IUnknown(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if(ref == 1)
         InterlockedIncrement(&This->numIfaces);
@@ -723,7 +721,7 @@ static ULONG WINAPI IUnknownImpl_Release(IUnknown *iface)
     IDirectSoundImpl *This = impl_from_IUnknown(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if (!ref && !InterlockedDecrement(&This->numIfaces))
         directsound_destroy(This);
@@ -759,7 +757,7 @@ static ULONG WINAPI IDirectSound8Impl_AddRef(IDirectSound8 *iface)
     IDirectSoundImpl *This = impl_from_IDirectSound8(iface);
     ULONG ref = InterlockedIncrement(&This->refds);
 
-    TRACE("(%p) refds=%d\n", This, ref);
+    TRACE("(%p) refds=%ld\n", This, ref);
 
     if(ref == 1)
         InterlockedIncrement(&This->numIfaces);
@@ -772,7 +770,7 @@ static ULONG WINAPI IDirectSound8Impl_Release(IDirectSound8 *iface)
     IDirectSoundImpl *This = impl_from_IDirectSound8(iface);
     ULONG ref = InterlockedDecrement(&(This->refds));
 
-    TRACE("(%p) refds=%d\n", This, ref);
+    TRACE("(%p) refds=%ld\n", This, ref);
 
     if (!ref && !InterlockedDecrement(&This->numIfaces))
         directsound_destroy(This);
@@ -803,7 +801,7 @@ static HRESULT WINAPI IDirectSound8Impl_GetCaps(IDirectSound8 *iface, DSCAPS *ds
         return DSERR_INVALIDPARAM;
     }
     if (dscaps->dwSize < sizeof(*dscaps)) {
-        WARN("invalid parameter: dscaps->dwSize = %d\n", dscaps->dwSize);
+        WARN("invalid parameter: dscaps->dwSize = %ld\n", dscaps->dwSize);
         return DSERR_INVALIDPARAM;
     }
 
@@ -831,7 +829,7 @@ static HRESULT WINAPI IDirectSound8Impl_GetCaps(IDirectSound8 *iface, DSCAPS *ds
     dscaps->dwPlayCpuOverheadSwBuffers     = This->device->drvcaps.dwPlayCpuOverheadSwBuffers;
 
     if (TRACE_ON(dsound)) {
-        TRACE("(flags=0x%08x:\n", dscaps->dwFlags);
+        TRACE("(flags=0x%08lx:\n", dscaps->dwFlags);
         _dump_DSCAPS(dscaps->dwFlags);
         TRACE(")\n");
     }
@@ -919,7 +917,7 @@ static HRESULT WINAPI IDirectSound8Impl_SetSpeakerConfig(IDirectSound8 *iface, D
 {
     IDirectSoundImpl *This = impl_from_IDirectSound8(iface);
 
-    TRACE("(%p,0x%08x)\n", This, config);
+    TRACE("(%p,0x%08lx)\n", This, config);
 
     if (!This->device) {
         WARN("not initialized\n");
@@ -981,7 +979,7 @@ HRESULT IDirectSoundImpl_Create(IUnknown *outer_unk, REFIID riid, void **ppv, BO
     TRACE("(%s, %p)\n", debugstr_guid(riid), ppv);
 
     *ppv = NULL;
-    obj = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*obj));
+    obj = calloc(1, sizeof(*obj));
     if (!obj) {
         WARN("out of memory\n");
         return DSERR_OUTOFMEMORY;
@@ -1193,6 +1191,6 @@ void DSOUND_ParseSpeakerConfig(DirectSoundDevice *device)
         break;
 
         default:
-            WARN("unknown speaker_config %u\n", device->speaker_config);
+            WARN("unknown speaker_config %lu\n", device->speaker_config);
     }
 }
