@@ -22,8 +22,6 @@
 
 
 #define COBJMACROS
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "dxdiag_private.h"
 #include "winver.h"
 #include "objidl.h"
@@ -70,7 +68,7 @@ static HRESULT WINAPI IDxDiagProviderImpl_QueryInterface(IDxDiagProvider *iface,
     if (IsEqualGUID(riid, &IID_IUnknown)
         || IsEqualGUID(riid, &IID_IDxDiagProvider)) {
         IUnknown_AddRef(iface);
-        *ppobj = This;
+        *ppobj = &This->IDxDiagProvider_iface;
         return S_OK;
     }
 
@@ -84,7 +82,7 @@ static ULONG WINAPI IDxDiagProviderImpl_AddRef(IDxDiagProvider *iface)
     IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
     ULONG refCount = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p)->(ref before=%u)\n", This, refCount - 1);
+    TRACE("(%p)->(ref before=%lu)\n", This, refCount - 1);
 
     DXDIAGN_LockModule();
 
@@ -96,7 +94,7 @@ static ULONG WINAPI IDxDiagProviderImpl_Release(IDxDiagProvider *iface)
     IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
     ULONG refCount = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p)->(ref before=%u)\n", This, refCount + 1);
+    TRACE("(%p)->(ref before=%lu)\n", This, refCount + 1);
 
     if (!refCount) {
         free_information_tree(This->info_root);
@@ -685,7 +683,7 @@ static HRESULT build_systeminfo_tree(IDxDiagContainerImpl_Container *node)
         return hr;
 
     /* FIXME: Roundoff should not be done with truncated division. */
-    swprintf(print_buf, ARRAY_SIZE(print_buf), L"%uMB RAM", (DWORD)(msex.ullTotalPhys / (1024 * 1024)));
+    swprintf(print_buf, ARRAY_SIZE(print_buf), L"%I64uMB RAM", msex.ullTotalPhys / (1024 * 1024));
     hr = add_bstr_property(node, L"szPhysicalMemoryEnglish", print_buf);
     if (FAILED(hr))
         return hr;
@@ -697,7 +695,7 @@ static HRESULT build_systeminfo_tree(IDxDiagContainerImpl_Container *node)
     args[0] = usedpage_mb;
     args[1] = availpage_mb;
     FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY, localized_pagefile_fmt,
-                   0, 0, print_buf, ARRAY_SIZE(print_buf), (__ms_va_list*)args);
+                   0, 0, print_buf, ARRAY_SIZE(print_buf), (va_list *)args);
 
     hr = add_bstr_property(node, L"szPageFileLocalized", print_buf);
     if (FAILED(hr))
@@ -792,7 +790,7 @@ static BOOL get_texture_memory(GUID *adapter, DWORD *available_mem)
     if (SUCCEEDED(hr))
     {
         dd_caps.dwCaps = DDSCAPS_LOCALVIDMEM | DDSCAPS_VIDEOMEMORY;
-        dd_caps.dwCaps2 = dd_caps.dwCaps3 = dd_caps.u1.dwCaps4 = 0;
+        dd_caps.dwCaps2 = dd_caps.dwCaps3 = dd_caps.dwCaps4 = 0;
         hr = IDirectDraw7_GetAvailableVidMem(pDirectDraw, &dd_caps, available_mem, NULL);
         IDirectDraw7_Release(pDirectDraw);
         if (SUCCEEDED(hr))
@@ -825,7 +823,7 @@ static const WCHAR *vendor_id_to_manufacturer_string(DWORD vendor_id)
             return vendors[i].name;
     }
 
-    FIXME("Unknown PCI vendor ID 0x%04x.\n", vendor_id);
+    FIXME("Unknown PCI vendor ID 0x%04lx.\n", vendor_id);
 
     return L"Unknown";
 }
@@ -887,8 +885,8 @@ static HRESULT fill_display_information_d3d(IDxDiagContainerImpl_Container *node
                 goto cleanup;
 
             swprintf(buffer, ARRAY_SIZE(buffer), L"%u.%u.%04u.%04u",
-                    HIWORD(adapter_info.DriverVersion.u.HighPart), LOWORD(adapter_info.DriverVersion.u.HighPart),
-                    HIWORD(adapter_info.DriverVersion.u.LowPart), LOWORD(adapter_info.DriverVersion.u.LowPart));
+                    HIWORD(adapter_info.DriverVersion.HighPart), LOWORD(adapter_info.DriverVersion.HighPart),
+                    HIWORD(adapter_info.DriverVersion.LowPart), LOWORD(adapter_info.DriverVersion.LowPart));
 
             hr = add_bstr_property(display_adapter, L"szDriverVersion", buffer);
             if (FAILED(hr))
@@ -1154,7 +1152,7 @@ static HRESULT fill_display_information_fallback(IDxDiagContainerImpl_Container
         return S_OK;
 
     dd_caps.dwCaps = DDSCAPS_LOCALVIDMEM | DDSCAPS_VIDEOMEMORY;
-    dd_caps.dwCaps2 = dd_caps.dwCaps3 = dd_caps.u1.dwCaps4 = 0;
+    dd_caps.dwCaps2 = dd_caps.dwCaps3 = dd_caps.dwCaps4 = 0;
     hr = IDirectDraw7_GetAvailableVidMem(pDirectDraw, &dd_caps, &tmp, NULL);
     if (SUCCEEDED(hr))
     {
@@ -1190,7 +1188,7 @@ static HRESULT fill_display_information_fallback(IDxDiagContainerImpl_Container
         if (surface_descr.dwFlags & DDSD_PIXELFORMAT)
         {
             hr = add_ui4_property(display_adapter, L"dwBpp",
-                    surface_descr.u4.ddpfPixelFormat.u1.dwRGBBitCount);
+                    surface_descr.ddpfPixelFormat.dwRGBBitCount);
             if (FAILED(hr))
                 goto cleanup;
         }
@@ -1548,16 +1546,16 @@ static HRESULT fill_filter_data_information(IDxDiagContainerImpl_Container *subc
 
     if (pRF->dwVersion == 1)
     {
-        for (j = 0; j < pRF->u.s1.cPins; j++)
-            if (pRF->u.s1.rgPins[j].bOutput)
+        for (j = 0; j < pRF->cPins; j++)
+            if (pRF->rgPins[j].bOutput)
                 dwNOutputs++;
             else
                 dwNInputs++;
     }
     else if (pRF->dwVersion == 2)
     {
-        for (j = 0; j < pRF->u.s2.cPins2; j++)
-            if (pRF->u.s2.rgPins2[j].dwFlags & REG_PINFLAG_B_OUTPUT)
+        for (j = 0; j < pRF->cPins2; j++)
+            if (pRF->rgPins2[j].dwFlags & REG_PINFLAG_B_OUTPUT)
                 dwNOutputs++;
             else
                 dwNInputs++;
@@ -1654,7 +1652,7 @@ static HRESULT build_directshowfilters_tree(IDxDiagContainerImpl_Container *node
         return hr;
 
     hr = ICreateDevEnum_CreateClassEnumerator(pCreateDevEnum, &CLSID_ActiveMovieCategories, &pEmCat, 0);
-    if (FAILED(hr))
+    if (hr != S_OK)
         goto cleanup;
 
     while (IEnumMoniker_Next(pEmCat, 1, &pMCat, NULL) == S_OK)
