@@ -188,9 +188,7 @@ BOOL WINAPI MapAndLoad(PCSTR pszImageName, PCSTR pszDllPath, PLOADED_IMAGE pLoad
     pLoadedImage->hFile            = hFile;
     pLoadedImage->MappedAddress    = mapping;
     pLoadedImage->FileHeader       = pNtHeader;
-    pLoadedImage->Sections         = (PIMAGE_SECTION_HEADER)
-        ((LPBYTE) &pNtHeader->OptionalHeader +
-         pNtHeader->FileHeader.SizeOfOptionalHeader);
+    pLoadedImage->Sections         = IMAGE_FIRST_SECTION(pNtHeader);
     pLoadedImage->NumberOfSections = pNtHeader->FileHeader.NumberOfSections;
     pLoadedImage->SizeOfImage      = GetFileSize(hFile, NULL);
     pLoadedImage->Characteristics  = pNtHeader->FileHeader.Characteristics;
