@@ -527,8 +527,10 @@ static HRESULT Array_shift(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigne
         hres = jsdisp_get_idx(jsthis, i, &v);
         if(hres == DISP_E_UNKNOWNNAME)
             hres = jsdisp_delete_idx(jsthis, i-1);
-        else if(SUCCEEDED(hres))
+        else if(SUCCEEDED(hres)) {
             hres = jsdisp_propput_idx(jsthis, i-1, v);
+            jsval_release(v);
+        }
     }
 
     if(SUCCEEDED(hres)) {
@@ -870,6 +872,8 @@ static HRESULT Array_splice(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsign
         }
 
         add_args = argc-2;
+    } else if (argc && ctx->version >= SCRIPTLANGUAGEVERSION_ES5) {
+        delete_cnt = length-start;
     }
 
     if(r) {
@@ -1148,6 +1152,8 @@ static HRESULT Array_filter(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsign
 
     if(r)
         *r = jsval_obj(arr);
+    else
+        jsdisp_release(arr);
 done:
     jsdisp_release(jsthis);
     return hres;
@@ -1352,14 +1358,16 @@ static HRESULT Array_map(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsigned
     if(argc > 1)
         context_this = argv[1];
 
-    hres = create_array(ctx, length, &array);
+    hres = create_array(ctx, 0, &array);
     if(FAILED(hres))
         goto done;
 
     for(k = 0; k < length; k++) {
         hres = jsdisp_get_idx(jsthis, k, &callback_args[0]);
-        if(hres == DISP_E_UNKNOWNNAME)
+        if(hres == DISP_E_UNKNOWNNAME) {
+            hres = S_OK;
             continue;
+        }
         if(FAILED(hres))
             break;
 
@@ -1418,8 +1426,10 @@ static HRESULT Array_reduce(script_ctx_t *ctx, jsval_t vthis, WORD flags, unsign
 
     for(k = 0; k < length; k++) {
         hres = jsdisp_get_idx(jsthis, k, &callback_args[1]);
-        if(hres == DISP_E_UNKNOWNNAME)
+        if(hres == DISP_E_UNKNOWNNAME) {
+            hres = S_OK;
             continue;
+        }
         if(FAILED(hres))
             break;
 
@@ -1663,9 +1673,8 @@ static HRESULT ArrayConstr_isArray(script_ctx_t *ctx, jsval_t vthis, WORD flags,
         return S_OK;
     }
 
-    obj = iface_to_jsdisp(get_object(argv[0]));
+    obj = to_jsdisp(get_object(argv[0]));
     if(r) *r = jsval_bool(obj && is_class(obj, JSCLASS_ARRAY));
-    if(obj) jsdisp_release(obj);
     return S_OK;
 }
 
