@@ -162,6 +162,23 @@ static inline BSTR local_name(call_frame_t *frame, int ref)
     return ref < 0 ? frame->function->params[-ref-1] : frame->function->variables[ref].name;
 }
 
+static jsval_t *get_detached_var_ref(scope_chain_t *scope, const WCHAR *name)
+{
+    struct vars_buffer *detached_vars = scope->detached_vars;
+    local_ref_t *ref;
+
+    if(!detached_vars)
+        return NULL;
+    ref = lookup_local(detached_vars->func_code, name, scope->scope_index);
+    return ref && ref->ref < 0 ? &detached_vars->var[-ref->ref - 1] : NULL;
+}
+
+static HRESULT get_detached_var_dispid(scope_chain_t *scope, const WCHAR *name, DISPID *id)
+{
+    jsval_t *var_ref = get_detached_var_ref(scope, name);
+    return var_ref ? jsdisp_get_idx_id(&scope->dispex, var_ref - scope->detached_vars->var, id) : DISP_E_UNKNOWNNAME;
+}
+
 /* Steals input reference even on failure. */
 static HRESULT stack_push_exprval(script_ctx_t *ctx, exprval_t *val)
 {
@@ -207,6 +224,7 @@ static BOOL stack_topn_exprval(script_ctx_t *ctx, unsigned n, exprval_t *r)
         unsigned off = get_number(v);
 
         if(!frame->base_scope->frame && off >= frame->arguments_off) {
+            jsdisp_t *jsobj;
             DISPID id;
             BSTR name;
             HRESULT hres = E_FAIL;
@@ -227,7 +245,19 @@ static BOOL stack_topn_exprval(script_ctx_t *ctx, unsigned n, exprval_t *r)
 
             while (1)
             {
-                if (scope->jsobj && SUCCEEDED(hres = jsdisp_get_id(scope->jsobj, name, 0, &id)))
+                hres = get_detached_var_dispid(scope, name, &id);
+                if (hres != DISP_E_UNKNOWNNAME)
+                {
+                    if (FAILED(hres))
+                    {
+                        r->type = EXPRVAL_INVALID;
+                        r->u.hres = hres;
+                        return FALSE;
+                    }
+                    jsobj = &scope->dispex;
+                    break;
+                }
+                if ((jsobj = to_jsdisp(scope->obj)) && SUCCEEDED(hres = jsdisp_get_id(jsobj, name, 0, &id)))
                     break;
                 if (scope == frame->base_scope)
                 {
@@ -238,10 +268,10 @@ static BOOL stack_topn_exprval(script_ctx_t *ctx, unsigned n, exprval_t *r)
                 scope = scope->next;
             }
 
-            *stack_top_ref(ctx, n+1) = jsval_obj(jsdisp_addref(scope->jsobj));
+            *stack_top_ref(ctx, n+1) = jsval_obj(jsdisp_addref(jsobj));
             *stack_top_ref(ctx, n) = jsval_number(id);
             r->type = EXPRVAL_IDREF;
-            r->u.idref.disp = scope->obj;
+            r->u.idref.disp = to_disp(jsobj);
             r->u.idref.id = id;
             return TRUE;
         }
@@ -422,9 +452,24 @@ static inline void clear_acc(script_ctx_t *ctx)
     ctx->acc = jsval_undefined();
 }
 
+static inline scope_chain_t *scope_from_dispex(jsdisp_t *dispex)
+{
+    return CONTAINING_RECORD(dispex, scope_chain_t, dispex);
+}
+
 static void scope_destructor(jsdisp_t *dispex)
 {
-    scope_chain_t *scope = CONTAINING_RECORD(dispex, scope_chain_t, dispex);
+    scope_chain_t *scope = scope_from_dispex(dispex);
+
+    if(scope->detached_vars) {
+        struct vars_buffer *vars = scope->detached_vars;
+        unsigned i, cnt = vars->argc;
+
+        release_bytecode(vars->func_code->bytecode);
+        for(i = 0; i < cnt; i++)
+            jsval_release(vars->var[i]);
+        free(vars);
+    }
 
     if(scope->next)
         scope_release(scope->next);
@@ -434,11 +479,53 @@ static void scope_destructor(jsdisp_t *dispex)
     free(scope);
 }
 
+static unsigned scope_idx_length(jsdisp_t *dispex)
+{
+    scope_chain_t *scope = scope_from_dispex(dispex);
+
+    return scope->detached_vars->argc;
+}
+
+static HRESULT scope_idx_get(jsdisp_t *dispex, unsigned idx, jsval_t *r)
+{
+    scope_chain_t *scope = scope_from_dispex(dispex);
+
+    return jsval_copy(scope->detached_vars->var[idx], r);
+}
+
+static HRESULT scope_idx_put(jsdisp_t *dispex, unsigned idx, jsval_t val)
+{
+    scope_chain_t *scope = scope_from_dispex(dispex);
+    jsval_t copy, *ref;
+    HRESULT hres;
+
+    hres = jsval_copy(val, &copy);
+    if(FAILED(hres))
+        return hres;
+
+    ref = &scope->detached_vars->var[idx];
+    jsval_release(*ref);
+    *ref = copy;
+    return S_OK;
+}
+
 static HRESULT scope_gc_traverse(struct gc_ctx *gc_ctx, enum gc_traverse_op op, jsdisp_t *dispex)
 {
-    scope_chain_t *scope = CONTAINING_RECORD(dispex, scope_chain_t, dispex);
+    scope_chain_t *scope = scope_from_dispex(dispex);
+    jsdisp_t *jsobj;
     HRESULT hres;
 
+    if(scope->detached_vars) {
+        struct vars_buffer *vars = scope->detached_vars;
+        unsigned i, cnt = vars->argc;
+
+        for(i = 0; i < cnt; i++) {
+            hres = gc_process_linked_val(gc_ctx, op, dispex, &vars->var[i]);
+            if(FAILED(hres))
+                return hres;
+        }
+    }
+
     if(scope->next) {
         hres = gc_process_linked_obj(gc_ctx, op, dispex, &scope->next->dispex, (void**)&scope->next);
         if(FAILED(hres))
@@ -454,7 +541,7 @@ static HRESULT scope_gc_traverse(struct gc_ctx *gc_ctx, enum gc_traverse_op op,
         return S_OK;
     }
 
-    return scope->jsobj ? gc_process_linked_obj(gc_ctx, op, dispex, scope->jsobj, (void**)&scope->obj) : S_OK;
+    return scope->obj && (jsobj = to_jsdisp(scope->obj)) ? gc_process_linked_obj(gc_ctx, op, dispex, jsobj, (void**)&scope->obj) : S_OK;
 }
 
 static const builtin_info_t scope_info = {
@@ -464,13 +551,13 @@ static const builtin_info_t scope_info = {
     NULL,
     scope_destructor,
     NULL,
-    NULL,
-    NULL,
-    NULL,
+    scope_idx_length,
+    scope_idx_get,
+    scope_idx_put,
     scope_gc_traverse
 };
 
-static HRESULT scope_push(script_ctx_t *ctx, scope_chain_t *scope, jsdisp_t *jsobj, IDispatch *obj, scope_chain_t **ret)
+static HRESULT scope_push(script_ctx_t *ctx, scope_chain_t *scope, IDispatch *obj, scope_chain_t **ret)
 {
     scope_chain_t *new_scope;
     HRESULT hres;
@@ -487,7 +574,6 @@ static HRESULT scope_push(script_ctx_t *ctx, scope_chain_t *scope, jsdisp_t *jso
 
     if (obj)
         IDispatch_AddRef(obj);
-    new_scope->jsobj = jsobj;
     new_scope->obj = obj;
     new_scope->frame = NULL;
     new_scope->next = scope ? scope_addref(scope) : NULL;
@@ -513,12 +599,9 @@ static HRESULT disp_get_id(script_ctx_t *ctx, IDispatch *disp, const WCHAR *name
     BSTR bstr;
     HRESULT hres;
 
-    jsdisp = iface_to_jsdisp(disp);
-    if(jsdisp) {
-        hres = jsdisp_get_id(jsdisp, name, flags, id);
-        jsdisp_release(jsdisp);
-        return hres;
-    }
+    jsdisp = to_jsdisp(disp);
+    if(jsdisp)
+        return jsdisp_get_id(jsdisp, name, flags, id);
 
     if(name_bstr) {
         bstr = name_bstr;
@@ -624,10 +707,34 @@ HRESULT jsval_strict_equal(jsval_t lval, jsval_t rval, BOOL *ret)
     return S_OK;
 }
 
+static HRESULT alloc_detached_vars(script_ctx_t *ctx, call_frame_t *frame, scope_chain_t *scope)
+{
+    function_code_t *func = frame->function;
+    unsigned i, argc;
+
+    argc = (scope == frame->base_scope) ? max(frame->argc, func->param_cnt) : 0;
+    if(!argc)
+        return S_OK;
+
+    if(!(scope->detached_vars = malloc(FIELD_OFFSET(struct vars_buffer, var[argc]))))
+        return E_OUTOFMEMORY;
+    scope->detached_vars->argc = argc;
+    scope->detached_vars->func_code = func;
+    bytecode_addref(func->bytecode);
+
+    for(i = 0; i < argc; i++) {
+        scope->detached_vars->var[i] = ctx->stack[frame->arguments_off + i];
+        ctx->stack[frame->arguments_off + i] = jsval_undefined();
+    }
+
+    return S_OK;
+}
+
 static HRESULT detach_scope(script_ctx_t *ctx, call_frame_t *frame, scope_chain_t *scope)
 {
     function_code_t *func = frame->function;
     unsigned int i, index;
+    jsdisp_t *jsobj;
     HRESULT hres;
 
     if (!scope->frame)
@@ -636,18 +743,22 @@ static HRESULT detach_scope(script_ctx_t *ctx, call_frame_t *frame, scope_chain_
     assert(scope->frame == frame);
     scope->frame = NULL;
 
-    if (!scope->jsobj)
-    {
-        assert(!scope->obj);
+    hres = alloc_detached_vars(ctx, frame, scope);
+    if (FAILED(hres))
+        return hres;
 
-        if (FAILED(hres = create_object(ctx, NULL, &scope->jsobj)))
+    if (!scope->obj)
+    {
+        if (FAILED(hres = create_dispex(ctx, NULL, NULL, &jsobj)))
             return hres;
-        scope->obj = to_disp(scope->jsobj);
+        scope->obj = to_disp(jsobj);
     }
+    else
+        jsobj = as_jsdisp(scope->obj);
 
     if (scope == frame->base_scope && func->name && func->local_ref == INVALID_LOCAL_REF &&
         ctx->version >= SCRIPTLANGUAGEVERSION_ES5)
-        jsdisp_propput_name(scope->jsobj, func->name, jsval_obj(jsdisp_addref(frame->function_instance)));
+        jsdisp_propput_name(jsobj, func->name, jsval_obj(frame->function_instance));
 
     index = scope->scope_index;
     for(i = 0; i < frame->function->local_scopes[index].locals_cnt; i++)
@@ -655,9 +766,11 @@ static HRESULT detach_scope(script_ctx_t *ctx, call_frame_t *frame, scope_chain_
         WCHAR *name = frame->function->local_scopes[index].locals[i].name;
         int ref = frame->function->local_scopes[index].locals[i].ref;
 
-        if (FAILED(hres = jsdisp_propput_name(scope->jsobj, name, ctx->stack[local_off(frame, ref)])))
+        if (ref < 0)
+            continue;
+        if (FAILED(hres = jsdisp_propput_name(jsobj, name, ctx->stack[local_off(frame, ref)])))
             return hres;
-        if (frame->function->variables[ref].func_id != -1 && scope != frame->base_scope
+        if (scope != frame->base_scope && frame->function->variables[ref].func_id != -1
                 && FAILED(hres = jsdisp_propput_name(frame->variable_obj, name, ctx->stack[local_off(frame, ref)])))
             return hres;
     }
@@ -687,7 +800,7 @@ static HRESULT detach_variable_object(script_ctx_t *ctx, call_frame_t *frame, BO
     TRACE("detaching %p\n", frame);
 
     assert(frame == frame->base_scope->frame);
-    assert(frame->variable_obj == frame->base_scope->jsobj);
+    assert(to_disp(frame->variable_obj) == frame->base_scope->obj);
 
     if(!from_release && !frame->arguments_obj) {
         hres = setup_arguments_object(ctx, frame);
@@ -782,15 +895,16 @@ static HRESULT identifier_eval(script_ctx_t *ctx, BSTR identifier, exprval_t *re
                     ret->u.val = jsval_obj(jsdisp_addref(scope->frame->function_instance));
                     return S_OK;
                 }
+            }else if((hres = get_detached_var_dispid(scope, identifier, &id)) != DISP_E_UNKNOWNNAME) {
+                if(SUCCEEDED(hres))
+                    exprval_set_disp_ref(ret, to_disp(&scope->dispex), id);
+                return hres;
             }
 
-            if (!scope->jsobj && !scope->obj)
+            if (!scope->obj)
                 continue;
 
-            if(scope->jsobj)
-                hres = jsdisp_get_id(scope->jsobj, identifier, fdexNameImplicit, &id);
-            else
-                hres = disp_get_id(ctx, scope->obj, identifier, identifier, fdexNameImplicit, &id);
+            hres = disp_get_id(ctx, scope->obj, identifier, identifier, fdexNameImplicit, &id);
             if(SUCCEEDED(hres)) {
                 exprval_set_disp_ref(ret, scope->obj, id);
                 return S_OK;
@@ -943,6 +1057,7 @@ static HRESULT scope_init_locals(script_ctx_t *ctx)
     unsigned int i, off, index;
     scope_chain_t *scope;
     BOOL detached_vars;
+    jsdisp_t *jsobj;
     HRESULT hres;
 
     scope = frame->scope;
@@ -954,13 +1069,14 @@ static HRESULT scope_init_locals(script_ctx_t *ctx)
         assert(frame->base_scope->frame == frame);
         frame->scope->frame = ctx->call_ctx;
     }
-    else if (!scope->jsobj)
+    else if (!scope->obj)
     {
-        assert(!scope->obj);
-        if (FAILED(hres = create_object(ctx, NULL, &scope->jsobj)))
+        if (FAILED(hres = create_dispex(ctx, NULL, NULL, &jsobj)))
             return hres;
-        scope->obj = to_disp(scope->jsobj);
+        scope->obj = to_disp(jsobj);
     }
+    else
+        jsobj = as_jsdisp(scope->obj);
 
     for(i = 0; i < frame->function->local_scopes[index].locals_cnt; i++)
     {
@@ -978,7 +1094,10 @@ static HRESULT scope_init_locals(script_ctx_t *ctx)
                 return hres;
             val = jsval_obj(func_obj);
             if (detached_vars && FAILED(hres = jsdisp_propput_name(frame->variable_obj, name, jsval_obj(func_obj))))
+            {
+                jsdisp_release(func_obj);
                 return hres;
+            }
         }
         else
         {
@@ -987,7 +1106,9 @@ static HRESULT scope_init_locals(script_ctx_t *ctx)
 
         if (detached_vars)
         {
-            if (FAILED(hres = jsdisp_propput_name(scope->jsobj, name, val)))
+            hres = jsdisp_propput_name(jsobj, name, val);
+            jsval_release(val);
+            if (FAILED(hres))
                 return hres;
         }
         else
@@ -1015,7 +1136,7 @@ static HRESULT interp_push_with_scope(script_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    hres = scope_push(ctx, ctx->call_ctx->scope, to_jsdisp(disp), disp, &ctx->call_ctx->scope);
+    hres = scope_push(ctx, ctx->call_ctx->scope, disp, &ctx->call_ctx->scope);
     IDispatch_Release(disp);
     return hres;
 }
@@ -1029,7 +1150,7 @@ static HRESULT interp_push_block_scope(script_ctx_t *ctx)
 
     TRACE("scope_index %u.\n", scope_index);
 
-    hres = scope_push(ctx, ctx->call_ctx->scope, NULL, NULL, &frame->scope);
+    hres = scope_push(ctx, ctx->call_ctx->scope, NULL, &frame->scope);
 
     if (FAILED(hres) || !scope_index)
         return hres;
@@ -1241,7 +1362,7 @@ static HRESULT interp_enter_catch(script_ctx_t *ctx)
     hres = jsdisp_propput_name(scope_obj, ident, v);
     jsval_release(v);
     if(SUCCEEDED(hres))
-        hres = scope_push(ctx, ctx->call_ctx->scope, scope_obj, to_disp(scope_obj), &ctx->call_ctx->scope);
+        hres = scope_push(ctx, ctx->call_ctx->scope, to_disp(scope_obj), &ctx->call_ctx->scope);
     jsdisp_release(scope_obj);
     return hres;
 }
@@ -1459,6 +1580,40 @@ static HRESULT interp_call_member(script_ctx_t *ctx)
             argn, stack_args(ctx, argn), do_ret ? &ctx->acc : NULL);
 }
 
+/* ECMA-262 5th Edition 15.1.2.1.1 */
+static HRESULT interp_call_eval(script_ctx_t *ctx)
+{
+    const unsigned argn = get_op_uint(ctx, 0);
+    const int do_ret = get_op_int(ctx, 1);
+    HRESULT hres = S_OK;
+    exprval_t exprval;
+    jsdisp_t *jsdisp;
+    BSTR identifier;
+    jsval_t v;
+
+    TRACE("%d %d\n", argn, do_ret);
+
+    identifier = SysAllocString(L"eval");
+    hres = identifier_eval(ctx, identifier, &exprval);
+    SysFreeString(identifier);
+    if(FAILED(hres))
+        return hres;
+
+    clear_acc(ctx);
+    hres = exprval_propget(ctx, &exprval, &v);
+    if(SUCCEEDED(hres)) {
+        if(is_object_instance(v) && (jsdisp = to_jsdisp(get_object(v))) && jsdisp->ctx == ctx && is_builtin_eval_func(jsdisp))
+            hres = builtin_eval(ctx, ctx->call_ctx, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
+                                argn, stack_args(ctx, argn), do_ret ? &ctx->acc : NULL);
+        else
+            hres = exprval_call(ctx, &exprval, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
+                                argn, stack_args(ctx, argn), do_ret ? &ctx->acc : NULL);
+        jsval_release(v);
+    }
+    exprval_release(&exprval);
+    return hres;
+}
+
 /* ECMA-262 3rd Edition    11.1.1 */
 static HRESULT interp_this(script_ctx_t *ctx)
 {
@@ -1731,7 +1886,7 @@ static HRESULT interp_obj_prop(script_ctx_t *ctx)
         jsdisp_t *func;
 
         assert(is_object_instance(val));
-        func = iface_to_jsdisp(get_object(val));
+        func = to_jsdisp(get_object(val));
 
         desc.mask = desc.flags;
         if(type == PROPERTY_DEFINITION_GETTER) {
@@ -1743,7 +1898,6 @@ static HRESULT interp_obj_prop(script_ctx_t *ctx)
         }
 
         hres = jsdisp_define_property(obj, name, &desc);
-        jsdisp_release(func);
     }
 
     jsval_release(val);
@@ -1887,15 +2041,12 @@ static HRESULT interp_instanceof(script_ctx_t *ctx)
         hres = JS_E_OBJECT_EXPECTED;
     else if(is_object_instance(prot)) {
         if(is_object_instance(v))
-            tmp = iface_to_jsdisp(get_object(v));
+            tmp = to_jsdisp(get_object(v));
         for(iter = tmp; !ret && iter; iter = iter->prototype) {
             hres = disp_cmp(get_object(prot), to_disp(iter), &ret);
             if(FAILED(hres))
                 break;
         }
-
-        if(tmp)
-            jsdisp_release(tmp);
     }else {
         FIXME("prototype is not an object\n");
         hres = E_FAIL;
@@ -2181,9 +2332,8 @@ static HRESULT typeof_string(jsval_t v, const WCHAR **ret)
     case JSV_OBJECT: {
         jsdisp_t *dispex;
 
-        if((dispex = iface_to_jsdisp(get_object(v)))) {
+        if((dispex = to_jsdisp(get_object(v)))) {
             *ret = is_class(dispex, JSCLASS_FUNCTION) ? L"function" : L"object";
-            jsdisp_release(dispex);
         }else {
             *ret = L"object";
         }
@@ -2966,7 +3116,7 @@ static void pop_call_frame(script_ctx_t *ctx)
     }
 
     if(frame->arguments_obj)
-        detach_arguments_object(frame->arguments_obj);
+        detach_arguments_object(frame);
     if(frame->scope)
         scope_release(frame->scope);
 
@@ -3231,7 +3381,7 @@ static HRESULT setup_scope(script_ctx_t *ctx, call_frame_t *frame, scope_chain_t
 
     frame->pop_variables = i;
 
-    hres = scope_push(ctx, scope_chain, variable_object, to_disp(variable_object), &scope);
+    hres = scope_push(ctx, scope_chain, to_disp(variable_object), &scope);
     if(FAILED(hres)) {
         stack_popn(ctx, ctx->stack_top - orig_stack);
         return hres;
