@@ -60,6 +60,7 @@
 #define IDS_ENUMERATOR_EXPECTED             0x1397
 #define IDS_REGEXP_EXPECTED                 0x1398
 #define IDS_REGEXP_SYNTAX_ERROR             0x1399
+#define IDS_UNEXPECTED_QUANTIFIER           0x139A
 #define IDS_EXCEPTION_THROWN                0x139E
 #define IDS_URI_INVALID_CHAR                0x13A0
 #define IDS_URI_INVALID_CODING              0x13A1
@@ -74,6 +75,7 @@
 #define IDS_NONCONFIGURABLE_REDEFINED       0x13D6
 #define IDS_NONWRITABLE_MODIFIED            0x13D7
 #define IDS_WRONG_THIS                      0x13FC
+#define IDS_KEY_NOT_OBJECT                  0x13FD
 /* FIXME: This is not compatible with native, but we would
  * conflict with IDS_UNSUPPORTED_ACTION otherwise */
 #define IDS_PROP_DESC_MISMATCH              0x1F00
