@@ -25,7 +25,6 @@
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
-#define NONAMELESSUNION
 #include "windef.h"
 #include "winbase.h"
 #include "wincon.h"
@@ -87,10 +86,10 @@ ULONGLONG WINAPI DECLSPEC_HOTPATCH GetTickCount64(void)
 
     do
     {
-        high = user_shared_data->u.TickCount.High1Time;
-        low = user_shared_data->u.TickCount.LowPart;
+        high = user_shared_data->TickCount.High1Time;
+        low = user_shared_data->TickCount.LowPart;
     }
-    while (high != user_shared_data->u.TickCount.High2Time);
+    while (high != user_shared_data->TickCount.High2Time);
     /* note: we ignore TickCountMultiplier */
     return (ULONGLONG)high << 32 | low;
 }
@@ -101,7 +100,7 @@ ULONGLONG WINAPI DECLSPEC_HOTPATCH GetTickCount64(void)
 DWORD WINAPI DECLSPEC_HOTPATCH GetTickCount(void)
 {
     /* note: we ignore TickCountMultiplier */
-    return user_shared_data->u.TickCount.LowPart;
+    return user_shared_data->TickCount.LowPart;
 }
 
 /***********************************************************************
@@ -566,7 +565,7 @@ BOOL WINAPI GetNamedPipeHandleStateA(
     WCHAR *username = NULL;
     BOOL ret;
 
-    WARN("%p %p %p %p %p %p %d: semi-stub\n", hNamedPipe, lpState, lpCurInstances,
+    WARN("%p %p %p %p %p %p %ld: semi-stub\n", hNamedPipe, lpState, lpCurInstances,
          lpMaxCollectionCount, lpCollectDataTimeout, lpUsername, nUsernameMaxSize);
 
     if (lpUsername && nUsernameMaxSize &&
@@ -593,7 +592,7 @@ BOOL WINAPI CallNamedPipeA(
     LPWSTR str = NULL;
     BOOL ret;
 
-    TRACE("%s %p %d %p %d %p %d\n",
+    TRACE("%s %p %ld %p %ld %p %ld\n",
            debugstr_a(lpNamedPipeName), lpInput, dwInputSize,
            lpOutput, dwOutputSize, lpBytesRead, nTimeout);
 
@@ -623,7 +622,7 @@ HANDLE WINAPI CreateMailslotA( LPCSTR lpName, DWORD nMaxMessageSize,
     HANDLE handle;
     LPWSTR name = NULL;
 
-    TRACE("%s %d %d %p\n", debugstr_a(lpName),
+    TRACE("%s %ld %ld %p\n", debugstr_a(lpName),
           nMaxMessageSize, lReadTimeout, sa);
 
     if( lpName )
@@ -665,7 +664,7 @@ HANDLE WINAPI CreateMailslotW( LPCWSTR lpName, DWORD nMaxMessageSize,
     LARGE_INTEGER timeout;
     IO_STATUS_BLOCK iosb;
 
-    TRACE("%s %d %d %p\n", debugstr_w(lpName),
+    TRACE("%s %ld %ld %p\n", debugstr_w(lpName),
           nMaxMessageSize, lReadTimeout, sa);
 
     if (!RtlDosPathNameToNtPathName_U( lpName, &nameW, NULL, NULL ))
@@ -767,7 +766,7 @@ BOOL WINAPI SetMailslotInfo( HANDLE hMailslot, DWORD dwReadTimeout)
     FILE_MAILSLOT_SET_INFORMATION info;
     IO_STATUS_BLOCK iosb;
 
-    TRACE("%p %d\n", hMailslot, dwReadTimeout);
+    TRACE("%p %ld\n", hMailslot, dwReadTimeout);
 
     if (dwReadTimeout != MAILSLOT_WAIT_FOREVER)
         info.ReadTimeout.QuadPart = (ULONGLONG)dwReadTimeout * -10000;
