@@ -385,7 +385,7 @@ static DWORD INSTR_inport( WORD port, int size, CONTEXT *context )
                      (WORD)context->SegCs, LOWORD(context->Eip));
             break;
         case 4:
-            TRACE_(io)( "0x%x < %08x @ %04x:%04x\n", port, res,
+            TRACE_(io)( "0x%x < %08lx @ %04x:%04x\n", port, res,
                      (WORD)context->SegCs, LOWORD(context->Eip));
             break;
         }
@@ -416,7 +416,7 @@ static void INSTR_outport( WORD port, int size, DWORD val, CONTEXT *context )
                     (WORD)context->SegCs, LOWORD(context->Eip));
             break;
         case 4:
-            TRACE_(io)("0x%x > %08x @ %04x:%04x\n", port, val,
+            TRACE_(io)("0x%x > %08lx @ %04x:%04x\n", port, val,
                     (WORD)context->SegCs, LOWORD(context->Eip));
             break;
         }
@@ -523,7 +523,7 @@ DWORD __wine_emulate_instruction( EXCEPTION_RECORD *rec, CONTEXT *context )
                 switch (instr[2])
                 {
                 case 0xc0:
-                    FIXME("mov %%eax, %%cr0 at 0x%08x, EAX=0x%08x\n",
+                    FIXME("mov %%eax, %%cr0 at 0x%08lx, EAX=0x%08lx\n",
                           context->Eip,context->Eax );
                           context->Eip += prefixlen+3;
                     return ExceptionContinueExecution;
@@ -546,12 +546,12 @@ DWORD __wine_emulate_instruction( EXCEPTION_RECORD *rec, CONTEXT *context )
                      * bit 7: PGE Enable global pages
                      * bit 8: PCE Enable performance counters at IPL3
                      */
-                    FIXME("mov %%cr4, %%eax at 0x%08x\n",context->Eip);
+                    FIXME("mov %%cr4, %%eax at 0x%08lx\n",context->Eip);
                     context->Eax = 0;
                     context->Eip += prefixlen+3;
                     return ExceptionContinueExecution;
                 case 0xc0: /* mov %cr0, %eax */
-                    FIXME("mov %%cr0, %%eax at 0x%08x\n",context->Eip);
+                    FIXME("mov %%cr0, %%eax at 0x%08lx\n",context->Eip);
                     context->Eax = 0x10; /* FIXME: set more bits ? */
                     context->Eip += prefixlen+3;
                     return ExceptionContinueExecution;
@@ -564,12 +564,12 @@ DWORD __wine_emulate_instruction( EXCEPTION_RECORD *rec, CONTEXT *context )
                 switch (instr[2])
                 {
                 case 0xc8: /* mov %dr1, %eax */
-                    TRACE("mov %%dr1, %%eax at 0x%08x\n",context->Eip);
+                    TRACE("mov %%dr1, %%eax at 0x%08lx\n",context->Eip);
                     context->Eax = context->Dr1;
                     context->Eip += prefixlen+3;
                     return ExceptionContinueExecution;
                 case 0xf8: /* mov %dr7, %eax */
-                    TRACE("mov %%dr7, %%eax at 0x%08x\n",context->Eip);
+                    TRACE("mov %%dr7, %%eax at 0x%08lx\n",context->Eip);
                     context->Eax = 0x400;
                     context->Eip += prefixlen+3;
                     return ExceptionContinueExecution;
