@@ -96,9 +96,6 @@ static void WINAPI DOSVM_DefaultHandler( CONTEXT *context )
  */
 void DOSVM_Exit( WORD retval )
 {
-    DWORD count;
-
-    ReleaseThunkLock( &count );
     ExitThread( retval );
 }
 
@@ -241,10 +238,10 @@ static void DOSVM_HardwareInterruptPM( CONTEXT *context, BYTE intnum )
  */
 BOOL DOSVM_EmulateInterruptPM( CONTEXT *context, BYTE intnum )
 {
-    TRACE_(relay)("\1Call DOS int 0x%02x ret=%04x:%08x\n"
-                  "  eax=%08x ebx=%08x ecx=%08x edx=%08x\n"
-                  "  esi=%08x edi=%08x ebp=%08x esp=%08x\n"
-                  "  ds=%04x es=%04x fs=%04x gs=%04x ss=%04x flags=%08x\n",
+    TRACE_(relay)("\1Call DOS int 0x%02x ret=%04lx:%08lx\n"
+                  "  eax=%08lx ebx=%08lx ecx=%08lx edx=%08lx\n"
+                  "  esi=%08lx edi=%08lx ebp=%08lx esp=%08lx\n"
+                  "  ds=%04lx es=%04lx fs=%04lx gs=%04lx ss=%04lx flags=%08lx\n",
                   intnum, context->SegCs, context->Eip,
                   context->Eax, context->Ebx, context->Ecx, context->Edx,
                   context->Esi, context->Edi, context->Ebp, context->Esp,
@@ -262,7 +259,7 @@ BOOL DOSVM_EmulateInterruptPM( CONTEXT *context, BYTE intnum )
 
         if (intnum != context->Eip / DOSVM_STUB_PM16)
             WARN( "interrupt stub has been modified "
-                  "(interrupt is %02x, interrupt stub is %02x)\n",
+                  "(interrupt is %02x, interrupt stub is %02lx)\n",
                   intnum, context->Eip/DOSVM_STUB_PM16 );
 
         TRACE( "builtin interrupt %02x has been branched to\n", intnum );
@@ -555,7 +552,7 @@ static void WINAPI DOSVM_Int1aHandler( CONTEXT *context )
             SET_CX( context, HIWORD(data->Ticks) );
             SET_DX( context, LOWORD(data->Ticks) );
             SET_AL( context, 0 ); /* FIXME: midnight flag is unsupported */
-            TRACE( "GET SYSTEM TIME - ticks=%d\n", data->Ticks );
+            TRACE( "GET SYSTEM TIME - ticks=%ld\n", data->Ticks );
         }
         break;
 
