@@ -26,10 +26,13 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "wine/winbase16.h"
 #include "wownt32.h"
 #include "winternl.h"
+#include "ddk/ntddk.h"
 #include "kernel16_private.h"
 #include "wine/exception.h"
 #include "wine/debug.h"
