@@ -23,7 +23,6 @@
 #include "mf_private.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
 
@@ -73,7 +72,7 @@ static ULONG WINAPI sample_copier_transform_AddRef(IMFTransform *iface)
     struct sample_copier *transform = impl_from_IMFTransform(iface);
     ULONG refcount = InterlockedIncrement(&transform->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -83,7 +82,7 @@ static ULONG WINAPI sample_copier_transform_Release(IMFTransform *iface)
     struct sample_copier *transform = impl_from_IMFTransform(iface);
     ULONG refcount = InterlockedDecrement(&transform->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -92,7 +91,7 @@ static ULONG WINAPI sample_copier_transform_Release(IMFTransform *iface)
         if (transform->buffer_type)
             IMFMediaType_Release(transform->buffer_type);
         DeleteCriticalSection(&transform->cs);
-        heap_free(transform);
+        free(transform);
     }
 
     return refcount;
@@ -121,7 +120,7 @@ static HRESULT WINAPI sample_copier_transform_GetStreamCount(IMFTransform *iface
 static HRESULT WINAPI sample_copier_transform_GetStreamIDs(IMFTransform *iface, DWORD input_size, DWORD *inputs,
         DWORD output_size, DWORD *outputs)
 {
-    TRACE("%p, %u, %p, %u, %p.\n", iface, input_size, inputs, output_size, outputs);
+    TRACE("%p, %lu, %p, %lu, %p.\n", iface, input_size, inputs, output_size, outputs);
 
     return E_NOTIMPL;
 }
@@ -130,7 +129,7 @@ static HRESULT WINAPI sample_copier_transform_GetInputStreamInfo(IMFTransform *i
 {
     struct sample_copier *transform = impl_from_IMFTransform(iface);
 
-    TRACE("%p, %u, %p.\n", iface, id, info);
+    TRACE("%p, %lu, %p.\n", iface, id, info);
 
     memset(info, 0, sizeof(*info));
 
@@ -146,7 +145,7 @@ static HRESULT WINAPI sample_copier_transform_GetOutputStreamInfo(IMFTransform *
 {
     struct sample_copier *transform = impl_from_IMFTransform(iface);
 
-    TRACE("%p, %u, %p.\n", iface, id, info);
+    TRACE("%p, %lu, %p.\n", iface, id, info);
 
     memset(info, 0, sizeof(*info));
 
@@ -172,7 +171,7 @@ static HRESULT WINAPI sample_copier_transform_GetAttributes(IMFTransform *iface,
 static HRESULT WINAPI sample_copier_transform_GetInputStreamAttributes(IMFTransform *iface, DWORD id,
         IMFAttributes **attributes)
 {
-    TRACE("%p, %u, %p.\n", iface, id, attributes);
+    TRACE("%p, %lu, %p.\n", iface, id, attributes);
 
     return E_NOTIMPL;
 }
@@ -180,21 +179,21 @@ static HRESULT WINAPI sample_copier_transform_GetInputStreamAttributes(IMFTransf
 static HRESULT WINAPI sample_copier_transform_GetOutputStreamAttributes(IMFTransform *iface, DWORD id,
         IMFAttributes **attributes)
 {
-    TRACE("%p, %u, %p.\n", iface, id, attributes);
+    TRACE("%p, %lu, %p.\n", iface, id, attributes);
 
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI sample_copier_transform_DeleteInputStream(IMFTransform *iface, DWORD id)
 {
-    TRACE("%p, %u.\n", iface, id);
+    TRACE("%p, %lu.\n", iface, id);
 
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI sample_copier_transform_AddInputStreams(IMFTransform *iface, DWORD streams, DWORD *ids)
 {
-    TRACE("%p, %u, %p.\n", iface, streams, ids);
+    TRACE("%p, %lu, %p.\n", iface, streams, ids);
 
     return E_NOTIMPL;
 }
@@ -205,7 +204,7 @@ static HRESULT WINAPI sample_copier_transform_GetInputAvailableType(IMFTransform
     static const GUID *types[] = { &MFMediaType_Video, &MFMediaType_Audio };
     HRESULT hr;
 
-    TRACE("%p, %u, %u, %p.\n", iface, id, index, type);
+    TRACE("%p, %lu, %lu, %p.\n", iface, id, index, type);
 
     if (id)
         return MF_E_INVALIDSTREAMNUMBER;
@@ -226,7 +225,7 @@ static HRESULT WINAPI sample_copier_transform_GetOutputAvailableType(IMFTransfor
     IMFMediaType *cloned_type = NULL;
     HRESULT hr = S_OK;
 
-    TRACE("%p, %u, %u, %p.\n", iface, id, index, type);
+    TRACE("%p, %lu, %lu, %p.\n", iface, id, index, type);
 
     EnterCriticalSection(&transform->cs);
     if (transform->buffer_type)
@@ -248,7 +247,7 @@ static HRESULT WINAPI sample_copier_transform_GetOutputAvailableType(IMFTransfor
     return hr;
 }
 
-static HRESULT sample_copier_get_buffer_size(IMFMediaType *type, DWORD *size)
+static HRESULT sample_copier_get_buffer_size(IMFMediaType *type, UINT32 *size)
 {
     GUID major, subtype;
     UINT64 frame_size;
@@ -282,7 +281,7 @@ static HRESULT sample_copier_get_buffer_size(IMFMediaType *type, DWORD *size)
 static HRESULT sample_copier_set_media_type(struct sample_copier *transform, BOOL input, DWORD id, IMFMediaType *type,
         DWORD flags)
 {
-    DWORD buffer_size;
+    UINT32 buffer_size;
     HRESULT hr = S_OK;
 
     if (id)
@@ -327,7 +326,7 @@ static HRESULT WINAPI sample_copier_transform_SetInputType(IMFTransform *iface,
 {
     struct sample_copier *transform = impl_from_IMFTransform(iface);
 
-    TRACE("%p, %u, %p, %#x.\n", iface, id, type, flags);
+    TRACE("%p, %lu, %p, %#lx.\n", iface, id, type, flags);
 
     return sample_copier_set_media_type(transform, TRUE, id, type, flags);
 }
@@ -336,7 +335,7 @@ static HRESULT WINAPI sample_copier_transform_SetOutputType(IMFTransform *iface,
 {
     struct sample_copier *transform = impl_from_IMFTransform(iface);
 
-    TRACE("%p, %u, %p, %#x.\n", iface, id, type, flags);
+    TRACE("%p, %lu, %p, %#lx.\n", iface, id, type, flags);
 
     return sample_copier_set_media_type(transform, FALSE, id, type, flags);
 }
@@ -372,7 +371,7 @@ static HRESULT WINAPI sample_copier_transform_GetInputCurrentType(IMFTransform *
 {
     struct sample_copier *transform = impl_from_IMFTransform(iface);
 
-    TRACE("%p, %u, %p.\n", iface, id, type);
+    TRACE("%p, %lu, %p.\n", iface, id, type);
 
     return sample_copier_get_current_type(transform, id, SAMPLE_COPIER_INPUT_TYPE_SET, type);
 }
@@ -381,7 +380,7 @@ static HRESULT WINAPI sample_copier_transform_GetOutputCurrentType(IMFTransform
 {
     struct sample_copier *transform = impl_from_IMFTransform(iface);
 
-    TRACE("%p, %u, %p.\n", iface, id, type);
+    TRACE("%p, %lu, %p.\n", iface, id, type);
 
     return sample_copier_get_current_type(transform, id, SAMPLE_COPIER_OUTPUT_TYPE_SET, type);
 }
@@ -391,7 +390,7 @@ static HRESULT WINAPI sample_copier_transform_GetInputStatus(IMFTransform *iface
     struct sample_copier *transform = impl_from_IMFTransform(iface);
     HRESULT hr = S_OK;
 
-    TRACE("%p, %u, %p.\n", iface, id, flags);
+    TRACE("%p, %lu, %p.\n", iface, id, flags);
 
     if (id)
         return MF_E_INVALIDSTREAMNUMBER;
@@ -432,7 +431,7 @@ static HRESULT WINAPI sample_copier_transform_SetOutputBounds(IMFTransform *ifac
 
 static HRESULT WINAPI sample_copier_transform_ProcessEvent(IMFTransform *iface, DWORD id, IMFMediaEvent *event)
 {
-    FIXME("%p, %u, %p.\n", iface, id, event);
+    FIXME("%p, %lu, %p.\n", iface, id, event);
 
     return E_NOTIMPL;
 }
@@ -464,7 +463,7 @@ static HRESULT WINAPI sample_copier_transform_ProcessInput(IMFTransform *iface,
     struct sample_copier *transform = impl_from_IMFTransform(iface);
     HRESULT hr = S_OK;
 
-    TRACE("%p, %u, %p, %#x.\n", iface, id, sample, flags);
+    TRACE("%p, %lu, %p, %#lx.\n", iface, id, sample, flags);
 
     if (id)
         return MF_E_INVALIDSTREAMNUMBER;
@@ -493,7 +492,7 @@ static HRESULT WINAPI sample_copier_transform_ProcessOutput(IMFTransform *iface,
     HRESULT hr = S_OK;
     LONGLONG time;
 
-    TRACE("%p, %#x, %u, %p, %p.\n", iface, flags, count, buffers, status);
+    TRACE("%p, %#lx, %lu, %p, %p.\n", iface, flags, count, buffers, status);
 
     EnterCriticalSection(&transform->cs);
     if (!(transform->flags & SAMPLE_COPIER_OUTPUT_TYPE_SET))
@@ -561,9 +560,9 @@ static const IMFTransformVtbl sample_copier_transform_vtbl =
     sample_copier_transform_ProcessOutput,
 };
 
-BOOL mf_is_sample_copier_transform(IUnknown *transform)
+BOOL mf_is_sample_copier_transform(IMFTransform *transform)
 {
-    return transform->lpVtbl == (IUnknownVtbl *)&sample_copier_transform_vtbl;
+    return transform->lpVtbl == &sample_copier_transform_vtbl;
 }
 
 /***********************************************************************
@@ -576,8 +575,7 @@ HRESULT WINAPI MFCreateSampleCopierMFT(IMFTransform **transform)
 
     TRACE("%p.\n", transform);
 
-    object = heap_alloc_zero(sizeof(*object));
-    if (!object)
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->IMFTransform_iface.lpVtbl = &sample_copier_transform_vtbl;
