@@ -712,7 +712,12 @@ static HRESULT WINAPI topology_CloneFrom(IMFTopology *iface, IMFTopology *src)
         for (j = 0; j < outputs->count; ++j)
         {
             DWORD input_index = outputs->streams[j].connection_stream;
-            TOPOID id = outputs->streams[j].connection->id;
+            TOPOID id;
+
+            if (!outputs->streams[j].connection)
+                continue;
+
+            id = outputs->streams[j].connection->id;
 
             /* Skip node lookup in destination topology, assuming same node order. */
             if (SUCCEEDED(hr = topology_get_node_by_id(topology, id, &node)))
@@ -2225,3 +2230,79 @@ HRESULT WINAPI MFCreateSequencerSource(IUnknown *reserved, IMFSequencerSource **
 
     return S_OK;
 }
+
+struct segment_offset
+{
+    IUnknown IUnknown_iface;
+    LONG refcount;
+    MFSequencerElementId id;
+    MFTIME timeoffset;
+};
+
+static inline struct segment_offset *impl_offset_from_IUnknown(IUnknown *iface)
+{
+    return CONTAINING_RECORD(iface, struct segment_offset, IUnknown_iface);
+}
+
+static HRESULT WINAPI segment_offset_QueryInterface(IUnknown *iface, REFIID riid, void **obj)
+{
+    if (IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IUnknown_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI segment_offset_AddRef(IUnknown *iface)
+{
+    struct segment_offset *offset = impl_offset_from_IUnknown(iface);
+    return InterlockedIncrement(&offset->refcount);
+}
+
+static ULONG WINAPI segment_offset_Release(IUnknown *iface)
+{
+    struct segment_offset *offset = impl_offset_from_IUnknown(iface);
+    ULONG refcount = InterlockedDecrement(&offset->refcount);
+
+    if (!refcount)
+        free(offset);
+
+    return refcount;
+}
+
+static const IUnknownVtbl segment_offset_vtbl =
+{
+    segment_offset_QueryInterface,
+    segment_offset_AddRef,
+    segment_offset_Release,
+};
+
+/***********************************************************************
+ *      MFCreateSequencerSegmentOffset (mf.@)
+ */
+HRESULT WINAPI MFCreateSequencerSegmentOffset(MFSequencerElementId id, MFTIME timeoffset, PROPVARIANT *ret)
+{
+    struct segment_offset *offset;
+
+    TRACE("%#lx, %s, %p.\n", id, debugstr_time(timeoffset), ret);
+
+    if (!ret)
+        return E_POINTER;
+
+    if (!(offset = calloc(1, sizeof(*offset))))
+        return E_OUTOFMEMORY;
+
+    offset->IUnknown_iface.lpVtbl = &segment_offset_vtbl;
+    offset->refcount = 1;
+    offset->id = id;
+    offset->timeoffset = timeoffset;
+
+    V_VT(ret) = VT_UNKNOWN;
+    V_UNKNOWN(ret) = &offset->IUnknown_iface;
+
+    return S_OK;
+}
