@@ -29,7 +29,6 @@
 #include "msado15_backcompat.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 #include "msado15_private.h"
 
@@ -108,10 +107,10 @@ static ULONG WINAPI connection_Release( _Connection *iface )
                 IUnknown_Release( connection->cp_connev.sinks[i] );
         }
         if (connection->session) IUnknown_Release( connection->session );
-        heap_free( connection->cp_connev.sinks );
-        heap_free( connection->provider );
-        heap_free( connection->datasource );
-        heap_free( connection );
+        free( connection->cp_connev.sinks );
+        free( connection->provider );
+        free( connection->datasource );
+        free( connection );
     }
     return refs;
 }
@@ -165,7 +164,7 @@ static HRESULT WINAPI connection_GetTypeInfoCount( _Connection *iface, UINT *cou
 static HRESULT WINAPI connection_GetTypeInfo( _Connection *iface, UINT index, LCID lcid, ITypeInfo **info )
 {
     struct connection *connection = impl_from_Connection( iface );
-    TRACE( "%p, %u, %u, %p\n", connection, index, lcid, info );
+    TRACE( "%p, %u, %lu, %p\n", connection, index, lcid, info );
     return get_typeinfo(Connection_tid, info);
 }
 
@@ -176,7 +175,7 @@ static HRESULT WINAPI connection_GetIDsOfNames( _Connection *iface, REFIID riid,
     HRESULT hr;
     ITypeInfo *typeinfo;
 
-    TRACE( "%p, %s, %p, %u, %u, %p\n", connection, debugstr_guid(riid), names, count, lcid, dispid );
+    TRACE( "%p, %s, %p, %u, %lu, %p\n", connection, debugstr_guid(riid), names, count, lcid, dispid );
 
     hr = get_typeinfo(Connection_tid, &typeinfo);
     if(SUCCEEDED(hr))
@@ -195,7 +194,7 @@ static HRESULT WINAPI connection_Invoke( _Connection *iface, DISPID member, REFI
     HRESULT hr;
     ITypeInfo *typeinfo;
 
-    TRACE( "%p, %d, %s, %d, %d, %p, %p, %p, %p\n", connection, member, debugstr_guid(riid), lcid, flags,
+    TRACE( "%p, %ld, %s, %ld, %d, %p, %p, %p, %p\n", connection, member, debugstr_guid(riid), lcid, flags,
            params, result, excep_info, arg_err );
 
     hr = get_typeinfo(Connection_tid, &typeinfo);
@@ -234,8 +233,8 @@ static HRESULT WINAPI connection_put_ConnectionString( _Connection *iface, BSTR
 
     TRACE( "%p, %s\n", connection, debugstr_w( str && !wcsstr( str, L"Password" ) ? L"<hidden>" : str ) );
 
-    if (str && !(source = strdupW( str ))) return E_OUTOFMEMORY;
-    heap_free( connection->datasource );
+    if (str && !(source = wcsdup( str ))) return E_OUTOFMEMORY;
+    free( connection->datasource );
     connection->datasource = source;
     return S_OK;
 }
@@ -251,7 +250,7 @@ static HRESULT WINAPI connection_get_CommandTimeout( _Connection *iface, LONG *t
 static HRESULT WINAPI connection_put_CommandTimeout( _Connection *iface, LONG timeout )
 {
     struct connection *connection = impl_from_Connection( iface );
-    TRACE( "%p, %d\n", connection, timeout );
+    TRACE( "%p, %ld\n", connection, timeout );
     connection->timeout = timeout;
     return S_OK;
 }
@@ -264,7 +263,7 @@ static HRESULT WINAPI connection_get_ConnectionTimeout( _Connection *iface, LONG
 
 static HRESULT WINAPI connection_put_ConnectionTimeout( _Connection *iface, LONG timeout )
 {
-    FIXME( "%p, %d\n", iface, timeout );
+    FIXME( "%p, %ld\n", iface, timeout );
     return E_NOTIMPL;
 }
 
@@ -301,7 +300,7 @@ static HRESULT WINAPI connection_Execute( _Connection *iface, BSTR command, VARI
     VARIANT source, active;
     IDispatch *dispatch;
 
-    FIXME( "%p, %s, %p, 0x%08x, %p Semi-stub\n", iface, debugstr_w(command), records_affected, options, record_set );
+    FIXME( "%p, %s, %p, 0x%08lx, %p Semi-stub\n", iface, debugstr_w(command), records_affected, options, record_set );
 
     if (connection->state == adStateClosed) return MAKE_ADO_HRESULT( adErrObjectClosed );
 
@@ -336,8 +335,10 @@ static HRESULT WINAPI connection_Execute( _Connection *iface, BSTR command, VARI
 
     if (records_affected)
     {
+        ADO_LONGPTR count;
+        _Recordset_get_RecordCount(recordset, &count);
         V_VT(records_affected) = VT_I4;
-        _Recordset_get_RecordCount(recordset, &V_I4(records_affected));
+        V_I4(records_affected) = count;
     }
 
     *record_set = recordset;
@@ -373,7 +374,7 @@ static HRESULT WINAPI connection_Open( _Connection *iface, BSTR connect_str, BST
     IDBCreateSession *session = NULL;
     HRESULT hr;
 
-    TRACE( "%p, %s, %s, %p, %08x\n", iface, debugstr_w(connect_str), debugstr_w(userid), password, options );
+    TRACE( "%p, %s, %s, %p, %08lx\n", iface, debugstr_w(connect_str), debugstr_w(userid), password, options );
 
     if (connection->state == adStateOpen) return MAKE_ADO_HRESULT( adErrObjectOpen );
     if (!connect_str) return E_FAIL;
@@ -409,7 +410,7 @@ done:
     }
     IDataInitialize_Release( datainit );
 
-    TRACE("ret 0x%08x\n", hr);
+    TRACE("ret 0x%08lx\n", hr);
     return hr;
 }
 
@@ -451,7 +452,7 @@ static HRESULT WINAPI connection_get_Attributes( _Connection *iface, LONG *attr
 
 static HRESULT WINAPI connection_put_Attributes( _Connection *iface, LONG attr )
 {
-    FIXME( "%p, %d\n", iface, attr );
+    FIXME( "%p, %ld\n", iface, attr );
     return E_NOTIMPL;
 }
 
@@ -516,8 +517,8 @@ static HRESULT WINAPI connection_put_Provider( _Connection *iface, BSTR str )
 
     if (!str) return MAKE_ADO_HRESULT(adErrInvalidArgument);
 
-    if (!(provider = strdupW( str ))) return E_OUTOFMEMORY;
-    heap_free( connection->provider );
+    if (!(provider = wcsdup( str ))) return E_OUTOFMEMORY;
+    free( connection->provider );
     connection->provider = provider;
     return S_OK;
 }
@@ -749,15 +750,16 @@ static HRESULT WINAPI connpoint_Advise( IConnectionPoint *iface, IUnknown *unk_s
         if (i == connpoint->sinks_size)
         {
             new_size = connpoint->sinks_size * 2;
-            if (!(tmp = heap_realloc_zero( connpoint->sinks, new_size * sizeof(*connpoint->sinks) )))
+            if (!(tmp = realloc( connpoint->sinks, new_size * sizeof(*connpoint->sinks) )))
                 return E_OUTOFMEMORY;
+            memset( tmp + connpoint->sinks_size, 0, (new_size - connpoint->sinks_size) * sizeof(*connpoint->sinks) );
             connpoint->sinks = tmp;
             connpoint->sinks_size = new_size;
         }
     }
     else
     {
-        if (!(connpoint->sinks = heap_alloc_zero( sizeof(*connpoint->sinks) ))) return E_OUTOFMEMORY;
+        if (!(connpoint->sinks = calloc( 1, sizeof(*connpoint->sinks) ))) return E_OUTOFMEMORY;
         connpoint->sinks_size = 1;
         i = 0;
     }
@@ -770,7 +772,7 @@ static HRESULT WINAPI connpoint_Advise( IConnectionPoint *iface, IUnknown *unk_s
 static HRESULT WINAPI connpoint_Unadvise( IConnectionPoint *iface, DWORD cookie )
 {
     struct connection_point *connpoint = impl_from_IConnectionPoint( iface );
-    TRACE( "%p, %u\n", connpoint, cookie );
+    TRACE( "%p, %lu\n", connpoint, cookie );
 
     if (!cookie || cookie > connpoint->sinks_size || !connpoint->sinks || !connpoint->sinks[cookie - 1])
         return E_FAIL;
@@ -858,7 +860,7 @@ HRESULT Connection_create( void **obj )
 {
     struct connection *connection;
 
-    if (!(connection = heap_alloc( sizeof(*connection) ))) return E_OUTOFMEMORY;
+    if (!(connection = malloc( sizeof(*connection) ))) return E_OUTOFMEMORY;
     connection->Connection_iface.lpVtbl = &connection_vtbl;
     connection->ISupportErrorInfo_iface.lpVtbl = &support_error_vtbl;
     connection->IConnectionPointContainer_iface.lpVtbl = &connpointcontainer_vtbl;
@@ -867,9 +869,9 @@ HRESULT Connection_create( void **obj )
     connection->state = adStateClosed;
     connection->timeout = 30;
     connection->datasource = NULL;
-    if (!(connection->provider = strdupW( L"MSDASQL" )))
+    if (!(connection->provider = wcsdup( L"MSDASQL" )))
     {
-        heap_free( connection );
+        free( connection );
         return E_OUTOFMEMORY;
     }
     connection->mode = adModeUnknown;
