@@ -40,7 +40,6 @@
 #include "mscoree_private.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL( mscoree );
 
@@ -168,7 +167,7 @@ static HRESULT RuntimeHost_GetDefaultDomain(RuntimeHost *This, const WCHAR *conf
     base_dirA = WtoA(base_dir);
     if (!base_dirA)
     {
-        HeapFree(GetProcessHeap(), 0, config_pathA);
+        free(config_pathA);
         res = E_OUTOFMEMORY;
         goto end;
     }
@@ -180,8 +179,8 @@ static HRESULT RuntimeHost_GetDefaultDomain(RuntimeHost *This, const WCHAR *conf
     TRACE("setting base_dir: %s, config_path: %s\n", base_dirA, config_pathA);
     mono_domain_set_config(*result, base_dirA, config_pathA);
 
-    HeapFree(GetProcessHeap(), 0, config_pathA);
-    HeapFree(GetProcessHeap(), 0, base_dirA);
+    free(config_pathA);
+    free(base_dirA);
 
 end:
 
@@ -388,7 +387,7 @@ static HRESULT RuntimeHost_AddDomain(RuntimeHost *This, const WCHAR *name, IUnkn
     }
 
     args[0] = mono_string_new(domain, nameA);
-    HeapFree(GetProcessHeap(), 0, nameA);
+    free(nameA);
 
     if (!args[0])
     {
@@ -943,10 +942,10 @@ static HRESULT WINAPI CLRRuntimeHost_ExecuteInDefaultAppDomain(ICLRRuntimeHost*
 
     domain_restore(prev_domain);
 
-    HeapFree(GetProcessHeap(), 0, filenameA);
-    HeapFree(GetProcessHeap(), 0, classA);
-    HeapFree(GetProcessHeap(), 0, argsA);
-    HeapFree(GetProcessHeap(), 0, methodA);
+    free(filenameA);
+    free(classA);
+    free(argsA);
+    free(methodA);
 
     return hr;
 }
@@ -1034,7 +1033,7 @@ HRESULT RuntimeHost_CreateManagedInstance(RuntimeHost *This, LPCWSTR name,
 
     domain_restore(prev_domain);
 
-    HeapFree(GetProcessHeap(), 0, nameA);
+    free(nameA);
 
     return hr;
 }
@@ -1082,7 +1081,7 @@ static void get_utf8_args(int *argc, char ***argv)
     }
     size += sizeof(char*);
 
-    *argv = HeapAlloc(GetProcessHeap(), 0, size);
+    *argv = malloc(size);
     current_arg = (char*)(*argv + *argc + 1);
 
     for (i=0; i<*argc; i++)
@@ -1325,7 +1324,7 @@ static void CDECL ReallyFixupVTable(struct dll_fixup *fixup)
     if (info != NULL)
         ICLRRuntimeInfo_Release(info);
 
-    HeapFree(GetProcessHeap(), 0, filenameA);
+    free(filenameA);
 
     if (!fixup->done)
     {
@@ -1346,7 +1345,7 @@ static void FixupVTableEntry(HMODULE hmodule, VTableFixup *vtable_fixup)
      * threads are clear. */
     struct dll_fixup *fixup;
 
-    fixup = HeapAlloc(GetProcessHeap(), 0, sizeof(*fixup));
+    fixup = malloc(sizeof(*fixup));
 
     fixup->dll = hmodule;
     fixup->thunk_code = HeapAlloc(dll_fixup_heap, 0, sizeof(struct vtable_fixup_thunk) * vtable_fixup->count);
@@ -1365,7 +1364,7 @@ static void FixupVTableEntry(HMODULE hmodule, VTableFixup *vtable_fixup)
         int i;
         struct vtable_fixup_thunk *thunks = fixup->thunk_code;
 
-        tokens = fixup->tokens = HeapAlloc(GetProcessHeap(), 0, sizeof(*tokens) * vtable_fixup->count);
+        tokens = fixup->tokens = malloc(sizeof(*tokens) * vtable_fixup->count);
         memcpy(tokens, vtable, sizeof(*tokens) * vtable_fixup->count);
         for (i=0; i<vtable_fixup->count; i++)
         {
@@ -1380,7 +1379,7 @@ static void FixupVTableEntry(HMODULE hmodule, VTableFixup *vtable_fixup)
     {
         ERR("unsupported vtable fixup flags %x\n", vtable_fixup->type);
         HeapFree(dll_fixup_heap, 0, fixup->thunk_code);
-        HeapFree(GetProcessHeap(), 0, fixup);
+        free(fixup);
         return;
     }
 
@@ -1446,7 +1445,7 @@ __int32 WINAPI _CorExeMain(void)
     filenameA = WtoA(filename);
     if (!filenameA)
     {
-        HeapFree(GetProcessHeap(), 0, argv);
+        free(argv);
         return -1;
     }
 
@@ -1463,12 +1462,12 @@ __int32 WINAPI _CorExeMain(void)
             if (parsed_config.private_path[i] == ';') number_of_private_paths++;
         if (parsed_config.private_path[wcslen(parsed_config.private_path) - 1] != ';') number_of_private_paths++;
         config_file_dir_size = (wcsrchr(config_file, '\\') - config_file) + 1;
-        priv_path = HeapAlloc(GetProcessHeap(), 0, (number_of_private_paths + 1) * sizeof(WCHAR *));
+        priv_path = malloc((number_of_private_paths + 1) * sizeof(WCHAR *));
         /* wcstok ignores trailing semicolons */
         temp = wcstok_s(parsed_config.private_path, scW, &save);
         for (i = 0; i < number_of_private_paths; i++)
         {
-            priv_path[i] = HeapAlloc(GetProcessHeap(), 0, (config_file_dir_size + wcslen(temp) + 1) * sizeof(WCHAR));
+            priv_path[i] = malloc((config_file_dir_size + wcslen(temp) + 1) * sizeof(WCHAR));
             memcpy(priv_path[i], config_file, config_file_dir_size * sizeof(WCHAR));
             wcscpy(priv_path[i] + config_file_dir_size, temp);
             temp = wcstok_s(NULL, scW, &save);
@@ -1517,7 +1516,7 @@ __int32 WINAPI _CorExeMain(void)
     else
         exit_code = -1;
 
-    HeapFree(GetProcessHeap(), 0, argv);
+    free(argv);
 
     if (domain)
     {
@@ -1575,8 +1574,8 @@ void runtimehost_uninit(void)
     HeapDestroy(dll_fixup_heap);
     LIST_FOR_EACH_ENTRY_SAFE(fixup, fixup2, &dll_fixups, struct dll_fixup, entry)
     {
-        HeapFree(GetProcessHeap(), 0, fixup->tokens);
-        HeapFree(GetProcessHeap(), 0, fixup);
+        free(fixup->tokens);
+        free(fixup);
     }
 }
 
@@ -1584,7 +1583,7 @@ HRESULT RuntimeHost_Construct(CLRRuntimeInfo *runtime_version, RuntimeHost** res
 {
     RuntimeHost *This;
 
-    This = HeapAlloc( GetProcessHeap(), 0, sizeof *This );
+    This = malloc(sizeof *This);
     if ( !This )
         return E_OUTOFMEMORY;
 
@@ -1668,7 +1667,7 @@ static BOOL try_create_registration_free_com(REFIID clsid, WCHAR *classname, UIN
     }
 
     QueryActCtxW(0, guid_info.hActCtx, &guid_info.ulAssemblyRosterIndex, AssemblyDetailedInformationInActivationContext, NULL, 0, &bytes_assembly_info);
-    assembly_info = heap_alloc(bytes_assembly_info);
+    assembly_info = malloc(bytes_assembly_info);
     if (!QueryActCtxW(0, guid_info.hActCtx, &guid_info.ulAssemblyRosterIndex,
             AssemblyDetailedInformationInActivationContext, assembly_info, bytes_assembly_info, &bytes_assembly_info))
     {
@@ -1706,7 +1705,7 @@ static BOOL try_create_registration_free_com(REFIID clsid, WCHAR *classname, UIN
     ret = TRUE;
 
 end:
-    heap_free(assembly_info);
+    free(assembly_info);
 
     if (guid_info.hActCtx)
         ReleaseActCtx(guid_info.hActCtx);
@@ -1914,7 +1913,7 @@ HRESULT create_monodata(REFCLSID clsid, LPVOID *ppObj)
 
             filenameA = WtoA(filename);
             assembly = mono_assembly_open(filenameA, &status);
-            HeapFree(GetProcessHeap(), 0, filenameA);
+            free(filenameA);
             if (!assembly)
             {
                 ERR("Cannot open assembly %s, status=%i\n", debugstr_w(filename), status);
@@ -1935,7 +1934,7 @@ HRESULT create_monodata(REFCLSID clsid, LPVOID *ppObj)
             *ns = '\0';
 
             klass = mono_class_from_name(image, classA, ns+1);
-            HeapFree(GetProcessHeap(), 0, classA);
+            free(classA);
             if (!klass)
             {
                 ERR("Couldn't get class from image\n");
