@@ -39,7 +39,6 @@
 #include "metahost.h"
 #include "fusion.h"
 #include "wine/list.h"
-#include "wine/heap.h"
 #include "mscoree_private.h"
 
 #include "wine/debug.h"
@@ -130,12 +129,15 @@ MonoThread* (CDECL *mono_thread_attach)(MonoDomain *domain);
 void (CDECL *mono_thread_manage)(void);
 void (CDECL *mono_trace_set_print_handler)(MonoPrintCallback callback);
 void (CDECL *mono_trace_set_printerr_handler)(MonoPrintCallback callback);
+static MonoAssembly* (CDECL *wine_mono_assembly_load_from_gac)(MonoAssemblyName *aname, MonoImageOpenStatus *status, int refonly);
 static void (CDECL *wine_mono_install_assembly_preload_hook)(WineMonoAssemblyPreLoadFunc func, void *user_data);
+static void (CDECL *wine_mono_install_assembly_preload_hook_v2)(WineMonoAssemblyPreLoadFunc func, void *user_data);
 
 static BOOL find_mono_dll(LPCWSTR path, LPWSTR dll_path);
 
 static MonoAssembly* CDECL mono_assembly_preload_hook_fn(MonoAssemblyName *aname, char **assemblies_path, void *user_data);
 static MonoAssembly* CDECL wine_mono_assembly_preload_hook_fn(MonoAssemblyName *aname, char **assemblies_path, int *search_path, void *user_data);
+static MonoAssembly* CDECL wine_mono_assembly_preload_hook_v2_fn(MonoAssemblyName *aname, char **assemblies_path, int *flags, void *user_data);
 
 static void CDECL mono_shutdown_callback_fn(MonoProfiler *prof);
 
@@ -251,7 +253,9 @@ static HRESULT load_mono(LPCWSTR mono_path)
         LOAD_OPT_MONO_FUNCTION(mono_set_crash_chaining, set_crash_chaining_dummy);
         LOAD_OPT_MONO_FUNCTION(mono_trace_set_print_handler, set_print_handler_dummy);
         LOAD_OPT_MONO_FUNCTION(mono_trace_set_printerr_handler, set_print_handler_dummy);
+        LOAD_OPT_MONO_FUNCTION(wine_mono_assembly_load_from_gac, NULL);
         LOAD_OPT_MONO_FUNCTION(wine_mono_install_assembly_preload_hook, NULL);
+        LOAD_OPT_MONO_FUNCTION(wine_mono_install_assembly_preload_hook_v2, NULL);
 
 #undef LOAD_OPT_MONO_FUNCTION
 
@@ -282,7 +286,9 @@ static HRESULT load_mono(LPCWSTR mono_path)
 
         mono_config_parse(NULL);
 
-        if (wine_mono_install_assembly_preload_hook)
+        if (wine_mono_install_assembly_preload_hook_v2)
+            wine_mono_install_assembly_preload_hook_v2(wine_mono_assembly_preload_hook_v2_fn, NULL);
+        else if (wine_mono_install_assembly_preload_hook)
             wine_mono_install_assembly_preload_hook(wine_mono_assembly_preload_hook_fn, NULL);
         else
             mono_install_assembly_preload_hook(mono_assembly_preload_hook_fn, NULL);
@@ -367,7 +373,7 @@ MonoDomain* get_root_domain(void)
 
         root_domain = mono_jit_init_version(exe_basename, "v4.0.30319");
 
-        HeapFree(GetProcessHeap(), 0, exe_basename);
+        free(exe_basename);
 
         is_mono_started = TRUE;
     }
@@ -789,7 +795,7 @@ static BOOL get_mono_path_dos(const WCHAR *dir, LPWSTR path)
         return FALSE;  /* No drive letter for this directory */
 
     len = lstrlenW( dir ) + lstrlenW( basedir ) + 1;
-    if (!(dos_dir = heap_alloc( len * sizeof(WCHAR) ))) return FALSE;
+    if (!(dos_dir = malloc( len * sizeof(WCHAR) ))) return FALSE;
     lstrcpyW( dos_dir, dir );
     lstrcatW( dos_dir, basedir );
 
@@ -797,7 +803,7 @@ static BOOL get_mono_path_dos(const WCHAR *dir, LPWSTR path)
     if (ret)
         lstrcpyW(path, dos_dir);
 
-    heap_free(dos_dir);
+    free(dos_dir);
 
     return ret;
 }
@@ -821,7 +827,7 @@ static BOOL get_mono_path_unix(const char *unix_dir, LPWSTR path)
 
     ret = get_mono_path_dos( dos_dir, path);
 
-    heap_free(dos_dir);
+    HeapFree(GetProcessHeap(), 0, dos_dir);
     return ret;
 }
 
@@ -845,13 +851,13 @@ static BOOL get_mono_path_datadir(LPWSTR path)
 
     if (!wcsncmp( data_dir, unix_prefix, wcslen(unix_prefix) )) return FALSE;
     data_dir += 4;  /* skip \??\ prefix */
-    package_dir = heap_alloc( (lstrlenW(data_dir) + lstrlenW(suffix) + 1) * sizeof(WCHAR));
+    package_dir = malloc((wcslen(data_dir) + wcslen(suffix) + 1) * sizeof(WCHAR));
     lstrcpyW( package_dir, data_dir );
     lstrcatW( package_dir, suffix );
 
     ret = get_mono_path_dos(package_dir, path);
 
-    heap_free(package_dir);
+    free(package_dir);
 
     return ret;
 }
@@ -921,7 +927,7 @@ static ULONG WINAPI InstalledRuntimeEnum_Release(IEnumUnknown* iface)
 
     if (ref == 0)
     {
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This);
     }
 
     return ref;
@@ -997,7 +1003,7 @@ static HRESULT WINAPI InstalledRuntimeEnum_Clone(IEnumUnknown *iface, IEnumUnkno
 
     TRACE("(%p)\n", iface);
 
-    new_enum = HeapAlloc(GetProcessHeap(), 0, sizeof(*new_enum));
+    new_enum = malloc(sizeof(*new_enum));
     if (!new_enum)
         return E_OUTOFMEMORY;
 
@@ -1175,7 +1181,7 @@ static HRESULT WINAPI CLRMetaHost_EnumerateInstalledRuntimes(ICLRMetaHost* iface
 
     TRACE("%p\n", ppEnumerator);
 
-    new_enum = HeapAlloc(GetProcessHeap(), 0, sizeof(*new_enum));
+    new_enum = malloc(sizeof(*new_enum));
     if (!new_enum)
         return E_OUTOFMEMORY;
 
@@ -1370,16 +1376,19 @@ HRESULT CLRMetaHostPolicy_CreateInstance(REFIID riid, void **ppobj)
  * Assembly search override settings:
  *
  * WINE_MONO_OVERRIDES=*,Gac=n
- *  Never search the GAC for libraries.
+ *  Never search the Windows GAC for libraries.
+ *
+ * WINE_MONO_OVERRIDES=*,MonoGac=n
+ *  Never search the Mono GAC for libraries.
  *
  * WINE_MONO_OVERRIDES=*,PrivatePath=n
  *  Never search the AppDomain search path for libraries.
  *
  * WINE_MONO_OVERRIDES=Microsoft.Xna.Framework,Gac=n
- *  Never search the GAC for Microsoft.Xna.Framework
+ *  Never search the Windows GAC for Microsoft.Xna.Framework
  *
  * WINE_MONO_OVERRIDES=Microsoft.Xna.Framework.*,Gac=n;Microsoft.Xna.Framework.GamerServices,Gac=y
- *  Never search the GAC for Microsoft.Xna.Framework, or any library starting
+ *  Never search the Windows GAC for Microsoft.Xna.Framework, or any library starting
  *  with Microsoft.Xna.Framework, except for Microsoft.Xna.Framework.GamerServices
  */
 
@@ -1387,7 +1396,8 @@ HRESULT CLRMetaHostPolicy_CreateInstance(REFIID riid, void **ppobj)
 #define ASSEMBLY_SEARCH_GAC 1
 #define ASSEMBLY_SEARCH_UNDEFINED 2
 #define ASSEMBLY_SEARCH_PRIVATEPATH 4
-#define ASSEMBLY_SEARCH_DEFAULT (ASSEMBLY_SEARCH_GAC|ASSEMBLY_SEARCH_PRIVATEPATH)
+#define ASSEMBLY_SEARCH_MONOGAC 8
+#define ASSEMBLY_SEARCH_DEFAULT (ASSEMBLY_SEARCH_GAC|ASSEMBLY_SEARCH_PRIVATEPATH|ASSEMBLY_SEARCH_MONOGAC)
 
 typedef struct override_entry {
     char *name;
@@ -1436,6 +1446,14 @@ static void parse_override_entry(override_entry *entry, const char *string, int
                         entry->flags &= ~ASSEMBLY_SEARCH_GAC;
                 }
                 break;
+            case 7:
+                if (!_strnicmp(string, "monogac", 7)) {
+                    if (IS_OPTION_TRUE(*value))
+                        entry->flags |= ASSEMBLY_SEARCH_MONOGAC;
+                    else if (IS_OPTION_FALSE(*value))
+                        entry->flags &= ~ASSEMBLY_SEARCH_MONOGAC;
+                }
+                break;
             case 11:
                 if (!_strnicmp(string, "privatepath", 11)) {
                     if (IS_OPTION_TRUE(*value))
@@ -1488,18 +1506,18 @@ static BOOL WINAPI parse_env_overrides(INIT_ONCE *once, void *param, void **cont
                 continue;
             }
 
-            entry = heap_alloc_zero(sizeof(*entry));
+            entry = calloc(1, sizeof(*entry));
             if (!entry)
             {
                 ERR("out of memory\n");
                 break;
             }
 
-            entry->name = heap_alloc_zero(basename_end - entry_start + 1);
+            entry->name = calloc(1, basename_end - entry_start + 1);
             if (!entry->name)
             {
                 ERR("out of memory\n");
-                heap_free(entry);
+                free(entry);
                 break;
             }
 
@@ -1563,7 +1581,7 @@ static DWORD get_basename_search_flags(const char *basename, MonoAssemblyName *a
     if (strcmp(basename, "Microsoft.Xna.Framework.*") == 0 &&
         mono_assembly_name_get_version(aname, NULL, NULL, NULL) == 4)
         /* Use FNA as a replacement for XNA4. */
-        return 0;
+        return ASSEMBLY_SEARCH_MONOGAC;
 
     return ASSEMBLY_SEARCH_UNDEFINED;
 }
@@ -1614,7 +1632,7 @@ static DWORD get_assembly_search_flags(MonoAssemblyName *aname)
         return result;
     }
 
-    name_copy = heap_alloc((strlen(name) + 3) * sizeof(WCHAR));
+    name_copy = malloc((strlen(name) + 3) * sizeof(WCHAR));
     if (!name_copy)
     {
         ERR("out of memory\n");
@@ -1644,7 +1662,7 @@ static DWORD get_assembly_search_flags(MonoAssemblyName *aname)
             result = ASSEMBLY_SEARCH_DEFAULT;
     }
 
-    heap_free(name_copy);
+    free(name_copy);
     if (appkey) RegCloseKey(appkey);
     if (userkey) RegCloseKey(userkey);
 
@@ -1699,7 +1717,7 @@ static MonoAssembly* mono_assembly_try_load(WCHAR *path)
     if (!(pathA = WtoA(path))) return NULL;
 
     result = mono_assembly_open(pathA, &stat);
-    HeapFree(GetProcessHeap(), 0, pathA);
+    free(pathA);
 
     if (result) TRACE("found: %s\n", debugstr_w(path));
     return result;
@@ -1707,11 +1725,20 @@ static MonoAssembly* mono_assembly_try_load(WCHAR *path)
 
 static MonoAssembly* CDECL mono_assembly_preload_hook_fn(MonoAssemblyName *aname, char **assemblies_path, void *user_data)
 {
-    int dummy;
-    return wine_mono_assembly_preload_hook_fn(aname, assemblies_path, &dummy, user_data);
+    int flags = 0;
+    return wine_mono_assembly_preload_hook_v2_fn(aname, assemblies_path, &flags, user_data);
 }
 
 static MonoAssembly* CDECL wine_mono_assembly_preload_hook_fn(MonoAssemblyName *aname, char **assemblies_path, int *halt_search, void *user_data)
+{
+    int flags = 0;
+    MonoAssembly* result = wine_mono_assembly_preload_hook_v2_fn(aname, assemblies_path, &flags, user_data);
+    if (flags & WINE_PRELOAD_SKIP_PRIVATE_PATH)
+        *halt_search = 1;
+    return result;
+}
+
+static MonoAssembly* CDECL wine_mono_assembly_preload_hook_v2_fn(MonoAssemblyName *aname, char **assemblies_path, int *flags, void *user_data)
 {
     HRESULT hr;
     MonoAssembly *result=NULL;
@@ -1734,7 +1761,7 @@ static MonoAssembly* CDECL wine_mono_assembly_preload_hook_fn(MonoAssemblyName *
     if (culture)
     {
         cultureW_size = MultiByteToWideChar(CP_UTF8, 0, culture, -1, NULL, 0);
-        cultureW = HeapAlloc(GetProcessHeap(), 0, cultureW_size * sizeof(WCHAR));
+        cultureW = malloc(cultureW_size * sizeof(WCHAR));
         if (cultureW) MultiByteToWideChar(CP_UTF8, 0, culture, -1, cultureW, cultureW_size);
     }
     else cultureW = NULL;
@@ -1747,7 +1774,7 @@ static MonoAssembly* CDECL wine_mono_assembly_preload_hook_fn(MonoAssemblyName *
     if (private_path && (search_flags & ASSEMBLY_SEARCH_PRIVATEPATH) != 0)
     {
         stringnameW_size = MultiByteToWideChar(CP_UTF8, 0, assemblyname, -1, NULL, 0);
-        stringnameW = HeapAlloc(GetProcessHeap(), 0, stringnameW_size * sizeof(WCHAR));
+        stringnameW = malloc(stringnameW_size * sizeof(WCHAR));
         if (stringnameW)
         {
             MultiByteToWideChar(CP_UTF8, 0, assemblyname, -1, stringnameW, stringnameW_size);
@@ -1779,25 +1806,23 @@ static MonoAssembly* CDECL wine_mono_assembly_preload_hook_fn(MonoAssemblyName *
                 result = mono_assembly_try_load(path);
                 if (result) break;
             }
-            HeapFree(GetProcessHeap(), 0, stringnameW);
+            free(stringnameW);
             if (result) goto done;
         }
     }
 
-    /* FIXME: We should search the given paths before the GAC. */
-
     if ((search_flags & ASSEMBLY_SEARCH_GAC) != 0)
     {
         stringnameW_size = MultiByteToWideChar(CP_UTF8, 0, stringname, -1, NULL, 0);
 
-        stringnameW = HeapAlloc(GetProcessHeap(), 0, stringnameW_size * sizeof(WCHAR));
+        stringnameW = malloc(stringnameW_size * sizeof(WCHAR));
         if (stringnameW)
         {
             MultiByteToWideChar(CP_UTF8, 0, stringname, -1, stringnameW, stringnameW_size);
 
             hr = get_file_from_strongname(stringnameW, path, MAX_PATH);
 
-            HeapFree(GetProcessHeap(), 0, stringnameW);
+            free(stringnameW);
         }
         else
             hr = E_OUTOFMEMORY;
@@ -1815,21 +1840,47 @@ static MonoAssembly* CDECL wine_mono_assembly_preload_hook_fn(MonoAssemblyName *
                 if (!result)
                     ERR("Failed to load %s, status=%u\n", debugstr_w(path), stat);
 
-                HeapFree(GetProcessHeap(), 0, pathA);
+                free(pathA);
+
+                if (result)
+                {
+                    *flags |= WINE_PRELOAD_SET_GAC;
+                    goto done;
+                }
             }
         }
     }
     else
         TRACE("skipping Windows GAC search due to override setting\n");
 
+    if (wine_mono_assembly_load_from_gac)
+    {
+        if (search_flags & ASSEMBLY_SEARCH_MONOGAC)
+        {
+            result = wine_mono_assembly_load_from_gac (aname, &stat, FALSE);
+
+            if (result)
+            {
+                TRACE("found in Mono GAC\n");
+                *flags |= WINE_PRELOAD_SET_GAC;
+                goto done;
+            }
+        }
+        else
+        {
+            *flags |= WINE_PRELOAD_SKIP_GAC;
+            TRACE("skipping Mono GAC search due to override setting\n");
+        }
+    }
+
     if ((search_flags & ASSEMBLY_SEARCH_PRIVATEPATH) == 0)
     {
         TRACE("skipping AppDomain search path due to override setting\n");
-        *halt_search = 1;
+        *flags |= WINE_PRELOAD_SKIP_PRIVATE_PATH;
     }
 
 done:
-    HeapFree(GetProcessHeap(), 0, cultureW);
+    free(cultureW);
     mono_free(stringname);
 
     return result;
