@@ -25,7 +25,6 @@
 #include "objbase.h"
 #include "rpcproxy.h"
 #include "msdasc.h"
-#include "wine/heap.h"
 #include "wine/debug.h"
 
 #include "msdasql.h"
@@ -409,7 +408,7 @@ static ULONG WINAPI session_Release(IUnknown *iface)
         TRACE( "destroying %p\n", session );
 
         IUnknown_Release(session->datasource);
-        heap_free( session );
+        free(session);
     }
     return refs;
 }
@@ -693,16 +692,15 @@ static ULONG WINAPI command_Release(ICommandText *iface)
     if (!refs)
     {
         TRACE( "destroying %p\n", command );
-        if (command->properties)
-            heap_free(command->properties);
+        free(command->properties);
         if (command->session)
             IUnknown_Release(command->session);
 
         if (command->hstmt)
             SQLFreeHandle(SQL_HANDLE_STMT, command->hstmt);
 
-        heap_free( command->query );
-        heap_free( command );
+        free(command->query);
+        free(command);
     }
     return refs;
 }
@@ -827,7 +825,7 @@ static ULONG WINAPI msdasql_rowset_Release(IRowset *iface)
         if (rowset->caller)
             IUnknown_Release(rowset->caller);
 
-        heap_free( rowset );
+        free(rowset);
     }
     return refs;
 }
@@ -1029,9 +1027,10 @@ static HRESULT WINAPI rowset_colsinfo_GetColumnInfo(IColumnsInfo *iface, DBORDIN
             dbcolumn[i].pTypeInfo = NULL;
             dbcolumn[i].iOrdinal = i+1;
 
-            ret = SQLColAttribute(rowset->hstmt, i+1, SQL_DESC_UNSIGNED, NULL, 0, NULL, &length);
+            ret = SQLColAttributesW (rowset->hstmt, i+1, SQL_DESC_UNSIGNED, NULL, 0, NULL, &length);
             if (!SQL_SUCCEEDED(ret))
             {
+                dump_sql_diag_records(SQL_HANDLE_STMT, rowset->hstmt);
                 CoTaskMemFree(ptr);
                 CoTaskMemFree(dbcolumn);
                 ERR("Failed to get column %d attribute\n", i+1);
@@ -1043,7 +1042,7 @@ static HRESULT WINAPI rowset_colsinfo_GetColumnInfo(IColumnsInfo *iface, DBORDIN
 
             dbcolumn[i].dwFlags = DBCOLUMNFLAGS_WRITE;
 
-            ret = SQLColAttribute(rowset->hstmt, i+1, SQL_DESC_LENGTH, NULL, 0, NULL, &length);
+            ret = SQLColAttributesW(rowset->hstmt, i+1, SQL_DESC_LENGTH, NULL, 0, NULL, &length);
             if (!SQL_SUCCEEDED(ret))
             {
                 CoTaskMemFree(ptr);
@@ -1062,7 +1061,7 @@ static HRESULT WINAPI rowset_colsinfo_GetColumnInfo(IColumnsInfo *iface, DBORDIN
             if (is_fixed_length(ColumnDataType))
                 dbcolumn[i].dwFlags |= DBCOLUMNFLAGS_ISFIXEDLENGTH;
 
-            ret = SQLColAttribute(rowset->hstmt, i+1, SQL_DESC_SCALE, NULL, 0, NULL, &length);
+            ret = SQLColAttributesW(rowset->hstmt, i+1, SQL_DESC_SCALE, NULL, 0, NULL, &length);
             if (!SQL_SUCCEEDED(ret))
             {
                 CoTaskMemFree(ptr);
@@ -1074,7 +1073,7 @@ static HRESULT WINAPI rowset_colsinfo_GetColumnInfo(IColumnsInfo *iface, DBORDIN
                 length = 255;
             dbcolumn[i].bScale = length;
 
-            ret = SQLColAttribute(rowset->hstmt, i+1, SQL_DESC_PRECISION, NULL, 0, NULL, &length);
+            ret = SQLColAttributesW(rowset->hstmt, i+1, SQL_DESC_PRECISION, NULL, 0, NULL, &length);
             if (!SQL_SUCCEEDED(ret))
             {
                 CoTaskMemFree(ptr);
@@ -1203,6 +1202,7 @@ static ULONG WINAPI column_rs_Release(IColumnsRowset *iface)
 static HRESULT WINAPI column_rs_GetAvailableColumns(IColumnsRowset *iface, DBORDINAL *count, DBID **columns)
 {
     struct msdasql_rowset *rowset = impl_from_IColumnsRowset( iface );
+    const DBORDINAL extra_columns = 3;
 
     TRACE("%p, %p, %p\n", rowset, count, columns);
 
@@ -1210,7 +1210,14 @@ static HRESULT WINAPI column_rs_GetAvailableColumns(IColumnsRowset *iface, DBORD
         return E_INVALIDARG;
 
     *count = 0;
-    *columns = NULL;
+    *columns = CoTaskMemAlloc(sizeof(DBID) * extra_columns);
+    if (!*columns)
+        return E_OUTOFMEMORY;
+
+    *count = extra_columns;
+    (*columns)[0] = DBCOLUMN_BASETABLENAME;
+    (*columns)[1] = DBCOLUMN_BASECOLUMNNAME;
+    (*columns)[2] = DBCOLUMN_KEYCOLUMN;
 
     return S_OK;
 }
@@ -1242,6 +1249,7 @@ static HRESULT WINAPI command_Execute(ICommandText *iface, IUnknown *outer, REFI
     RETCODE ret;
     SQLHSTMT hstmt = command->hstmt;
     SQLLEN results = -1;
+    BOOL free_hstmt = TRUE;
 
     TRACE("%p, %p, %s, %p %p %p\n", command, outer, debugstr_guid(riid), params, affected, rowset);
 
@@ -1255,10 +1263,11 @@ static HRESULT WINAPI command_Execute(ICommandText *iface, IUnknown *outer, REFI
         return E_FAIL;
     }
 
-    *rowset = NULL;
+    if (rowset)
+        *rowset = NULL;
     if (!wcsnicmp( command->query, L"select ", 7 ))
     {
-        msrowset = heap_alloc(sizeof(*msrowset));
+        msrowset = malloc(sizeof(*msrowset));
         if (!msrowset)
             return E_OUTOFMEMORY;
 
@@ -1272,21 +1281,23 @@ static HRESULT WINAPI command_Execute(ICommandText *iface, IUnknown *outer, REFI
         msrowset->refs = 1;
         ICommandText_QueryInterface(iface, &IID_IUnknown, (void**)&msrowset->caller);
         msrowset->hstmt = hstmt;
+        free_hstmt = FALSE;
 
         hr = IRowset_QueryInterface(&msrowset->IRowset_iface, riid, (void**)rowset);
         IRowset_Release(&msrowset->IRowset_iface);
     }
-    else
+
+    if (affected)
     {
         ret = SQLRowCount(hstmt, &results);
         if (ret != SQL_SUCCESS)
             ERR("SQLRowCount failed (%d)\n", ret);
 
-        SQLFreeStmt(hstmt, SQL_CLOSE);
+        *affected = results;
     }
 
-    if (affected)
-        *affected = results;
+    if (free_hstmt)
+        SQLFreeStmt(hstmt, SQL_CLOSE);
 
     return hr;
 }
@@ -1323,8 +1334,7 @@ static HRESULT WINAPI command_GetCommandText(ICommandText *iface, GUID *dialect,
         hr = DB_S_DIALECTIGNORED;
     }
 
-    *commandstr = heap_alloc((lstrlenW(command->query)+1)*sizeof(WCHAR));
-    wcscpy(*commandstr, command->query);
+    *commandstr = wcsdup(command->query);
     return hr;
 }
 
@@ -1336,15 +1346,13 @@ static HRESULT WINAPI command_SetCommandText(ICommandText *iface, REFGUID dialec
     if (!IsEqualGUID(&DBGUID_DEFAULT, dialect))
         FIXME("Currently non Default Dialect isn't supported\n");
 
-    heap_free(command->query);
+    free(command->query);
 
     if (commandstr)
     {
-        command->query = heap_alloc((lstrlenW(commandstr)+1)*sizeof(WCHAR));
+        command->query = wcsdup(commandstr);
         if (!command->query)
             return E_OUTOFMEMORY;
-
-        wcscpy(command->query, commandstr);
     }
     else
         command->query = NULL;
@@ -1772,7 +1780,7 @@ static HRESULT WINAPI createcommand_CreateCommand(IDBCreateCommand *iface, IUnkn
     if (outer)
         FIXME("Outer not currently supported\n");
 
-    command = heap_alloc(sizeof(*command));
+    command = malloc(sizeof(*command));
     if (!command)
         return E_OUTOFMEMORY;
 
@@ -1788,7 +1796,7 @@ static HRESULT WINAPI createcommand_CreateCommand(IDBCreateCommand *iface, IUnkn
     command->hstmt = NULL;
 
     command->prop_count = ARRAY_SIZE(msdasql_init_props);
-    command->properties = heap_alloc(sizeof(msdasql_init_props));
+    command->properties = malloc(sizeof(msdasql_init_props));
     memcpy(command->properties, msdasql_init_props, sizeof(msdasql_init_props));
 
     IUnknown_QueryInterface(&session->session_iface, &IID_IUnknown, (void**)&command->session);
@@ -1912,7 +1920,7 @@ HRESULT create_db_session(REFIID riid, IUnknown *datasource, HDBC hdbc, void **u
     struct msdasql_session *session;
     HRESULT hr;
 
-    session = heap_alloc(sizeof(*session));
+    session = malloc(sizeof(*session));
     if (!session)
         return E_OUTOFMEMORY;
 
