@@ -48,12 +48,15 @@ static HRESULT navigate_href_new_window(HTMLElement *element, nsAString *href_st
     IUri *uri;
     HRESULT hres;
 
+    if(!element->node.doc->window->base.outer_window)
+        return S_OK;
+
     nsAString_GetData(href_str, &href);
-    hres = create_relative_uri(element->node.doc->basedoc.window, href, &uri);
+    hres = create_relative_uri(element->node.doc->window->base.outer_window, href, &uri);
     if(FAILED(hres))
         return hres;
 
-    hres = navigate_new_window(element->node.doc->basedoc.window, uri, target, NULL, NULL);
+    hres = navigate_new_window(element->node.doc->window->base.outer_window, uri, target, NULL, NULL);
     IUri_Release(uri);
     return hres;
 }
@@ -110,7 +113,10 @@ static HRESULT navigate_href(HTMLElement *element, nsAString *href_str, nsAStrin
     const PRUnichar *href;
     HRESULT hres;
 
-    window = get_target_window(element->node.doc->basedoc.window, target_str, &use_new_window);
+    if(!element->node.doc->window->base.outer_window)
+        return S_OK;
+
+    window = get_target_window(element->node.doc->window->base.outer_window, target_str, &use_new_window);
     if(!window) {
         if(use_new_window) {
             const PRUnichar *target;
@@ -183,7 +189,7 @@ static IUri *get_anchor_uri(HTMLAnchorElement *anchor)
         nsAString_GetData(&href_str, &href);
         create_uri(href, 0, &uri);
     }else {
-        ERR("GetHref failed: %08x\n", nsres);
+        ERR("GetHref failed: %08lx\n", nsres);
     }
 
     nsAString_Finish(&href_str);
@@ -283,7 +289,7 @@ static HRESULT WINAPI HTMLAnchorElement_get_href(IHTMLAnchorElement *iface, BSTR
         nsAString_GetData(&href_str, &href);
         hres = nsuri_to_url(href, TRUE, p);
     }else {
-        ERR("GetHref failed: %08x\n", nsres);
+        ERR("GetHref failed: %08lx\n", nsres);
         hres = E_FAIL;
     }
 
@@ -517,6 +523,10 @@ static HRESULT WINAPI HTMLAnchorElement_get_port(IHTMLAnchorElement *iface, BSTR
     IUri_Release(uri);
     if(FAILED(hres))
         return hres;
+    if(hres != S_OK) {
+        *p = NULL;
+        return S_OK;
+    }
 
     len = swprintf(buf, ARRAY_SIZE(buf), L"%u", port);
     str = SysAllocStringLen(buf, len);
@@ -556,6 +566,11 @@ static HRESULT WINAPI HTMLAnchorElement_get_protocol(IHTMLAnchorElement *iface,
     IUri_Release(uri);
     if(FAILED(hres))
         return hres;
+    if(hres != S_OK) {
+        SysFreeString(scheme);
+        *p = NULL;
+        return S_OK;
+    }
 
     len = SysStringLen(scheme);
     if(len) {
@@ -783,39 +798,40 @@ static const IHTMLAnchorElementVtbl HTMLAnchorElementVtbl = {
     HTMLAnchorElement_blur
 };
 
-static inline HTMLAnchorElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline HTMLAnchorElement *impl_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, HTMLAnchorElement, element.node);
+    return CONTAINING_RECORD(iface, HTMLAnchorElement, element.node.event_target.dispex);
 }
 
-static HRESULT HTMLAnchorElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *HTMLAnchorElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLAnchorElement *This = impl_from_HTMLDOMNode(iface);
+    HTMLAnchorElement *This = impl_from_DispatchEx(dispex);
 
-    *ppv = NULL;
+    if(IsEqualGUID(&IID_IHTMLAnchorElement, riid))
+        return &This->IHTMLAnchorElement_iface;
 
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = &This->IHTMLAnchorElement_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        TRACE("(%p)->(IID_IDispatch %p)\n", This, ppv);
-        *ppv = &This->IHTMLAnchorElement_iface;
-    }else if(IsEqualGUID(&IID_IHTMLAnchorElement, riid)) {
-        TRACE("(%p)->(IID_IHTMLAnchorElement %p)\n", This, ppv);
-        *ppv = &This->IHTMLAnchorElement_iface;
-    }
-
-    if(*ppv) {
-        IUnknown_AddRef((IUnknown*)*ppv);
-        return S_OK;
-    }
-
-    return HTMLElement_QI(&This->element.node, riid, ppv);
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
-static HRESULT HTMLAnchorElement_handle_event(HTMLDOMNode *iface, DWORD eid, nsIDOMEvent *event, BOOL *prevent_default)
+static void HTMLAnchorElement_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
 {
-    HTMLAnchorElement *This = impl_from_HTMLDOMNode(iface);
+    HTMLAnchorElement *This = impl_from_DispatchEx(dispex);
+    HTMLElement_traverse(dispex, cb);
+
+    if(This->nsanchor)
+        note_cc_edge((nsISupports*)This->nsanchor, "nsanchor", cb);
+}
+
+static void HTMLAnchorElement_unlink(DispatchEx *dispex)
+{
+    HTMLAnchorElement *This = impl_from_DispatchEx(dispex);
+    HTMLElement_unlink(dispex);
+    unlink_ref(&This->nsanchor);
+}
+
+static HRESULT HTMLAnchorElement_handle_event(DispatchEx *dispex, eventid_t eid, nsIDOMEvent *event, BOOL *prevent_default)
+{
+    HTMLAnchorElement *This = impl_from_DispatchEx(dispex);
     nsAString href_str, target_str;
     nsresult nsres;
 
@@ -823,14 +839,14 @@ static HRESULT HTMLAnchorElement_handle_event(HTMLDOMNode *iface, DWORD eid, nsI
         nsAString_Init(&href_str, NULL);
         nsres = nsIDOMHTMLAnchorElement_GetHref(This->nsanchor, &href_str);
         if (NS_FAILED(nsres)) {
-            ERR("Could not get anchor href: %08x\n", nsres);
+            ERR("Could not get anchor href: %08lx\n", nsres);
             goto fallback;
         }
 
         nsAString_Init(&target_str, NULL);
         nsres = nsIDOMHTMLAnchorElement_GetTarget(This->nsanchor, &target_str);
         if (NS_FAILED(nsres)) {
-            ERR("Could not get anchor target: %08x\n", nsres);
+            ERR("Could not get anchor target: %08lx\n", nsres);
             goto fallback;
         }
 
@@ -841,47 +857,26 @@ fallback:
         nsAString_Finish(&target_str);
     }
 
-    return HTMLElement_handle_event(&This->element.node, eid, event, prevent_default);
-}
-
-static void HTMLAnchorElement_traverse(HTMLDOMNode *iface, nsCycleCollectionTraversalCallback *cb)
-{
-    HTMLAnchorElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsanchor)
-        note_cc_edge((nsISupports*)This->nsanchor, "This->nsanchor", cb);
-}
-
-static void HTMLAnchorElement_unlink(HTMLDOMNode *iface)
-{
-    HTMLAnchorElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsanchor) {
-        nsIDOMHTMLAnchorElement *nsanchor = This->nsanchor;
-
-        This->nsanchor = NULL;
-        nsIDOMHTMLAnchorElement_Release(nsanchor);
-    }
+    return HTMLElement_handle_event(&This->element.node.event_target.dispex, eid, event, prevent_default);
 }
 
 static const NodeImplVtbl HTMLAnchorElementImplVtbl = {
-    &CLSID_HTMLAnchorElement,
-    HTMLAnchorElement_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLAnchorElement_handle_event,
-    HTMLElement_get_attr_col,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    HTMLAnchorElement_traverse,
-    HTMLAnchorElement_unlink
+    .clsid                 = &CLSID_HTMLAnchorElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t HTMLAnchorElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLAnchorElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLAnchorElement_traverse,
+        .unlink         = HTMLAnchorElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLAnchorElement_handle_event
 };
 
 static const tid_t HTMLAnchorElement_iface_tids[] = {
@@ -891,8 +886,8 @@ static const tid_t HTMLAnchorElement_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLAnchorElement_dispex = {
-    L"HTMLAnchorElement",
-    NULL,
+    "HTMLAnchorElement",
+    &HTMLAnchorElement_event_target_vtbl.dispex_vtbl,
     DispHTMLAnchorElement_tid,
     HTMLAnchorElement_iface_tids,
     HTMLElement_init_dispex_info
@@ -903,7 +898,7 @@ HRESULT HTMLAnchorElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, H
     HTMLAnchorElement *ret;
     nsresult nsres;
 
-    ret = heap_alloc_zero(sizeof(HTMLAnchorElement));
+    ret = calloc(1, sizeof(HTMLAnchorElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
