@@ -374,21 +374,6 @@ static inline HTMLLinkElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
     return CONTAINING_RECORD(iface, HTMLLinkElement, element.node);
 }
 
-static HRESULT HTMLLinkElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
-{
-    HTMLLinkElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(IsEqualGUID(&IID_IHTMLLinkElement, riid)) {
-        TRACE("(%p)->(IID_IHTMLLinkElement %p)\n", This, ppv);
-        *ppv = &This->IHTMLLinkElement_iface;
-    }else {
-        return HTMLElement_QI(&This->element.node, riid, ppv);
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
 static HRESULT HTMLLinkElementImpl_put_disabled(HTMLDOMNode *iface, VARIANT_BOOL v)
 {
     HTMLLinkElement *This = impl_from_HTMLDOMNode(iface);
@@ -401,45 +386,55 @@ static HRESULT HTMLLinkElementImpl_get_disabled(HTMLDOMNode *iface, VARIANT_BOOL
     return IHTMLLinkElement_get_disabled(&This->IHTMLLinkElement_iface, p);
 }
 
-static void HTMLLinkElement_traverse(HTMLDOMNode *iface, nsCycleCollectionTraversalCallback *cb)
+static inline HTMLLinkElement *impl_from_DispatchEx(DispatchEx *iface)
 {
-    HTMLLinkElement *This = impl_from_HTMLDOMNode(iface);
+    return CONTAINING_RECORD(iface, HTMLLinkElement, element.node.event_target.dispex);
+}
+
+static void *HTMLLinkElement_query_interface(DispatchEx *dispex, REFIID riid)
+{
+    HTMLLinkElement *This = impl_from_DispatchEx(dispex);
+
+    if(IsEqualGUID(&IID_IHTMLLinkElement, riid))
+        return &This->IHTMLLinkElement_iface;
+
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
+}
+
+static void HTMLLinkElement_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
+{
+    HTMLLinkElement *This = impl_from_DispatchEx(dispex);
+    HTMLElement_traverse(dispex, cb);
 
     if(This->nslink)
-        note_cc_edge((nsISupports*)This->nslink, "This->nslink", cb);
+        note_cc_edge((nsISupports*)This->nslink, "nslink", cb);
 }
 
-static void HTMLLinkElement_unlink(HTMLDOMNode *iface)
+static void HTMLLinkElement_unlink(DispatchEx *dispex)
 {
-    HTMLLinkElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nslink) {
-        nsIDOMHTMLLinkElement *nslink = This->nslink;
-
-        This->nslink = NULL;
-        nsIDOMHTMLLinkElement_Release(nslink);
-    }
+    HTMLLinkElement *This = impl_from_DispatchEx(dispex);
+    HTMLElement_unlink(dispex);
+    unlink_ref(&This->nslink);
 }
 static const NodeImplVtbl HTMLLinkElementImplVtbl = {
-    &CLSID_HTMLLinkElement,
-    HTMLLinkElement_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLElement_dispatch_nsevent_hook,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col,
-    NULL,
-    HTMLLinkElementImpl_put_disabled,
-    HTMLLinkElementImpl_get_disabled,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    HTMLLinkElement_traverse,
-    HTMLLinkElement_unlink
+    .clsid                 = &CLSID_HTMLLinkElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+    .put_disabled          = HTMLLinkElementImpl_put_disabled,
+    .get_disabled          = HTMLLinkElementImpl_get_disabled,
+};
+
+static const event_target_vtbl_t HTMLLinkElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLLinkElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLLinkElement_traverse,
+        .unlink         = HTMLLinkElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
 };
 
 static const tid_t HTMLLinkElement_iface_tids[] = {
@@ -448,8 +443,8 @@ static const tid_t HTMLLinkElement_iface_tids[] = {
     0
 };
 static dispex_static_data_t HTMLLinkElement_dispex = {
-    L"HTMLLinkElement",
-    NULL,
+    "HTMLLinkElement",
+    &HTMLLinkElement_event_target_vtbl.dispex_vtbl,
     DispHTMLLinkElement_tid,
     HTMLLinkElement_iface_tids,
     HTMLElement_init_dispex_info
