@@ -36,32 +36,14 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
-static HRESULT get_url(HTMLLocation *This, const WCHAR **ret)
-{
-    if(!This->window || !This->window->base.outer_window || !This->window->base.outer_window->url) {
-        FIXME("No current URL\n");
-        return E_NOTIMPL;
-    }
-
-    *ret = This->window->base.outer_window->url;
-    return S_OK;
-}
-
 static IUri *get_uri(HTMLLocation *This)
 {
-    if(!This->window || !This->window->base.outer_window)
-        return NULL;
-    return This->window->base.outer_window->uri;
+    return This->window->uri;
 }
 
 static HRESULT get_url_components(HTMLLocation *This, URL_COMPONENTSW *url)
 {
-    const WCHAR *doc_url;
-    HRESULT hres;
-
-    hres = get_url(This, &doc_url);
-    if(FAILED(hres))
-        return hres;
+    const WCHAR *doc_url = This->window->url ? This->window->url : L"about:blank";
 
     if(!InternetCrackUrlW(doc_url, 0, 0, url)) {
         FIXME("InternetCrackUrlW failed: 0x%08lx\n", GetLastError());
@@ -80,54 +62,19 @@ static inline HTMLLocation *impl_from_IHTMLLocation(IHTMLLocation *iface)
 static HRESULT WINAPI HTMLLocation_QueryInterface(IHTMLLocation *iface, REFIID riid, void **ppv)
 {
     HTMLLocation *This = impl_from_IHTMLLocation(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        *ppv = &This->IHTMLLocation_iface;
-    }else if(IsEqualGUID(&IID_IHTMLLocation, riid)) {
-        *ppv = &This->IHTMLLocation_iface;
-    }else if(IsEqualGUID(&IID_IMarshal, riid)) {
-        *ppv = NULL;
-        FIXME("(%p)->(IID_IMarshal %p)\n", This, ppv);
-        return E_NOINTERFACE;
-    }else if(dispex_query_interface(&This->dispex, riid, ppv)) {
-        return *ppv ? S_OK : E_NOINTERFACE;
-    }else {
-        *ppv = NULL;
-        WARN("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-        return E_NOINTERFACE;
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return IDispatchEx_QueryInterface(&This->dispex.IDispatchEx_iface, riid, ppv);
 }
 
 static ULONG WINAPI HTMLLocation_AddRef(IHTMLLocation *iface)
 {
     HTMLLocation *This = impl_from_IHTMLLocation(iface);
-    LONG ref = InterlockedIncrement(&This->ref);
-
-    TRACE("(%p) ref=%ld\n", This, ref);
-
-    return ref;
+    return IDispatchEx_AddRef(&This->dispex.IDispatchEx_iface);
 }
 
 static ULONG WINAPI HTMLLocation_Release(IHTMLLocation *iface)
 {
     HTMLLocation *This = impl_from_IHTMLLocation(iface);
-    LONG ref = InterlockedDecrement(&This->ref);
-
-    TRACE("(%p) ref=%ld\n", This, ref);
-
-    if(!ref) {
-        if(This->window)
-            This->window->location = NULL;
-        release_dispex(&This->dispex);
-        free(This);
-    }
-
-    return ref;
+    return IDispatchEx_Release(&This->dispex.IDispatchEx_iface);
 }
 
 static HRESULT WINAPI HTMLLocation_GetTypeInfoCount(IHTMLLocation *iface, UINT *pctinfo)
@@ -167,12 +114,7 @@ static HRESULT WINAPI HTMLLocation_put_href(IHTMLLocation *iface, BSTR v)
 
     TRACE("(%p)->(%s)\n", This, debugstr_w(v));
 
-    if(!This->window || !This->window->base.outer_window) {
-        FIXME("No window available\n");
-        return E_FAIL;
-    }
-
-    return navigate_url(This->window->base.outer_window, v, This->window->base.outer_window->uri, BINDING_NAVIGATED);
+    return navigate_url(This->window, v, get_uri(This), BINDING_NAVIGATED);
 }
 
 static HRESULT WINAPI HTMLLocation_get_href(IHTMLLocation *iface, BSTR *p)
@@ -295,10 +237,8 @@ static HRESULT WINAPI HTMLLocation_get_protocol(IHTMLLocation *iface, BSTR *p)
     if(!p)
         return E_POINTER;
 
-    if(!(uri = get_uri(This))) {
-        FIXME("No current URI\n");
-        return E_NOTIMPL;
-    }
+    if(!(uri = get_uri(This)))
+        return (*p = SysAllocString(L"about:")) ? S_OK : E_OUTOFMEMORY;
 
     hres = IUri_GetSchemeName(uri, &protocol);
     if(FAILED(hres))
@@ -388,8 +328,8 @@ static HRESULT WINAPI HTMLLocation_get_hostname(IHTMLLocation *iface, BSTR *p)
         return E_POINTER;
 
     if(!(uri = get_uri(This))) {
-        FIXME("No current URI\n");
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
 
     hres = IUri_GetHost(uri, &hostname);
@@ -425,8 +365,8 @@ static HRESULT WINAPI HTMLLocation_get_port(IHTMLLocation *iface, BSTR *p)
         return E_POINTER;
 
     if(!(uri = get_uri(This))) {
-        FIXME("No current URI\n");
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
 
     hres = IUri_GetPort(uri, &port);
@@ -466,10 +406,8 @@ static HRESULT WINAPI HTMLLocation_get_pathname(IHTMLLocation *iface, BSTR *p)
     if(!p)
         return E_POINTER;
 
-    if(!(uri = get_uri(This))) {
-        FIXME("No current URI\n");
-        return E_NOTIMPL;
-    }
+    if(!(uri = get_uri(This)))
+        return (*p = SysAllocString(L"blank")) ? S_OK : E_OUTOFMEMORY;
 
     hres = IUri_GetPath(uri, &path);
     if(FAILED(hres))
@@ -504,8 +442,8 @@ static HRESULT WINAPI HTMLLocation_get_search(IHTMLLocation *iface, BSTR *p)
         return E_POINTER;
 
     if(!(uri = get_uri(This))) {
-        FIXME("No current URI\n");
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
 
     hres = IUri_GetQuery(uri, &query);
@@ -529,11 +467,6 @@ static HRESULT WINAPI HTMLLocation_put_hash(IHTMLLocation *iface, BSTR v)
 
     TRACE("(%p)->(%s)\n", This, debugstr_w(v));
 
-    if(!This->window || !This->window->base.outer_window) {
-        FIXME("No window available\n");
-        return E_FAIL;
-    }
-
     if(hash[0] != '#') {
         unsigned size = (1 /* # */ + wcslen(v) + 1) * sizeof(WCHAR);
         if(!(hash = malloc(size)))
@@ -542,7 +475,7 @@ static HRESULT WINAPI HTMLLocation_put_hash(IHTMLLocation *iface, BSTR v)
         memcpy(hash + 1, v, size - sizeof(WCHAR));
     }
 
-    hres = navigate_url(This->window->base.outer_window, hash, This->window->base.outer_window->uri, BINDING_NAVIGATED);
+    hres = navigate_url(This->window, hash, get_uri(This), BINDING_NAVIGATED);
 
     if(hash != v)
         free(hash);
@@ -562,8 +495,8 @@ static HRESULT WINAPI HTMLLocation_get_hash(IHTMLLocation *iface, BSTR *p)
         return E_POINTER;
 
     if(!(uri = get_uri(This))) {
-        FIXME("No current URI\n");
-        return E_NOTIMPL;
+        *p = NULL;
+        return S_OK;
     }
 
     hres = IUri_GetFragment(uri, &hash);
@@ -585,18 +518,13 @@ static HRESULT WINAPI HTMLLocation_reload(IHTMLLocation *iface, VARIANT_BOOL fla
 
     TRACE("(%p)->(%x)\n", This, flag);
 
-    if(!This->window) {
-        FIXME("No window available\n");
-        return E_FAIL;
-    }
-
     /* reload is supposed to fail if called from a script with different origin, but IE doesn't care */
-    if(!is_main_content_window(This->window->base.outer_window)) {
+    if(!is_main_content_window(This->window)) {
         FIXME("Unsupported on iframe\n");
         return E_NOTIMPL;
     }
 
-    return reload_page(This->window->base.outer_window);
+    return reload_page(This->window);
 }
 
 static HRESULT WINAPI HTMLLocation_replace(IHTMLLocation *iface, BSTR bstr)
@@ -605,13 +533,7 @@ static HRESULT WINAPI HTMLLocation_replace(IHTMLLocation *iface, BSTR bstr)
 
     TRACE("(%p)->(%s)\n", This, debugstr_w(bstr));
 
-    if(!This->window || !This->window->base.outer_window) {
-        FIXME("No window available\n");
-        return E_FAIL;
-    }
-
-    return navigate_url(This->window->base.outer_window, bstr, This->window->base.outer_window->uri,
-            BINDING_NAVIGATED|BINDING_REPLACE);
+    return navigate_url(This->window, bstr, get_uri(This), BINDING_NAVIGATED | BINDING_REPLACE);
 }
 
 static HRESULT WINAPI HTMLLocation_assign(IHTMLLocation *iface, BSTR bstr)
@@ -660,32 +582,78 @@ static const IHTMLLocationVtbl HTMLLocationVtbl = {
     HTMLLocation_toString
 };
 
+static inline HTMLLocation *impl_from_DispatchEx(DispatchEx *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLLocation, dispex);
+}
+
+static void *HTMLLocation_query_interface(DispatchEx *dispex, REFIID riid)
+{
+    HTMLLocation *This = impl_from_DispatchEx(dispex);
+
+    if(IsEqualGUID(&IID_IHTMLLocation, riid))
+        return &This->IHTMLLocation_iface;
+    if(IsEqualGUID(&IID_IMarshal, riid)) {
+        FIXME("(%p)->(IID_IMarshal)\n", This);
+        return NULL;
+    }
+
+    return NULL;
+}
+
+static void HTMLLocation_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
+{
+    HTMLLocation *This = impl_from_DispatchEx(dispex);
+    if(This->window)
+        note_cc_edge((nsISupports*)&This->window->base.IHTMLWindow2_iface, "window", cb);
+}
+
+static void HTMLLocation_unlink(DispatchEx *dispex)
+{
+    HTMLLocation *This = impl_from_DispatchEx(dispex);
+    if(This->window) {
+        HTMLOuterWindow *window = This->window;
+        This->window = NULL;
+        IHTMLWindow2_Release(&window->base.IHTMLWindow2_iface);
+    }
+}
+
+static void HTMLLocation_destructor(DispatchEx *dispex)
+{
+    HTMLLocation *This = impl_from_DispatchEx(dispex);
+    free(This);
+}
+
+static const dispex_static_data_vtbl_t HTMLLocation_dispex_vtbl = {
+    .query_interface  = HTMLLocation_query_interface,
+    .destructor       = HTMLLocation_destructor,
+    .traverse         = HTMLLocation_traverse,
+    .unlink           = HTMLLocation_unlink
+};
+
 static const tid_t HTMLLocation_iface_tids[] = {
     IHTMLLocation_tid,
     0
 };
 static dispex_static_data_t HTMLLocation_dispex = {
-    L"Object",
-    NULL,
+    "Location",
+    &HTMLLocation_dispex_vtbl,
     DispHTMLLocation_tid,
     HTMLLocation_iface_tids
 };
 
-
-HRESULT HTMLLocation_Create(HTMLInnerWindow *window, HTMLLocation **ret)
+HRESULT create_location(HTMLOuterWindow *window, HTMLLocation **ret)
 {
     HTMLLocation *location;
 
-    location = malloc(sizeof(*location));
-    if(!location)
+    if(!(location = calloc(1, sizeof(*location))))
         return E_OUTOFMEMORY;
 
     location->IHTMLLocation_iface.lpVtbl = &HTMLLocationVtbl;
-    location->ref = 1;
     location->window = window;
+    IHTMLWindow2_AddRef(&window->base.IHTMLWindow2_iface);
 
-    init_dispatch(&location->dispex, (IUnknown*)&location->IHTMLLocation_iface, &HTMLLocation_dispex,
-                  dispex_compat_mode(&window->event_target.dispex));
+    init_dispatch(&location->dispex, &HTMLLocation_dispex, COMPAT_MODE_QUIRKS);
 
     *ret = location;
     return S_OK;
