@@ -29,6 +29,7 @@
 #include "wine/debug.h"
 
 #include "mshtml_private.h"
+#include "htmlevent.h"
 #include "pluginhost.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
@@ -278,7 +279,7 @@ static HRESULT WINAPI HTMLObjectElement_put_width(IHTMLObjectElement *iface, VAR
     nsres = nsIDOMHTMLObjectElement_SetWidth(This->nsobject, &width_str);
     nsAString_Finish(&width_str);
     if(NS_FAILED(nsres)) {
-        FIXME("SetWidth failed: %08x\n", nsres);
+        FIXME("SetWidth failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -305,7 +306,7 @@ static HRESULT WINAPI HTMLObjectElement_get_width(IHTMLObjectElement *iface, VAR
         V_BSTR(p) = SysAllocString(width);
         hres = V_BSTR(p) ? S_OK : E_OUTOFMEMORY;
     }else {
-        ERR("GetWidth failed: %08x\n", nsres);
+        ERR("GetWidth failed: %08lx\n", nsres);
         hres = E_FAIL;
     }
 
@@ -336,7 +337,7 @@ static HRESULT WINAPI HTMLObjectElement_put_height(IHTMLObjectElement *iface, VA
     nsres = nsIDOMHTMLObjectElement_SetHeight(This->nsobject, &height_str);
     nsAString_Finish(&height_str);
     if(NS_FAILED(nsres)) {
-        FIXME("SetHeight failed: %08x\n", nsres);
+        FIXME("SetHeight failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -363,7 +364,7 @@ static HRESULT WINAPI HTMLObjectElement_get_height(IHTMLObjectElement *iface, VA
         V_BSTR(p) = SysAllocString(height);
         hres = V_BSTR(p) ? S_OK : E_OUTOFMEMORY;
     }else {
-        ERR("GetHeight failed: %08x\n", nsres);
+        ERR("GetHeight failed: %08lx\n", nsres);
         hres = E_FAIL;
     }
 
@@ -423,7 +424,7 @@ static HRESULT WINAPI HTMLObjectElement_get_altHtml(IHTMLObjectElement *iface, B
 static HRESULT WINAPI HTMLObjectElement_put_vspace(IHTMLObjectElement *iface, LONG v)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -436,7 +437,7 @@ static HRESULT WINAPI HTMLObjectElement_get_vspace(IHTMLObjectElement *iface, LO
 
     nsres = nsIDOMHTMLObjectElement_GetVspace(This->nsobject, p);
     if(NS_FAILED(nsres)) {
-        ERR("GetVspace failed: %08x\n", nsres);
+        ERR("GetVspace failed: %08lx\n", nsres);
         return E_FAIL;
     }
 
@@ -446,7 +447,7 @@ static HRESULT WINAPI HTMLObjectElement_get_vspace(IHTMLObjectElement *iface, LO
 static HRESULT WINAPI HTMLObjectElement_put_hspace(IHTMLObjectElement *iface, LONG v)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    FIXME("(%p)->(%d)\n", This, v);
+    FIXME("(%p)->(%ld)\n", This, v);
     return E_NOTIMPL;
 }
 
@@ -636,62 +637,6 @@ static inline HTMLObjectElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
     return CONTAINING_RECORD(iface, HTMLObjectElement, plugin_container.element.node);
 }
 
-static HRESULT HTMLObjectElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
-{
-    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        *ppv = &This->IHTMLObjectElement_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        *ppv = &This->IHTMLObjectElement_iface;
-    }else if(IsEqualGUID(&IID_IHTMLObjectElement, riid)) {
-        *ppv = &This->IHTMLObjectElement_iface;
-    }else if(IsEqualGUID(&IID_IHTMLObjectElement2, riid)) {
-        *ppv = &This->IHTMLObjectElement2_iface;
-    }else if(IsEqualGUID(&IID_HTMLPluginContainer, riid)) {
-        /* Special pseudo-interface returning HTMLPluginContainse struct. */
-        *ppv = &This->plugin_container;
-        node_addref(&This->plugin_container.element.node);
-        return S_OK;
-    }else {
-        HRESULT hres;
-
-        hres = HTMLElement_QI(&This->plugin_container.element.node, riid, ppv);
-        if(hres == E_NOINTERFACE && This->plugin_container.plugin_host && This->plugin_container.plugin_host->plugin_unk) {
-            IUnknown *plugin_iface, *ret;
-
-            hres = IUnknown_QueryInterface(This->plugin_container.plugin_host->plugin_unk, riid, (void**)&plugin_iface);
-            if(hres == S_OK) {
-                hres = wrap_iface(plugin_iface, (IUnknown*)&This->IHTMLObjectElement_iface, &ret);
-                IUnknown_Release(plugin_iface);
-                if(FAILED(hres))
-                    return hres;
-
-                TRACE("returning plugin iface %p wrapped to %p\n", plugin_iface, ret);
-                *ppv = ret;
-                return S_OK;
-            }
-        }
-
-        return hres;
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
-static void HTMLObjectElement_destructor(HTMLDOMNode *iface)
-{
-    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->plugin_container.plugin_host)
-        detach_plugin_host(This->plugin_container.plugin_host);
-
-    HTMLElement_destructor(&This->plugin_container.element.node);
-}
-
 static HRESULT HTMLObjectElement_get_readystate(HTMLDOMNode *iface, BSTR *p)
 {
     HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
@@ -699,64 +644,121 @@ static HRESULT HTMLObjectElement_get_readystate(HTMLDOMNode *iface, BSTR *p)
     return E_NOTIMPL;
 }
 
-static HRESULT HTMLObjectElement_get_dispid(HTMLDOMNode *iface, BSTR name,
-        DWORD grfdex, DISPID *pid)
+static inline HTMLObjectElement *impl_from_DispatchEx(DispatchEx *iface)
 {
-    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
-
-    TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(name), grfdex, pid);
-
-    return get_plugin_dispid(&This->plugin_container, name, pid);
+    return CONTAINING_RECORD(iface, HTMLObjectElement, plugin_container.element.node.event_target.dispex);
 }
 
-static HRESULT HTMLObjectElement_invoke(HTMLDOMNode *iface, DISPID id, LCID lcid,
-        WORD flags, DISPPARAMS *params, VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
+static void *HTMLObjectElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+    void *elem_iface;
 
-    TRACE("(%p)->(%d)\n", This, id);
+    if(IsEqualGUID(&IID_IHTMLObjectElement, riid))
+        return &This->IHTMLObjectElement_iface;
+    if(IsEqualGUID(&IID_IHTMLObjectElement2, riid))
+        return &This->IHTMLObjectElement2_iface;
+    if(IsEqualGUID(&IID_HTMLPluginContainer, riid)) {
+        /* Special pseudo-interface returning HTMLPluginContainer struct. */
+        return &This->plugin_container;
+    }
+
+    elem_iface = HTMLElement_query_interface(&This->plugin_container.element.node.event_target.dispex, riid);
+    if(!elem_iface && This->plugin_container.plugin_host && This->plugin_container.plugin_host->plugin_unk) {
+        IUnknown *plugin_iface, *ret;
+        HRESULT hres = IUnknown_QueryInterface(This->plugin_container.plugin_host->plugin_unk, riid, (void**)&plugin_iface);
+
+        if(hres == S_OK) {
+            hres = wrap_iface(plugin_iface, (IUnknown*)&This->IHTMLObjectElement_iface, &ret);
+            IUnknown_Release(plugin_iface);
+            if(FAILED(hres)) {
+                ERR("wrap_iface failed: %08lx\n", hres);
+                return NULL;
+            }
+
+            TRACE("returning plugin iface %p wrapped to %p\n", plugin_iface, ret);
+            return ret;
+        }
+    }
+
+    return elem_iface;
+}
+
+static void HTMLObjectElement_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
+{
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+    HTMLElement_traverse(dispex, cb);
+
+    if(This->nsobject)
+        note_cc_edge((nsISupports*)This->nsobject, "nsobject", cb);
+}
+
+static void HTMLObjectElement_unlink(DispatchEx *dispex)
+{
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+    HTMLElement_unlink(dispex);
+    unlink_ref(&This->nsobject);
+}
+
+static void HTMLObjectElement_destructor(DispatchEx *dispex)
+{
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+
+    if(This->plugin_container.plugin_host)
+        detach_plugin_host(This->plugin_container.plugin_host);
+
+    HTMLElement_destructor(&This->plugin_container.element.node.event_target.dispex);
+}
+
+static HRESULT HTMLObjectElement_get_dispid(DispatchEx *dispex, BSTR name, DWORD grfdex, DISPID *dispid)
+{
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+
+    TRACE("(%p)->(%s %lx %p)\n", This, debugstr_w(name), grfdex, dispid);
+
+    return get_plugin_dispid(&This->plugin_container, name, dispid);
+}
+
+static HRESULT HTMLObjectElement_dispex_get_name(DispatchEx *dispex, DISPID id, BSTR *name)
+{
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+
+    FIXME("(%p)->(%lx %p)\n", This, id, name);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT HTMLObjectElement_invoke(DispatchEx *dispex, DISPID id, LCID lcid, WORD flags, DISPPARAMS *params,
+        VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
+{
+    HTMLObjectElement *This = impl_from_DispatchEx(dispex);
+
+    TRACE("(%p)->(%ld)\n", This, id);
 
     return invoke_plugin_prop(&This->plugin_container, id, lcid, flags, params, res, ei);
 }
 
-static void HTMLObjectElement_traverse(HTMLDOMNode *iface, nsCycleCollectionTraversalCallback *cb)
-{
-    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsobject)
-        note_cc_edge((nsISupports*)This->nsobject, "This->nsobject", cb);
-}
-
-static void HTMLObjectElement_unlink(HTMLDOMNode *iface)
-{
-    HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsobject) {
-        nsIDOMHTMLObjectElement *nsobject = This->nsobject;
-
-        This->nsobject = NULL;
-        nsIDOMHTMLObjectElement_Release(nsobject);
-    }
-}
-
 static const NodeImplVtbl HTMLObjectElementImplVtbl = {
-    &CLSID_HTMLObjectElement,
-    HTMLObjectElement_QI,
-    HTMLObjectElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    HTMLObjectElement_get_readystate,
-    HTMLObjectElement_get_dispid,
-    HTMLObjectElement_invoke,
-    NULL,
-    HTMLObjectElement_traverse,
-    HTMLObjectElement_unlink
+    .clsid                 = &CLSID_HTMLObjectElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+    .get_readystate        = HTMLObjectElement_get_readystate,
+};
+
+static const event_target_vtbl_t HTMLObjectElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLObjectElement_query_interface,
+        .destructor     = HTMLObjectElement_destructor,
+        .traverse       = HTMLObjectElement_traverse,
+        .unlink         = HTMLObjectElement_unlink,
+        .get_dispid     = HTMLObjectElement_get_dispid,
+        .get_name       = HTMLObjectElement_dispex_get_name,
+        .invoke         = HTMLObjectElement_invoke
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
 };
 
 static const tid_t HTMLObjectElement_iface_tids[] = {
@@ -766,8 +768,8 @@ static const tid_t HTMLObjectElement_iface_tids[] = {
     0
 };
 static dispex_static_data_t HTMLObjectElement_dispex = {
-    L"HTMLObjectElement",
-    NULL,
+    "HTMLObjectElement",
+    &HTMLObjectElement_event_target_vtbl.dispex_vtbl,
     DispHTMLObjectElement_tid,
     HTMLObjectElement_iface_tids,
     HTMLElement_init_dispex_info
@@ -778,7 +780,7 @@ HRESULT HTMLObjectElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, H
     HTMLObjectElement *ret;
     nsresult nsres;
 
-    ret = heap_alloc_zero(sizeof(*ret));
+    ret = calloc(1, sizeof(*ret));
     if(!ret)
         return E_OUTOFMEMORY;
 
@@ -981,46 +983,38 @@ static const IHTMLEmbedElementVtbl HTMLEmbedElementVtbl = {
     HTMLEmbedElement_get_height
 };
 
-static inline HTMLEmbed *embed_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline HTMLEmbed *embed_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, HTMLEmbed, element.node);
+    return CONTAINING_RECORD(iface, HTMLEmbed, element.node.event_target.dispex);
 }
 
-static HRESULT HTMLEmbedElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *HTMLEmbedElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLEmbed *This = embed_from_HTMLDOMNode(iface);
+    HTMLEmbed *This = embed_from_DispatchEx(dispex);
 
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+    if(IsEqualGUID(&IID_IHTMLEmbedElement, riid))
+        return &This->IHTMLEmbedElement_iface;
 
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        *ppv = &This->IHTMLEmbedElement_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        *ppv = &This->IHTMLEmbedElement_iface;
-    }else if(IsEqualGUID(&IID_IHTMLEmbedElement, riid)) {
-        *ppv = &This->IHTMLEmbedElement_iface;
-    }else {
-        return HTMLElement_QI(&This->element.node, riid, ppv);
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
-}
-
-static void HTMLEmbedElement_destructor(HTMLDOMNode *iface)
-{
-    HTMLEmbed *This = embed_from_HTMLDOMNode(iface);
-
-    HTMLElement_destructor(&This->element.node);
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
 static const NodeImplVtbl HTMLEmbedElementImplVtbl = {
-    &CLSID_HTMLEmbed,
-    HTMLEmbedElement_QI,
-    HTMLEmbedElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col
+    .clsid                 = &CLSID_HTMLEmbed,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col
+};
+
+static const event_target_vtbl_t HTMLEmbedElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLEmbedElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLElement_traverse,
+        .unlink         = HTMLElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
 };
 
 static const tid_t HTMLEmbedElement_iface_tids[] = {
@@ -1029,8 +1023,8 @@ static const tid_t HTMLEmbedElement_iface_tids[] = {
     0
 };
 static dispex_static_data_t HTMLEmbedElement_dispex = {
-    L"HTMLEmbedElement",
-    NULL,
+    "HTMLEmbedElement",
+    &HTMLEmbedElement_event_target_vtbl.dispex_vtbl,
     DispHTMLEmbed_tid,
     HTMLEmbedElement_iface_tids,
     HTMLElement_init_dispex_info
@@ -1040,7 +1034,7 @@ HRESULT HTMLEmbedElement_Create(HTMLDocumentNode *doc, nsIDOMElement *nselem, HT
 {
     HTMLEmbed *ret;
 
-    ret = heap_alloc_zero(sizeof(*ret));
+    ret = calloc(1, sizeof(*ret));
     if(!ret)
         return E_OUTOFMEMORY;
 
