@@ -28,6 +28,7 @@
 #include "wine/debug.h"
 
 #include "mshtml_private.h"
+#include "htmlevent.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -440,79 +441,54 @@ static const IHTMLTableCellVtbl HTMLTableCellVtbl = {
     HTMLTableCell_get_cellIndex
 };
 
-static inline HTMLTableCell *HTMLTableCell_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline HTMLTableCell *HTMLTableCell_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, HTMLTableCell, element.node);
+    return CONTAINING_RECORD(iface, HTMLTableCell, element.node.event_target.dispex);
 }
 
-static HRESULT HTMLTableCell_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *HTMLTableCell_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLTableCell *This = HTMLTableCell_from_HTMLDOMNode(iface);
+    HTMLTableCell *This = HTMLTableCell_from_DispatchEx(dispex);
 
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = &This->IHTMLTableCell_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        TRACE("(%p)->(IID_IDispatch %p)\n", This, ppv);
-        *ppv = &This->IHTMLTableCell_iface;
-    }else if(IsEqualGUID(&IID_IHTMLTableCell, riid)) {
-        TRACE("(%p)->(IID_IHTMLTableCell %p)\n", This, ppv);
-        *ppv = &This->IHTMLTableCell_iface;
-    }else {
-        return HTMLElement_QI(&This->element.node, riid, ppv);
-    }
+    if(IsEqualGUID(&IID_IHTMLTableCell, riid))
+        return &This->IHTMLTableCell_iface;
 
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
-static void HTMLTableCell_destructor(HTMLDOMNode *iface)
+static void HTMLTableCell_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
 {
-    HTMLTableCell *This = HTMLTableCell_from_HTMLDOMNode(iface);
-
-    HTMLElement_destructor(&This->element.node);
-}
-
-static void HTMLTableCell_traverse(HTMLDOMNode *iface, nsCycleCollectionTraversalCallback *cb)
-{
-    HTMLTableCell *This = HTMLTableCell_from_HTMLDOMNode(iface);
+    HTMLTableCell *This = HTMLTableCell_from_DispatchEx(dispex);
+    HTMLElement_traverse(dispex, cb);
 
     if(This->nscell)
-        note_cc_edge((nsISupports*)This->nscell, "This->nstablecell", cb);
+        note_cc_edge((nsISupports*)This->nscell, "nstablecell", cb);
 }
 
-static void HTMLTableCell_unlink(HTMLDOMNode *iface)
+static void HTMLTableCell_unlink(DispatchEx *dispex)
 {
-    HTMLTableCell *This = HTMLTableCell_from_HTMLDOMNode(iface);
-
-    if(This->nscell) {
-        nsIDOMHTMLTableCellElement *nscell = This->nscell;
-
-        This->nscell = NULL;
-        nsIDOMHTMLTableCellElement_Release(nscell);
-    }
+    HTMLTableCell *This = HTMLTableCell_from_DispatchEx(dispex);
+    HTMLElement_unlink(dispex);
+    unlink_ref(&This->nscell);
 }
 
 static const NodeImplVtbl HTMLTableCellImplVtbl = {
-    &CLSID_HTMLTableCell,
-    HTMLTableCell_QI,
-    HTMLTableCell_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLElement_dispatch_nsevent_hook,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    HTMLTableCell_traverse,
-    HTMLTableCell_unlink
+    .clsid                 = &CLSID_HTMLTableCell,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t HTMLTableCell_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLTableCell_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLTableCell_traverse,
+        .unlink         = HTMLTableCell_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
 };
 
 static const tid_t HTMLTableCell_iface_tids[] = {
@@ -522,8 +498,8 @@ static const tid_t HTMLTableCell_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLTableCell_dispex = {
-    L"HTMLTableDataCellElement",
-    NULL,
+    "HTMLTableDataCellElement",
+    &HTMLTableCell_event_target_vtbl.dispex_vtbl,
     DispHTMLTableCell_tid,
     HTMLTableCell_iface_tids,
     HTMLElement_init_dispex_info
@@ -890,76 +866,54 @@ static const IHTMLTableRowVtbl HTMLTableRowVtbl = {
     HTMLTableRow_deleteCell
 };
 
-static inline HTMLTableRow *HTMLTableRow_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline HTMLTableRow *HTMLTableRow_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, HTMLTableRow, element.node);
+    return CONTAINING_RECORD(iface, HTMLTableRow, element.node.event_target.dispex);
 }
 
-static HRESULT HTMLTableRow_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *HTMLTableRow_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLTableRow *This = HTMLTableRow_from_HTMLDOMNode(iface);
+    HTMLTableRow *This = HTMLTableRow_from_DispatchEx(dispex);
 
-    *ppv = NULL;
+    if(IsEqualGUID(&IID_IHTMLTableRow, riid))
+        return &This->IHTMLTableRow_iface;
 
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = &This->IHTMLTableRow_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        TRACE("(%p)->(IID_IDispatch %p)\n", This, ppv);
-        *ppv = &This->IHTMLTableRow_iface;
-    }else if(IsEqualGUID(&IID_IHTMLTableRow, riid)) {
-        TRACE("(%p)->(IID_IHTMLTableRow %p)\n", This, ppv);
-        *ppv = &This->IHTMLTableRow_iface;
-    }
-
-    if(*ppv) {
-        IUnknown_AddRef((IUnknown*)*ppv);
-        return S_OK;
-    }
-
-    return HTMLElement_QI(&This->element.node, riid, ppv);
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
-static void HTMLTableRow_traverse(HTMLDOMNode *iface, nsCycleCollectionTraversalCallback *cb)
+static void HTMLTableRow_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
 {
-    HTMLTableRow *This = HTMLTableRow_from_HTMLDOMNode(iface);
+    HTMLTableRow *This = HTMLTableRow_from_DispatchEx(dispex);
+    HTMLElement_traverse(dispex, cb);
 
     if(This->nsrow)
-        note_cc_edge((nsISupports*)This->nsrow, "This->nstablerow", cb);
+        note_cc_edge((nsISupports*)This->nsrow, "nstablerow", cb);
 }
 
-static void HTMLTableRow_unlink(HTMLDOMNode *iface)
+static void HTMLTableRow_unlink(DispatchEx *dispex)
 {
-    HTMLTableRow *This = HTMLTableRow_from_HTMLDOMNode(iface);
-
-    if(This->nsrow) {
-        nsIDOMHTMLTableRowElement *nsrow = This->nsrow;
-
-        This->nsrow = NULL;
-        nsIDOMHTMLTableRowElement_Release(nsrow);
-    }
+    HTMLTableRow *This = HTMLTableRow_from_DispatchEx(dispex);
+    HTMLElement_unlink(dispex);
+    unlink_ref(&This->nsrow);
 }
 
 static const NodeImplVtbl HTMLTableRowImplVtbl = {
-    &CLSID_HTMLTableRow,
-    HTMLTableRow_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    HTMLElement_dispatch_nsevent_hook,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    HTMLTableRow_traverse,
-    HTMLTableRow_unlink
+    .clsid                 = &CLSID_HTMLTableRow,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t HTMLTableRow_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLTableRow_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLTableRow_traverse,
+        .unlink         = HTMLTableRow_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
 };
 
 static const tid_t HTMLTableRow_iface_tids[] = {
@@ -969,8 +923,8 @@ static const tid_t HTMLTableRow_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLTableRow_dispex = {
-    L"HTMLTableRowElement",
-    NULL,
+    "HTMLTableRowElement",
+    &HTMLTableRow_event_target_vtbl.dispex_vtbl,
     DispHTMLTableRow_tid,
     HTMLTableRow_iface_tids,
     HTMLElement_init_dispex_info
@@ -1911,60 +1865,39 @@ static const IHTMLTable3Vtbl HTMLTable3Vtbl = {
     HTMLTable3_get_summary
 };
 
-static inline HTMLTable *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline HTMLTable *impl_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, HTMLTable, element.node);
+    return CONTAINING_RECORD(iface, HTMLTable, element.node.event_target.dispex);
 }
 
-static HRESULT HTMLTable_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *HTMLTable_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    HTMLTable *This = impl_from_HTMLDOMNode(iface);
+    HTMLTable *This = impl_from_DispatchEx(dispex);
 
-    *ppv = NULL;
+    if(IsEqualGUID(&IID_IHTMLTable, riid))
+        return &This->IHTMLTable_iface;
+    if(IsEqualGUID(&IID_IHTMLTable2, riid))
+        return &This->IHTMLTable2_iface;
+    if(IsEqualGUID(&IID_IHTMLTable3, riid))
+        return &This->IHTMLTable3_iface;
 
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = &This->IHTMLTable_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        TRACE("(%p)->(IID_IDispatch %p)\n", This, ppv);
-        *ppv = &This->IHTMLTable_iface;
-    }else if(IsEqualGUID(&IID_IHTMLTable, riid)) {
-        TRACE("(%p)->(IID_IHTMLTable %p)\n", This, ppv);
-        *ppv = &This->IHTMLTable_iface;
-    }else if(IsEqualGUID(&IID_IHTMLTable2, riid)) {
-        TRACE("(%p)->(IID_IHTMLTable2 %p)\n", This, ppv);
-        *ppv = &This->IHTMLTable2_iface;
-    }else if(IsEqualGUID(&IID_IHTMLTable3, riid)) {
-        TRACE("(%p)->(IID_IHTMLTable3 %p)\n", This, ppv);
-        *ppv = &This->IHTMLTable3_iface;
-    }
-
-    if(*ppv) {
-        IUnknown_AddRef((IUnknown*)*ppv);
-        return S_OK;
-    }
-
-    return HTMLElement_QI(&This->element.node, riid, ppv);
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
-static void HTMLTable_traverse(HTMLDOMNode *iface, nsCycleCollectionTraversalCallback *cb)
+static void HTMLTable_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
 {
-    HTMLTable *This = impl_from_HTMLDOMNode(iface);
+    HTMLTable *This = impl_from_DispatchEx(dispex);
+    HTMLElement_traverse(dispex, cb);
 
     if(This->nstable)
-        note_cc_edge((nsISupports*)This->nstable, "This->nstable", cb);
+        note_cc_edge((nsISupports*)This->nstable, "nstable", cb);
 }
 
-static void HTMLTable_unlink(HTMLDOMNode *iface)
+static void HTMLTable_unlink(DispatchEx *dispex)
 {
-    HTMLTable *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nstable) {
-        nsIDOMHTMLTableElement *nstable = This->nstable;
-
-        This->nstable = NULL;
-        nsIDOMHTMLTableElement_Release(nstable);
-    }
+    HTMLTable *This = impl_from_DispatchEx(dispex);
+    HTMLElement_unlink(dispex);
+    unlink_ref(&This->nstable);
 }
 
 static const cpc_entry_t HTMLTable_cpc[] = {
@@ -1974,25 +1907,22 @@ static const cpc_entry_t HTMLTable_cpc[] = {
 };
 
 static const NodeImplVtbl HTMLTableImplVtbl = {
-    &CLSID_HTMLTable,
-    HTMLTable_QI,
-    HTMLElement_destructor,
-    HTMLTable_cpc,
-    HTMLElement_clone,
-    HTMLElement_dispatch_nsevent_hook,
-    HTMLElement_handle_event,
-    HTMLElement_get_attr_col,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    HTMLTable_traverse,
-    HTMLTable_unlink
+    .clsid                 = &CLSID_HTMLTable,
+    .cpc_entries           = HTMLTable_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t HTMLTable_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= HTMLTable_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLTable_traverse,
+        .unlink         = HTMLTable_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
 };
 
 static const tid_t HTMLTable_iface_tids[] = {
@@ -2004,8 +1934,8 @@ static const tid_t HTMLTable_iface_tids[] = {
 };
 
 static dispex_static_data_t HTMLTable_dispex = {
-    L"HTMLTableElement",
-    NULL,
+    "HTMLTableElement",
+    &HTMLTable_event_target_vtbl.dispex_vtbl,
     DispHTMLTable_tid,
     HTMLTable_iface_tids,
     HTMLElement_init_dispex_info
