@@ -36,8 +36,6 @@ typedef struct {
     IHTMLSelectionObject IHTMLSelectionObject_iface;
     IHTMLSelectionObject2 IHTMLSelectionObject2_iface;
 
-    LONG ref;
-
     nsISelection *nsselection;
     HTMLDocumentNode *doc;
 
@@ -53,56 +51,19 @@ static HRESULT WINAPI HTMLSelectionObject_QueryInterface(IHTMLSelectionObject *i
                                                          REFIID riid, void **ppv)
 {
     HTMLSelectionObject *This = impl_from_IHTMLSelectionObject(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
-
-    if(IsEqualGUID(&IID_IUnknown, riid)) {
-        *ppv = &This->IHTMLSelectionObject_iface;
-    }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        *ppv = &This->IHTMLSelectionObject_iface;
-    }else if(IsEqualGUID(&IID_IHTMLSelectionObject, riid)) {
-        *ppv = &This->IHTMLSelectionObject_iface;
-    }else if(IsEqualGUID(&IID_IHTMLSelectionObject2, riid)) {
-        *ppv = &This->IHTMLSelectionObject2_iface;
-    }else if(dispex_query_interface(&This->dispex, riid, ppv)) {
-        return *ppv ? S_OK : E_NOINTERFACE;
-    }else {
-        *ppv = NULL;
-        WARN("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppv);
-        return E_NOINTERFACE;
-    }
-
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return IDispatchEx_QueryInterface(&This->dispex.IDispatchEx_iface, riid, ppv);
 }
 
 static ULONG WINAPI HTMLSelectionObject_AddRef(IHTMLSelectionObject *iface)
 {
     HTMLSelectionObject *This = impl_from_IHTMLSelectionObject(iface);
-    LONG ref = InterlockedIncrement(&This->ref);
-
-    TRACE("(%p) ref=%d\n", This, ref);
-
-    return ref;
+    return IDispatchEx_AddRef(&This->dispex.IDispatchEx_iface);
 }
 
 static ULONG WINAPI HTMLSelectionObject_Release(IHTMLSelectionObject *iface)
 {
     HTMLSelectionObject *This = impl_from_IHTMLSelectionObject(iface);
-    LONG ref = InterlockedDecrement(&This->ref);
-
-    TRACE("(%p) ref=%d\n", This, ref);
-
-    if(!ref) {
-        if(This->nsselection)
-            nsISelection_Release(This->nsselection);
-        if(This->doc)
-            list_remove(&This->entry);
-        release_dispex(&This->dispex);
-        heap_free(This);
-    }
-
-    return ref;
+    return IDispatchEx_Release(&This->dispex.IDispatchEx_iface);
 }
 
 static HRESULT WINAPI HTMLSelectionObject_GetTypeInfoCount(IHTMLSelectionObject *iface, UINT *pctinfo)
@@ -160,28 +121,33 @@ static HRESULT WINAPI HTMLSelectionObject_createRange(IHTMLSelectionObject *ifac
 
             TRACE("nsrange_cnt = 0\n");
 
-            if(!This->doc->nsdoc) {
-                WARN("nsdoc is NULL\n");
+            if(!This->doc->dom_document) {
+                WARN("dom_document is NULL\n");
                 return E_UNEXPECTED;
             }
 
-            nsres = nsIDOMHTMLDocument_GetBody(This->doc->nsdoc, &nsbody);
+            if(!This->doc->html_document) {
+                FIXME("Not implemented for XML document\n");
+                return E_NOTIMPL;
+            }
+
+            nsres = nsIDOMHTMLDocument_GetBody(This->doc->html_document, &nsbody);
             if(NS_FAILED(nsres) || !nsbody) {
-                ERR("Could not get body: %08x\n", nsres);
+                ERR("Could not get body: %08lx\n", nsres);
                 return E_FAIL;
             }
 
             nsres = nsISelection_Collapse(This->nsselection, (nsIDOMNode*)nsbody, 0);
             nsIDOMHTMLElement_Release(nsbody);
             if(NS_FAILED(nsres))
-                ERR("Collapse failed: %08x\n", nsres);
+                ERR("Collapse failed: %08lx\n", nsres);
         }else if(nsrange_cnt > 1) {
-            FIXME("range_cnt = %d\n", nsrange_cnt);
+            FIXME("range_cnt = %ld\n", nsrange_cnt);
         }
 
         nsres = nsISelection_GetRangeAt(This->nsselection, 0, &nsrange);
         if(NS_FAILED(nsres))
-            ERR("GetRangeAt failed: %08x\n", nsres);
+            ERR("GetRangeAt failed: %08lx\n", nsres);
     }
 
     hres = HTMLTxtRange_Create(This->doc, nsrange, &range_obj);
@@ -324,14 +290,61 @@ static const IHTMLSelectionObject2Vtbl HTMLSelectionObject2Vtbl = {
     HTMLSelectionObject2_get_typeDetail
 };
 
+static inline HTMLSelectionObject *impl_from_DispatchEx(DispatchEx *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLSelectionObject, dispex);
+}
+
+static void *HTMLSelectionObject_query_interface(DispatchEx *dispex, REFIID riid)
+{
+    HTMLSelectionObject *This = impl_from_DispatchEx(dispex);
+
+    if(IsEqualGUID(&IID_IHTMLSelectionObject, riid))
+        return &This->IHTMLSelectionObject_iface;
+    if(IsEqualGUID(&IID_IHTMLSelectionObject2, riid))
+        return &This->IHTMLSelectionObject2_iface;
+
+    return NULL;
+}
+
+static void HTMLSelectionObject_traverse(DispatchEx *dispex, nsCycleCollectionTraversalCallback *cb)
+{
+    HTMLSelectionObject *This = impl_from_DispatchEx(dispex);
+    if(This->nsselection)
+        note_cc_edge((nsISupports*)This->nsselection, "nsselection", cb);
+}
+
+static void HTMLSelectionObject_unlink(DispatchEx *dispex)
+{
+    HTMLSelectionObject *This = impl_from_DispatchEx(dispex);
+    unlink_ref(&This->nsselection);
+    if(This->doc) {
+        This->doc = NULL;
+        list_remove(&This->entry);
+    }
+}
+
+static void HTMLSelectionObject_destructor(DispatchEx *dispex)
+{
+    HTMLSelectionObject *This = impl_from_DispatchEx(dispex);
+    free(This);
+}
+
+static const dispex_static_data_vtbl_t HTMLSelectionObject_dispex_vtbl = {
+    .query_interface  = HTMLSelectionObject_query_interface,
+    .destructor       = HTMLSelectionObject_destructor,
+    .traverse         = HTMLSelectionObject_traverse,
+    .unlink           = HTMLSelectionObject_unlink
+};
+
 static const tid_t HTMLSelectionObject_iface_tids[] = {
     IHTMLSelectionObject_tid,
     IHTMLSelectionObject2_tid,
     0
 };
 static dispex_static_data_t HTMLSelectionObject_dispex = {
-    L"MSSelection",
-    NULL,
+    "MSSelection",
+    &HTMLSelectionObject_dispex_vtbl,
     IHTMLSelectionObject_tid, /* FIXME: We have a test for that, but it doesn't expose IHTMLSelectionObject2 iface. */
     HTMLSelectionObject_iface_tids
 };
@@ -340,16 +353,14 @@ HRESULT HTMLSelectionObject_Create(HTMLDocumentNode *doc, nsISelection *nsselect
 {
     HTMLSelectionObject *selection;
 
-    selection = heap_alloc(sizeof(HTMLSelectionObject));
+    selection = malloc(sizeof(HTMLSelectionObject));
     if(!selection)
         return E_OUTOFMEMORY;
 
-    init_dispatch(&selection->dispex, (IUnknown*)&selection->IHTMLSelectionObject_iface,
-                  &HTMLSelectionObject_dispex, dispex_compat_mode(&doc->node.event_target.dispex));
+    init_dispatch(&selection->dispex, &HTMLSelectionObject_dispex, dispex_compat_mode(&doc->node.event_target.dispex));
 
     selection->IHTMLSelectionObject_iface.lpVtbl = &HTMLSelectionObjectVtbl;
     selection->IHTMLSelectionObject2_iface.lpVtbl = &HTMLSelectionObject2Vtbl;
-    selection->ref = 1;
     selection->nsselection = nsselection; /* We shouldn't call AddRef here */
 
     selection->doc = doc;
