@@ -24,6 +24,9 @@
 
 #include "windef.h"
 #include "winternl.h"
+#include "winbase.h"
+#include "winver.h"
+#include "imagehlp.h"
 #include "wine/exception.h"
 #include "wine/debug.h"
 #include "msvcrt.h"
@@ -583,7 +586,9 @@ DEFINE_RTTI_DATA1( bad_cast, 0, &exception_rtti_base_descriptor, ".?AVbad_cast@@
 DEFINE_RTTI_DATA2( __non_rtti_object, 0, &bad_typeid_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AV__non_rtti_object@@" )
 #endif
 
+#if _MSVCR_VER >= 100
 DEFINE_CXX_EXCEPTION0( exception, exception_dtor )
+#endif
 DEFINE_CXX_DATA1( bad_typeid, &exception_cxx_type_info, bad_typeid_dtor )
 DEFINE_CXX_DATA1( bad_cast, &exception_cxx_type_info, bad_cast_dtor )
 DEFINE_CXX_DATA2( __non_rtti_object, &bad_typeid_cxx_type_info,
@@ -605,7 +610,11 @@ void msvcrt_init_exception(void *base)
     init_bad_cast_rtti(base);
     init___non_rtti_object_rtti(base);
 
+#if _MSVCR_VER >= 100
     init_exception_cxx(base);
+#else
+    init_exception_cxx_type_info(base);
+#endif
     init_bad_typeid_cxx(base);
     init_bad_cast_cxx(base);
     init___non_rtti_object_cxx(base);
@@ -624,6 +633,15 @@ void throw_bad_alloc(void)
 }
 #endif
 
+#if _MSVCR_VER >= 100
+void throw_exception(const char* msg)
+{
+    exception e;
+    __exception_ctor(&e, msg, &exception_vtable);
+    _CxxThrowException(&e, &exception_exception_type);
+}
+#endif
+
 /******************************************************************
  *		?set_terminate@@YAP6AXXZP6AXXZ@Z (MSVCRT.@)
  *
@@ -1128,351 +1146,11 @@ const char * __thiscall type_info_name_internal_method(type_info * _this, struct
     return type_info_name(_this);
 }
 
-#endif /* _MSVCR_VER >= 80 */
-
-/* std::exception_ptr class helpers */
-typedef struct
-{
-    EXCEPTION_RECORD *rec;
-    int *ref; /* not binary compatible with native msvcr100 */
-} exception_ptr;
-
-#if _MSVCR_VER >= 100
-
-/*********************************************************************
- * ?__ExceptionPtrCreate@@YAXPAX@Z
- * ?__ExceptionPtrCreate@@YAXPEAX@Z
- */
-void __cdecl __ExceptionPtrCreate(exception_ptr *ep)
-{
-    TRACE("(%p)\n", ep);
-
-    ep->rec = NULL;
-    ep->ref = NULL;
-}
-
-#ifdef __ASM_USE_THISCALL_WRAPPER
-extern void call_dtor(const cxx_exception_type *type, void *func, void *object);
-
-__ASM_GLOBAL_FUNC( call_dtor,
-                   "movl 12(%esp),%ecx\n\t"
-                   "call *8(%esp)\n\t"
-                   "ret" );
-#elif __x86_64__
-static inline void call_dtor(const cxx_exception_type *type, unsigned int dtor, void *object)
-{
-    char *base = RtlPcToFileHeader((void*)type, (void**)&base);
-    void (__cdecl *func)(void*) = (void*)(base + dtor);
-    func(object);
-}
-#else
-#define call_dtor(type, func, object) ((void (__thiscall*)(void*))(func))(object)
-#endif
-
-/*********************************************************************
- * ?__ExceptionPtrDestroy@@YAXPAX@Z
- * ?__ExceptionPtrDestroy@@YAXPEAX@Z
- */
-void __cdecl __ExceptionPtrDestroy(exception_ptr *ep)
-{
-    TRACE("(%p)\n", ep);
-
-    if (!ep->rec)
-        return;
-
-    if (!InterlockedDecrement(ep->ref))
-    {
-        if (ep->rec->ExceptionCode == CXX_EXCEPTION)
-        {
-            const cxx_exception_type *type = (void*)ep->rec->ExceptionInformation[2];
-            void *obj = (void*)ep->rec->ExceptionInformation[1];
-
-            if (type && type->destructor) call_dtor(type, type->destructor, obj);
-            HeapFree(GetProcessHeap(), 0, obj);
-        }
-
-        HeapFree(GetProcessHeap(), 0, ep->rec);
-        HeapFree(GetProcessHeap(), 0, ep->ref);
-    }
-}
-
-/*********************************************************************
- * ?__ExceptionPtrCopy@@YAXPAXPBX@Z
- * ?__ExceptionPtrCopy@@YAXPEAXPEBX@Z
- */
-void __cdecl __ExceptionPtrCopy(exception_ptr *ep, const exception_ptr *copy)
-{
-    TRACE("(%p %p)\n", ep, copy);
-
-    /* don't destroy object stored in ep */
-    *ep = *copy;
-    if (ep->ref)
-        InterlockedIncrement(copy->ref);
-}
-
-/*********************************************************************
- * ?__ExceptionPtrAssign@@YAXPAXPBX@Z
- * ?__ExceptionPtrAssign@@YAXPEAXPEBX@Z
- */
-void __cdecl __ExceptionPtrAssign(exception_ptr *ep, const exception_ptr *assign)
-{
-    TRACE("(%p %p)\n", ep, assign);
-
-    /* don't destroy object stored in ep */
-    if (ep->ref)
-        InterlockedDecrement(ep->ref);
-
-    *ep = *assign;
-    if (ep->ref)
-        InterlockedIncrement(ep->ref);
-}
-
-#endif /* _MSVCR_VER >= 100 */
-
-/*********************************************************************
- * ?__ExceptionPtrRethrow@@YAXPBX@Z
- * ?__ExceptionPtrRethrow@@YAXPEBX@Z
- */
-void __cdecl __ExceptionPtrRethrow(const exception_ptr *ep)
-{
-    TRACE("(%p)\n", ep);
-
-    if (!ep->rec)
-    {
-        static const char *exception_msg = "bad exception";
-        exception e;
-
-        exception_ctor(&e, &exception_msg);
-        _CxxThrowException(&e, &exception_exception_type);
-        return;
-    }
-
-    RaiseException(ep->rec->ExceptionCode, ep->rec->ExceptionFlags & (~EH_UNWINDING),
-            ep->rec->NumberParameters, ep->rec->ExceptionInformation);
-}
-
-#if _MSVCR_VER >= 100
-
-#ifdef __i386__
-extern void call_copy_ctor( void *func, void *this, void *src, int has_vbase );
-#else
-static inline void call_copy_ctor( void *func, void *this, void *src, int has_vbase )
-{
-    TRACE( "calling copy ctor %p object %p src %p\n", func, this, src );
-    if (has_vbase)
-        ((void (__cdecl*)(void*, void*, BOOL))func)(this, src, 1);
-    else
-        ((void (__cdecl*)(void*, void*))func)(this, src);
-}
-#endif
-
-/*********************************************************************
- * ?__ExceptionPtrCurrentException@@YAXPAX@Z
- * ?__ExceptionPtrCurrentException@@YAXPEAX@Z
- */
-#ifndef __x86_64__
-void __cdecl __ExceptionPtrCurrentException(exception_ptr *ep)
-{
-    EXCEPTION_RECORD *rec = msvcrt_get_thread_data()->exc_record;
-
-    TRACE("(%p)\n", ep);
-
-    if (!rec)
-    {
-        ep->rec = NULL;
-        ep->ref = NULL;
-        return;
-    }
-
-    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
-    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
-
-    *ep->rec = *rec;
-    *ep->ref = 1;
-
-    if (ep->rec->ExceptionCode == CXX_EXCEPTION)
-    {
-        const cxx_exception_type *et = (void*)ep->rec->ExceptionInformation[2];
-        const cxx_type_info *ti;
-        void **data, *obj;
-
-        ti = et->type_info_table->info[0];
-        data = HeapAlloc(GetProcessHeap(), 0, ti->size);
-
-        obj = (void*)ep->rec->ExceptionInformation[1];
-        if (ti->flags & CLASS_IS_SIMPLE_TYPE)
-        {
-            memcpy(data, obj, ti->size);
-            if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
-        }
-        else if (ti->copy_ctor)
-        {
-            call_copy_ctor(ti->copy_ctor, data, get_this_pointer(&ti->offsets, obj),
-                    ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
-        }
-        else
-            memcpy(data, get_this_pointer(&ti->offsets, obj), ti->size);
-        ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
-    }
-    return;
-}
-#else
-void __cdecl __ExceptionPtrCurrentException(exception_ptr *ep)
-{
-    EXCEPTION_RECORD *rec = msvcrt_get_thread_data()->exc_record;
-
-    TRACE("(%p)\n", ep);
-
-    if (!rec)
-    {
-        ep->rec = NULL;
-        ep->ref = NULL;
-        return;
-    }
-
-    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
-    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
-
-    *ep->rec = *rec;
-    *ep->ref = 1;
-
-    if (ep->rec->ExceptionCode == CXX_EXCEPTION)
-    {
-        const cxx_exception_type *et = (void*)ep->rec->ExceptionInformation[2];
-        const cxx_type_info *ti;
-        void **data, *obj;
-        char *base = RtlPcToFileHeader((void*)et, (void**)&base);
-
-        ti = (const cxx_type_info*)(base + ((const cxx_type_info_table*)(base + et->type_info_table))->info[0]);
-        data = HeapAlloc(GetProcessHeap(), 0, ti->size);
-
-        obj = (void*)ep->rec->ExceptionInformation[1];
-        if (ti->flags & CLASS_IS_SIMPLE_TYPE)
-        {
-            memcpy(data, obj, ti->size);
-            if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
-        }
-        else if (ti->copy_ctor)
-        {
-            call_copy_ctor(base + ti->copy_ctor, data, get_this_pointer(&ti->offsets, obj),
-                    ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
-        }
-        else
-            memcpy(data, get_this_pointer(&ti->offsets, obj), ti->size);
-        ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
-    }
-    return;
-}
-#endif
-
-#endif /* _MSVCR_VER >= 100 */
-
-#if _MSVCR_VER >= 110
-/*********************************************************************
- * ?__ExceptionPtrToBool@@YA_NPBX@Z
- * ?__ExceptionPtrToBool@@YA_NPEBX@Z
- */
-bool __cdecl __ExceptionPtrToBool(exception_ptr *ep)
-{
-    return !!ep->rec;
-}
-#endif
-
-#if _MSVCR_VER >= 100
-
-/*********************************************************************
- * ?__ExceptionPtrCopyException@@YAXPAXPBX1@Z
- * ?__ExceptionPtrCopyException@@YAXPEAXPEBX1@Z
- */
-#ifndef __x86_64__
-void __cdecl __ExceptionPtrCopyException(exception_ptr *ep,
-        exception *object, const cxx_exception_type *type)
-{
-    const cxx_type_info *ti;
-    void **data;
-
-    __ExceptionPtrDestroy(ep);
-
-    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
-    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
-    *ep->ref = 1;
-
-    memset(ep->rec, 0, sizeof(EXCEPTION_RECORD));
-    ep->rec->ExceptionCode = CXX_EXCEPTION;
-    ep->rec->ExceptionFlags = EH_NONCONTINUABLE;
-    ep->rec->NumberParameters = 3;
-    ep->rec->ExceptionInformation[0] = CXX_FRAME_MAGIC_VC6;
-    ep->rec->ExceptionInformation[2] = (ULONG_PTR)type;
-
-    ti = type->type_info_table->info[0];
-    data = HeapAlloc(GetProcessHeap(), 0, ti->size);
-    if (ti->flags & CLASS_IS_SIMPLE_TYPE)
-    {
-        memcpy(data, object, ti->size);
-        if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
-    }
-    else if (ti->copy_ctor)
-    {
-        call_copy_ctor(ti->copy_ctor, data, get_this_pointer(&ti->offsets, object),
-                ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
-    }
-    else
-        memcpy(data, get_this_pointer(&ti->offsets, object), ti->size);
-    ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
-}
-#else
-void __cdecl __ExceptionPtrCopyException(exception_ptr *ep,
-        exception *object, const cxx_exception_type *type)
-{
-    const cxx_type_info *ti;
-    void **data;
-    char *base;
-
-    RtlPcToFileHeader((void*)type, (void**)&base);
-    __ExceptionPtrDestroy(ep);
-
-    ep->rec = HeapAlloc(GetProcessHeap(), 0, sizeof(EXCEPTION_RECORD));
-    ep->ref = HeapAlloc(GetProcessHeap(), 0, sizeof(int));
-    *ep->ref = 1;
-
-    memset(ep->rec, 0, sizeof(EXCEPTION_RECORD));
-    ep->rec->ExceptionCode = CXX_EXCEPTION;
-    ep->rec->ExceptionFlags = EH_NONCONTINUABLE;
-    ep->rec->NumberParameters = 4;
-    ep->rec->ExceptionInformation[0] = CXX_FRAME_MAGIC_VC6;
-    ep->rec->ExceptionInformation[2] = (ULONG_PTR)type;
-    ep->rec->ExceptionInformation[3] = (ULONG_PTR)base;
-
-    ti = (const cxx_type_info*)(base + ((const cxx_type_info_table*)(base + type->type_info_table))->info[0]);
-    data = HeapAlloc(GetProcessHeap(), 0, ti->size);
-    if (ti->flags & CLASS_IS_SIMPLE_TYPE)
-    {
-        memcpy(data, object, ti->size);
-        if (ti->size == sizeof(void *)) *data = get_this_pointer(&ti->offsets, *data);
-    }
-    else if (ti->copy_ctor)
-    {
-        call_copy_ctor(base + ti->copy_ctor, data, get_this_pointer(&ti->offsets, object),
-                ti->flags & CLASS_HAS_VIRTUAL_BASE_CLASS);
-    }
-    else
-        memcpy(data, get_this_pointer(&ti->offsets, object), ti->size);
-    ep->rec->ExceptionInformation[1] = (ULONG_PTR)data;
-}
-#endif
-
-bool __cdecl __ExceptionPtrCompare(const exception_ptr *ep1, const exception_ptr *ep2)
-{
-    return ep1->rec == ep2->rec;
-}
-
-#endif /* _MSVCR_VER >= 100 */
-
-#if _MSVCR_VER >= 80
 void* __cdecl __AdjustPointer(void *obj, const this_ptr_offsets *off)
 {
     return get_this_pointer(off, obj);
 }
+
 #endif
 
 #if _MSVCR_VER >= 140
