@@ -729,13 +729,13 @@ static void test__Gettnames(void)
     {
         size = GetLocaleInfoA(MAKELCID(LANG_ENGLISH, SORT_DEFAULT),
                               time_data[i], buf, sizeof(buf));
-        ok(size, "GetLocaleInfo failed: %x\n", GetLastError());
+        ok(size, "GetLocaleInfo failed: %lx\n", GetLastError());
         ok(!strcmp(ret->str[i], buf), "ret->str[%i] = %s, expected %s\n", i, ret->str[i], buf);
     }
 
     ok(ret->wstr[0] != NULL, "ret->wstr[0] = NULL\n");
     ok(ret->str[42] + strlen(ret->str[42])+1 != (char*)ret->wstr[0],
-            "ret->str[42] = %p len = %d, ret->wstr[0] = %p\n",
+            "ret->str[42] = %p len = %Id, ret->wstr[0] = %p\n",
             ret->str[42], strlen(ret->str[42]), ret->wstr[0]);
     free(ret);
 
@@ -747,7 +747,7 @@ static void test__Gettnames(void)
     {
         size = GetLocaleInfoA(MAKELCID(LANG_GERMAN, SORT_DEFAULT),
                               time_data[i], buf, sizeof(buf));
-        ok(size, "GetLocaleInfo failed: %x\n", GetLastError());
+        ok(size, "GetLocaleInfo failed: %lx\n", GetLastError());
         ok(!strcmp(ret->str[i], buf), "ret->str[%i] = %s, expected %s\n", i, ret->str[i], buf);
     }
     free(ret);
