@@ -23,16 +23,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "msvcrt.h"
+#include "winver.h"
+#include "imagehlp.h"
 
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 
-/* TODO:
- * - document a bit (grammar + functions)
- * - back-port this new code into tools/winedump/msmangle.c
- */
-
 /* How data types qualifiers are stored:
  * M (in the following definitions) is defined for
  * 'A', 'B', 'C' and 'D' as follows
@@ -48,9 +45,11 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
  *      in data fields:
  *              same as for arguments and also the following
  *              ?<M>x   {<M>}x
- *              
+ *
  */
 
+#define UNDNAME_NO_COMPLEX_TYPE (0x8000)
+
 struct array
 {
     unsigned            start;          /* first valid reference in array */
@@ -520,6 +519,28 @@ static BOOL get_qualified_type(struct datatype_t *ct, struct parsed_symbol* sym,
     ct->right = NULL;
     ct->flags = 0;
 
+    /* parse managed handle information */
+    if (sym->current[0] == '$' && sym->current[1] == 'A')
+    {
+        sym->current += 2;
+
+        switch (qualif)
+        {
+        case 'A':
+        case 'B':
+            ref = " %";
+            break;
+        case 'P':
+        case 'Q':
+        case 'R':
+        case 'S':
+            ref = " ^";
+            break;
+        default:
+            return FALSE;
+        }
+    }
+
     if (get_qualifier(sym, &xdt2, &class))
     {
         unsigned            mark = sym->stack.num;
@@ -592,7 +613,8 @@ static char* get_literal_string(struct parsed_symbol* sym)
         if (!((*sym->current >= 'A' && *sym->current <= 'Z') ||
               (*sym->current >= 'a' && *sym->current <= 'z') ||
               (*sym->current >= '0' && *sym->current <= '9') ||
-              *sym->current == '_' || *sym->current == '$')) {
+              *sym->current == '_' || *sym->current == '$' ||
+              *sym->current == '<' || *sym->current == '>')) {
             TRACE("Failed at '%c' in %s\n", *sym->current, debugstr_a(ptr));
             return NULL;
         }
