@@ -17,7 +17,6 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
 
 #include <stdarg.h>
 
@@ -206,7 +205,7 @@ static HRESULT WINAPI bsc_OnDataAvailable(
 
     do
     {
-        hr = IStream_Read(pstgmed->u.pstm, buf, sizeof(buf), &read);
+        hr = IStream_Read(pstgmed->pstm, buf, sizeof(buf), &read);
         if(FAILED(hr))
             break;
 
