@@ -54,7 +54,7 @@ static void test_device(const WCHAR *service_name, const MIB_IF_ROW2 *row)
 
     if (status != STATUS_SUCCESS)
     {
-        skip( "Couldn't open the device (status = %d)\n", status );
+        skip( "Couldn't open the device (status = %ld)\n", status );
         return;
     }
 
@@ -63,8 +63,8 @@ static void test_device(const WCHAR *service_name, const MIB_IF_ROW2 *row)
     iosb.Information = 0xdeadbeef;
     status = NtDeviceIoControlFile( netdev, NULL, NULL, NULL, &iosb,
             IOCTL_NDIS_QUERY_GLOBAL_STATS, &oid, sizeof(oid), &medium, sizeof(medium) );
-    ok(status == STATUS_INVALID_PARAMETER, "got status %#x\n", status);
-    ok(iosb.Status == 0xdeadbeef, "got %#x\n", iosb.Status);
+    ok(status == STATUS_INVALID_PARAMETER, "got status %#lx\n", status);
+    ok(iosb.Status == 0xdeadbeef, "got %#lx\n", iosb.Status);
     ok(iosb.Information == 0xdeadbeef, "got size %#Ix\n", iosb.Information);
 
     oid = OID_GEN_MEDIA_SUPPORTED;
