@@ -18,13 +18,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#ifdef __x86_64__
+#if defined(__x86_64__) && !defined(__arm64ec__)
 
 #include <stdlib.h>
 #include <stdarg.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
@@ -33,9 +31,11 @@
 #include "wine/list.h"
 #include "ntdll_misc.h"
 #include "wine/debug.h"
+#include "ntsyscalls.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(unwind);
 WINE_DECLARE_DEBUG_CHANNEL(seh);
+WINE_DECLARE_DEBUG_CHANNEL(relay);
 WINE_DECLARE_DEBUG_CHANNEL(threadname);
 
 typedef struct _SCOPE_TABLE
@@ -80,6 +80,15 @@ struct MSVCRT_JUMP_BUFFER
     M128A   Xmm15;
 };
 
+
+/*******************************************************************
+ *         syscalls
+ */
+#define SYSCALL_ENTRY(id,name,args) __ASM_SYSCALL_FUNC( id, name )
+ALL_SYSCALLS64
+#undef SYSCALL_ENTRY
+
+
 /***********************************************************************
  * Definitions for Win32 unwind tables
  */
@@ -282,7 +291,7 @@ static NTSTATUS virtual_unwind( ULONG type, DISPATCHER_CONTEXT *dispatch, CONTEX
     {
         struct unwind_builtin_dll_params params = { type, dispatch, context };
 
-        status = NTDLL_UNIX_CALL( unwind_builtin_dll, &params );
+        status = WINE_UNIX_CALL( unix_unwind_builtin_dll, &params );
         if (!status && dispatch->LanguageHandler && !module)
         {
             FIXME( "calling personality routine in system library not supported yet\n" );
@@ -348,36 +357,63 @@ __ASM_GLOBAL_FUNC( RtlCaptureContext,
                    "fxsave 0x100(%rcx)\n\t"         /* context->FltSave */
                    "ret" );
 
-static DWORD __cdecl nested_exception_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *frame,
-                                               CONTEXT *context, EXCEPTION_REGISTRATION_RECORD **dispatcher )
+DWORD __cdecl nested_exception_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *frame,
+                                        CONTEXT *context, EXCEPTION_REGISTRATION_RECORD **dispatcher )
 {
     if (!(rec->ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND)))
-        rec->ExceptionFlags |= EH_NESTED_CALL;
+        return ExceptionNestedException;
 
     return ExceptionContinueSearch;
 }
 
+/***********************************************************************
+ *		exception_handler_call_wrapper
+ */
+#ifdef __WINE_PE_BUILD
+DWORD WINAPI exception_handler_call_wrapper( EXCEPTION_RECORD *rec, void *frame,
+                                      CONTEXT *context, DISPATCHER_CONTEXT *dispatch );
+
+C_ASSERT( offsetof(DISPATCHER_CONTEXT, LanguageHandler) == 0x30 );
+
+__ASM_GLOBAL_FUNC( exception_handler_call_wrapper,
+                   ".seh_endprologue\n\t"
+                   "subq $0x28, %rsp\n\t"
+                   ".seh_stackalloc 0x28\n\t"
+                   "callq *0x30(%r9)\n\t"       /* dispatch->LanguageHandler */
+                   "nop\n\t"                    /* avoid epilogue so handler is called */
+                   "addq $0x28, %rsp\n\t"
+                   "ret\n\t"
+                   ".seh_handler " __ASM_NAME("nested_exception_handler") ", @except\n\t" )
+#else
+static DWORD exception_handler_call_wrapper( EXCEPTION_RECORD *rec, void *frame,
+                                             CONTEXT *context, DISPATCHER_CONTEXT *dispatch )
+{
+    EXCEPTION_REGISTRATION_RECORD wrapper_frame;
+    DWORD res;
+
+    wrapper_frame.Handler = nested_exception_handler;
+    __wine_push_frame( &wrapper_frame );
+    res = dispatch->LanguageHandler( rec, (void *)dispatch->EstablisherFrame, context, dispatch );
+    __wine_pop_frame( &wrapper_frame );
+    return res;
+}
+#endif
+
 /**********************************************************************
  *           call_handler
  *
  * Call a single exception handler.
- * FIXME: Handle nested exceptions.
  */
 static DWORD call_handler( EXCEPTION_RECORD *rec, CONTEXT *context, DISPATCHER_CONTEXT *dispatch )
 {
-    EXCEPTION_REGISTRATION_RECORD frame;
     DWORD res;
 
-    frame.Handler = nested_exception_handler;
-    __wine_push_frame( &frame );
-
     TRACE_(seh)( "calling handler %p (rec=%p, frame=%p context=%p, dispatch=%p)\n",
                  dispatch->LanguageHandler, rec, (void *)dispatch->EstablisherFrame, dispatch->ContextRecord, dispatch );
-    res = dispatch->LanguageHandler( rec, (void *)dispatch->EstablisherFrame, context, dispatch );
+    res = exception_handler_call_wrapper( rec, (void *)dispatch->EstablisherFrame, context, dispatch );
     TRACE_(seh)( "handler at %p returned %lu\n", dispatch->LanguageHandler, res );
 
     rec->ExceptionFlags &= EH_NONCONTINUABLE;
-    __wine_pop_frame( &frame );
     return res;
 }
 
@@ -448,7 +484,8 @@ static NTSTATUS call_stack_handlers( EXCEPTION_RECORD *rec, CONTEXT *orig_contex
             case ExceptionContinueSearch:
                 break;
             case ExceptionNestedException:
-                FIXME_(seh)( "nested exception\n" );
+                rec->ExceptionFlags |= EH_NESTED_CALL;
+                TRACE_(seh)( "nested exception\n" );
                 break;
             case ExceptionCollidedUnwind: {
                 ULONG64 frame;
@@ -478,7 +515,8 @@ static NTSTATUS call_stack_handlers( EXCEPTION_RECORD *rec, CONTEXT *orig_contex
             case ExceptionContinueSearch:
                 break;
             case ExceptionNestedException:
-                FIXME_(seh)( "nested exception\n" );
+                rec->ExceptionFlags |= EH_NESTED_CALL;
+                TRACE_(seh)( "nested exception\n" );
                 break;
             case ExceptionCollidedUnwind: {
                 ULONG64 frame;
@@ -526,7 +564,7 @@ NTSTATUS WINAPI dispatch_exception( EXCEPTION_RECORD *rec, CONTEXT *context )
     }
     else if (rec->ExceptionCode == EXCEPTION_WINE_NAME_THREAD && rec->ExceptionInformation[0] == 0x1000)
     {
-        if ((DWORD)rec->ExceptionInformation[2] == -1)
+        if ((DWORD)rec->ExceptionInformation[2] == -1 || (DWORD)rec->ExceptionInformation[2] == GetCurrentThreadId())
             WARN_(threadname)( "Thread renamed to %s\n", debugstr_a((char *)rec->ExceptionInformation[1]) );
         else
             WARN_(threadname)( "Thread ID %04lx renamed to %s\n", (DWORD)rec->ExceptionInformation[2],
@@ -559,10 +597,6 @@ NTSTATUS WINAPI dispatch_exception( EXCEPTION_RECORD *rec, CONTEXT *context )
                      context->R12, context->R13, context->R14, context->R15 );
     }
 
-    /* Legends of Runeterra depends on having SegDs == SegSs in an exception
-     * handler. */
-    context->SegDs = context->SegSs;
-
     if (call_vectored_handlers( rec, context ) == EXCEPTION_CONTINUE_EXECUTION)
         NtContinue( context, FALSE );
 
@@ -574,94 +608,90 @@ NTSTATUS WINAPI dispatch_exception( EXCEPTION_RECORD *rec, CONTEXT *context )
 }
 
 
-NTSTATUS WINAPI dispatch_wow_exception( EXCEPTION_RECORD *rec_ptr, CONTEXT *context_ptr )
-{
-    char buffer[sizeof(CONTEXT) + sizeof(CONTEXT_EX) + sizeof(XSTATE) + 128];
-    CONTEXT *context;
-    CONTEXT_EX *context_ex;
-    EXCEPTION_RECORD rec = *rec_ptr;
-
-    RtlInitializeExtendedContext( buffer, context_ptr->ContextFlags, &context_ex );
-    context = RtlLocateLegacyContext( context_ex, NULL );
-    RtlCopyContext( context, context_ptr->ContextFlags, context_ptr );
-    pWow64PrepareForException( &rec, context );
-    return dispatch_exception( &rec, context );
-}
-
-
 /*******************************************************************
  *		KiUserExceptionDispatcher (NTDLL.@)
  */
 __ASM_GLOBAL_FUNC( KiUserExceptionDispatcher,
-                  "mov 0x98(%rsp),%rcx\n\t" /* context->Rsp */
-                  "movw %cs,%ax\n\t"
-                  "cmpw %ax,0x38(%rsp)\n\t" /* context->SegCs */
-                  "je 1f\n\t"
-                  "mov %rsp,%rdx\n\t" /* context */
-                  "lea 0x4f0(%rsp),%rcx\n\t" /* rec */
-                  "movq %r14,%rsp\n\t"  /* switch to 64-bit stack */
-                  "call " __ASM_NAME("dispatch_wow_exception") "\n\t"
-                  "int3\n"
-                  "1:\tmov 0xf8(%rsp),%rdx\n\t" /* context->Rip */
-                  "mov %rdx,-0x8(%rcx)\n\t"
-                  "mov %rbp,-0x10(%rcx)\n\t"
-                  "mov %rdi,-0x18(%rcx)\n\t"
-                  "mov %rsi,-0x20(%rcx)\n\t"
-                  "lea -0x20(%rcx),%rbp\n\t"
-                  "mov %rsp,%rdx\n\t" /* context */
-                  "lea 0x4f0(%rsp),%rcx\n\t" /* rec */
-                  __ASM_SEH(".seh_pushreg %rbp\n\t")
-                  __ASM_SEH(".seh_pushreg %rdi\n\t")
-                  __ASM_SEH(".seh_pushreg %rsi\n\t")
-                  __ASM_SEH(".seh_setframe %rbp,0\n\t")
-                  __ASM_SEH(".seh_endprologue\n\t")
-
-                  __ASM_CFI(".cfi_signal_frame\n\t")
-                  __ASM_CFI(".cfi_adjust_cfa_offset 0x20\n\t")
-                  __ASM_CFI(".cfi_def_cfa %rbp,0x20\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rip,0x18\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rbp,0x10\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rdi,0x8\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rsi,0\n\t")
+                   __ASM_SEH(".seh_pushframe\n\t")
+                   __ASM_SEH(".seh_stackalloc 0x590\n\t")
+                   __ASM_SEH(".seh_savereg %rbx,0x90\n\t")
+                   __ASM_SEH(".seh_savereg %rbp,0xa0\n\t")
+                   __ASM_SEH(".seh_savereg %rsi,0xa8\n\t")
+                   __ASM_SEH(".seh_savereg %rdi,0xb0\n\t")
+                   __ASM_SEH(".seh_savereg %r12,0xd8\n\t")
+                   __ASM_SEH(".seh_savereg %r13,0xe0\n\t")
+                   __ASM_SEH(".seh_savereg %r14,0xe8\n\t")
+                   __ASM_SEH(".seh_savereg %r15,0xf0\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
+                   __ASM_CFI(".cfi_signal_frame\n\t")
+                   __ASM_CFI(".cfi_def_cfa_offset 0\n\t")
+                   __ASM_CFI(".cfi_offset %rbx,0x90\n\t")
+                   __ASM_CFI(".cfi_offset %rbp,0xa0\n\t")
+                   __ASM_CFI(".cfi_offset %rsi,0xa8\n\t")
+                   __ASM_CFI(".cfi_offset %rdi,0xb0\n\t")
+                   __ASM_CFI(".cfi_offset %r12,0xd8\n\t")
+                   __ASM_CFI(".cfi_offset %r13,0xe0\n\t")
+                   __ASM_CFI(".cfi_offset %r14,0xe8\n\t")
+                   __ASM_CFI(".cfi_offset %r15,0xf0\n\t")
+                   __ASM_CFI(".cfi_offset %rip,0x590\n\t")
+                   __ASM_CFI(".cfi_offset %rsp,0x5a8\n\t")
+                   "cld\n\t"
+                   /* some anticheats need this exact instruction here */
+                   "mov " __ASM_NAME("pWow64PrepareForException") "(%rip),%rax\n\t"
+                   "test %rax,%rax\n\t"
+                   "jz 1f\n\t"
+                   "mov %rsp,%rdx\n\t"           /* context */
+                   "lea 0x4f0(%rsp),%rcx\n\t"    /* rec */
+                   "call *%rax\n"
+                   "1:\tmov %rsp,%rdx\n\t" /* context */
+                   "lea 0x4f0(%rsp),%rcx\n\t" /* rec */
                    "call " __ASM_NAME("dispatch_exception") "\n\t"
-                  "int3")
+                   "int3" )
 
 
 /*******************************************************************
  *		KiUserApcDispatcher (NTDLL.@)
  */
-void WINAPI dispatch_apc( CONTEXT *context, ULONG_PTR arg1, ULONG_PTR arg2, ULONG_PTR arg3,
-                          void (CALLBACK *func)(ULONG_PTR,ULONG_PTR,ULONG_PTR,CONTEXT*) )
-{
-    func( arg1, arg2, arg3, context );
-    NtContinue( context, TRUE );
-}
-
 __ASM_GLOBAL_FUNC( KiUserApcDispatcher,
-                  "addq $0x8,%rsp\n\t"
-                  "mov 0x98(%rcx),%r10\n\t" /* context->Rsp */
-                  "mov 0xf8(%rcx),%r11\n\t" /* context->Rip */
-                  "mov %r11,-0x8(%r10)\n\t"
-                  "mov %rbp,-0x10(%r10)\n\t"
-                  "lea -0x10(%r10),%rbp\n\t"
-                  __ASM_SEH(".seh_pushreg %rbp\n\t")
-                  __ASM_SEH(".seh_setframe %rbp,0\n\t")
-                  __ASM_SEH(".seh_endprologue\n\t")
-                  __ASM_CFI(".cfi_signal_frame\n\t")
-                  __ASM_CFI(".cfi_adjust_cfa_offset 0x10\n\t")
-                  __ASM_CFI(".cfi_def_cfa %rbp,0x10\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rip,0x8\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rbp,0\n\t")
-                   "call " __ASM_NAME("dispatch_apc") "\n\t"
-                   "int3")
+                   __ASM_SEH(".seh_pushframe\n\t")
+                   __ASM_SEH(".seh_stackalloc 0x4d0\n\t")  /* sizeof(CONTEXT) */
+                   __ASM_SEH(".seh_savereg %rbx,0x90\n\t")
+                   __ASM_SEH(".seh_savereg %rbp,0xa0\n\t")
+                   __ASM_SEH(".seh_savereg %rsi,0xa8\n\t")
+                   __ASM_SEH(".seh_savereg %rdi,0xb0\n\t")
+                   __ASM_SEH(".seh_savereg %r12,0xd8\n\t")
+                   __ASM_SEH(".seh_savereg %r13,0xe0\n\t")
+                   __ASM_SEH(".seh_savereg %r14,0xe8\n\t")
+                   __ASM_SEH(".seh_savereg %r15,0xf0\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
+                   __ASM_CFI(".cfi_signal_frame\n\t")
+                   __ASM_CFI(".cfi_def_cfa_offset 0\n\t")
+                   __ASM_CFI(".cfi_offset %rbx,0x90\n\t")
+                   __ASM_CFI(".cfi_offset %rbp,0xa0\n\t")
+                   __ASM_CFI(".cfi_offset %rsi,0xa8\n\t")
+                   __ASM_CFI(".cfi_offset %rdi,0xb0\n\t")
+                   __ASM_CFI(".cfi_offset %r12,0xd8\n\t")
+                   __ASM_CFI(".cfi_offset %r13,0xe0\n\t")
+                   __ASM_CFI(".cfi_offset %r14,0xe8\n\t")
+                   __ASM_CFI(".cfi_offset %r15,0xf0\n\t")
+                   __ASM_CFI(".cfi_offset %rip,0x4d0\n\t")
+                   __ASM_CFI(".cfi_offset %rsp,0x4e8\n\t")
+                   "movq 0x00(%rsp),%rcx\n\t"  /* context->P1Home = arg1 */
+                   "movq 0x08(%rsp),%rdx\n\t"  /* context->P2Home = arg2 */
+                   "movq 0x10(%rsp),%r8\n\t"   /* context->P3Home = arg3 */
+                   "movq 0x18(%rsp),%rax\n\t"  /* context->P4Home = func */
+                   "movq %rsp,%r9\n\t"         /* context */
+                   "callq *%rax\n\t"
+                   "movq %rsp,%rcx\n\t"        /* context */
+                   "movl $1,%edx\n\t"          /* alertable */
+                   "call " __ASM_NAME("NtContinue") "\n\t"
+                   "int3" )
 
 
 /*******************************************************************
  *		KiUserCallbackDispatcher (NTDLL.@)
- *
- * FIXME: not binary compatible
  */
-void WINAPI KiUserCallbackDispatcher( ULONG id, void *args, ULONG len )
+void WINAPI dispatch_callback( void *args, ULONG len, ULONG id )
 {
     NTSTATUS status;
 
@@ -679,6 +709,27 @@ void WINAPI KiUserCallbackDispatcher( ULONG id, void *args, ULONG len )
 
     RtlRaiseStatus( status );
 }
+__ASM_GLOBAL_FUNC( KiUserCallbackDispatcher,
+                   __ASM_SEH(".seh_pushframe\n\t")
+                   __ASM_SEH(".seh_stackalloc 0x30\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
+                   __ASM_CFI(".cfi_signal_frame\n\t")
+                   __ASM_CFI(".cfi_def_cfa_offset 0\n\t")
+                   __ASM_CFI(".cfi_offset %rip,0x30\n\t")
+                   __ASM_CFI(".cfi_offset %rsp,0x48\n\t")
+                   "movq 0x20(%rsp),%rcx\n\t"  /* args */
+                   "movl 0x28(%rsp),%edx\n\t"  /* len */
+                   "movl 0x2c(%rsp),%r8d\n\t"  /* id */
+                   "call " __ASM_NAME("dispatch_callback") )
+
+
+/**************************************************************************
+ *              RtlIsEcCode (NTDLL.@)
+ */
+BOOLEAN WINAPI RtlIsEcCode( const void *ptr )
+{
+    return FALSE;
+}
 
 
 static ULONG64 get_int_reg( CONTEXT *context, int reg )
@@ -689,14 +740,14 @@ static ULONG64 get_int_reg( CONTEXT *context, int reg )
 static void set_int_reg( CONTEXT *context, KNONVOLATILE_CONTEXT_POINTERS *ctx_ptr, int reg, ULONG64 *val )
 {
     *(&context->Rax + reg) = *val;
-    if (ctx_ptr) ctx_ptr->u2.IntegerContext[reg] = val;
+    if (ctx_ptr) ctx_ptr->IntegerContext[reg] = val;
 }
 
 static void set_float_reg( CONTEXT *context, KNONVOLATILE_CONTEXT_POINTERS *ctx_ptr, int reg, M128A *val )
 {
     /* Use a memcpy() to avoid issues if val is misaligned. */
-    memcpy(&context->u.s.Xmm0 + reg, val, sizeof(*val));
-    if (ctx_ptr) ctx_ptr->u.FloatingContext[reg] = val;
+    memcpy(&context->Xmm0 + reg, val, sizeof(*val));
+    if (ctx_ptr) ctx_ptr->FloatingContext[reg] = val;
 }
 
 static int get_opcode_size( struct opcode op )
@@ -993,6 +1044,8 @@ PVOID WINAPI RtlVirtualUnwind( ULONG type, ULONG64 base, ULONG64 pc,
 struct unwind_exception_frame
 {
     EXCEPTION_REGISTRATION_RECORD frame;
+    char dummy[0x10]; /* Layout 'dispatch' accessed from unwind_exception_handler() so it is above register
+                       * save space when .seh handler is used. */
     DISPATCHER_CONTEXT *dispatch;
 };
 
@@ -1001,8 +1054,8 @@ struct unwind_exception_frame
  *
  * Handler for exceptions happening while calling an unwind handler.
  */
-static DWORD __cdecl unwind_exception_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *frame,
-                                               CONTEXT *context, EXCEPTION_REGISTRATION_RECORD **dispatcher )
+DWORD __cdecl unwind_exception_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *frame,
+                                        CONTEXT *context, EXCEPTION_REGISTRATION_RECORD **dispatcher )
 {
     struct unwind_exception_frame *unwind_frame = (struct unwind_exception_frame *)frame;
     DISPATCHER_CONTEXT *dispatch = (DISPATCHER_CONTEXT *)dispatcher;
@@ -1021,27 +1074,57 @@ static DWORD __cdecl unwind_exception_handler( EXCEPTION_RECORD *rec, EXCEPTION_
     return ExceptionCollidedUnwind;
 }
 
+/***********************************************************************
+ *		unwind_handler_call_wrapper
+ */
+#ifdef __WINE_PE_BUILD
+DWORD WINAPI unwind_handler_call_wrapper( EXCEPTION_RECORD *rec, void *frame,
+                                   CONTEXT *context, DISPATCHER_CONTEXT *dispatch );
+
+C_ASSERT( sizeof(struct unwind_exception_frame) == 0x28 );
+C_ASSERT( offsetof(struct unwind_exception_frame, dispatch) == 0x20 );
+C_ASSERT( offsetof(DISPATCHER_CONTEXT, LanguageHandler) == 0x30 );
+
+__ASM_GLOBAL_FUNC( unwind_handler_call_wrapper,
+                   ".seh_endprologue\n\t"
+                   "subq $0x28,%rsp\n\t"
+                   ".seh_stackalloc 0x28\n\t"
+                   "movq %r9,0x20(%rsp)\n\t"   /* unwind_exception_frame->dispatch */
+                   "callq *0x30(%r9)\n\t"      /* dispatch->LanguageHandler */
+                   "nop\n\t"                   /* avoid epilogue so handler is called */
+                   "addq $0x28, %rsp\n\t"
+                   "ret\n\t"
+                   ".seh_handler " __ASM_NAME("unwind_exception_handler") ", @except, @unwind\n\t" )
+#else
+static DWORD unwind_handler_call_wrapper( EXCEPTION_RECORD *rec, void *frame,
+                                          CONTEXT *context, DISPATCHER_CONTEXT *dispatch )
+{
+    struct unwind_exception_frame wrapper_frame;
+    DWORD res;
+
+    wrapper_frame.frame.Handler = unwind_exception_handler;
+    wrapper_frame.dispatch = dispatch;
+    __wine_push_frame( &wrapper_frame.frame );
+    res = dispatch->LanguageHandler( rec, (void *)dispatch->EstablisherFrame, dispatch->ContextRecord, dispatch );
+    __wine_pop_frame( &wrapper_frame.frame );
+    return res;
+}
+#endif
+
 /**********************************************************************
  *           call_unwind_handler
  *
  * Call a single unwind handler.
  */
-static DWORD call_unwind_handler( EXCEPTION_RECORD *rec, DISPATCHER_CONTEXT *dispatch )
+DWORD call_unwind_handler( EXCEPTION_RECORD *rec, DISPATCHER_CONTEXT *dispatch )
 {
-    struct unwind_exception_frame frame;
     DWORD res;
 
-    frame.frame.Handler = unwind_exception_handler;
-    frame.dispatch = dispatch;
-    __wine_push_frame( &frame.frame );
-
     TRACE( "calling handler %p (rec=%p, frame=%p context=%p, dispatch=%p)\n",
            dispatch->LanguageHandler, rec, (void *)dispatch->EstablisherFrame, dispatch->ContextRecord, dispatch );
-    res = dispatch->LanguageHandler( rec, (void *)dispatch->EstablisherFrame, dispatch->ContextRecord, dispatch );
+    res = unwind_handler_call_wrapper( rec, (void *)dispatch->EstablisherFrame, dispatch->ContextRecord, dispatch );
     TRACE( "handler %p returned %lx\n", dispatch->LanguageHandler, res );
 
-    __wine_pop_frame( &frame.frame );
-
     switch (res)
     {
     case ExceptionContinueSearch:
@@ -1218,29 +1301,29 @@ void CDECL RtlRestoreContext( CONTEXT *context, EXCEPTION_RECORD *rec )
     if (rec && rec->ExceptionCode == STATUS_LONGJUMP && rec->NumberParameters >= 1)
     {
         struct MSVCRT_JUMP_BUFFER *jmp = (struct MSVCRT_JUMP_BUFFER *)rec->ExceptionInformation[0];
-        context->Rbx       = jmp->Rbx;
-        context->Rsp       = jmp->Rsp;
-        context->Rbp       = jmp->Rbp;
-        context->Rsi       = jmp->Rsi;
-        context->Rdi       = jmp->Rdi;
-        context->R12       = jmp->R12;
-        context->R13       = jmp->R13;
-        context->R14       = jmp->R14;
-        context->R15       = jmp->R15;
-        context->Rip       = jmp->Rip;
-        context->u.s.Xmm6  = jmp->Xmm6;
-        context->u.s.Xmm7  = jmp->Xmm7;
-        context->u.s.Xmm8  = jmp->Xmm8;
-        context->u.s.Xmm9  = jmp->Xmm9;
-        context->u.s.Xmm10 = jmp->Xmm10;
-        context->u.s.Xmm11 = jmp->Xmm11;
-        context->u.s.Xmm12 = jmp->Xmm12;
-        context->u.s.Xmm13 = jmp->Xmm13;
-        context->u.s.Xmm14 = jmp->Xmm14;
-        context->u.s.Xmm15 = jmp->Xmm15;
-        context->MxCsr     = jmp->MxCsr;
-        context->u.FltSave.MxCsr = jmp->MxCsr;
-        context->u.FltSave.ControlWord = jmp->FpCsr;
+        context->Rbx   = jmp->Rbx;
+        context->Rsp   = jmp->Rsp;
+        context->Rbp   = jmp->Rbp;
+        context->Rsi   = jmp->Rsi;
+        context->Rdi   = jmp->Rdi;
+        context->R12   = jmp->R12;
+        context->R13   = jmp->R13;
+        context->R14   = jmp->R14;
+        context->R15   = jmp->R15;
+        context->Rip   = jmp->Rip;
+        context->Xmm6  = jmp->Xmm6;
+        context->Xmm7  = jmp->Xmm7;
+        context->Xmm8  = jmp->Xmm8;
+        context->Xmm9  = jmp->Xmm9;
+        context->Xmm10 = jmp->Xmm10;
+        context->Xmm11 = jmp->Xmm11;
+        context->Xmm12 = jmp->Xmm12;
+        context->Xmm13 = jmp->Xmm13;
+        context->Xmm14 = jmp->Xmm14;
+        context->Xmm15 = jmp->Xmm15;
+        context->MxCsr = jmp->MxCsr;
+        context->FltSave.MxCsr = jmp->MxCsr;
+        context->FltSave.ControlWord = jmp->FpCsr;
     }
     else if (rec && rec->ExceptionCode == STATUS_UNWIND_CONSOLIDATE && rec->NumberParameters >= 1)
     {
@@ -1580,9 +1663,41 @@ USHORT WINAPI RtlCaptureStackBackTrace( ULONG skip, ULONG count, PVOID *buffer,
 }
 
 
+/***********************************************************************
+ *           RtlUserThreadStart (NTDLL.@)
+ */
+#ifdef __WINE_PE_BUILD
+__ASM_GLOBAL_FUNC( RtlUserThreadStart,
+                   "subq $0x28,%rsp\n\t"
+                   ".seh_stackalloc 0x28\n\t"
+                   ".seh_endprologue\n\t"
+                   "movq %rdx,%r8\n\t"
+                   "movq %rcx,%rdx\n\t"
+                   "xorq %rcx,%rcx\n\t"
+                   "movq " __ASM_NAME( "pBaseThreadInitThunk" ) "(%rip),%r9\n\t"
+                   "call *%r9\n\t"
+                   "int3\n\t"
+                   ".seh_handler " __ASM_NAME("call_unhandled_exception_handler") ", @except" )
+#else
+void WINAPI RtlUserThreadStart( PRTL_THREAD_START_ROUTINE entry, void *arg )
+{
+    __TRY
+    {
+        pBaseThreadInitThunk( 0, (LPTHREAD_START_ROUTINE)entry, arg );
+    }
+    __EXCEPT(call_unhandled_exception_filter)
+    {
+        NtTerminateProcess( GetCurrentProcess(), GetExceptionCode() );
+    }
+    __ENDTRY
+}
+#endif
+
+
 /***********************************************************************
  *           signal_start_thread
  */
+extern void CDECL DECLSPEC_NORETURN signal_start_thread( CONTEXT *ctx );
 __ASM_GLOBAL_FUNC( signal_start_thread,
                    "movq %rcx,%rbx\n\t"        /* context */
                    /* clear the thread stack */
@@ -1600,17 +1715,28 @@ __ASM_GLOBAL_FUNC( signal_start_thread,
                    "call " __ASM_NAME("NtContinue") )
 
 
+/******************************************************************
+ *		LdrInitializeThunk (NTDLL.@)
+ */
+void WINAPI LdrInitializeThunk( CONTEXT *context, ULONG_PTR unk2, ULONG_PTR unk3, ULONG_PTR unk4 )
+{
+    loader_init( context, (void **)&context->Rcx );
+    TRACE_(relay)( "\1Starting thread proc %p (arg=%p)\n", (void *)context->Rcx, (void *)context->Rdx );
+    signal_start_thread( context );
+}
+
+
 /**********************************************************************
  *		DbgBreakPoint   (NTDLL.@)
  */
-__ASM_STDCALL_FUNC( DbgBreakPoint, 0, "int $3; ret"
+__ASM_GLOBAL_FUNC( DbgBreakPoint, "int $3; ret"
                     "\n\tnop; nop; nop; nop; nop; nop; nop; nop"
                     "\n\tnop; nop; nop; nop; nop; nop" );
 
 /**********************************************************************
  *		DbgUserBreakPoint   (NTDLL.@)
  */
-__ASM_STDCALL_FUNC( DbgUserBreakPoint, 0, "int $3; ret"
+__ASM_GLOBAL_FUNC( DbgUserBreakPoint, "int $3; ret"
                     "\n\tnop; nop; nop; nop; nop; nop; nop; nop"
                     "\n\tnop; nop; nop; nop; nop; nop" );
 
