@@ -23,7 +23,6 @@
 #include <limits.h>
 #include <sys/types.h>
 
-#define NONAMELESSUNION
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "winternl.h"
@@ -157,6 +156,16 @@ int __cdecl __wine_dbg_header( enum __wine_debug_class cls, struct __wine_debug_
     return info->out_pos;
 }
 
+/***********************************************************************
+ *		__wine_dbg_write  (NTDLL.@)
+ */
+int WINAPI __wine_dbg_write( const char *str, unsigned int len )
+{
+    struct wine_dbg_write_params params = { str, len };
+
+    return WINE_UNIX_CALL( unix_wine_dbg_write, &params );
+}
+
 /***********************************************************************
  *		__wine_dbg_output  (NTDLL.@)
  */
@@ -237,64 +246,6 @@ void WINAPI RtlExitUserThread( ULONG status )
 }
 
 
-/***********************************************************************
- *           RtlUserThreadStart (NTDLL.@)
- */
-#ifdef __i386__
-__ASM_STDCALL_FUNC( RtlUserThreadStart, 8,
-                   "movl %ebx,8(%esp)\n\t"  /* arg */
-                   "movl %eax,4(%esp)\n\t"  /* entry */
-                   "jmp " __ASM_NAME("call_thread_func") )
-
-/* wrapper to call BaseThreadInitThunk */
-extern void DECLSPEC_NORETURN call_thread_func_wrapper( void *thunk, PRTL_THREAD_START_ROUTINE entry, void *arg );
-__ASM_GLOBAL_FUNC( call_thread_func_wrapper,
-                  "pushl %ebp\n\t"
-                  __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
-                  __ASM_CFI(".cfi_rel_offset %ebp,0\n\t")
-                  "movl %esp,%ebp\n\t"
-                  __ASM_CFI(".cfi_def_cfa_register %ebp\n\t")
-                   "subl $4,%esp\n\t"
-                   "andl $~0xf,%esp\n\t"
-                   "xorl %ecx,%ecx\n\t"
-                   "movl 12(%ebp),%edx\n\t"
-                   "movl 16(%ebp),%eax\n\t"
-                   "movl %eax,(%esp)\n\t"
-                   "call *8(%ebp)" )
-
-void DECLSPEC_HIDDEN call_thread_func( PRTL_THREAD_START_ROUTINE entry, void *arg )
-{
-    __TRY
-    {
-        TRACE_(relay)( "\1Starting thread proc %p (arg=%p)\n", entry, arg );
-        call_thread_func_wrapper( pBaseThreadInitThunk, entry, arg );
-    }
-    __EXCEPT(call_unhandled_exception_filter)
-    {
-        NtTerminateProcess( GetCurrentProcess(), GetExceptionCode() );
-    }
-    __ENDTRY
-}
-
-#else  /* __i386__ */
-
-void WINAPI RtlUserThreadStart( PRTL_THREAD_START_ROUTINE entry, void *arg )
-{
-    __TRY
-    {
-        TRACE_(relay)( "\1Starting thread proc %p (arg=%p)\n", entry, arg );
-        pBaseThreadInitThunk( 0, (LPTHREAD_START_ROUTINE)entry, arg );
-    }
-    __EXCEPT(call_unhandled_exception_filter)
-    {
-        NtTerminateProcess( GetCurrentProcess(), GetExceptionCode() );
-    }
-    __ENDTRY
-}
-
-#endif  /* __i386__ */
-
-
 /***********************************************************************
  *              RtlCreateUserThread   (NTDLL.@)
  */
@@ -454,6 +405,24 @@ TEB_ACTIVE_FRAME * WINAPI RtlGetFrame(void)
 }
 
 
+/******************************************************************************
+ *              RtlIsCurrentThread  (NTDLL.@)
+ */
+BOOLEAN WINAPI RtlIsCurrentThread( HANDLE handle )
+{
+    return handle == NtCurrentThread() || !NtCompareObjects( handle, NtCurrentThread() );
+}
+
+
+/***********************************************************************
+ *           _errno  (NTDLL.@)
+ */
+int * CDECL _errno(void)
+{
+    return (int *)&NtCurrentTeb()->TlsSlots[NTDLL_TLS_ERRNO];
+}
+
+
 /***********************************************************************
  * Fibers
  ***********************************************************************/
