@@ -387,8 +387,8 @@ static void fill_caps(__u32 pixelformat, __u32 width, __u32 height,
     caps->media_type.cbFormat = sizeof(VIDEOINFOHEADER);
     /* We reallocate the caps array, so pbFormat has to be set after all caps
      * have been enumerated. */
-    caps->config.MaxFrameInterval = 10000000 * max_fps;
-    caps->config.MinFrameInterval = 10000000 * min_fps;
+    caps->config.MaxFrameInterval = 10000000 / max_fps;
+    caps->config.MinFrameInterval = 10000000 / min_fps;
     caps->config.MaxOutputSize.cx = width;
     caps->config.MaxOutputSize.cy = height;
     caps->config.MinOutputSize.cx = width;
@@ -528,8 +528,8 @@ static NTSTATUS v4l_device_create( void *args )
             else if (frmival.type == V4L2_FRMIVAL_TYPE_STEPWISE
                     || frmival.type == V4L2_FRMIVAL_TYPE_CONTINUOUS)
             {
-                max_fps = frmival.stepwise.max.denominator / frmival.stepwise.max.numerator;
-                min_fps = frmival.stepwise.min.denominator / frmival.stepwise.min.numerator;
+                min_fps = frmival.stepwise.max.denominator / frmival.stepwise.max.numerator;
+                max_fps = frmival.stepwise.min.denominator / frmival.stepwise.min.numerator;
             }
         }
         else
@@ -558,8 +558,8 @@ static NTSTATUS v4l_device_create( void *args )
     }
 
     TRACE("Format: %d bpp - %dx%d.\n", device->current_caps->video_info.bmiHeader.biBitCount,
-            device->current_caps->video_info.bmiHeader.biWidth,
-            device->current_caps->video_info.bmiHeader.biHeight);
+            (int)device->current_caps->video_info.bmiHeader.biWidth,
+            (int)device->current_caps->video_info.bmiHeader.biHeight);
 
     *params->device = (ULONG_PTR)device;
     return S_OK;
@@ -593,6 +593,8 @@ const unixlib_entry_t __wine_unix_call_funcs[] =
     v4l_device_read_frame,
 };
 
+C_ASSERT( ARRAYSIZE(__wine_unix_call_funcs) == unix_funcs_count );
+
 #ifdef _WIN64
 
 typedef ULONG PTR32;
@@ -857,6 +859,8 @@ const unixlib_entry_t __wine_unix_call_wow64_funcs[] =
     wow64_v4l_device_read_frame,
 };
 
+C_ASSERT( ARRAYSIZE(__wine_unix_call_wow64_funcs) == unix_funcs_count );
+
 #endif /* _WIN64 */
 
 #endif /* HAVE_LINUX_VIDEODEV2_H */
