@@ -184,7 +184,7 @@ static DWORD calc_arg_size(MIDL_STUB_MESSAGE *pStubMsg, PFORMAT_STRING pFormat)
         break;
     case FC_BOGUS_ARRAY:
         pFormat = ComputeConformance(pStubMsg, NULL, pFormat + 4, *(const WORD*)&pFormat[2]);
-        TRACE("conformance = %ld\n", pStubMsg->MaxCount);
+        TRACE("conformance = %Id\n", pStubMsg->MaxCount);
         pFormat = ComputeVariance(pStubMsg, NULL, pFormat, pStubMsg->MaxCount);
         size = ComplexStructSize(pStubMsg, pFormat);
         size *= pStubMsg->MaxCount;
@@ -849,8 +849,8 @@ static LONG_PTR do_ndr_client_call( const MIDL_STUB_DESC *stub_desc, const PFORM
     return retval;
 }
 
-LONG_PTR CDECL DECLSPEC_HIDDEN ndr_client_call( PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pFormat,
-                                                void **stack_top, void **fpu_stack )
+LONG_PTR CDECL ndr_client_call( PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pFormat,
+                                void **stack_top, void **fpu_stack )
 {
     /* pointer to start of stack where arguments start */
     MIDL_STUB_MESSAGE stubMsg;
@@ -873,7 +873,7 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr_client_call( PMIDL_STUB_DESC pStubDesc, PFORM
 
     TRACE("pStubDesc %p, pFormat %p, ...\n", pStubDesc, pFormat);
 
-    TRACE("NDR Version: 0x%x\n", pStubDesc->Version);
+    TRACE("NDR Version: 0x%lx\n", pStubDesc->Version);
 
     if (pProcHeader->Oi_flags & Oi_HAS_RPCFLAGS)
     {
@@ -891,7 +891,7 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr_client_call( PMIDL_STUB_DESC pStubDesc, PFORM
     TRACE("stack size: 0x%x\n", stack_size);
     TRACE("proc num: %d\n", procedure_number);
     TRACE("Oi_flags = 0x%02x\n", pProcHeader->Oi_flags);
-    TRACE("MIDL stub version = 0x%x\n", pStubDesc->MIDLVersion);
+    TRACE("MIDL stub version = 0x%lx\n", pStubDesc->MIDLVersion);
 
     pHandleFormat = pFormat;
 
@@ -998,19 +998,21 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr_client_call( PMIDL_STUB_DESC pStubDesc, PFORM
                 number_of_params, Oif_flags, ext_flags, pProcHeader);
     }
 
-    TRACE("RetVal = 0x%lx\n", RetVal);
+    TRACE("RetVal = 0x%Ix\n", RetVal);
     return RetVal;
 }
 
 #ifdef __x86_64__
 
 __ASM_GLOBAL_FUNC( NdrClientCall2,
-                   "movq %r8,0x18(%rsp)\n\t"
-                   "movq %r9,0x20(%rsp)\n\t"
-                   "leaq 0x18(%rsp),%r8\n\t"
-                   "xorq %r9,%r9\n\t"
                    "subq $0x28,%rsp\n\t"
+                   __ASM_SEH(".seh_stackalloc 0x28\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    __ASM_CFI(".cfi_adjust_cfa_offset 0x28\n\t")
+                   "movq %r8,0x40(%rsp)\n\t"
+                   "movq %r9,0x48(%rsp)\n\t"
+                   "leaq 0x40(%rsp),%r8\n\t"
+                   "xorq %r9,%r9\n\t"
                    "call " __ASM_NAME("ndr_client_call") "\n\t"
                    "addq $0x28,%rsp\n\t"
                    __ASM_CFI(".cfi_adjust_cfa_offset -0x28\n\t")
@@ -1023,12 +1025,12 @@ __ASM_GLOBAL_FUNC( NdrClientCall2,
  */
 CLIENT_CALL_RETURN WINAPIV NdrClientCall2( PMIDL_STUB_DESC desc, PFORMAT_STRING format, ... )
 {
-    __ms_va_list args;
+    va_list args;
     LONG_PTR ret;
 
-    __ms_va_start( args, format );
+    va_start( args, format );
     ret = ndr_client_call( desc, format, va_arg( args, void ** ), NULL );
-    __ms_va_end( args );
+    va_end( args );
     return *(CLIENT_CALL_RETURN *)&ret;
 }
 
@@ -1144,7 +1146,6 @@ __ASM_GLOBAL_FUNC( call_server_func,
 #elif defined __arm__
 LONG_PTR __cdecl call_server_func(SERVER_ROUTINE func, unsigned char *args, unsigned int stack_size);
 __ASM_GLOBAL_FUNC( call_server_func,
-                   ".arm\n\t"
                    "push {r4, r5, LR}\n\t"
                    "mov r4, r0\n\t"
                    "mov r5, SP\n\t"
@@ -1153,6 +1154,7 @@ __ASM_GLOBAL_FUNC( call_server_func,
                    "beq 5f\n\t"
                    "sub SP, SP, r2\n\t"
                    "tst r3, #1\n\t"
+                   "it eq\n\t"
                    "subeq SP, SP, #4\n\t"
                    "1:\tsub r2, r2, #4\n\t"
                    "ldr r0, [r1, r2]\n\t"
@@ -1179,7 +1181,10 @@ __ASM_GLOBAL_FUNC( call_server_func,
 LONG_PTR __cdecl call_server_func(SERVER_ROUTINE func, unsigned char *args, unsigned int stack_size);
 __ASM_GLOBAL_FUNC( call_server_func,
                    "stp x29, x30, [sp, #-16]!\n\t"
+                   __ASM_SEH(".seh_save_fplr_x 16\n\t")
                    "mov x29, sp\n\t"
+                   __ASM_SEH(".seh_set_fp\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    "add x9, x2, #15\n\t"
                    "lsr x9, x9, #4\n\t"
                    "sub sp, sp, x9, lsl #4\n\t"
@@ -1246,7 +1251,7 @@ static LONG_PTR *stub_do_args(MIDL_STUB_MESSAGE *pStubMsg,
         case STUBLESS_FREE:
             if (params[i].attr.ServerAllocSize)
             {
-                HeapFree(GetProcessHeap(), 0, *(void **)pArg);
+                free(*(void **)pArg);
             }
             else if (param_needs_alloc(params[i].attr) &&
                      (!params[i].attr.MustFree || params[i].attr.IsSimpleRef))
@@ -1277,8 +1282,7 @@ static LONG_PTR *stub_do_args(MIDL_STUB_MESSAGE *pStubMsg,
             break;
         case STUBLESS_UNMARSHAL:
             if (params[i].attr.ServerAllocSize)
-                *(void **)pArg = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                           params[i].attr.ServerAllocSize * 8);
+                *(void **)pArg = calloc(params[i].attr.ServerAllocSize, 8);
 
             if (params[i].attr.IsIn)
                 call_unmarshaller(pStubMsg, &pArg, &params[i], 0);
@@ -1344,7 +1348,7 @@ LONG WINAPI NdrStubCall2(
     pFormat = pServerInfo->ProcString + pServerInfo->FmtStringOffset[pRpcMsg->ProcNum];
     pProcHeader = (const NDR_PROC_HEADER *)&pFormat[0];
 
-    TRACE("NDR Version: 0x%x\n", pStubDesc->Version);
+    TRACE("NDR Version: 0x%lx\n", pStubDesc->Version);
 
     if (pProcHeader->Oi_flags & Oi_HAS_RPCFLAGS)
     {
@@ -1361,37 +1365,6 @@ LONG WINAPI NdrStubCall2(
 
     TRACE("Oi_flags = 0x%02x\n", pProcHeader->Oi_flags);
 
-    /* binding */
-    switch (pProcHeader->handle_type)
-    {
-    /* explicit binding: parse additional section */
-    case 0:
-        switch (*pFormat) /* handle_type */
-        {
-        case FC_BIND_PRIMITIVE: /* explicit primitive */
-            pFormat += sizeof(NDR_EHD_PRIMITIVE);
-            break;
-        case FC_BIND_GENERIC: /* explicit generic */
-            pFormat += sizeof(NDR_EHD_GENERIC);
-            break;
-        case FC_BIND_CONTEXT: /* explicit context */
-            pFormat += sizeof(NDR_EHD_CONTEXT);
-            break;
-        default:
-            ERR("bad explicit binding handle type (0x%02x)\n", pProcHeader->handle_type);
-            RpcRaiseException(RPC_X_BAD_STUB_DATA);
-        }
-        break;
-    case FC_BIND_GENERIC: /* implicit generic */
-    case FC_BIND_PRIMITIVE: /* implicit primitive */
-    case FC_CALLBACK_HANDLE: /* implicit callback */
-    case FC_AUTO_HANDLE: /* implicit auto handle */
-        break;
-    default:
-        ERR("bad implicit binding handle type (0x%02x)\n", pProcHeader->handle_type);
-        RpcRaiseException(RPC_X_BAD_STUB_DATA);
-    }
-
     if (pProcHeader->Oi_flags & Oi_OBJECT_PROC)
         NdrStubInitialize(pRpcMsg, &stubMsg, pStubDesc, pChannel);
     else
@@ -1415,9 +1388,47 @@ LONG WINAPI NdrStubCall2(
 
     TRACE("allocating memory for stack of size %x\n", stack_size);
 
-    args = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, stack_size);
+    args = calloc(1, stack_size);
     stubMsg.StackTop = args; /* used by conformance of top-level objects */
 
+    /* binding */
+    switch (pProcHeader->handle_type)
+    {
+    /* explicit binding: parse additional section */
+    case 0:
+        switch (*pFormat) /* handle_type */
+        {
+        case FC_BIND_PRIMITIVE: /* explicit primitive */
+            {
+                const NDR_EHD_PRIMITIVE *pDesc = (const NDR_EHD_PRIMITIVE *)pFormat;
+                if (pDesc->flag)
+                    **(handle_t **)ARG_FROM_OFFSET(stubMsg.StackTop, pDesc->offset) = pRpcMsg->Handle;
+                else
+                    *(handle_t *)ARG_FROM_OFFSET(stubMsg.StackTop, pDesc->offset) = pRpcMsg->Handle;
+                pFormat += sizeof(NDR_EHD_PRIMITIVE);
+                break;
+            }
+        case FC_BIND_GENERIC: /* explicit generic */
+            pFormat += sizeof(NDR_EHD_GENERIC);
+            break;
+        case FC_BIND_CONTEXT: /* explicit context */
+            pFormat += sizeof(NDR_EHD_CONTEXT);
+            break;
+        default:
+            ERR("bad explicit binding handle type (0x%02x)\n", pProcHeader->handle_type);
+            RpcRaiseException(RPC_X_BAD_STUB_DATA);
+        }
+        break;
+    case FC_BIND_GENERIC: /* implicit generic */
+    case FC_BIND_PRIMITIVE: /* implicit primitive */
+    case FC_CALLBACK_HANDLE: /* implicit callback */
+    case FC_AUTO_HANDLE: /* implicit auto handle */
+        break;
+    default:
+        ERR("bad implicit binding handle type (0x%02x)\n", pProcHeader->handle_type);
+        RpcRaiseException(RPC_X_BAD_STUB_DATA);
+    }
+
     /* add the implicit This pointer as the first arg to the function if we
      * are calling an object method */
     if (pThis)
@@ -1496,7 +1507,7 @@ LONG WINAPI NdrStubCall2(
 
                 if (retval_ptr)
                 {
-                    TRACE("stub implementation returned 0x%lx\n", retval);
+                    TRACE("stub implementation returned 0x%Ix\n", retval);
                     *retval_ptr = retval;
                 }
                 else
@@ -1554,7 +1565,7 @@ LONG WINAPI NdrStubCall2(
         NdrFullPointerXlatFree(stubMsg.FullPtrXlatTables);
 
     /* free server function stack */
-    HeapFree(GetProcessHeap(), 0, args);
+    free(args);
 
     return S_OK;
 }
@@ -1616,7 +1627,7 @@ static void do_ndr_async_client_call( const MIDL_STUB_DESC *pStubDesc, PFORMAT_S
     /* Later NDR language versions probably won't be backwards compatible */
     if (pStubDesc->Version > 0x60001)
     {
-        FIXME("Incompatible stub description version: 0x%x\n", pStubDesc->Version);
+        FIXME("Incompatible stub description version: 0x%lx\n", pStubDesc->Version);
         RpcRaiseException(RPC_X_WRONG_STUB_VERSION);
     }
 
@@ -1657,7 +1668,7 @@ static void do_ndr_async_client_call( const MIDL_STUB_DESC *pStubDesc, PFORMAT_S
     NdrClientInitializeNew(pRpcMsg, pStubMsg, pStubDesc, procedure_number);
 
     TRACE("Oi_flags = 0x%02x\n", pProcHeader->Oi_flags);
-    TRACE("MIDL stub version = 0x%x\n", pStubDesc->MIDLVersion);
+    TRACE("MIDL stub version = 0x%lx\n", pStubDesc->MIDLVersion);
 
     /* needed for conformance of top-level objects */
     pStubMsg->StackTop = I_RpcAllocate(async_call_data->stack_size);
@@ -1792,8 +1803,8 @@ static void do_ndr_async_client_call( const MIDL_STUB_DESC *pStubDesc, PFORMAT_S
     }
 }
 
-LONG_PTR CDECL DECLSPEC_HIDDEN ndr_async_client_call( PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pFormat,
-                                                      void **stack_top )
+LONG_PTR CDECL ndr_async_client_call( PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pFormat,
+                                      void **stack_top )
 {
     LONG_PTR ret = 0;
     const NDR_PROC_HEADER *pProcHeader = (const NDR_PROC_HEADER *)&pFormat[0];
@@ -1808,7 +1819,7 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr_async_client_call( PMIDL_STUB_DESC pStubDesc,
         }
         __EXCEPT_ALL
         {
-            FIXME("exception %x during ndr_async_client_call()\n", GetExceptionCode());
+            FIXME("exception %lx during ndr_async_client_call()\n", GetExceptionCode());
             ret = GetExceptionCode();
         }
         __ENDTRY
@@ -1816,7 +1827,7 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr_async_client_call( PMIDL_STUB_DESC pStubDesc,
     else
         do_ndr_async_client_call( pStubDesc, pFormat, stack_top);
 
-    TRACE("returning %ld\n", ret);
+    TRACE("returning %Id\n", ret);
     return ret;
 }
 
@@ -1897,7 +1908,7 @@ cleanup:
     I_RpcFree(pStubMsg->StackTop);
     I_RpcFree(async_call_data);
 
-    TRACE("-- 0x%x\n", status);
+    TRACE("-- 0x%lx\n", status);
     return status;
 }
 
@@ -1923,12 +1934,12 @@ __ASM_GLOBAL_FUNC( NdrAsyncClientCall,
  */
 CLIENT_CALL_RETURN WINAPIV NdrAsyncClientCall( PMIDL_STUB_DESC desc, PFORMAT_STRING format, ... )
 {
-    __ms_va_list args;
+    va_list args;
     LONG_PTR ret;
 
-    __ms_va_start( args, format );
+    va_start( args, format );
     ret = ndr_async_client_call( desc, format, va_arg( args, void ** ));
-    __ms_va_end( args );
+    va_end( args );
     return *(CLIENT_CALL_RETURN *)&ret;
 }
 
@@ -1963,7 +1974,7 @@ void RPC_ENTRY NdrAsyncServerCall(PRPC_MESSAGE pRpcMsg)
     pFormat = pServerInfo->ProcString + pServerInfo->FmtStringOffset[pRpcMsg->ProcNum];
     pProcHeader = (const NDR_PROC_HEADER *)&pFormat[0];
 
-    TRACE("NDR Version: 0x%x\n", pStubDesc->Version);
+    TRACE("NDR Version: 0x%lx\n", pStubDesc->Version);
 
     async_call_data = I_RpcAllocate(sizeof(*async_call_data) + sizeof(MIDL_STUB_MESSAGE) + sizeof(RPC_MESSAGE));
     if (!async_call_data) RpcRaiseException(RPC_X_NO_MEMORY);
@@ -2040,7 +2051,7 @@ void RPC_ENTRY NdrAsyncServerCall(PRPC_MESSAGE pRpcMsg)
 
     TRACE("allocating memory for stack of size %x\n", async_call_data->stack_size);
 
-    args = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, async_call_data->stack_size);
+    args = calloc(1, async_call_data->stack_size);
     async_call_data->pStubMsg->StackTop = args; /* used by conformance of top-level objects */
 
     pAsync = I_RpcAllocate(sizeof(*pAsync));
@@ -2143,7 +2154,7 @@ RPC_STATUS NdrpCompleteAsyncServerCall(RPC_ASYNC_STATE *pAsync, void *Reply)
 
     if (async_call_data->retval_ptr)
     {
-        TRACE("stub implementation returned 0x%lx\n", *(LONG_PTR *)Reply);
+        TRACE("stub implementation returned 0x%Ix\n", *(LONG_PTR *)Reply);
         *async_call_data->retval_ptr = *(LONG_PTR *)Reply;
     }
     else
@@ -2158,7 +2169,7 @@ RPC_STATUS NdrpCompleteAsyncServerCall(RPC_ASYNC_STATE *pAsync, void *Reply)
             if (async_call_data->pProcHeader->Oi_flags & Oi_OBJECT_PROC)
             {
                 ERR("objects not supported\n");
-                HeapFree(GetProcessHeap(), 0, async_call_data->pStubMsg->StackTop);
+                free(async_call_data->pStubMsg->StackTop);
                 I_RpcFree(async_call_data);
                 I_RpcFree(pAsync);
                 RpcRaiseException(RPC_X_BAD_STUB_DATA);
@@ -2204,7 +2215,7 @@ RPC_STATUS NdrpCompleteAsyncServerCall(RPC_ASYNC_STATE *pAsync, void *Reply)
 #endif
 
     /* free server function stack */
-    HeapFree(GetProcessHeap(), 0, async_call_data->pStubMsg->StackTop);
+    free(async_call_data->pStubMsg->StackTop);
     I_RpcFree(async_call_data);
     I_RpcFree(pAsync);
 
@@ -2214,12 +2225,12 @@ RPC_STATUS NdrpCompleteAsyncServerCall(RPC_ASYNC_STATE *pAsync, void *Reply)
 static const RPC_SYNTAX_IDENTIFIER ndr_syntax_id =
     {{0x8a885d04, 0x1ceb, 0x11c9, {0x9f, 0xe8, 0x08, 0x00, 0x2b, 0x10, 0x48, 0x60}}, {2, 0}};
 
-LONG_PTR CDECL DECLSPEC_HIDDEN ndr64_client_call( MIDL_STUBLESS_PROXY_INFO *info,
+LONG_PTR CDECL ndr64_client_call( MIDL_STUBLESS_PROXY_INFO *info,
         ULONG proc, void *retval, void **stack_top, void **fpu_stack )
 {
     ULONG_PTR i;
 
-    TRACE("info %p, proc %u, retval %p, stack_top %p, fpu_stack %p\n",
+    TRACE("info %p, proc %lu, retval %p, stack_top %p, fpu_stack %p\n",
             info, proc, retval, stack_top, fpu_stack);
 
     for (i = 0; i < info->nCount; ++i)
@@ -2246,11 +2257,13 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr64_client_call( MIDL_STUBLESS_PROXY_INFO *info
 #ifdef __x86_64__
 
 __ASM_GLOBAL_FUNC( NdrClientCall3,
-                   "movq %r9,0x20(%rsp)\n\t"
-                   "leaq 0x20(%rsp),%r9\n\t"
-                   "pushq $0\n\t"
-                   "subq $0x20,%rsp\n\t"
+                   "subq $0x28,%rsp\n\t"
+                   __ASM_SEH(".seh_stackalloc 0x28\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    __ASM_CFI(".cfi_adjust_cfa_offset 0x28\n\t")
+                   "movq %r9,0x48(%rsp)\n\t"
+                   "leaq 0x48(%rsp),%r9\n\t"
+                   "movq $0,0x20(%rsp)\n\t"
                    "call " __ASM_NAME("ndr64_client_call") "\n\t"
                    "addq $0x28,%rsp\n\t"
                    __ASM_CFI(".cfi_adjust_cfa_offset -0x28\n\t")
@@ -2263,23 +2276,23 @@ __ASM_GLOBAL_FUNC( NdrClientCall3,
  */
 CLIENT_CALL_RETURN WINAPIV NdrClientCall3( MIDL_STUBLESS_PROXY_INFO *info, ULONG proc, void *retval, ... )
 {
-    __ms_va_list args;
+    va_list args;
     LONG_PTR ret;
 
-    __ms_va_start( args, retval );
+    va_start( args, retval );
     ret = ndr64_client_call( info, proc, retval, va_arg( args, void ** ), NULL );
-    __ms_va_end( args );
+    va_end( args );
     return *(CLIENT_CALL_RETURN *)&ret;
 }
 
 #endif
 
-LONG_PTR CDECL DECLSPEC_HIDDEN ndr64_async_client_call( MIDL_STUBLESS_PROXY_INFO *info,
+LONG_PTR CDECL ndr64_async_client_call( MIDL_STUBLESS_PROXY_INFO *info,
         ULONG proc, void *retval, void **stack_top, void **fpu_stack )
 {
     ULONG_PTR i;
 
-    TRACE("info %p, proc %u, retval %p, stack_top %p, fpu_stack %p\n",
+    TRACE("info %p, proc %lu, retval %p, stack_top %p, fpu_stack %p\n",
             info, proc, retval, stack_top, fpu_stack);
 
     for (i = 0; i < info->nCount; ++i)
@@ -2306,11 +2319,13 @@ LONG_PTR CDECL DECLSPEC_HIDDEN ndr64_async_client_call( MIDL_STUBLESS_PROXY_INFO
 #ifdef __x86_64__
 
 __ASM_GLOBAL_FUNC( Ndr64AsyncClientCall,
-                   "movq %r9,0x20(%rsp)\n\t"
-                   "leaq 0x20(%rsp),%r9\n\t"
-                   "pushq $0\n\t"
-                   "subq $0x20,%rsp\n\t"
+                   "subq $0x28,%rsp\n\t"
+                   __ASM_SEH(".seh_stackalloc 0x28\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    __ASM_CFI(".cfi_adjust_cfa_offset 0x28\n\t")
+                   "movq %r9,0x48(%rsp)\n\t"
+                   "leaq 0x48(%rsp),%r9\n\t"
+                   "movq $0,0x20(%rsp)\n\t"
                    "call " __ASM_NAME("ndr64_async_client_call") "\n\t"
                    "addq $0x28,%rsp\n\t"
                    __ASM_CFI(".cfi_adjust_cfa_offset -0x28\n\t")
@@ -2323,12 +2338,12 @@ __ASM_GLOBAL_FUNC( Ndr64AsyncClientCall,
  */
 CLIENT_CALL_RETURN WINAPIV Ndr64AsyncClientCall( MIDL_STUBLESS_PROXY_INFO *info, ULONG proc, void *retval, ... )
 {
-    __ms_va_list args;
+    va_list args;
     LONG_PTR ret;
 
-    __ms_va_start( args, retval );
+    va_start( args, retval );
     ret = ndr64_async_client_call( info, proc, retval, va_arg( args, void ** ), NULL );
-    __ms_va_end( args );
+    va_end( args );
     return *(CLIENT_CALL_RETURN *)&ret;
 }
 
