@@ -147,9 +147,7 @@ static BOOL add_handled_dll( const WCHAR *name )
         WCHAR **new_dlls;
         unsigned int new_count = max( 64, handled_total * 2 );
 
-        if (handled_dlls) new_dlls = HeapReAlloc( GetProcessHeap(), 0, handled_dlls,
-                                                  new_count * sizeof(*handled_dlls) );
-        else new_dlls = HeapAlloc( GetProcessHeap(), 0, new_count * sizeof(*handled_dlls) );
+        new_dlls = realloc( handled_dlls, new_count * sizeof(*handled_dlls) );
         if (!new_dlls) return FALSE;
         handled_dlls = new_dlls;
         handled_total = new_count;
@@ -275,7 +273,7 @@ static BOOL build_fake_dll( HANDLE file, const WCHAR *name )
     DWORD size, header_size = lfanew + sizeof(*nt);
 
     info.handle = file;
-    buffer = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, header_size + 8 * sizeof(IMAGE_SECTION_HEADER) );
+    buffer = calloc( 1, header_size + 8 * sizeof(IMAGE_SECTION_HEADER) );
 
     dos = (IMAGE_DOS_HEADER *)buffer;
     dos->e_magic    = IMAGE_DOS_SIGNATURE;
@@ -361,7 +359,7 @@ static BOOL build_fake_dll( HANDLE file, const WCHAR *name )
     nt->OptionalHeader.SizeOfImage   = ALIGN( info.mem_pos, section_alignment );
     ret = xwrite( &info, buffer, header_size, 0 );
 done:
-    HeapFree( GetProcessHeap(), 0, buffer );
+    free( buffer );
     return ret;
 }
 
@@ -387,7 +385,7 @@ static void create_directories( const WCHAR *name )
     WCHAR *path, *p;
 
     /* create the directory/directories */
-    path = HeapAlloc(GetProcessHeap(), 0, (lstrlenW(name) + 1)*sizeof(WCHAR));
+    path = malloc((wcslen(name) + 1) * sizeof(WCHAR));
     lstrcpyW(path, name);
 
     p = wcschr(path, '\\');
@@ -399,7 +397,7 @@ static void create_directories( const WCHAR *name )
         *p = '\\';
         p = wcschr(p+1, '\\');
     }
-    HeapFree(GetProcessHeap(), 0, path);
+    free(path);
 }
 
 static inline WCHAR *prepend( WCHAR *buffer, const WCHAR *str, size_t len )
@@ -448,7 +446,7 @@ static void *load_fake_dll( const WCHAR *name, SIZE_T *size )
     while ((path = enum_load_path( i++ ))) maxlen = max( maxlen, lstrlenW(path) );
     maxlen += ARRAY_SIZE(pe_dir) + len + 1;
 
-    if (!(file = HeapAlloc( GetProcessHeap(), 0, maxlen * sizeof(WCHAR) ))) return NULL;
+    if (!(file = malloc( maxlen * sizeof(WCHAR) ))) return NULL;
 
     pos = maxlen - len - 1;
     lstrcpyW( file + pos, name );
@@ -478,7 +476,7 @@ static void *load_fake_dll( const WCHAR *name, SIZE_T *size )
     }
 
 done:
-    HeapFree( GetProcessHeap(), 0, file );
+    free( file );
     if (res == 1) return data;
     return NULL;
 }
@@ -667,7 +665,7 @@ static WCHAR* create_winsxs_dll_path( const xmlstr_t *arch, const xmlstr_t *name
     path_len = GetWindowsDirectoryW( NULL, 0 ) + ARRAY_SIZE( L"\\winsxs\\" )
         + arch->len + name->len + key->len + version->len + 19;
 
-    path = HeapAlloc( GetProcessHeap(), 0, path_len * sizeof(WCHAR) );
+    path = malloc( path_len * sizeof(WCHAR) );
     GetWindowsDirectoryW( path, path_len );
     lstrcatW( path, L"\\winsxs\\" );
     append_manifest_filename( arch, name, key, version, lang, path, path_len );
@@ -686,7 +684,7 @@ static BOOL create_manifest( const xmlstr_t *arch, const xmlstr_t *name, const x
     path_len = GetWindowsDirectoryW( NULL, 0 ) + ARRAY_SIZE( L"\\winsxs\\manifests\\" )
         + arch->len + name->len + key->len + version->len + 18 + ARRAY_SIZE( L".manifest" );
 
-    path = HeapAlloc( GetProcessHeap(), 0, path_len * sizeof(WCHAR) );
+    path = malloc( path_len * sizeof(WCHAR) );
     GetWindowsDirectoryW( path, path_len );
     lstrcatW( path, L"\\winsxs\\manifests\\" );
     append_manifest_filename( arch, name, key, version, lang, path, path_len );
@@ -706,7 +704,7 @@ static BOOL create_manifest( const xmlstr_t *arch, const xmlstr_t *name, const x
         CloseHandle( handle );
         if (!ret) DeleteFileW( path );
     }
-    HeapFree( GetProcessHeap(), 0, path );
+    free( path );
     return ret;
 }
 
@@ -764,9 +762,8 @@ static BOOL CALLBACK register_manifest( HMODULE module, const WCHAR *type, WCHAR
 
             if (!error && dest && name.ptr)
             {
-                struct delay_copy *add = HeapAlloc( GetProcessHeap(), 0,
-                        sizeof(*add) + (dll_data->src_len + name.len +
-                            dest_len + name.len + 1) * sizeof(WCHAR) );
+                struct delay_copy *add = malloc( sizeof(*add) +
+                        (dll_data->src_len + name.len + dest_len + name.len + 1) * sizeof(WCHAR) );
                 add->src = add->data;
                 memcpy( add->src, dll_data->src_dir, dll_data->src_len * sizeof(WCHAR) );
                 MultiByteToWideChar( CP_UTF8, 0, name.ptr, name.len,
@@ -783,7 +780,7 @@ static BOOL CALLBACK register_manifest( HMODULE module, const WCHAR *type, WCHAR
         }
 
         if (!xmlstr_cmp( &elem, "assemblyIdentity" )) continue;
-        HeapFree( GetProcessHeap(), 0, dest );
+        free( dest );
         dest = NULL;
         while (next_xml_attr( &buffer, &attr_name, &attr_value, &error ))
         {
@@ -802,7 +799,7 @@ static BOOL CALLBACK register_manifest( HMODULE module, const WCHAR *type, WCHAR
             }
             if (!arch.len)  /* fixup the architecture */
             {
-                char *new_buffer = HeapAlloc( GetProcessHeap(), 0, len + sizeof(current_arch) );
+                char *new_buffer = malloc( len + sizeof(current_arch) );
                 memcpy( new_buffer, manifest, arch.ptr - manifest );
                 strcpy( new_buffer + (arch.ptr - manifest), current_arch );
                 memcpy( new_buffer + strlen(new_buffer), arch.ptr, len - (arch.ptr - manifest) );
@@ -810,7 +807,7 @@ static BOOL CALLBACK register_manifest( HMODULE module, const WCHAR *type, WCHAR
                 arch.len = strlen( current_arch );
                 dest = create_winsxs_dll_path( &arch, &name, &key, &version, &lang );
                 create_manifest( &arch, &name, &key, &version, &lang, new_buffer, len + arch.len );
-                HeapFree( GetProcessHeap(), 0, new_buffer );
+                free( new_buffer );
             }
             else
             {
@@ -820,7 +817,7 @@ static BOOL CALLBACK register_manifest( HMODULE module, const WCHAR *type, WCHAR
             dest_len = wcslen( dest );
         }
     }
-    HeapFree( GetProcessHeap(), 0, dest );
+    free( dest );
 
     return TRUE;
 }
@@ -835,11 +832,11 @@ static BOOL CALLBACK register_resource( HMODULE module, LPCWSTR type, LPWSTR nam
 
     if (!str) return FALSE;
     lenW = MultiByteToWideChar( CP_UTF8, 0, str, lenA, NULL, 0 ) + 1;
-    if (!(buffer = HeapAlloc( GetProcessHeap(), 0, lenW * sizeof(WCHAR) ))) return FALSE;
+    if (!(buffer = malloc( lenW * sizeof(WCHAR) ))) return FALSE;
     MultiByteToWideChar( CP_UTF8, 0, str, lenA, buffer, lenW );
     buffer[lenW - 1] = 0;
     *hr = IRegistrar_StringRegister( registrar, buffer );
-    HeapFree( GetProcessHeap(), 0, buffer );
+    free( buffer );
     return TRUE;
 }
 
@@ -945,7 +942,7 @@ static void delay_copy_files( struct list *delay_copy )
         ret = read_file( copy->src, &data, &size );
         if (ret != 1)
         {
-            HeapFree( GetProcessHeap(), 0, copy );
+            free( copy );
             continue;
         }
 
@@ -957,7 +954,7 @@ static void delay_copy_files( struct list *delay_copy )
             CloseHandle( h );
             if (!ret) DeleteFileW( copy->dest );
         }
-        HeapFree( GetProcessHeap(), 0, copy );
+        free( copy );
     }
 }
 
@@ -1012,11 +1009,11 @@ static BOOL create_wildcard_dlls( const WCHAR *dirname, const WCHAR *wildcard, B
     if (build_dir) maxlen = lstrlenW(build_dir) + ARRAY_SIZE(L"\\programs") + 1;
     for (i = 0; (path = enum_load_path(i)); i++) maxlen = max( maxlen, lstrlenW(path) );
     maxlen += 2 * max_dll_name_len + 2 + ARRAY_SIZE(pe_dir) + 10; /* ".dll" */
-    if (!(file = HeapAlloc( GetProcessHeap(), 0, maxlen * sizeof(WCHAR) ))) return FALSE;
+    if (!(file = malloc( maxlen * sizeof(WCHAR) ))) return FALSE;
 
-    if (!(dest = HeapAlloc( GetProcessHeap(), 0, (lstrlenW(dirname) + max_dll_name_len) * sizeof(WCHAR) )))
+    if (!(dest = malloc( (wcslen(dirname) + max_dll_name_len) * sizeof(WCHAR) )))
     {
-        HeapFree( GetProcessHeap(), 0, file );
+        free( file );
         return FALSE;
     }
     lstrcpyW( dest, dirname );
@@ -1038,8 +1035,8 @@ static BOOL create_wildcard_dlls( const WCHAR *dirname, const WCHAR *wildcard, B
         lstrcpyW( file, path );
         install_lib_dir( dest, file, wildcard, NULL, delete );
     }
-    HeapFree( GetProcessHeap(), 0, file );
-    HeapFree( GetProcessHeap(), 0, dest );
+    free( file );
+    free( dest );
     return TRUE;
 }
 
@@ -1101,7 +1098,7 @@ void cleanup_fake_dlls(void)
 {
     if (file_buffer) VirtualFree( file_buffer, 0, MEM_RELEASE );
     file_buffer = NULL;
-    HeapFree( GetProcessHeap(), 0, handled_dlls );
+    free( handled_dlls );
     handled_dlls = NULL;
     handled_count = handled_total = 0;
     if (registrar) IRegistrar_Release( registrar );
