@@ -131,7 +131,7 @@ static ULONG WINAPI IQueryAssociations_fnAddRef(IQueryAssociations *iface)
   IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
   ULONG refCount = InterlockedIncrement(&This->ref);
 
-  TRACE("(%p)->(ref before=%u)\n",This, refCount - 1);
+  TRACE("(%p)->(ref before=%lu)\n",This, refCount - 1);
 
   return refCount;
 }
@@ -146,7 +146,7 @@ static ULONG WINAPI IQueryAssociations_fnRelease(IQueryAssociations *iface)
   IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
   ULONG refCount = InterlockedDecrement(&This->ref);
 
-  TRACE("(%p)->(ref before=%u)\n",This, refCount + 1);
+  TRACE("(%p)->(ref before=%lu)\n",This, refCount + 1);
 
   if (!refCount)
   {
@@ -185,7 +185,7 @@ static HRESULT WINAPI IQueryAssociations_fnInit(
     IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
     LONG ret;
 
-    TRACE("(%p)->(%d,%s,%p,%p)\n", iface,
+    TRACE("(%p)->(%ld,%s,%p,%p)\n", iface,
                                     cfFlags,
                                     debugstr_w(pszAssoc),
                                     hkeyProgid,
@@ -193,7 +193,7 @@ static HRESULT WINAPI IQueryAssociations_fnInit(
     if (hWnd != NULL)
         FIXME("hwnd != NULL not supported\n");
     if (cfFlags != 0)
-	FIXME("unsupported flags: %x\n", cfFlags);
+	FIXME("unsupported flags: %lx\n", cfFlags);
 
     RegCloseKey(This->hkeySource);
     if (This->hkeySource != This->hkeyProgID)
@@ -247,7 +247,7 @@ static HRESULT WINAPI IQueryAssociations_fnInit(
                             0,
                             KEY_READ,
                             &This->hkeyProgID);
-        heap_free(progId);
+        free(progId);
 
         return S_OK;
     }
@@ -272,13 +272,13 @@ static HRESULT ASSOC_GetValue(HKEY hkey, const WCHAR *name, void **data, DWORD *
     return HRESULT_FROM_WIN32(ret);
   if (!size)
     return E_FAIL;
-  *data = heap_alloc(size);
+  *data = malloc(size);
   if (!*data)
     return E_OUTOFMEMORY;
   ret = RegQueryValueExW(hkey, name, 0, NULL, (LPBYTE)*data, &size);
   if (ret != ERROR_SUCCESS)
   {
-    heap_free(*data);
+    free(*data);
     return HRESULT_FROM_WIN32(ret);
   }
   if(data_size)
@@ -304,7 +304,7 @@ static HRESULT ASSOC_GetCommand(IQueryAssociationsImpl *This, const WCHAR *extra
       HKEY hkeyFile;
 
       ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, filetype, 0, KEY_READ, &hkeyFile);
-      heap_free(filetype);
+      free(filetype);
 
       if (ret == ERROR_SUCCESS)
       {
@@ -336,7 +336,7 @@ static HRESULT ASSOC_GetCommand(IQueryAssociationsImpl *This, const WCHAR *extra
           }
 
           max_subkey_len++;
-          extra_from_reg = heap_alloc(max_subkey_len * sizeof(WCHAR));
+          extra_from_reg = malloc(max_subkey_len * sizeof(WCHAR));
           if (!extra_from_reg)
           {
               RegCloseKey(hkeyShell);
@@ -346,7 +346,7 @@ static HRESULT ASSOC_GetCommand(IQueryAssociationsImpl *This, const WCHAR *extra
           ret = RegEnumKeyExW(hkeyShell, 0, extra_from_reg, &max_subkey_len, NULL, NULL, NULL, NULL);
           if (ret)
           {
-              heap_free(extra_from_reg);
+              free(extra_from_reg);
               RegCloseKey(hkeyShell);
               return HRESULT_FROM_WIN32(ret);
           }
@@ -356,7 +356,7 @@ static HRESULT ASSOC_GetCommand(IQueryAssociationsImpl *This, const WCHAR *extra
 
   /* open verb subkey */
   ret = RegOpenKeyExW(hkeyShell, extra, 0, KEY_READ, &hkeyVerb);
-  heap_free(extra_from_reg);
+  free(extra_from_reg);
   RegCloseKey(hkeyShell);
   if (ret) return HRESULT_FROM_WIN32(ret);
 
@@ -407,7 +407,7 @@ static HRESULT ASSOC_GetExecutable(IQueryAssociationsImpl *This,
       *len = SearchPathW(NULL, pszStart, NULL, pathlen, path, NULL);
   }
 
-  heap_free(pszCommand);
+  free(pszCommand);
   if (!*len)
     return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
   return S_OK;
@@ -439,7 +439,7 @@ static HRESULT ASSOC_ReturnString(ASSOCF flags, LPWSTR out, DWORD *outlen, LPCWS
     HRESULT hr = S_OK;
     DWORD len;
 
-    TRACE("flags=0x%08x, data=%s\n", flags, debugstr_w(data));
+    TRACE("flags=0x%08lx, data=%s\n", flags, debugstr_w(data));
 
     if (!out)
     {
@@ -502,10 +502,10 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
   HRESULT hr;
   WCHAR path[MAX_PATH];
 
-  TRACE("(%p)->(0x%08x, %u, %s, %p, %p)\n", This, flags, str, debugstr_w(pszExtra), pszOut, pcchOut);
+  TRACE("(%p)->(0x%08lx, %u, %s, %p, %p)\n", This, flags, str, debugstr_w(pszExtra), pszOut, pcchOut);
 
   if (flags & unimplemented_flags)
-    FIXME("%08x: unimplemented flags\n", flags & unimplemented_flags);
+    FIXME("%08lx: unimplemented flags\n", flags & unimplemented_flags);
 
   if (!pcchOut)
     return E_UNEXPECTED;
@@ -522,7 +522,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
       if (SUCCEEDED(hr))
       {
         hr = ASSOC_ReturnString(flags, pszOut, pcchOut, command, lstrlenW(command) + 1);
-        heap_free(command);
+        free(command);
       }
       return hr;
     }
@@ -546,7 +546,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
           return HRESULT_FROM_WIN32(ERROR_NO_ASSOCIATION);
       }
       hr = ASSOC_ReturnString(flags, pszOut, pcchOut, docName, lstrlenW(docName) + 1);
-      heap_free(docName);
+      free(docName);
       return hr;
     }
 
@@ -565,7 +565,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
       retval = GetFileVersionInfoSizeW(path, &size);
       if (!retval)
         goto get_friendly_name_fail;
-      verinfoW = heap_alloc_zero(retval);
+      verinfoW = calloc(1, retval);
       if (!verinfoW)
         return E_OUTOFMEMORY;
       if (!GetFileVersionInfoW(path, 0, retval, verinfoW))
@@ -584,7 +584,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetString(
             len = lstrlenW(bufW) + 1;
             TRACE("found FileDescription: %s\n", debugstr_w(bufW));
             hr = ASSOC_ReturnString(flags, pszOut, pcchOut, bufW, len);
-            heap_free(verinfoW);
+            free(verinfoW);
             return hr;
           }
         }
@@ -594,7 +594,7 @@ get_friendly_name_fail:
       PathStripPathW(path);
       TRACE("using filename: %s\n", debugstr_w(path));
       hr = ASSOC_ReturnString(flags, pszOut, pcchOut, path, lstrlenW(path) + 1);
-      heap_free(verinfoW);
+      free(verinfoW);
       return hr;
     }
 
@@ -608,7 +608,7 @@ get_friendly_name_fail:
       ret = RegGetValueW(This->hkeySource, NULL, L"Content Type", RRF_RT_REG_SZ, NULL, NULL, &size);
       if (ret != ERROR_SUCCESS)
         return HRESULT_FROM_WIN32(ret);
-      contentType = heap_alloc(size);
+      contentType = malloc(size);
       if (contentType != NULL)
       {
         ret = RegGetValueW(This->hkeySource, NULL, L"Content Type", RRF_RT_REG_SZ, NULL, contentType, &size);
@@ -616,7 +616,7 @@ get_friendly_name_fail:
           hr = ASSOC_ReturnString(flags, pszOut, pcchOut, contentType, lstrlenW(contentType) + 1);
         else
           hr = HRESULT_FROM_WIN32(ret);
-        heap_free(contentType);
+        free(contentType);
       }
       else
         hr = E_OUTOFMEMORY;
@@ -632,7 +632,7 @@ get_friendly_name_fail:
       ret = RegGetValueW(This->hkeyProgID, L"DefaultIcon", NULL, RRF_RT_REG_SZ, NULL, NULL, &size);
       if (ret == ERROR_SUCCESS)
       {
-        WCHAR *icon = heap_alloc(size);
+        WCHAR *icon = malloc(size);
         if (icon)
         {
           ret = RegGetValueW(This->hkeyProgID, L"DefaultIcon", NULL, RRF_RT_REG_SZ, NULL, icon, &size);
@@ -640,7 +640,7 @@ get_friendly_name_fail:
             hr = ASSOC_ReturnString(flags, pszOut, pcchOut, icon, lstrlenW(icon) + 1);
           else
             hr = HRESULT_FROM_WIN32(ret);
-          heap_free(icon);
+          free(icon);
         }
         else
           hr = E_OUTOFMEMORY;
@@ -708,7 +708,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetKey(
 {
   IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
 
-  FIXME("(%p,0x%8x,0x%8x,%s,%p)-stub!\n", This, cfFlags, assockey,
+  FIXME("(%p,0x%8lx,0x%8x,%s,%p)-stub!\n", This, cfFlags, assockey,
         debugstr_w(pszExtra), phkeyOut);
   return E_NOTIMPL;
 }
@@ -739,11 +739,11 @@ static HRESULT WINAPI IQueryAssociations_fnGetData(IQueryAssociations *iface,
     DWORD size;
     HRESULT hres;
 
-    TRACE("(%p,0x%8x,0x%8x,%s,%p,%p)\n", This, cfFlags, assocdata,
+    TRACE("(%p,0x%8lx,0x%8x,%s,%p,%p)\n", This, cfFlags, assocdata,
             debugstr_w(pszExtra), pvOut, pcbOut);
 
     if(cfFlags)
-        FIXME("Unsupported flags: %x\n", cfFlags);
+        FIXME("Unsupported flags: %lx\n", cfFlags);
 
     switch(assocdata) {
     case ASSOCDATA_EDITFLAGS:
@@ -753,7 +753,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetData(IQueryAssociations *iface,
         hres = ASSOC_GetValue(This->hkeyProgID, L"EditFlags", &data, &size);
         if(SUCCEEDED(hres) && pcbOut)
             hres = ASSOC_ReturnData(pvOut, pcbOut, data, size);
-        heap_free(data);
+        free(data);
         return hres;
     default:
         FIXME("Unsupported ASSOCDATA value: %d\n", assocdata);
@@ -791,7 +791,7 @@ static HRESULT WINAPI IQueryAssociations_fnGetEnum(
 {
   IQueryAssociationsImpl *This = impl_from_IQueryAssociations(iface);
 
-  FIXME("(%p,0x%8x,0x%8x,%s,%s,%p)-stub!\n", This, cfFlags, assocenum,
+  FIXME("(%p,0x%8lx,0x%8x,%s,%s,%p)-stub!\n", This, cfFlags, assocenum,
         debugstr_w(pszExtra), debugstr_guid(riid), ppvOut);
   return E_NOTIMPL;
 }
@@ -844,7 +844,7 @@ static ULONG WINAPI ApplicationAssociationRegistration_AddRef(IApplicationAssoci
     IApplicationAssociationRegistrationImpl *This = impl_from_IApplicationAssociationRegistration(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
     return ref;
 }
 
@@ -853,7 +853,7 @@ static ULONG WINAPI ApplicationAssociationRegistration_Release(IApplicationAssoc
     IApplicationAssociationRegistrationImpl *This = impl_from_IApplicationAssociationRegistration(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p) ref=%d\n", This, ref);
+    TRACE("(%p) ref=%ld\n", This, ref);
 
     if (!ref) {
         SHFree(This);
@@ -1042,7 +1042,7 @@ HRESULT WINAPI ApplicationAssociationRegistration_Constructor(IUnknown *outer, R
     if (FAILED(hr))
         SHFree(This);
 
-    TRACE("returning 0x%x with %p\n", hr, *ppv);
+    TRACE("returning 0x%lx with %p\n", hr, *ppv);
     return hr;
 }
 
@@ -1069,7 +1069,7 @@ static ULONG WINAPI enumassochandlers_AddRef(IEnumAssocHandlers *iface)
     struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
 
-    TRACE("(%p)->(%u)\n", This, ref);
+    TRACE("(%p)->(%lu)\n", This, ref);
     return ref;
 }
 
@@ -1078,7 +1078,7 @@ static ULONG WINAPI enumassochandlers_Release(IEnumAssocHandlers *iface)
     struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
-    TRACE("(%p)->(%u)\n", This, ref);
+    TRACE("(%p)->(%lu)\n", This, ref);
 
     if (!ref)
         SHFree(This);
@@ -1091,7 +1091,7 @@ static HRESULT WINAPI enumassochandlers_Next(IEnumAssocHandlers *iface, ULONG co
 {
     struct enumassochandlers *This = impl_from_IEnumAssocHandlers(iface);
 
-    FIXME("(%p)->(%u %p %p): stub\n", This, count, handlers, fetched);
+    FIXME("(%p)->(%lu %p %p): stub\n", This, count, handlers, fetched);
 
     return E_NOTIMPL;
 }
