@@ -41,7 +41,6 @@
 #include "shlwapi.h"
 #include "shell32_main.h"
 #include "shfldr.h"
-#include "undocshell.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(shell);
@@ -317,7 +316,7 @@ static DWORD SHELL32_AnsiToUnicodeBuf(LPCSTR aPath, LPWSTR *wPath, DWORD minChar
 	if (len < minChars)
 	  len = minChars;
 
-	*wPath = heap_alloc(len * sizeof(WCHAR));
+	*wPath = malloc(len * sizeof(WCHAR));
 	if (*wPath)
 	{
 	  MultiByteToWideChar(CP_ACP, 0, aPath, -1, *wPath, len);
@@ -396,7 +395,7 @@ static DWORD SHNotifyCreateDirectoryA(LPCSTR path, LPSECURITY_ATTRIBUTES sec)
 	if (!retCode)
 	{
 	  retCode = SHNotifyCreateDirectoryW(wPath, sec);
-	  heap_free(wPath);
+	  free(wPath);
 	}
 	return retCode;
 }
@@ -450,7 +449,7 @@ static DWORD SHNotifyRemoveDirectoryA(LPCSTR path)
 	if (!retCode)
 	{
 	  retCode = SHNotifyRemoveDirectoryW(wPath);
-	  heap_free(wPath);
+	  free(wPath);
 	}
 	return retCode;
 }
@@ -514,7 +513,7 @@ static DWORD SHNotifyDeleteFileA(LPCSTR path)
 	if (!retCode)
 	{
 	  retCode = SHNotifyDeleteFileW(wPath);
-	  heap_free(wPath);
+	  free(wPath);
 	}
 	return retCode;
 }
@@ -710,7 +709,7 @@ int WINAPI SHCreateDirectoryExA(HWND hWnd, LPCSTR path, LPSECURITY_ATTRIBUTES se
 	if (!retCode)
 	{
 	  retCode = SHCreateDirectoryExW(hWnd, wPath, sec);
-	  heap_free(wPath);
+	  free(wPath);
 	}
 	return retCode;
 }
@@ -882,12 +881,12 @@ int WINAPI SHFileOperationA(LPSHFILEOPSTRUCTA lpFileOp)
 	    if (retCode == ERROR_ACCESS_DENIED && (GetVersion() & 0x80000000))
 	      retCode = S_OK;
 
-	    heap_free(ForFree); /* we cannot use wString, it was changed */
+	    free(ForFree); /* we cannot use wString, it was changed */
 	    break;
 	  }
 	  else
 	  {
-	    wString = ForFree = heap_alloc(size * sizeof(WCHAR));
+	    wString = ForFree = malloc(size * sizeof(WCHAR));
 	    if (ForFree) continue;
 	    retCode = ERROR_OUTOFMEMORY;
 	    nFileOp.fAnyOperationsAborted = TRUE;
@@ -926,8 +925,7 @@ typedef struct
 
 static inline void grow_list(FILE_LIST *list)
 {
-    FILE_ENTRY *new = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, list->feFiles,
-                                  list->num_alloc * 2 * sizeof(*new) );
+    FILE_ENTRY *new = _recalloc(list->feFiles, list->num_alloc * 2, sizeof(*new));
     list->feFiles = new;
     list->num_alloc *= 2;
 }
@@ -939,18 +937,18 @@ static void add_file_to_entry(FILE_ENTRY *feFile, LPCWSTR szFile)
     DWORD dwLen = lstrlenW(szFile) + 1;
     LPCWSTR ptr;
 
-    feFile->szFullPath = heap_alloc(dwLen * sizeof(WCHAR));
+    feFile->szFullPath = malloc(dwLen * sizeof(WCHAR));
     lstrcpyW(feFile->szFullPath, szFile);
 
     ptr = StrRChrW(szFile, NULL, '\\');
     if (ptr)
     {
         dwLen = ptr - szFile + 1;
-        feFile->szDirectory = heap_alloc(dwLen * sizeof(WCHAR));
+        feFile->szDirectory = malloc(dwLen * sizeof(WCHAR));
         lstrcpynW(feFile->szDirectory, szFile, dwLen);
 
         dwLen = lstrlenW(feFile->szFullPath) - dwLen + 1;
-        feFile->szFilename = heap_alloc(dwLen * sizeof(WCHAR));
+        feFile->szFilename = malloc(dwLen * sizeof(WCHAR));
         lstrcpyW(feFile->szFilename, ptr + 1); /* skip over backslash */
     }
     feFile->bFromWildcard = FALSE;
@@ -966,7 +964,7 @@ static LPWSTR wildcard_to_file(LPCWSTR szWildCard, LPCWSTR szFileName)
     dwDirLen = ptr - szWildCard + 1;
 
     dwFullLen = dwDirLen + lstrlenW(szFileName) + 1;
-    szFullPath = heap_alloc(dwFullLen * sizeof(WCHAR));
+    szFullPath = malloc(dwFullLen * sizeof(WCHAR));
 
     lstrcpynW(szFullPath, szWildCard, dwDirLen + 1);
     lstrcatW(szFullPath, szFileName);
@@ -994,7 +992,7 @@ static void parse_wildcard_files(FILE_LIST *flList, LPCWSTR szFile, LPDWORD pdwL
         file->bFromWildcard = TRUE;
         file->attributes = wfd.dwFileAttributes;
         if (IsAttribDir(file->attributes)) flList->bAnyDirectories = TRUE;
-        heap_free(szFullPath);
+        free(szFullPath);
     }
 
     FindClose(hFile);
@@ -1020,8 +1018,8 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
     /* empty list */
     if (!szFiles[0])
         return ERROR_ACCESS_DENIED;
-        
-    flList->feFiles = heap_alloc_zero(flList->num_alloc * sizeof(FILE_ENTRY));
+
+    flList->feFiles = calloc(flList->num_alloc, sizeof(FILE_ENTRY));
 
     while (*ptr)
     {
@@ -1078,12 +1076,12 @@ static void destroy_file_list(FILE_LIST *flList)
 
     for (i = 0; i < flList->dwNumFiles; i++)
     {
-        heap_free(flList->feFiles[i].szDirectory);
-        heap_free(flList->feFiles[i].szFilename);
-        heap_free(flList->feFiles[i].szFullPath);
+        free(flList->feFiles[i].szDirectory);
+        free(flList->feFiles[i].szFilename);
+        free(flList->feFiles[i].szFullPath);
     }
 
-    heap_free(flList->feFiles);
+    free(flList->feFiles);
 }
 
 static void copy_dir_to_dir(FILE_OPERATION *op, const FILE_ENTRY *feFrom, LPCWSTR szDestPath)
@@ -1212,9 +1210,9 @@ static int copy_files(FILE_OPERATION *op, const FILE_LIST *flFrom, FILE_LIST *fl
             /* Free all but the first entry. */
             for (i = 1; i < flTo->dwNumFiles; i++)
             {
-                heap_free(flTo->feFiles[i].szDirectory);
-                heap_free(flTo->feFiles[i].szFilename);
-                heap_free(flTo->feFiles[i].szFullPath);
+                free(flTo->feFiles[i].szDirectory);
+                free(flTo->feFiles[i].szFilename);
+                free(flTo->feFiles[i].szFullPath);
             }
 
             flTo->dwNumFiles = 1;
@@ -1739,10 +1737,10 @@ HRESULT WINAPI SHPathPrepareForWriteW(HWND hwnd, IUnknown *modless, LPCWSTR path
     WCHAR* last_slash;
     WCHAR* temppath=NULL;
 
-    TRACE("%p %p %s 0x%08x\n", hwnd, modless, debugstr_w(path), flags);
+    TRACE("%p %p %s 0x%08lx\n", hwnd, modless, debugstr_w(path), flags);
 
     if (flags & ~(SHPPFW_DIRCREATE|SHPPFW_ASKDIRCREATE|SHPPFW_IGNOREFILENAME))
-        FIXME("unimplemented flags 0x%08x\n", flags);
+        FIXME("unimplemented flags 0x%08lx\n", flags);
 
     /* cut off filename if necessary */
     if (flags & SHPPFW_IGNOREFILENAME)
@@ -1752,7 +1750,7 @@ HRESULT WINAPI SHPathPrepareForWriteW(HWND hwnd, IUnknown *modless, LPCWSTR path
             len = 1;
         else
             len = last_slash - path + 1;
-        temppath = heap_alloc(len * sizeof(WCHAR));
+        temppath = malloc(len * sizeof(WCHAR));
         if (!temppath)
             return E_OUTOFMEMORY;
         StrCpyNW(temppath, path, len);
@@ -1775,7 +1773,7 @@ HRESULT WINAPI SHPathPrepareForWriteW(HWND hwnd, IUnknown *modless, LPCWSTR path
     /* check if we can access the directory */
     res = GetFileAttributesW(realpath);
 
-    heap_free(temppath);
+    free(temppath);
 
     if (res == INVALID_FILE_ATTRIBUTES)
     {
@@ -1796,7 +1794,7 @@ HRESULT WINAPI SHPathPrepareForWriteW(HWND hwnd, IUnknown *modless, LPCWSTR path
 
 HRESULT WINAPI SHMultiFileProperties(IDataObject *pdtobj, DWORD flags)
 {
-    FIXME("stub: %p %u\n", pdtobj, flags);
+    FIXME("stub: %p %lu\n", pdtobj, flags);
     return E_NOTIMPL;
 }
 
@@ -1836,7 +1834,7 @@ static ULONG WINAPI file_operation_AddRef(IFileOperation *iface)
     struct file_operation *operation = impl_from_IFileOperation(iface);
     ULONG ref = InterlockedIncrement(&operation->ref);
 
-    TRACE("(%p): ref=%u.\n", iface, ref);
+    TRACE("(%p): ref=%lu.\n", iface, ref);
 
     return ref;
 }
@@ -1846,11 +1844,11 @@ static ULONG WINAPI file_operation_Release(IFileOperation *iface)
     struct file_operation *operation = impl_from_IFileOperation(iface);
     ULONG ref = InterlockedDecrement(&operation->ref);
 
-    TRACE("(%p): ref=%u.\n", iface, ref);
+    TRACE("(%p): ref=%lu.\n", iface, ref);
 
     if (!ref)
     {
-        HeapFree(GetProcessHeap(), 0, operation);
+        free(operation);
     }
 
     return ref;
@@ -1865,14 +1863,14 @@ static HRESULT WINAPI file_operation_Advise(IFileOperation *iface, IFileOperatio
 
 static HRESULT WINAPI file_operation_Unadvise(IFileOperation *iface, DWORD cookie)
 {
-    FIXME("(%p, %x): stub.\n", iface, cookie);
+    FIXME("(%p, %lx): stub.\n", iface, cookie);
 
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI file_operation_SetOperationFlags(IFileOperation *iface, DWORD flags)
 {
-    FIXME("(%p, %x): stub.\n", iface, flags);
+    FIXME("(%p, %lx): stub.\n", iface, flags);
 
     return E_NOTIMPL;
 }
@@ -1982,7 +1980,7 @@ static HRESULT WINAPI file_operation_DeleteItems(IFileOperation *iface, IUnknown
 static HRESULT WINAPI file_operation_NewItem(IFileOperation *iface, IShellItem *folder, DWORD attributes,
         LPCWSTR name, LPCWSTR template, IFileOperationProgressSink *sink)
 {
-    FIXME("(%p, %p, %x, %s, %s, %p): stub.\n", iface, folder, attributes,
+    FIXME("(%p, %p, %lx, %s, %s, %p): stub.\n", iface, folder, attributes,
           debugstr_w(name), debugstr_w(template), sink);
 
     return E_NOTIMPL;
@@ -2034,7 +2032,7 @@ HRESULT WINAPI IFileOperation_Constructor(IUnknown *outer, REFIID riid, void **o
     struct file_operation *object;
     HRESULT hr;
 
-    object = heap_alloc_zero(sizeof(*object));
+    object = calloc(1, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
