@@ -28,6 +28,7 @@
 #include "shlwapi.h"
 #include "wininet.h"
 #include "intshcut.h"
+#include "winternl.h"
 
 static const char* TEST_URL_1 = "http://www.winehq.org/tests?date=10/10/1923";
 static const char* TEST_URL_2 = "http://localhost:8080/tests%2e.html?date=Mon%2010/10/1923";
@@ -407,7 +408,32 @@ static struct {
     {"file://%24%25foobar", "file://$%foobar"}
 };
 
-/* ################ */
+static struct
+{
+    const WCHAR *url;
+    const WCHAR *expect;
+    DWORD flags;
+} TEST_URL_UNESCAPEW[] =
+{
+    { L"file://foo/bar", L"file://foo/bar" },
+    { L"file://fo%20o%5Ca/bar", L"file://fo o\\a/bar" },
+    { L"file://%24%25foobar", L"file://$%foobar" },
+    { L"file:///C:/Program Files", L"file:///C:/Program Files" },
+    { L"file:///C:/Program Files", L"file:///C:/Program Files", URL_UNESCAPE_AS_UTF8 },
+    { L"file:///C:/Program%20Files", L"file:///C:/Program Files" },
+    { L"file:///C:/Program%20Files", L"file:///C:/Program Files", URL_UNESCAPE_AS_UTF8 },
+    { L"file://foo/%E4%B8%AD%E6%96%87/bar", L"file://foo/\xe4\xb8\xad\xe6\x96\x87/bar" }, /* with 3 btyes utf-8 */
+    { L"file://foo/%E4%B8%AD%E6%96%87/bar", L"file://foo/\x4e2d\x6587/bar", URL_UNESCAPE_AS_UTF8 },
+    /* mix corrupt and good utf-8 */
+    { L"file://foo/%E4%AD%E6%96%87/bar", L"file://foo/\xfffd\x6587/bar", URL_UNESCAPE_AS_UTF8 },
+    { L"file://foo/%F0%9F%8D%B7/bar", L"file://foo/\xf0\x9f\x8d\xb7/bar" }, /* with 4 btyes utf-8 */
+    { L"file://foo/%F0%9F%8D%B7/bar", L"file://foo/\xd83c\xdf77/bar", URL_UNESCAPE_AS_UTF8 },
+    /* non-escaped chars between multi-byte escaped chars */
+    { L"file://foo/%E4%B8%ADabc%E6%96%87/bar", L"file://foo/\x4e2d""abc""\x6587/bar", URL_UNESCAPE_AS_UTF8 },
+    { L"file://foo/%E4B8%AD/bar", L"file://foo/\xfffd""B8\xfffd/bar", URL_UNESCAPE_AS_UTF8 },
+    { L"file://foo/%E4%G8%AD/bar", L"file://foo/\xfffd""%G8\xfffd/bar", URL_UNESCAPE_AS_UTF8 },
+    { L"file://foo/%G4%B8%AD/bar", L"file://foo/%G4\xfffd\xfffd/bar", URL_UNESCAPE_AS_UTF8 },
+};
 
 static const  struct {
     const char *path;
@@ -1391,17 +1417,15 @@ static void test_UrlIs(void)
 
 static void test_UrlUnescape(void)
 {
+    WCHAR urlW[INTERNET_MAX_URL_LENGTH], bufferW[INTERNET_MAX_URL_LENGTH];
     CHAR szReturnUrl[INTERNET_MAX_URL_LENGTH];
-    WCHAR ret_urlW[INTERNET_MAX_URL_LENGTH];
-    WCHAR *urlW, *expected_urlW;
-    DWORD dwEscaped;
-    size_t i;
+    DWORD dwEscaped, unescaped;
+    BOOL utf8_support = TRUE;
     static char inplace[] = "file:///C:/Program%20Files";
     static char another_inplace[] = "file:///C:/Program%20Files";
     static const char expected[] = "file:///C:/Program Files";
-    static WCHAR inplaceW[] = L"file:///C:/Program Files";
-    static WCHAR another_inplaceW[] = L"file:///C:/Program%20Files";
     HRESULT res;
+    int i;
 
     for (i = 0; i < ARRAY_SIZE(TEST_URL_UNESCAPE); i++) {
         dwEscaped=INTERNET_MAX_URL_LENGTH;
@@ -1418,21 +1442,43 @@ static void test_UrlUnescape(void)
             "UrlUnescapeA returned 0x%lx (expected E_INVALIDARG) for \"%s\"\n",
             res, TEST_URL_UNESCAPE[i].url);
         ok(strcmp(szReturnUrl,"")==0, "Expected empty string\n");
+    }
 
-        dwEscaped = INTERNET_MAX_URL_LENGTH;
-        urlW = GetWideString(TEST_URL_UNESCAPE[i].url);
-        expected_urlW = GetWideString(TEST_URL_UNESCAPE[i].expect);
-        res = UrlUnescapeW(urlW, ret_urlW, &dwEscaped, 0);
-        ok(res == S_OK,
-            "UrlUnescapeW returned 0x%lx (expected S_OK) for \"%s\"\n",
-            res, TEST_URL_UNESCAPE[i].url);
+    unescaped = INTERNET_MAX_URL_LENGTH;
+    lstrcpyW(urlW, L"%F0%9F%8D%B7");
+    res = UrlUnescapeW(urlW, NULL, &unescaped, URL_UNESCAPE_AS_UTF8 | URL_UNESCAPE_INPLACE);
+    ok(res == S_OK, "Got %#lx.\n", res);
+    if (!wcscmp(urlW, L"\xf0\x9f\x8d\xb7"))
+    {
+        utf8_support = FALSE;
+        win_skip("Skip URL_UNESCAPE_AS_UTF8 tests for pre-win7 systems.\n");
+    }
 
-        WideCharToMultiByte(CP_ACP,0,ret_urlW,-1,szReturnUrl,INTERNET_MAX_URL_LENGTH,0,0);
-        ok(lstrcmpW(ret_urlW, expected_urlW)==0,
-            "Expected \"%s\", but got \"%s\" from \"%s\" flags %08lx\n",
-            TEST_URL_UNESCAPE[i].expect, szReturnUrl, TEST_URL_UNESCAPE[i].url, 0L);
-        FreeWideString(urlW);
-        FreeWideString(expected_urlW);
+    for (i = 0; i < ARRAYSIZE(TEST_URL_UNESCAPEW); i++)
+    {
+        if (TEST_URL_UNESCAPEW[i].flags & URL_UNESCAPE_AS_UTF8 && !utf8_support)
+            continue;
+
+        lstrcpyW(urlW, TEST_URL_UNESCAPEW[i].url);
+
+        memset(bufferW, 0xff, sizeof(bufferW));
+        unescaped = INTERNET_MAX_URL_LENGTH;
+        res = UrlUnescapeW(urlW, bufferW, &unescaped, TEST_URL_UNESCAPEW[i].flags);
+        ok(res == S_OK, "[%d]: returned %#lx.\n", i, res);
+        ok(unescaped == wcslen(TEST_URL_UNESCAPEW[i].expect), "[%d]: got unescaped %ld.\n", i, unescaped);
+        ok(!wcscmp(bufferW, TEST_URL_UNESCAPEW[i].expect), "[%d]: got result %s.\n", i, debugstr_w(bufferW));
+
+        /* Test with URL_UNESCAPE_INPLACE */
+        unescaped = INTERNET_MAX_URL_LENGTH;
+        res = UrlUnescapeW(urlW, NULL, &unescaped, TEST_URL_UNESCAPEW[i].flags | URL_UNESCAPE_INPLACE);
+        ok(res == S_OK, "[%d]: returned %#lx.\n", i, res);
+        ok(unescaped == INTERNET_MAX_URL_LENGTH, "[%d]: got unescaped %ld.\n", i, unescaped);
+        ok(!wcscmp(urlW, TEST_URL_UNESCAPEW[i].expect), "[%d]: got result %s.\n", i, debugstr_w(urlW));
+
+        lstrcpyW(urlW, TEST_URL_UNESCAPEW[i].url);
+        unescaped = wcslen(TEST_URL_UNESCAPEW[i].expect) - 1;
+        res = UrlUnescapeW(urlW, bufferW, &unescaped, TEST_URL_UNESCAPEW[i].flags);
+        ok(res == E_POINTER, "[%d]: returned %#lx.\n", i, res);
     }
 
     dwEscaped = sizeof(inplace);
@@ -1445,17 +1491,6 @@ static void test_UrlUnescape(void)
     res = UrlUnescapeA(another_inplace, NULL, NULL, URL_UNESCAPE_INPLACE);
     ok(res == S_OK, "UrlUnescapeA returned 0x%lx (expected S_OK)\n", res);
     ok(!strcmp(another_inplace, expected), "got %s expected %s\n", another_inplace, expected);
-
-    dwEscaped = sizeof(inplaceW);
-    res = UrlUnescapeW(inplaceW, NULL, &dwEscaped, URL_UNESCAPE_INPLACE);
-    ok(res == S_OK, "UrlUnescapeW returned 0x%lx (expected S_OK)\n", res);
-    ok(dwEscaped == 50, "got %ld expected 50\n", dwEscaped);
-
-    /* if we set the buffer pointer to NULL, the string apparently still gets converted (Google Lively does this) */
-    res = UrlUnescapeW(another_inplaceW, NULL, NULL, URL_UNESCAPE_INPLACE);
-    ok(res == S_OK, "UrlUnescapeW returned 0x%lx (expected S_OK)\n", res);
-
-    ok(lstrlenW(another_inplaceW) == 24, "got %d expected 24\n", lstrlenW(another_inplaceW));
 }
 
 static const struct parse_url_test_t {
