@@ -66,7 +66,7 @@ static void __cdecl test_invalid_parameter_handler(const wchar_t *expression,
     ok(function == NULL, "function is not NULL\n");
     ok(file == NULL, "file is not NULL\n");
     ok(line == 0, "line = %u\n", line);
-    ok(arg == 0, "arg = %lx\n", (UINT_PTR)arg);
+    ok(arg == 0, "arg = %Ix\n", arg);
 }
 
 _ACRTIMP int __cdecl _o_tolower(int);
@@ -152,6 +152,42 @@ static void test_strtod(void)
     test_strtod_str_errno("2.47e-324", 0, 9, ERANGE);
 }
 
+static void test_strtof(void)
+{
+    static const struct {
+        const char *str;
+        int len;
+        float ret;
+        int err;
+    } tests[] = {
+        { "12.1", 4, 12.1f },
+        { "-13.721", 7, -13.721f },
+        { "1.e40", 5, INFINITY, ERANGE },
+        { "-1.e40", 6, -INFINITY, ERANGE },
+        { "0.0", 3, 0.0f },
+        { "-0.0", 4, 0.0f },
+        { "1.4e-45", 7, 1.4e-45f },
+        { "-1.4e-45", 8, -1.4e-45f },
+        { "1.e-60", 6, 0, ERANGE },
+        { "-1.e-60", 7, 0, ERANGE },
+    };
+
+    char *end;
+    float f;
+    int i;
+
+    for (i=0; i<ARRAY_SIZE(tests); i++)
+    {
+        errno = 0xdeadbeef;
+        f = strtof(tests[i].str, &end);
+        ok(f == tests[i].ret, "%d) f = %.16e\n", i, f);
+        ok(end == tests[i].str + tests[i].len, "%d) len = %d\n",
+                i, (int)(end - tests[i].str));
+        ok(errno == tests[i].err || (!tests[i].err && errno == 0xdeadbeef),
+                "%d) errno = %d\n", i, errno);
+    }
+}
+
 static void test__memicmp(void)
 {
     static const char *s1 = "abc";
@@ -444,15 +480,47 @@ static void test__strnicmp(void)
     static const char str2[] = "test";
     int ret;
 
+    SET_EXPECT(invalid_parameter_handler);
+    errno = 0xdeadbeef;
+    ret = _strnicmp(str1, NULL, 2);
+    CHECK_CALLED(invalid_parameter_handler);
+    ok(ret == _NLSCMPERROR, "got %d.\n", ret);
+    ok(errno == EINVAL, "Unexpected errno %d.\n", errno);
+
     SET_EXPECT(invalid_parameter_handler);
     errno = 0xdeadbeef;
     ret = _strnicmp(str1, str2, -1);
-    todo_wine CHECK_CALLED(invalid_parameter_handler);
-    todo_wine ok(ret == _NLSCMPERROR, "got %d.\n", ret);
-    todo_wine ok(errno == EINVAL, "Unexpected errno %d.\n", errno);
+    CHECK_CALLED(invalid_parameter_handler);
+    ok(ret == _NLSCMPERROR, "got %d.\n", ret);
+    ok(errno == EINVAL, "Unexpected errno %d.\n", errno);
+
+    ret = _strnicmp(str1, str2, 0);
+    ok(!ret, "got %d.\n", ret);
 
     ret = _strnicmp(str1, str2, 0x7fffffff);
     ok(!ret, "got %d.\n", ret);
+
+    /* If numbers of characters to compare is too big return error */
+    SET_EXPECT(invalid_parameter_handler);
+    errno = 0xdeadbeef;
+    ret = _strnicmp(str1, str2, 0x80000000);
+    CHECK_CALLED(invalid_parameter_handler);
+    ok(ret == _NLSCMPERROR, "got %d.\n", ret);
+    ok(errno == EINVAL, "Unexpected errno %d.\n", errno);
+}
+
+static void test_wcsnicmp(void)
+{
+    static const wchar_t str1[] = L"TEST";
+    static const wchar_t str2[] = L"test";
+    int ret;
+
+    errno = 0xdeadbeef;
+    ret = wcsnicmp(str1, str2, -1);
+    ok(!ret, "got %d.\n", ret);
+
+    ret = wcsnicmp(str1, str2, 0x7fffffff);
+    ok(!ret, "got %d.\n", ret);
 }
 
 static void test_SpecialCasing(void)
@@ -552,12 +620,44 @@ static void test__mbbtype_l(void)
     }
 }
 
+static void test_strcmp(void)
+{
+    int ret = strcmp( "abc", "abcd" );
+    ok( ret == -1, "wrong ret %d\n", ret );
+    ret = strcmp( "", "abc" );
+    ok( ret == -1, "wrong ret %d\n", ret );
+    ret = strcmp( "abc", "ab\xa0" );
+    ok( ret == -1, "wrong ret %d\n", ret );
+    ret = strcmp( "ab\xb0", "ab\xa0" );
+    ok( ret == 1, "wrong ret %d\n", ret );
+    ret = strcmp( "ab\xc2", "ab\xc2" );
+    ok( ret == 0, "wrong ret %d\n", ret );
+
+    ret = strncmp( "abc", "abcd", 3 );
+    ok( ret == 0, "wrong ret %d\n", ret );
+    ret = strncmp( "", "abc", 3 );
+    ok( ret == -1, "wrong ret %d\n", ret );
+    ret = strncmp( "abc", "ab\xa0", 4 );
+    ok( ret == -1, "wrong ret %d\n", ret );
+    ret = strncmp( "ab\xb0", "ab\xa0", 3 );
+    ok( ret == 1, "wrong ret %d\n", ret );
+    ret = strncmp( "ab\xb0", "ab\xa0", 2 );
+    ok( ret == 0, "wrong ret %d\n", ret );
+    ret = strncmp( "ab\xc2", "ab\xc2", 3 );
+    ok( ret == 0, "wrong ret %d\n", ret );
+    ret = strncmp( "abc", "abd", 0 );
+    ok( ret == 0, "wrong ret %d\n", ret );
+    ret = strncmp( "abc", "abc", 12 );
+    ok( ret == 0, "wrong ret %d\n", ret );
+}
+
 START_TEST(string)
 {
     ok(_set_invalid_parameter_handler(test_invalid_parameter_handler) == NULL,
             "Invalid parameter handler was already set\n");
 
     test_strtod();
+    test_strtof();
     test__memicmp();
     test__memicmp_l();
     test___strncnt();
@@ -565,6 +665,8 @@ START_TEST(string)
     test_mbsspn();
     test_wcstok();
     test__strnicmp();
+    test_wcsnicmp();
     test_SpecialCasing();
     test__mbbtype_l();
+    test_strcmp();
 }
