@@ -1184,6 +1184,9 @@ static HRESULT Global_LBound(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
     case VT_VARIANT|VT_ARRAY|VT_BYREF:
         sa = *V_ARRAYREF(arg);
         break;
+    case VT_EMPTY:
+    case VT_NULL:
+        return MAKE_VBSERROR(VBSE_TYPE_MISMATCH);
     default:
         FIXME("arg %s not supported\n", debugstr_variant(arg));
         return E_NOTIMPL;
@@ -1222,6 +1225,9 @@ static HRESULT Global_UBound(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
     case VT_VARIANT|VT_ARRAY|VT_BYREF:
         sa = *V_ARRAYREF(arg);
         break;
+    case VT_EMPTY:
+    case VT_NULL:
+        return MAKE_VBSERROR(VBSE_TYPE_MISMATCH);
     default:
         FIXME("arg %s not supported\n", debugstr_variant(arg));
         return E_NOTIMPL;
@@ -2172,6 +2178,12 @@ static HRESULT Global_Second(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt,
     return FAILED(hres) ? hres : return_short(res, st.wSecond);
 }
 
+static HRESULT Global_SetLocale(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
+{
+    FIXME("\n");
+    return E_NOTIMPL;
+}
+
 static HRESULT Global_DateValue(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
 {
     FIXME("\n");
@@ -2493,6 +2505,8 @@ static HRESULT Global_TypeName(BuiltinDisp *This, VARIANT *arg, unsigned args_cn
         case VT_NULL:
             return return_string(res, L"Null");
         case VT_DISPATCH:
+            if (!V_DISPATCH(arg))
+                return return_string(res, L"Nothing");
             if (SUCCEEDED(IDispatch_GetTypeInfo(V_DISPATCH(arg), 0, GetUserDefaultLCID(), &typeinfo)))
             {
                 hres = ITypeInfo_GetDocumentation(typeinfo, MEMBERID_NIL, &name, NULL, NULL, NULL);
@@ -2572,12 +2586,12 @@ static HRESULT Global_Join(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, V
 
 static HRESULT Global_Split(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    BSTR str, string, delimiter = NULL;
+    BSTR string, delimiter = NULL;
     int count, max, mode, len, start, end, ret, delimiterlen = 1;
     int i, *indices = NULL, *new_indices, indices_max = 8;
     SAFEARRAYBOUND bounds;
     SAFEARRAY *sa = NULL;
-    VARIANT *data, var;
+    VARIANT *data;
     HRESULT hres = S_OK;
 
     TRACE("%s %u...\n", debugstr_variant(args), args_cnt);
@@ -2685,19 +2699,13 @@ static HRESULT Global_Split(BuiltinDisp *This, VARIANT *args, unsigned args_cnt,
 
     start = 0;
     for (i = 0; i < count; i++) {
-        str = SysAllocStringLen(string + start, indices[i] - start);
-        if (!str) {
+        V_VT(&data[i]) = VT_BSTR;
+        V_BSTR(&data[i]) = SysAllocStringLen(string + start, indices[i] - start);
+
+        if (!V_BSTR(&data[i])) {
             hres = E_OUTOFMEMORY;
             break;
         }
-        V_VT(&var) = VT_BSTR;
-        V_BSTR(&var) = str;
-
-        hres = VariantCopyInd(data+i, &var);
-        if(FAILED(hres)) {
-            SafeArrayUnaccessData(sa);
-            goto error;
-        }
         start = indices[i]+delimiterlen;
     }
     SafeArrayUnaccessData(sa);
@@ -3048,6 +3056,12 @@ static HRESULT Global_FormatPercent(BuiltinDisp *This, VARIANT *args, unsigned a
     return return_bstr(res, str);
 }
 
+static HRESULT Global_GetLocale(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
+{
+    FIXME("\n");
+    return E_NOTIMPL;
+}
+
 static HRESULT Global_FormatDateTime(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
     int format = 0;
@@ -3139,32 +3153,47 @@ static HRESULT Global_MonthName(BuiltinDisp *This, VARIANT *args, unsigned args_
     return return_bstr(res, ret);
 }
 
-static HRESULT Global_Round(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
+static HRESULT Global_Round(BuiltinDisp *This, VARIANT *args, unsigned args_cnt, VARIANT *res)
 {
-    double n;
+    int decimal_places = 0;
+    double d;
     HRESULT hres;
 
-    TRACE("%s\n", debugstr_variant(arg));
+    TRACE("%s %s\n", debugstr_variant(args), args_cnt == 2 ? debugstr_variant(args + 1) : "0");
+
+    assert(args_cnt == 1 || args_cnt == 2);
 
     if(!res)
         return S_OK;
 
-    switch(V_VT(arg)) {
+    if(args_cnt == 2) {
+       if (V_VT(args + 1) != VT_ERROR) {
+           hres = to_int(args + 1, &decimal_places);
+           if (FAILED(hres))
+              return hres;
+       }
+    }
+
+    switch(V_VT(args)) {
     case VT_I2:
     case VT_I4:
     case VT_BOOL:
-        *res = *arg;
+        *res = *args;
         return S_OK;
     case VT_R8:
-        n = V_R8(arg);
+        d = V_R8(args);
         break;
     default:
-        hres = to_double(arg, &n);
+        hres = to_double(args, &d);
         if(FAILED(hres))
             return hres;
     }
 
-    return return_double(res, round(n));
+    hres = VarR8Round(d, decimal_places, &d);
+    if(FAILED(hres))
+        return hres;
+
+    return return_double(res, d);
 }
 
 static HRESULT Global_Escape(BuiltinDisp *This, VARIANT *arg, unsigned args_cnt, VARIANT *res)
@@ -3270,6 +3299,7 @@ static const builtin_prop_t global_props[] = {
     {L"FormatDateTime",            Global_FormatDateTime, 0, 1, 2},
     {L"FormatNumber",              Global_FormatNumber, 0, 1, 5},
     {L"FormatPercent",             Global_FormatPercent, 0, 1, 5},
+    {L"GetLocale",                 Global_GetLocale, 0, 0},
     {L"GetObject",                 Global_GetObject, 0, 0, 2},
     {L"GetRef",                    Global_GetRef, 0, 1},
     {L"Hex",                       Global_Hex, 0, 1},
@@ -3316,6 +3346,7 @@ static const builtin_prop_t global_props[] = {
     {L"ScriptEngineMajorVersion",  Global_ScriptEngineMajorVersion, 0, 0},
     {L"ScriptEngineMinorVersion",  Global_ScriptEngineMinorVersion, 0, 0},
     {L"Second",                    Global_Second, 0, 1},
+    {L"SetLocale",                 Global_SetLocale, 0, 0, 1},
     {L"Sgn",                       Global_Sgn, 0, 1},
     {L"Sin",                       Global_Sin, 0, 1},
     {L"Space",                     Global_Space, 0, 1},
