@@ -52,7 +52,7 @@ static HRESULT WINAPI client_security_QueryInterface(
     if ( IsEqualGUID( riid, &IID_IClientSecurity ) ||
          IsEqualGUID( riid, &IID_IUnknown ) )
     {
-        *ppvObject = cs;
+        *ppvObject = &cs->IClientSecurity_iface;
     }
     else
     {
@@ -264,7 +264,7 @@ static HRESULT WINAPI wbem_services_QueryInterface(
     if ( IsEqualGUID( riid, &IID_IWbemServices ) ||
          IsEqualGUID( riid, &IID_IUnknown ) )
     {
-        *ppvObject = ws;
+        *ppvObject = &ws->IWbemServices_iface;
     }
     else if ( IsEqualGUID( riid, &IID_IClientSecurity ) )
     {
@@ -1058,7 +1058,7 @@ static HRESULT WINAPI wbem_context_Clone(
 
     TRACE("%p, %p\n", iface, newcopy);
 
-    if (SUCCEEDED(hr = WbemContext_create( (void **)&cloned_context )))
+    if (SUCCEEDED(hr = WbemContext_create( (void **)&cloned_context, &IID_IWbemContext )))
     {
         LIST_FOR_EACH_ENTRY( value, &context->values, struct wbem_context_value, entry )
         {
@@ -1225,12 +1225,16 @@ static const IWbemContextVtbl wbem_context_vtbl =
     wbem_context_DeleteAll,
 };
 
-HRESULT WbemContext_create( void **obj )
+HRESULT WbemContext_create( void **obj, REFIID riid )
 {
     struct wbem_context *context;
 
     TRACE("(%p)\n", obj);
 
+    if ( !IsEqualGUID( riid, &IID_IWbemContext ) &&
+         !IsEqualGUID( riid, &IID_IUnknown ) )
+        return E_NOINTERFACE;
+
     if (!(context = malloc( sizeof(*context) ))) return E_OUTOFMEMORY;
 
     context->IWbemContext_iface.lpVtbl = &wbem_context_vtbl;
