@@ -108,7 +108,7 @@ static WINDOWPROC *find_winproc( WNDPROC func, BOOL ansi )
 
 /* return the window proc for a given handle, or NULL for an invalid handle,
  * or WINPROC_PROC16 for a handle to a 16-bit proc. */
-WINDOWPROC *get_winproc_ptr( WNDPROC handle )
+static WINDOWPROC *get_winproc_ptr( WNDPROC handle )
 {
     UINT index = LOWORD(handle);
     if ((ULONG_PTR)handle >> 16 != WINPROC_HANDLE) return NULL;
@@ -249,8 +249,8 @@ DLGPROC get_dialog_proc( DLGPROC ret, BOOL ansi )
 static void init_user(void)
 {
     gdi_init();
-    winstation_init();
     sysparams_init();
+    winstation_init();
     register_desktop_class();
 }
 
@@ -1125,14 +1125,6 @@ static const struct builtin_class_descr builtin_classes[] =
         .extra = DLGWINDOWEXTRA,
         .cursor = IDC_ARROW,
     },
-    /* edit */
-    {
-        .name = "Edit",
-        .style = CS_DBLCLKS | CS_PARENTDC,
-        .proc = WINPROC_EDIT,
-        .extra = sizeof(UINT64),
-        .cursor = IDC_IBEAM,
-    },
     /* icon title */
     {
         .name = MAKEINTRESOURCEA(ICONTITLE_CLASS_ATOM),
@@ -1233,7 +1225,19 @@ static void register_builtins(void)
     ULONG ret_len, i;
     void *ret_ptr;
 
+    /* 64-bit Windows use sizeof(UINT64) for all processes, while 32-bit Windows use 6 for extra
+     * bytes size. Civilization II depends on the size being 6, so we use that even in wow64. */
+    const struct builtin_class_descr edit_class =
+    {
+        .name = "Edit",
+        .style = CS_DBLCLKS | CS_PARENTDC,
+        .proc = WINPROC_EDIT,
+        .extra = sizeof(void *) == 4 || NtCurrentTeb()->WowTebOffset ? 6 : sizeof(UINT64),
+        .cursor = IDC_IBEAM,
+    };
+
     for (i = 0; i < ARRAYSIZE(builtin_classes); i++) register_builtin( &builtin_classes[i] );
+    register_builtin( &edit_class );
     KeUserModeCallback( NtUserInitBuiltinClasses, NULL, 0, &ret_ptr, &ret_len );
 }
 
