@@ -91,12 +91,17 @@ HHOOK WINAPI NtUserSetWindowsHookEx( HINSTANCE inst, UNICODE_STRING *module, DWO
             id == WH_SYSMSGFILTER)
         {
             /* these can only be global */
-            RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
+            RtlSetLastWin32Error( ERROR_GLOBAL_ONLY_HOOK );
             return 0;
         }
     }
     else  /* system-global hook */
     {
+        if (id == WH_JOURNALRECORD || id == WH_JOURNALPLAYBACK)
+        {
+            RtlSetLastWin32Error( ERROR_ACCESS_DENIED );
+            return 0;
+        }
         if (id == WH_KEYBOARD_LL || id == WH_MOUSE_LL) inst = 0;
         else if (!inst)
         {
@@ -195,7 +200,8 @@ static UINT get_ll_hook_timeout(void)
  * Call hook either in current thread or send message to the destination
  * thread.
  */
-static LRESULT call_hook( struct win_hook_params *info, const WCHAR *module )
+static LRESULT call_hook( struct win_hook_params *info, const WCHAR *module, size_t lparam_size,
+                          size_t message_size, BOOL ansi )
 {
     DWORD_PTR ret = 0;
 
@@ -230,72 +236,82 @@ static LRESULT call_hook( struct win_hook_params *info, const WCHAR *module )
     else if (info->proc)
     {
         struct user_thread_info *thread_info = get_user_thread_info();
+        size_t size, lparam_offset = 0, message_offset = 0;
+        size_t lparam_ret_size = lparam_size;
         HHOOK prev = thread_info->hook;
         BOOL prev_unicode = thread_info->hook_unicode;
         struct win_hook_params *params = info;
-        ULONG size = sizeof(*params);
-        ULONG lparam_ret_size = params->lparam_size;
-        CREATESTRUCTW *cs = NULL;
+        size_t reply_size;
         void *ret_ptr;
         ULONG ret_len;
 
-        if (params->lparam_size)
-        {
-            lparam_ret_size = params->lparam_size;
+        size = FIELD_OFFSET( struct win_hook_params, module[module ? lstrlenW( module ) + 1 : 1] );
 
+        if (lparam_size)
+        {
             if (params->id == WH_CBT && params->code == HCBT_CREATEWND)
             {
-                cs = ((CBT_CREATEWNDW *)params->lparam)->lpcs;
-                params->lparam = 0;
-                lparam_ret_size = 0;
-                params->lparam_size = sizeof(*cs);
-                if (!IS_INTRESOURCE( cs->lpszName ))
-                    params->lparam_size += (wcslen( cs->lpszName ) + 1) * sizeof(WCHAR);
-                if (!IS_INTRESOURCE( cs->lpszClass ))
-                    params->lparam_size += (wcslen( cs->lpszClass ) + 1) * sizeof(WCHAR);
+                CBT_CREATEWNDW *cbtc = (CBT_CREATEWNDW *)params->lparam;
+                message_size = user_message_size( (HWND)params->wparam, WM_NCCREATE,
+                                                  0, (LPARAM)cbtc->lpcs, TRUE, FALSE, &reply_size );
+                lparam_size = lparam_ret_size = 0;
             }
 
-            size += params->lparam_size;
+            if (lparam_size)
+            {
+                lparam_offset = (size + 15) & ~15; /* align offset */
+                size = lparam_offset + lparam_size;
+            }
+
+            if (message_size)
+            {
+                message_offset = (size + 15) & ~15; /* align offset */
+                size = message_offset + message_size;
+            }
         }
 
-        if (module && module[0]) size += (lstrlenW( module ) + 1) * sizeof(WCHAR);
-        if (size != sizeof(*params))
+        if (size > sizeof(*info))
         {
             if (!(params = malloc( size ))) return 0;
-            *params = *info;
+            memcpy( params, info, FIELD_OFFSET( struct win_hook_params, module ));
         }
+        if (module)
+            wcscpy( params->module, module );
+        else
+            params->module[0] = 0;
 
-        if (params->lparam_size)
+        if (lparam_size)
+            memcpy( (char *)params + lparam_offset, (const void *)params->lparam, lparam_size );
+
+        if (message_size)
         {
-            if (cs)
+            switch (params->id)
             {
-                CREATESTRUCTW *params_cs = (CREATESTRUCTW *)(params + 1);
-                WCHAR *ptr = (WCHAR *)(params_cs + 1);
-                const void *inline_ptr = (void *)0xffffffff;
-
-                *params_cs = *cs;
-                if (!IS_INTRESOURCE( cs->lpszName ))
+            case WH_CBT:
                 {
-                    UINT len = wcslen( cs->lpszName ) + 1;
-                    memcpy( ptr, cs->lpszName, len * sizeof(WCHAR) );
-                    ptr += len;
-                    params_cs->lpszName = inline_ptr;
+                    CBT_CREATEWNDW *cbtc = (CBT_CREATEWNDW *)params->lparam;
+                    LPARAM lp = (LPARAM)cbtc->lpcs;
+                    pack_user_message( (char *)params + message_offset, message_size,
+                                       WM_CREATE, 0, lp, FALSE );
                 }
-                if (!IS_INTRESOURCE( cs->lpszClass ))
+                break;
+            case WH_CALLWNDPROC:
                 {
-                    wcscpy( ptr, cs->lpszClass );
-                    params_cs->lpszClass = inline_ptr;
+                    CWPSTRUCT *cwp = (CWPSTRUCT *)((char *)params + lparam_offset);
+                    pack_user_message( (char *)params + message_offset, message_size,
+                                       cwp->message, cwp->wParam, cwp->lParam, ansi );
                 }
-            }
-            else
-            {
-                memcpy( params + 1, (const void *)params->lparam, params->lparam_size );
+                break;
+            case WH_CALLWNDPROCRET:
+                {
+                    CWPRETSTRUCT *cwpret = (CWPRETSTRUCT *)((char *)params + lparam_offset);
+                    pack_user_message( (char *)params + message_offset, message_size,
+                                       cwpret->message, cwpret->wParam, cwpret->lParam, ansi );
+                }
+                break;
             }
         }
 
-        if (module && module[0])
-            wcscpy( (WCHAR *)((char *)(params + 1) + params->lparam_size), module );
-
         /*
          * Windows protects from stack overflow in recursive hook calls. Different Windows
          * allow different depths.
@@ -316,7 +332,7 @@ static LRESULT call_hook( struct win_hook_params *info, const WCHAR *module )
         thread_info->hook_call_depth++;
         ret = KeUserModeCallback( NtUserCallWindowsHook, params, size, &ret_ptr, &ret_len );
         if (ret_len && ret_len == lparam_ret_size)
-            memcpy( (void *)params->lparam, ret_ptr, lparam_ret_size );
+            memcpy( (void *)params->lparam, ret_ptr, ret_len );
         thread_info->hook = prev;
         thread_info->hook_unicode = prev_unicode;
         thread_info->hook_call_depth--;
@@ -363,7 +379,7 @@ LRESULT WINAPI NtUserCallNextHookEx( HHOOK hhook, INT code, WPARAM wparam, LPARA
     info.wparam = wparam;
     info.lparam = lparam;
     info.prev_unicode = thread_info->hook_unicode;
-    return call_hook( &info, module );
+    return call_hook( &info, module, 0, 0, FALSE );
 }
 
 LRESULT call_current_hook( HHOOK hhook, INT code, WPARAM wparam, LPARAM lparam )
@@ -396,10 +412,11 @@ LRESULT call_current_hook( HHOOK hhook, INT code, WPARAM wparam, LPARAM lparam )
     info.wparam = wparam;
     info.lparam = lparam;
     info.prev_unicode = TRUE;  /* assume Unicode for this function */
-    return call_hook( &info, module );
+    return call_hook( &info, module, 0, 0, FALSE );
 }
 
-LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, size_t lparam_size )
+LRESULT call_message_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, size_t lparam_size,
+                            size_t message_size, BOOL ansi )
 {
     struct user_thread_info *thread_info = get_user_thread_info();
     struct win_hook_params info;
@@ -440,8 +457,7 @@ LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, size_t lpara
     info.code   = code;
     info.wparam = wparam;
     info.lparam = lparam;
-    info.lparam_size = lparam_size;
-    ret = call_hook( &info, module );
+    ret = call_hook( &info, module, lparam_size, message_size, ansi );
 
     SERVER_START_REQ( finish_hook_chain )
     {
@@ -452,6 +468,11 @@ LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, size_t lpara
     return ret;
 }
 
+LRESULT call_hooks( INT id, INT code, WPARAM wparam, LPARAM lparam, size_t lparam_size )
+{
+    return call_message_hooks( id, code, wparam, lparam, lparam_size, 0, FALSE );
+}
+
 /***********************************************************************
  *           NtUserSetWinEventHook   (win32u.@)
  */
