@@ -109,37 +109,79 @@ static ULONG WINAPI effect_impl_Release( IWineForceFeedbackEffectImpl *iface )
     return ref;
 }
 
+static int effect_reorient_direction( const WineForceFeedbackEffectParameters *params, Vector3 *direction )
+{
+    int sign = +1;
+
+    switch (params->type)
+    {
+    case WineForceFeedbackEffectType_Constant:
+        *direction = params->constant.direction;
+        sign = params->constant.direction.X < 0 ? -1 : +1;
+        break;
+
+    case WineForceFeedbackEffectType_Ramp:
+        *direction = params->ramp.start_vector;
+        sign = params->ramp.start_vector.X < 0 ? -1 : +1;
+        break;
+
+    case WineForceFeedbackEffectType_Periodic_SineWave:
+    case WineForceFeedbackEffectType_Periodic_TriangleWave:
+    case WineForceFeedbackEffectType_Periodic_SquareWave:
+    case WineForceFeedbackEffectType_Periodic_SawtoothWaveDown:
+    case WineForceFeedbackEffectType_Periodic_SawtoothWaveUp:
+        *direction = params->periodic.direction;
+        break;
+
+    case WineForceFeedbackEffectType_Condition_Spring:
+    case WineForceFeedbackEffectType_Condition_Damper:
+    case WineForceFeedbackEffectType_Condition_Inertia:
+    case WineForceFeedbackEffectType_Condition_Friction:
+        *direction = params->condition.direction;
+        sign = -1;
+        break;
+    }
+
+    direction->X *= -sign;
+    direction->Y *= -sign;
+    direction->Z *= -sign;
+
+    return sign;
+}
+
 static HRESULT WINAPI effect_impl_put_Parameters( IWineForceFeedbackEffectImpl *iface, WineForceFeedbackEffectParameters params,
                                                   WineForceFeedbackEffectEnvelope *envelope )
 {
     struct effect *impl = impl_from_IWineForceFeedbackEffectImpl( iface );
+    Vector3 direction = {0};
+    double magnitude = 0;
     DWORD count = 0;
     HRESULT hr;
+    int sign;
 
     TRACE( "iface %p, params %p, envelope %p.\n", iface, &params, envelope );
 
     EnterCriticalSection( &impl->cs );
+
+    sign = effect_reorient_direction( &params, &direction );
+    /* Y and Z axes seems to be always ignored, is it really the case? */
+    magnitude += direction.X * direction.X;
+
     switch (params.type)
     {
     case WineForceFeedbackEffectType_Constant:
         impl->repeat_count = params.constant.repeat_count;
-        impl->constant_force.lMagnitude = round( params.constant.gain * params.constant.direction.X * 10000 );
-        impl->params.dwDuration = params.constant.duration.Duration / 10;
-        impl->params.dwStartDelay = params.constant.start_delay.Duration / 10;
-        if (impl->axes[count] == DIJOFS_X) impl->directions[count++] = round( -params.constant.direction.X * 10000 );
-        if (impl->axes[count] == DIJOFS_Y) impl->directions[count++] = round( -params.constant.direction.Y * 10000 );
-        if (impl->axes[count] == DIJOFS_Z) impl->directions[count++] = round( -params.constant.direction.Z * 10000 );
+        impl->constant_force.lMagnitude = sign * round( params.constant.gain * sqrt( magnitude ) * 10000 );
+        impl->params.dwDuration = min( max( params.constant.duration.Duration / 10, 0 ), INFINITE );
+        impl->params.dwStartDelay = min( max( params.constant.start_delay.Duration / 10, 0 ), INFINITE );
         break;
 
     case WineForceFeedbackEffectType_Ramp:
         impl->repeat_count = params.ramp.repeat_count;
-        impl->ramp_force.lStart = round( params.ramp.gain * params.ramp.start_vector.X * 10000 );
+        impl->ramp_force.lStart = sign * round( params.ramp.gain * sqrt( magnitude ) * 10000 );
         impl->ramp_force.lEnd = round( params.ramp.gain * params.ramp.end_vector.X * 10000 );
-        impl->params.dwDuration = params.ramp.duration.Duration / 10;
-        impl->params.dwStartDelay = params.ramp.start_delay.Duration / 10;
-        if (impl->axes[count] == DIJOFS_X) impl->directions[count++] = round( -params.ramp.start_vector.X * 10000 );
-        if (impl->axes[count] == DIJOFS_Y) impl->directions[count++] = round( -params.ramp.start_vector.Y * 10000 );
-        if (impl->axes[count] == DIJOFS_Z) impl->directions[count++] = round( -params.ramp.start_vector.Z * 10000 );
+        impl->params.dwDuration = min( max( params.ramp.duration.Duration / 10, 0 ), INFINITE );
+        impl->params.dwStartDelay = min( max( params.ramp.start_delay.Duration / 10, 0 ), INFINITE );
         break;
 
     case WineForceFeedbackEffectType_Periodic_SineWave:
@@ -152,11 +194,8 @@ static HRESULT WINAPI effect_impl_put_Parameters( IWineForceFeedbackEffectImpl *
         impl->periodic.dwPeriod = 1000000 / params.periodic.frequency;
         impl->periodic.dwPhase = round( params.periodic.phase * 36000 );
         impl->periodic.lOffset = round( params.periodic.bias * 10000 );
-        impl->params.dwDuration = params.periodic.duration.Duration / 10;
-        impl->params.dwStartDelay = params.periodic.start_delay.Duration / 10;
-        if (impl->axes[count] == DIJOFS_X) impl->directions[count++] = round( -params.periodic.direction.X * 10000 );
-        if (impl->axes[count] == DIJOFS_Y) impl->directions[count++] = round( -params.periodic.direction.Y * 10000 );
-        if (impl->axes[count] == DIJOFS_Z) impl->directions[count++] = round( -params.periodic.direction.Z * 10000 );
+        impl->params.dwDuration = min( max( params.periodic.duration.Duration / 10, 0 ), INFINITE );
+        impl->params.dwStartDelay = min( max( params.periodic.start_delay.Duration / 10, 0 ), INFINITE );
         break;
 
     case WineForceFeedbackEffectType_Condition_Spring:
@@ -172,18 +211,19 @@ static HRESULT WINAPI effect_impl_put_Parameters( IWineForceFeedbackEffectImpl *
         impl->condition.lOffset = round( params.condition.bias * 10000 );
         impl->params.dwDuration = -1;
         impl->params.dwStartDelay = 0;
-        if (impl->axes[count] == DIJOFS_X) impl->directions[count++] = round( params.condition.direction.X * 10000 );
-        if (impl->axes[count] == DIJOFS_Y) impl->directions[count++] = round( params.condition.direction.Y * 10000 );
-        if (impl->axes[count] == DIJOFS_Z) impl->directions[count++] = round( params.condition.direction.Z * 10000 );
         break;
     }
 
+    if (impl->axes[count] == DIJOFS_X) impl->directions[count++] = round( direction.X * 10000 );
+    if (impl->axes[count] == DIJOFS_Y) impl->directions[count++] = round( direction.Y * 10000 );
+    if (impl->axes[count] == DIJOFS_Z) impl->directions[count++] = round( direction.Z * 10000 );
+
     if (!envelope) impl->params.lpEnvelope = NULL;
     else
     {
-        impl->envelope.dwAttackTime = envelope->attack_duration.Duration / 10;
+        impl->envelope.dwAttackTime = min( max( envelope->attack_duration.Duration / 10, 0 ), INFINITE );
         impl->envelope.dwAttackLevel = round( envelope->attack_gain * 10000 );
-        impl->envelope.dwFadeTime = impl->params.dwDuration - envelope->release_duration.Duration / 10;
+        impl->envelope.dwFadeTime = impl->params.dwDuration - min( max( envelope->release_duration.Duration / 10, 0 ), INFINITE );
         impl->envelope.dwFadeLevel = round( envelope->release_gain * 10000 );
         impl->params.lpEnvelope = &impl->envelope;
     }
@@ -606,7 +646,6 @@ static HRESULT WINAPI motor_load_effect_async( IUnknown *invoker, IUnknown *para
     {
         if (effect->effect) IDirectInputEffect_Release( effect->effect );
         effect->effect = dinput_effect;
-        IDirectInputEffect_AddRef( effect->effect );
     }
 
     LeaveCriticalSection( &effect->cs );
