@@ -248,7 +248,7 @@ static struct dds_format {
     { { sizeof(DDS_PIXELFORMAT), DDPF_RGB,       0, 32, 0x3FF00000,0xFFC00,0x3FF,0xC0000000 },
       &GUID_WICPixelFormat32bppR10G10B10A2, 32, DXGI_FORMAT_R10G10B10A2_UNORM },
     { { sizeof(DDS_PIXELFORMAT), DDPF_RGB,       0, 32, 0x3FF,0xFFC00,0x3FF00000,0xC0000000 },
-      &GUID_WICPixelFormat32bppRGBA1010102, 32, DXGI_FORMAT_UNKNOWN },
+      &GUID_WICPixelFormat32bppRGBA1010102, 32, DXGI_FORMAT_R10G10B10A2_UNORM },
     { { sizeof(DDS_PIXELFORMAT), DDPF_RGB ,      0, 32, 0xFFFF,0xFFFF0000,0,0 },
       &GUID_WICPixelFormatUndefined,        0,  DXGI_FORMAT_R16G16_UNORM },
     { { sizeof(DDS_PIXELFORMAT), DDPF_RGB ,      0, 32, 0xFFFFFFFF,0,0,0 },
@@ -793,9 +793,9 @@ static ULONG WINAPI DdsFrameDecode_Release(IWICBitmapFrameDecode *iface)
     TRACE("(%p) refcount=%lu\n", iface, ref);
 
     if (ref == 0) {
-        if (This->pixel_data != This->block_data) HeapFree(GetProcessHeap(), 0, This->pixel_data);
-        HeapFree(GetProcessHeap(), 0, This->block_data);
-        HeapFree(GetProcessHeap(), 0, This);
+        if (This->pixel_data != This->block_data) free(This->pixel_data);
+        free(This->block_data);
+        free(This);
     }
 
     return ref;
@@ -884,7 +884,7 @@ static HRESULT WINAPI DdsFrameDecode_CopyPixels(IWICBitmapFrameDecode *iface,
 
     if (!This->pixel_data) {
         if (is_compressed(This->info.format)) {
-            This->pixel_data = HeapAlloc(GetProcessHeap(), 0, frame_size);
+            This->pixel_data = malloc(frame_size);
             if (!This->pixel_data) {
                 hr = E_OUTOFMEMORY;
                 goto end;
@@ -1045,7 +1045,7 @@ static HRESULT DdsFrameDecode_CreateInstance(DdsFrameDecode **frame_decode)
 {
     DdsFrameDecode *result;
 
-    result = HeapAlloc(GetProcessHeap(), 0, sizeof(*result));
+    result = malloc(sizeof(*result));
     if (!result) return E_OUTOFMEMORY;
 
     result->IWICBitmapFrameDecode_iface.lpVtbl = &DdsFrameDecode_Vtbl;
@@ -1104,7 +1104,7 @@ static ULONG WINAPI DdsDecoder_Release(IWICBitmapDecoder *iface)
         This->lock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->lock);
         if (This->stream) IStream_Release(This->stream);
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This);
     }
 
     return ref;
@@ -1403,7 +1403,7 @@ static HRESULT WINAPI DdsDecoder_Dds_GetFrame(IWICDdsDecoder *iface,
     frame_decode->info.height_in_blocks = frame_height_in_blocks;
     frame_decode->info.pixel_format = This->info.pixel_format;
     frame_decode->info.pixel_format_bpp = This->info.pixel_format_bpp;
-    frame_decode->block_data = HeapAlloc(GetProcessHeap(), 0, frame_size);
+    frame_decode->block_data = malloc(frame_size);
     frame_decode->pixel_data = NULL;
     hr = IStream_Seek(This->stream, seek, SEEK_SET, NULL);
     if (hr != S_OK) goto end;
@@ -1565,7 +1565,7 @@ static ULONG WINAPI DdsFrameEncode_Release(IWICBitmapFrameEncode *iface)
     if (ref == 0)
     {
         IWICBitmapEncoder_Release(&This->parent->IWICBitmapEncoder_iface);
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This);
     }
 
     return ref;
@@ -1752,7 +1752,7 @@ HRESULT DdsDecoder_CreateInstance(REFIID iid, void** ppv)
 
     *ppv = NULL;
 
-    This = HeapAlloc(GetProcessHeap(), 0, sizeof(DdsDecoder));
+    This = malloc(sizeof(DdsDecoder));
     if (!This) return E_OUTOFMEMORY;
 
     This->IWICBitmapDecoder_iface.lpVtbl = &DdsDecoder_Vtbl;
@@ -1884,7 +1884,7 @@ static HRESULT WINAPI DdsEncoder_Dds_CreateNewFrame(IWICDdsEncoder *iface,
         goto end;
     }
 
-    result = HeapAlloc(GetProcessHeap(), 0, sizeof(*result));
+    result = malloc(sizeof(*result));
     if (!result)
     {
         hr = E_OUTOFMEMORY;
@@ -1966,7 +1966,7 @@ static ULONG WINAPI DdsEncoder_Release(IWICBitmapEncoder *iface)
         This->lock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->lock);
         if (This->stream) IStream_Release(This->stream);
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This);
     }
 
     return ref;
@@ -2129,7 +2129,7 @@ HRESULT DdsEncoder_CreateInstance( REFIID iid, void **ppv)
 
     *ppv = NULL;
 
-    This = HeapAlloc(GetProcessHeap(), 0, sizeof(DdsEncoder));
+    This = malloc(sizeof(DdsEncoder));
     if (!This) return E_OUTOFMEMORY;
 
     This->IWICBitmapEncoder_iface.lpVtbl = &DdsEncoder_Vtbl;
