@@ -18,7 +18,6 @@
 
 #include <stdarg.h>
 
-#define NONAMELESSUNION
 #define COBJMACROS
 
 #include "windef.h"
@@ -93,7 +92,7 @@ static ULONG WINAPI CommonDecoder_Release(IWICBitmapDecoder *iface)
         This->lock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->lock);
         decoder_destroy(This->decoder);
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This);
     }
 
     return ref;
@@ -311,8 +310,8 @@ static ULONG WINAPI CommonDecoderFrame_Release(IWICBitmapFrameDecode *iface)
     if (ref == 0)
     {
         IWICBitmapDecoder_Release(&This->parent->IWICBitmapDecoder_iface);
-        HeapFree(GetProcessHeap(), 0, This->metadata_blocks);
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This->metadata_blocks);
+        free(This);
     }
 
     return ref;
@@ -494,7 +493,7 @@ static HRESULT WINAPI CommonDecoderFrame_GetColorContexts(IWICBitmapFrameDecode
                 {
                     hr = IWICColorContext_InitializeFromMemory(ppIColorContexts[i], profile, profile_len);
 
-                    HeapFree(GetProcessHeap(), 0, profile);
+                    free(profile);
                 }
 
                 if (FAILED(hr))
@@ -733,7 +732,7 @@ static HRESULT WINAPI CommonDecoder_GetFrame(IWICBitmapDecoder *iface,
 
     if (SUCCEEDED(hr))
     {
-        result = HeapAlloc(GetProcessHeap(), 0, sizeof(*result));
+        result = malloc(sizeof(*result));
         if (!result)
             hr = E_OUTOFMEMORY;
     }
@@ -755,7 +754,7 @@ static HRESULT WINAPI CommonDecoder_GetFrame(IWICBitmapDecoder *iface,
             hr = CommonDecoderFrame_InitializeMetadata(result);
 
         if (FAILED(hr))
-            HeapFree(GetProcessHeap(), 0, result);
+            free(result);
     }
 
     LeaveCriticalSection(&This->lock);
@@ -786,7 +785,7 @@ HRESULT CommonDecoder_CreateInstance(struct decoder *decoder,
 
     TRACE("(%s,%s,%p)\n", debugstr_guid(&decoder_info->clsid), debugstr_guid(iid), ppv);
 
-    This = HeapAlloc(GetProcessHeap(), 0, sizeof(*This));
+    This = malloc(sizeof(*This));
     if (!This)
     {
         decoder_destroy(decoder);
