@@ -31,9 +31,6 @@
 #include <sys/ioctl.h>
 #include <unistd.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
-
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
@@ -1138,17 +1135,17 @@ NTSTATUS android_dispatch_ioctl( void *arg )
         {
             irp->IoStatus.Information = 0;
             NtUserGetThreadInfo()->driver_data = params->client_id;
-            irp->IoStatus.u.Status = func( irp->AssociatedIrp.SystemBuffer, in_size,
-                                           irpsp->Parameters.DeviceIoControl.OutputBufferLength,
-                                           &irp->IoStatus.Information );
+            irp->IoStatus.Status = func( irp->AssociatedIrp.SystemBuffer, in_size,
+                                         irpsp->Parameters.DeviceIoControl.OutputBufferLength,
+                                         &irp->IoStatus.Information );
             NtUserGetThreadInfo()->driver_data = 0;
         }
-        else irp->IoStatus.u.Status = STATUS_INVALID_PARAMETER;
+        else irp->IoStatus.Status = STATUS_INVALID_PARAMETER;
     }
     else
     {
         FIXME( "ioctl %x not supported\n", (int)irpsp->Parameters.DeviceIoControl.IoControlCode );
-        irp->IoStatus.u.Status = STATUS_NOT_SUPPORTED;
+        irp->IoStatus.Status = STATUS_NOT_SUPPORTED;
     }
     return STATUS_SUCCESS;
 }
