@@ -21,6 +21,7 @@
 #include <stdio.h>
 
 #include "wined3d_private.h"
+#include "wined3d_gl.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_shader);
 WINE_DECLARE_DEBUG_CHANNEL(d3d);
@@ -49,7 +50,7 @@ struct atifs_ffp_desc
     struct ffp_frag_desc parent;
     GLuint shader;
     unsigned int num_textures_used;
-    enum atifs_constant_value constants[WINED3D_MAX_TEXTURES];
+    enum atifs_constant_value constants[WINED3D_MAX_FFP_TEXTURES];
 };
 
 struct atifs_private_data
@@ -319,15 +320,15 @@ static GLuint register_for_arg(unsigned int arg, const struct wined3d_gl_info *g
     return ret;
 }
 
-static GLuint find_tmpreg(const struct texture_stage_op op[WINED3D_MAX_TEXTURES])
+static GLuint find_tmpreg(const struct texture_stage_op op[WINED3D_MAX_FFP_TEXTURES])
 {
     int lowest_read = -1;
     int lowest_write = -1;
     int i;
-    BOOL tex_used[WINED3D_MAX_TEXTURES];
+    BOOL tex_used[WINED3D_MAX_FFP_TEXTURES];
 
     memset(tex_used, 0, sizeof(tex_used));
-    for (i = 0; i < WINED3D_MAX_TEXTURES; ++i)
+    for (i = 0; i < WINED3D_MAX_FFP_TEXTURES; ++i)
     {
         if (op[i].cop == WINED3D_TOP_DISABLE)
             break;
@@ -464,7 +465,7 @@ static BOOL op_reads_constant(const struct texture_stage_op *op)
             || (op->aarg2 & WINED3DTA_SELECTMASK) == WINED3DTA_CONSTANT;
 }
 
-static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_TEXTURES],
+static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_FFP_TEXTURES],
         const struct wined3d_gl_info *gl_info, enum atifs_constant_value *constants)
 {
     GLuint ret = GL_EXTCALL(glGenFragmentShadersATI(1));
@@ -489,7 +490,7 @@ static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_TEXTUR
     checkGLcall("GL_EXTCALL(glBeginFragmentShaderATI())");
 
     /* Pass 1: Generate sampling instructions for perturbation maps */
-    for (stage = 0; stage < gl_info->limits.textures; ++stage)
+    for (stage = 0; stage < gl_info->limits.ffp_textures; ++stage)
     {
         if (op[stage].cop == WINED3D_TOP_DISABLE)
             break;
@@ -516,7 +517,7 @@ static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_TEXTUR
     }
 
     /* Pass 2: Generate perturbation calculations */
-    for (stage = 0; stage < gl_info->limits.textures; ++stage)
+    for (stage = 0; stage < gl_info->limits.ffp_textures; ++stage)
     {
         GLuint argmodextra_x, argmodextra_y;
         struct color_fixup_desc fixup;
@@ -567,7 +568,7 @@ static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_TEXTUR
     }
 
     /* Pass 3: Generate sampling instructions for regular textures */
-    for (stage = 0; stage < gl_info->limits.textures; ++stage)
+    for (stage = 0; stage < gl_info->limits.ffp_textures; ++stage)
     {
         if (op[stage].cop == WINED3D_TOP_DISABLE)
             break;
@@ -601,7 +602,7 @@ static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_TEXTUR
     }
 
     /* Pass 4: Generate the arithmetic instructions */
-    for (stage = 0; stage < WINED3D_MAX_TEXTURES; ++stage)
+    for (stage = 0; stage < WINED3D_MAX_FFP_TEXTURES; ++stage)
     {
         if (op[stage].cop == WINED3D_TOP_DISABLE)
         {
@@ -933,7 +934,7 @@ static GLuint gen_ati_shader(const struct texture_stage_op op[WINED3D_MAX_TEXTUR
     constants[ATIFS_CONST_TFACTOR - GL_CON_0_ATI] = ATIFS_CONSTANT_TFACTOR;
 
     /* Assign unused constants to avoid reloading due to unused <-> bump matrix switches. */
-    for (stage = 0; stage < WINED3D_MAX_TEXTURES; ++stage)
+    for (stage = 0; stage < WINED3D_MAX_FFP_TEXTURES; ++stage)
     {
         if (constants[stage] == ATIFS_CONSTANT_UNUSED)
             constants[stage] = ATIFS_CONSTANT_BUMP;
@@ -1033,7 +1034,7 @@ static void set_tex_op_atifs(struct wined3d_context *context, const struct wined
             return;
         }
         new_desc->num_textures_used = 0;
-        for (i = 0; i < d3d_info->limits.ffp_blend_stages; ++i)
+        for (i = 0; i < d3d_info->ffp_fragment_caps.max_blend_stages; ++i)
         {
             if (settings.op[i].cop == WINED3D_TOP_DISABLE)
                 break;
@@ -1056,14 +1057,14 @@ static void set_tex_op_atifs(struct wined3d_context *context, const struct wined
         if (mapped_stage != WINED3D_UNMAPPED_STAGE)
         {
             wined3d_context_gl_active_texture(context_gl, gl_info, mapped_stage);
-            texture_activate_dimensions(state->textures[i], gl_info);
+            texture_activate_dimensions(wined3d_state_get_ffp_texture(state, i), gl_info);
         }
     }
 
     GL_EXTCALL(glBindFragmentShaderATI(desc->shader));
     ctx_priv->last_shader = desc;
 
-    for (i = 0; i < WINED3D_MAX_TEXTURES; i++)
+    for (i = 0; i < WINED3D_MAX_FFP_TEXTURES; ++i)
     {
         if (last_shader && last_shader->constants[i] == desc->constants[i])
             continue;
@@ -1268,7 +1269,8 @@ static void atifs_enable(const struct wined3d_context *context, BOOL enable)
 
 static void atifs_get_caps(const struct wined3d_adapter *adapter, struct fragment_caps *caps)
 {
-    caps->wined3d_caps = WINED3D_FRAGMENT_CAP_PROJ_CONTROL;
+    memset(caps, 0, sizeof(*caps));
+    caps->proj_control = true;
     caps->PrimitiveMiscCaps = WINED3DPMISCCAPS_TSSARGTEMP               |
             WINED3DPMISCCAPS_PERSTAGECONSTANT;
     caps->TextureOpCaps =  WINED3DTEXOPCAPS_DISABLE                     |
@@ -1311,11 +1313,11 @@ static void atifs_get_caps(const struct wined3d_adapter *adapter, struct fragmen
      * The proper fix for this is not to use GL_ATI_fragment_shader on cards newer than the
      * r200 series and use an ARB or GLSL shader instead
      */
-    caps->MaxTextureBlendStages   = WINED3D_MAX_TEXTURES;
-    caps->MaxSimultaneousTextures = 6;
+    caps->max_blend_stages = WINED3D_MAX_FFP_TEXTURES;
+    caps->max_textures = 6;
 }
 
-static unsigned int atifs_get_emul_mask(const struct wined3d_gl_info *gl_info)
+static unsigned int atifs_get_emul_mask(const struct wined3d_adapter *adapter)
 {
     return GL_EXT_EMUL_ARB_MULTITEXTURE | GL_EXT_EMUL_EXT_FOG_COORD;
 }
