@@ -22,8 +22,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_shader);
@@ -399,7 +397,7 @@ static const enum wined3d_shader_resource_type resource_type_table[] =
  * and possibly a relative addressing token.
  * Return the number of tokens read */
 static unsigned int shader_get_param(const struct wined3d_sm1_data *priv, const DWORD *ptr,
-        DWORD *token, DWORD *addr_token)
+        unsigned int *token, unsigned int *addr_token)
 {
     unsigned int count = 1;
 
@@ -428,9 +426,9 @@ static unsigned int shader_get_param(const struct wined3d_sm1_data *priv, const
     return count;
 }
 
-static const struct wined3d_sm1_opcode_info *shader_get_opcode(const struct wined3d_sm1_data *priv, DWORD token)
+static const struct wined3d_sm1_opcode_info *shader_get_opcode(const struct wined3d_sm1_data *priv, unsigned int token)
 {
-    DWORD shader_version = WINED3D_SHADER_VERSION(priv->shader_version.major, priv->shader_version.minor);
+    unsigned int shader_version = WINED3D_SHADER_VERSION(priv->shader_version.major, priv->shader_version.minor);
     const struct wined3d_sm1_opcode_info *opcode_table = priv->opcode_table;
     unsigned int i = 0;
 
@@ -504,7 +502,7 @@ static unsigned int shader_skip_unrecognized(const struct wined3d_sm1_data *priv
     /* TODO: Think of a good name for 0x80000000 and replace it with a constant */
     while (*ptr & 0x80000000)
     {
-        DWORD token, addr_token = 0;
+        unsigned int token, addr_token = 0;
         struct wined3d_shader_src_param rel_addr;
 
         tokens_read += shader_get_param(priv, ptr, &token, &addr_token);
@@ -538,13 +536,13 @@ static void *shader_sm1_init(const DWORD *byte_code, size_t byte_code_size,
     struct wined3d_sm1_data *priv;
     BYTE major, minor;
 
-    TRACE("Version: 0x%08x.\n", *byte_code);
+    TRACE("Version: 0x%08lx.\n", *byte_code);
 
     major = WINED3D_SM1_VERSION_MAJOR(*byte_code);
     minor = WINED3D_SM1_VERSION_MINOR(*byte_code);
     if (WINED3D_SHADER_VERSION(major, minor) > WINED3D_SHADER_VERSION(3, 0))
     {
-        WARN("Invalid shader version %u.%u (%#x).\n", major, minor, *byte_code);
+        WARN("Invalid shader version %u.%u (%#lx).\n", major, minor, *byte_code);
         return NULL;
     }
 
@@ -567,7 +565,7 @@ static void *shader_sm1_init(const DWORD *byte_code, size_t byte_code_size,
             break;
 
         default:
-            FIXME("Unrecognized shader type %#x.\n", *byte_code >> 16);
+            FIXME("Unrecognized shader type %#lx.\n", *byte_code >> 16);
             heap_free(priv);
             return NULL;
     }
@@ -595,7 +593,7 @@ static void shader_sm1_read_header(void *data, const DWORD **ptr, struct wined3d
 static void shader_sm1_read_src_param(struct wined3d_sm1_data *priv, const DWORD **ptr,
         struct wined3d_shader_src_param *src_param, struct wined3d_shader_src_param *src_rel_addr)
 {
-    DWORD token, addr_token;
+    unsigned int token, addr_token;
 
     *ptr += shader_get_param(priv, *ptr, &token, &addr_token);
     if (token & WINED3D_SM1_ADDRESS_MODE_RELATIVE)
@@ -612,7 +610,7 @@ static void shader_sm1_read_src_param(struct wined3d_sm1_data *priv, const DWORD
 static void shader_sm1_read_dst_param(struct wined3d_sm1_data *priv, const DWORD **ptr,
         struct wined3d_shader_dst_param *dst_param, struct wined3d_shader_src_param *dst_rel_addr)
 {
-    DWORD token, addr_token;
+    unsigned int token, addr_token;
 
     *ptr += shader_get_param(priv, *ptr, &token, &addr_token);
     if (token & WINED3D_SM1_ADDRESS_MODE_RELATIVE)
@@ -725,7 +723,7 @@ static void shader_sm1_read_instruction(void *data, const DWORD **ptr, struct wi
 {
     const struct wined3d_sm1_opcode_info *opcode_info;
     struct wined3d_sm1_data *priv = data;
-    DWORD opcode_token;
+    unsigned int opcode_token;
     unsigned int i;
     const DWORD *p;
 
