@@ -26,6 +26,10 @@
 
 #include "wine/debug.h"
 
+#include "initguid.h"
+
+#include "codecapi.h"
+
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
 WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
@@ -53,14 +57,20 @@ struct h264_decoder
     IMFAttributes *attributes;
     IMFAttributes *output_attributes;
 
+    UINT64 sample_time;
     IMFMediaType *input_type;
     MFT_INPUT_STREAM_INFO input_info;
     IMFMediaType *output_type;
     MFT_OUTPUT_STREAM_INFO output_info;
+    IMFMediaType *stream_type;
 
-    struct wg_format wg_format;
-    struct wg_transform *wg_transform;
+    wg_transform_t wg_transform;
     struct wg_sample_queue *wg_sample_queue;
+
+    IMFVideoSampleAllocatorEx *allocator;
+    BOOL allocator_initialized;
+    IMFTransform *copier;
+    IMFMediaBuffer *temp_buffer;
 };
 
 static struct h264_decoder *impl_from_IMFTransform(IMFTransform *iface)
@@ -70,12 +80,23 @@ static struct h264_decoder *impl_from_IMFTransform(IMFTransform *iface)
 
 static HRESULT try_create_wg_transform(struct h264_decoder *decoder)
 {
+    /* Call of Duty: Black Ops 3 doesn't care about the ProcessInput/ProcessOutput
+     * return values, it calls them in a specific order and expects the decoder
+     * transform to be able to queue its input buffers. We need to use a buffer list
+     * to match its expectations.
+     */
+    struct wg_transform_attrs attrs =
+    {
+        .output_plane_align = 15,
+        .input_queue_length = 15,
+    };
     struct wg_format input_format;
     struct wg_format output_format;
+    UINT32 low_latency;
 
     if (decoder->wg_transform)
         wg_transform_destroy(decoder->wg_transform);
-    decoder->wg_transform = NULL;
+    decoder->wg_transform = 0;
 
     mf_media_type_to_wg_format(decoder->input_type, &input_format);
     if (input_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
@@ -93,7 +114,10 @@ static HRESULT try_create_wg_transform(struct h264_decoder *decoder)
     output_format.u.video.fps_d = 0;
     output_format.u.video.fps_n = 0;
 
-    if (!(decoder->wg_transform = wg_transform_create(&input_format, &output_format)))
+    if (SUCCEEDED(IMFAttributes_GetUINT32(decoder->attributes, &MF_LOW_LATENCY, &low_latency)))
+        attrs.low_latency = !!low_latency;
+
+    if (!(decoder->wg_transform = wg_transform_create(&input_format, &output_format, &attrs)))
         return E_FAIL;
 
     return S_OK;
@@ -102,8 +126,8 @@ static HRESULT try_create_wg_transform(struct h264_decoder *decoder)
 static HRESULT fill_output_media_type(struct h264_decoder *decoder, IMFMediaType *media_type)
 {
     IMFMediaType *default_type = decoder->output_type;
-    struct wg_format *wg_format = &decoder->wg_format;
     UINT32 value, width, height;
+    MFVideoArea aperture;
     UINT64 ratio;
     GUID subtype;
     HRESULT hr;
@@ -113,7 +137,8 @@ static HRESULT fill_output_media_type(struct h264_decoder *decoder, IMFMediaType
 
     if (FAILED(hr = IMFMediaType_GetUINT64(media_type, &MF_MT_FRAME_SIZE, &ratio)))
     {
-        ratio = (UINT64)wg_format->u.video.width << 32 | wg_format->u.video.height;
+        if (FAILED(IMFMediaType_GetUINT64(decoder->stream_type, &MF_MT_FRAME_SIZE, &ratio)))
+            ratio = (UINT64)1920 << 32 | 1080;
         if (FAILED(hr = IMFMediaType_SetUINT64(media_type, &MF_MT_FRAME_SIZE, ratio)))
             return hr;
     }
@@ -122,14 +147,16 @@ static HRESULT fill_output_media_type(struct h264_decoder *decoder, IMFMediaType
 
     if (FAILED(hr = IMFMediaType_GetItem(media_type, &MF_MT_FRAME_RATE, NULL)))
     {
-        ratio = (UINT64)wg_format->u.video.fps_n << 32 | wg_format->u.video.fps_d;
+        if (FAILED(IMFMediaType_GetUINT64(decoder->stream_type, &MF_MT_FRAME_RATE, &ratio)))
+            ratio = (UINT64)30000 << 32 | 1001;
         if (FAILED(hr = IMFMediaType_SetUINT64(media_type, &MF_MT_FRAME_RATE, ratio)))
             return hr;
     }
 
     if (FAILED(hr = IMFMediaType_GetItem(media_type, &MF_MT_PIXEL_ASPECT_RATIO, NULL)))
     {
-        ratio = (UINT64)1 << 32 | 1; /* FIXME: read it from format */
+        if (FAILED(IMFMediaType_GetUINT64(decoder->stream_type, &MF_MT_PIXEL_ASPECT_RATIO, &ratio)))
+            ratio = (UINT64)1 << 32 | 1;
         if (FAILED(hr = IMFMediaType_SetUINT64(media_type, &MF_MT_PIXEL_ASPECT_RATIO, ratio)))
             return hr;
     }
@@ -183,16 +210,9 @@ static HRESULT fill_output_media_type(struct h264_decoder *decoder, IMFMediaType
     }
 
     if (FAILED(hr = IMFMediaType_GetItem(media_type, &MF_MT_MINIMUM_DISPLAY_APERTURE, NULL))
-            && !IsRectEmpty(&wg_format->u.video.padding))
+            && SUCCEEDED(hr = IMFMediaType_GetBlob(decoder->stream_type, &MF_MT_MINIMUM_DISPLAY_APERTURE,
+            (BYTE *)&aperture, sizeof(aperture), &value)))
     {
-        MFVideoArea aperture =
-        {
-            .OffsetX = {.value = wg_format->u.video.padding.left},
-            .OffsetY = {.value = wg_format->u.video.padding.top},
-            .Area.cx = wg_format->u.video.width - wg_format->u.video.padding.right - wg_format->u.video.padding.left,
-            .Area.cy = wg_format->u.video.height - wg_format->u.video.padding.bottom - wg_format->u.video.padding.top,
-        };
-
         if (FAILED(hr = IMFMediaType_SetBlob(media_type, &MF_MT_MINIMUM_DISPLAY_APERTURE,
                 (BYTE *)&aperture, sizeof(aperture))))
             return hr;
@@ -201,6 +221,31 @@ static HRESULT fill_output_media_type(struct h264_decoder *decoder, IMFMediaType
     return S_OK;
 }
 
+static HRESULT init_allocator(struct h264_decoder *decoder)
+{
+    HRESULT hr;
+
+    if (decoder->allocator_initialized)
+        return S_OK;
+
+    if (FAILED(hr = IMFTransform_SetInputType(decoder->copier, 0, decoder->output_type, 0)))
+        return hr;
+    if (FAILED(hr = IMFTransform_SetOutputType(decoder->copier, 0, decoder->output_type, 0)))
+        return hr;
+
+    if (FAILED(hr = IMFVideoSampleAllocatorEx_InitializeSampleAllocatorEx(decoder->allocator, 10, 10,
+            decoder->attributes, decoder->output_type)))
+        return hr;
+    decoder->allocator_initialized = TRUE;
+    return S_OK;
+}
+
+static void uninit_allocator(struct h264_decoder *decoder)
+{
+    IMFVideoSampleAllocatorEx_UninitializeSampleAllocator(decoder->allocator);
+    decoder->allocator_initialized = FALSE;
+}
+
 static HRESULT WINAPI transform_QueryInterface(IMFTransform *iface, REFIID iid, void **out)
 {
     struct h264_decoder *decoder = impl_from_IMFTransform(iface);
@@ -240,6 +285,10 @@ static ULONG WINAPI transform_Release(IMFTransform *iface)
 
     if (!refcount)
     {
+        IMFTransform_Release(decoder->copier);
+        IMFVideoSampleAllocatorEx_Release(decoder->allocator);
+        if (decoder->temp_buffer)
+            IMFMediaBuffer_Release(decoder->temp_buffer);
         if (decoder->wg_transform)
             wg_transform_destroy(decoder->wg_transform);
         if (decoder->input_type)
@@ -250,7 +299,6 @@ static ULONG WINAPI transform_Release(IMFTransform *iface)
             IMFAttributes_Release(decoder->output_attributes);
         if (decoder->attributes)
             IMFAttributes_Release(decoder->attributes);
-
         wg_sample_queue_destroy(decoder->wg_sample_queue);
         free(decoder);
     }
@@ -448,10 +496,9 @@ static HRESULT WINAPI transform_SetInputType(IMFTransform *iface, DWORD id, IMFM
 
     if (SUCCEEDED(IMFMediaType_GetUINT64(type, &MF_MT_FRAME_SIZE, &frame_size)))
     {
-        decoder->wg_format.u.video.width = frame_size >> 32;
-        decoder->wg_format.u.video.height = (UINT32)frame_size;
-        decoder->output_info.cbSize = decoder->wg_format.u.video.width
-                * decoder->wg_format.u.video.height * 2;
+        if (FAILED(hr = IMFMediaType_SetUINT64(decoder->stream_type, &MF_MT_FRAME_SIZE, frame_size)))
+            WARN("Failed to update stream type frame size, hr %#lx\n", hr);
+        decoder->output_info.cbSize = (frame_size >> 32) * (UINT32)frame_size * 2;
     }
 
     return S_OK;
@@ -460,8 +507,8 @@ static HRESULT WINAPI transform_SetInputType(IMFTransform *iface, DWORD id, IMFM
 static HRESULT WINAPI transform_SetOutputType(IMFTransform *iface, DWORD id, IMFMediaType *type, DWORD flags)
 {
     struct h264_decoder *decoder = impl_from_IMFTransform(iface);
+    UINT64 frame_size, stream_frame_size;
     GUID major, subtype;
-    UINT64 frame_size;
     HRESULT hr;
     ULONG i;
 
@@ -483,9 +530,10 @@ static HRESULT WINAPI transform_SetOutputType(IMFTransform *iface, DWORD id, IMF
     if (i == ARRAY_SIZE(h264_decoder_output_types))
         return MF_E_INVALIDMEDIATYPE;
 
-    if (FAILED(hr = IMFMediaType_GetUINT64(type, &MF_MT_FRAME_SIZE, &frame_size))
-            || (frame_size >> 32) != decoder->wg_format.u.video.width
-            || (UINT32)frame_size != decoder->wg_format.u.video.height)
+    if (FAILED(hr = IMFMediaType_GetUINT64(type, &MF_MT_FRAME_SIZE, &frame_size)))
+        return MF_E_INVALIDMEDIATYPE;
+    if (SUCCEEDED(IMFMediaType_GetUINT64(decoder->stream_type, &MF_MT_FRAME_SIZE, &stream_frame_size))
+            && frame_size != stream_frame_size)
         return MF_E_INVALIDMEDIATYPE;
     if (flags & MFT_SET_TYPE_TEST_ONLY)
         return S_OK;
@@ -548,8 +596,15 @@ static HRESULT WINAPI transform_GetOutputCurrentType(IMFTransform *iface, DWORD
 
 static HRESULT WINAPI transform_GetInputStatus(IMFTransform *iface, DWORD id, DWORD *flags)
 {
-    FIXME("iface %p, id %#lx, flags %p stub!\n", iface, id, flags);
-    return E_NOTIMPL;
+    struct h264_decoder *decoder = impl_from_IMFTransform(iface);
+
+    TRACE("iface %p, id %#lx, flags %p.\n", iface, id, flags);
+
+    if (!decoder->wg_transform)
+        return MF_E_TRANSFORM_TYPE_NOT_SET;
+
+    *flags = MFT_INPUT_STATUS_ACCEPT_DATA;
+    return S_OK;
 }
 
 static HRESULT WINAPI transform_GetOutputStatus(IMFTransform *iface, DWORD *flags)
@@ -572,8 +627,34 @@ static HRESULT WINAPI transform_ProcessEvent(IMFTransform *iface, DWORD id, IMFM
 
 static HRESULT WINAPI transform_ProcessMessage(IMFTransform *iface, MFT_MESSAGE_TYPE message, ULONG_PTR param)
 {
-    FIXME("iface %p, message %#x, param %Ix stub!\n", iface, message, param);
-    return S_OK;
+    struct h264_decoder *decoder = impl_from_IMFTransform(iface);
+    HRESULT hr;
+
+    TRACE("iface %p, message %#x, param %Ix.\n", iface, message, param);
+
+    switch (message)
+    {
+    case MFT_MESSAGE_SET_D3D_MANAGER:
+        if (FAILED(hr = IMFVideoSampleAllocatorEx_SetDirectXManager(decoder->allocator, (IUnknown *)param)))
+            return hr;
+
+        uninit_allocator(decoder);
+        if (param)
+            decoder->output_info.dwFlags |= MFT_OUTPUT_STREAM_PROVIDES_SAMPLES;
+        else
+            decoder->output_info.dwFlags &= ~MFT_OUTPUT_STREAM_PROVIDES_SAMPLES;
+        return S_OK;
+
+    case MFT_MESSAGE_COMMAND_DRAIN:
+        return wg_transform_drain(decoder->wg_transform);
+
+    case MFT_MESSAGE_COMMAND_FLUSH:
+        return wg_transform_flush(decoder->wg_transform);
+
+    default:
+        FIXME("Ignoring message %#x.\n", message);
+        return S_OK;
+    }
 }
 
 static HRESULT WINAPI transform_ProcessInput(IMFTransform *iface, DWORD id, IMFSample *sample, DWORD flags)
@@ -588,14 +669,69 @@ static HRESULT WINAPI transform_ProcessInput(IMFTransform *iface, DWORD id, IMFS
     return wg_transform_push_mf(decoder->wg_transform, sample, decoder->wg_sample_queue);
 }
 
+static HRESULT output_sample(struct h264_decoder *decoder, IMFSample **out, IMFSample *src_sample)
+{
+    MFT_OUTPUT_DATA_BUFFER output[1];
+    IMFSample *sample;
+    DWORD status;
+    HRESULT hr;
+
+    if (FAILED(hr = init_allocator(decoder)))
+    {
+        ERR("Failed to initialize allocator, hr %#lx.\n", hr);
+        return hr;
+    }
+    if (FAILED(hr = IMFVideoSampleAllocatorEx_AllocateSample(decoder->allocator, &sample)))
+        return hr;
+
+    if (FAILED(hr = IMFTransform_ProcessInput(decoder->copier, 0, src_sample, 0)))
+    {
+        IMFSample_Release(sample);
+        return hr;
+    }
+    output[0].pSample = sample;
+    if (FAILED(hr = IMFTransform_ProcessOutput(decoder->copier, 0, 1, output, &status)))
+    {
+        IMFSample_Release(sample);
+        return hr;
+    }
+    *out = sample;
+    return S_OK;
+}
+
+static HRESULT handle_stream_type_change(struct h264_decoder *decoder, const struct wg_format *format)
+{
+    UINT64 frame_size, frame_rate;
+    HRESULT hr;
+
+    if (decoder->stream_type)
+        IMFMediaType_Release(decoder->stream_type);
+    if (!(decoder->stream_type = mf_media_type_from_wg_format(format)))
+        return E_OUTOFMEMORY;
+
+    if (SUCCEEDED(IMFMediaType_GetUINT64(decoder->output_type, &MF_MT_FRAME_RATE, &frame_rate))
+            && FAILED(hr = IMFMediaType_SetUINT64(decoder->stream_type, &MF_MT_FRAME_RATE, frame_rate)))
+            WARN("Failed to update stream type frame size, hr %#lx\n", hr);
+
+    if (FAILED(hr = IMFMediaType_GetUINT64(decoder->stream_type, &MF_MT_FRAME_SIZE, &frame_size)))
+        return hr;
+    decoder->output_info.cbSize = (frame_size >> 32) * (UINT32)frame_size * 2;
+    uninit_allocator(decoder);
+
+    return MF_E_TRANSFORM_STREAM_CHANGE;
+}
+
 static HRESULT WINAPI transform_ProcessOutput(IMFTransform *iface, DWORD flags, DWORD count,
         MFT_OUTPUT_DATA_BUFFER *samples, DWORD *status)
 {
     struct h264_decoder *decoder = impl_from_IMFTransform(iface);
     struct wg_format wg_format;
     UINT32 sample_size;
-    UINT64 frame_rate;
+    LONGLONG duration;
+    IMFSample *sample;
+    UINT64 frame_size, frame_rate;
     GUID subtype;
+    DWORD size;
     HRESULT hr;
 
     TRACE("iface %p, flags %#lx, count %lu, samples %p, status %p.\n", iface, flags, count, samples, status);
@@ -607,34 +743,65 @@ static HRESULT WINAPI transform_ProcessOutput(IMFTransform *iface, DWORD flags,
         return MF_E_TRANSFORM_TYPE_NOT_SET;
 
     *status = samples->dwStatus = 0;
-    if (!samples->pSample)
+    if (!(sample = samples->pSample) && !(decoder->output_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES))
         return E_INVALIDARG;
 
     if (FAILED(hr = IMFMediaType_GetGUID(decoder->output_type, &MF_MT_SUBTYPE, &subtype)))
         return hr;
-    if (FAILED(hr = MFCalculateImageSize(&subtype, decoder->wg_format.u.video.width,
-            decoder->wg_format.u.video.height, &sample_size)))
+    if (FAILED(hr = IMFMediaType_GetUINT64(decoder->output_type, &MF_MT_FRAME_SIZE, &frame_size)))
+        return hr;
+    if (FAILED(hr = MFCalculateImageSize(&subtype, frame_size >> 32, (UINT32)frame_size, &sample_size)))
         return hr;
 
-    if (SUCCEEDED(hr = wg_transform_read_mf(decoder->wg_transform, samples->pSample,
+    if (decoder->output_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES)
+    {
+        if (decoder->temp_buffer)
+        {
+            if (FAILED(IMFMediaBuffer_GetMaxLength(decoder->temp_buffer, &size)) || size < sample_size)
+            {
+                IMFMediaBuffer_Release(decoder->temp_buffer);
+                decoder->temp_buffer = NULL;
+            }
+        }
+        if (!decoder->temp_buffer && FAILED(hr = MFCreateMemoryBuffer(sample_size, &decoder->temp_buffer)))
+            return hr;
+        if (FAILED(hr = MFCreateSample(&sample)))
+            return hr;
+        if (FAILED(hr = IMFSample_AddBuffer(sample, decoder->temp_buffer)))
+        {
+            IMFSample_Release(sample);
+            return hr;
+        }
+    }
+
+    if (SUCCEEDED(hr = wg_transform_read_mf(decoder->wg_transform, sample,
             sample_size, &wg_format, &samples->dwStatus)))
+    {
         wg_sample_queue_flush(decoder->wg_sample_queue, false);
 
+        if (FAILED(IMFMediaType_GetUINT64(decoder->input_type, &MF_MT_FRAME_RATE, &frame_rate)))
+            frame_rate = (UINT64)30000 << 32 | 1001;
+
+        duration = (UINT64)10000000 * (UINT32)frame_rate / (frame_rate >> 32);
+        if (FAILED(IMFSample_SetSampleTime(sample, decoder->sample_time)))
+            WARN("Failed to set sample time\n");
+        if (FAILED(IMFSample_SetSampleDuration(sample, duration)))
+            WARN("Failed to set sample duration\n");
+        decoder->sample_time += duration;
+    }
+
     if (hr == MF_E_TRANSFORM_STREAM_CHANGE)
     {
-        decoder->wg_format = wg_format;
-        decoder->output_info.cbSize = ALIGN_SIZE(decoder->wg_format.u.video.width, 0xf)
-                * ALIGN_SIZE(decoder->wg_format.u.video.height, 0xf) * 2;
-
-        /* keep the frame rate that was requested, GStreamer doesn't provide any */
-        if (SUCCEEDED(IMFMediaType_GetUINT64(decoder->output_type, &MF_MT_FRAME_RATE, &frame_rate)))
-        {
-            decoder->wg_format.u.video.fps_n = frame_rate >> 32;
-            decoder->wg_format.u.video.fps_d = (UINT32)frame_rate;
-        }
-
         samples[0].dwStatus |= MFT_OUTPUT_DATA_BUFFER_FORMAT_CHANGE;
         *status |= MFT_OUTPUT_DATA_BUFFER_FORMAT_CHANGE;
+        hr = handle_stream_type_change(decoder, &wg_format);
+    }
+
+    if (decoder->output_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES)
+    {
+        if (hr == S_OK && FAILED(hr = output_sample(decoder, &samples->pSample, sample)))
+            ERR("Failed to output sample, hr %#lx.\n", hr);
+        IMFSample_Release(sample);
     }
 
     return hr;
@@ -683,13 +850,14 @@ HRESULT h264_decoder_create(REFIID riid, void **ret)
         },
     };
     static const struct wg_format input_format = {.major_type = WG_MAJOR_TYPE_VIDEO_H264};
-    struct wg_transform *transform;
+    struct wg_transform_attrs attrs = {0};
+    wg_transform_t transform;
     struct h264_decoder *decoder;
     HRESULT hr;
 
     TRACE("riid %s, ret %p.\n", debugstr_guid(riid), ret);
 
-    if (!(transform = wg_transform_create(&input_format, &output_format)))
+    if (!(transform = wg_transform_create(&input_format, &output_format, &attrs)))
     {
         ERR_(winediag)("GStreamer doesn't support H.264 decoding, please install appropriate plugins\n");
         return E_FAIL;
@@ -701,11 +869,6 @@ HRESULT h264_decoder_create(REFIID riid, void **ret)
 
     decoder->IMFTransform_iface.lpVtbl = &transform_vtbl;
     decoder->refcount = 1;
-    decoder->wg_format.u.video.format = WG_VIDEO_FORMAT_UNKNOWN;
-    decoder->wg_format.u.video.width = 1920;
-    decoder->wg_format.u.video.height = 1080;
-    decoder->wg_format.u.video.fps_n = 30000;
-    decoder->wg_format.u.video.fps_d = 1001;
 
     decoder->input_info.dwFlags = MFT_INPUT_STREAM_WHOLE_SAMPLES | MFT_INPUT_STREAM_SINGLE_SAMPLE_PER_BUFFER
             | MFT_INPUT_STREAM_FIXED_SAMPLE_SIZE;
@@ -714,24 +877,41 @@ HRESULT h264_decoder_create(REFIID riid, void **ret)
             | MFT_OUTPUT_STREAM_FIXED_SAMPLE_SIZE;
     decoder->output_info.cbSize = 1920 * 1088 * 2;
 
+    if (FAILED(hr = MFCreateMediaType(&decoder->stream_type)))
+        goto failed;
     if (FAILED(hr = MFCreateAttributes(&decoder->attributes, 16)))
         goto failed;
     if (FAILED(hr = IMFAttributes_SetUINT32(decoder->attributes, &MF_LOW_LATENCY, 0)))
         goto failed;
+    if (FAILED(hr = IMFAttributes_SetUINT32(decoder->attributes, &MF_SA_D3D11_AWARE, TRUE)))
+        goto failed;
+    if (FAILED(hr = IMFAttributes_SetUINT32(decoder->attributes, &AVDecVideoAcceleration_H264, TRUE)))
+        goto failed;
+
     if (FAILED(hr = MFCreateAttributes(&decoder->output_attributes, 0)))
         goto failed;
     if (FAILED(hr = wg_sample_queue_create(&decoder->wg_sample_queue)))
         goto failed;
+    if (FAILED(hr = MFCreateVideoSampleAllocatorEx(&IID_IMFVideoSampleAllocatorEx, (void **)&decoder->allocator)))
+        goto failed;
+    if (FAILED(hr = MFCreateSampleCopierMFT(&decoder->copier)))
+        goto failed;
 
     *ret = &decoder->IMFTransform_iface;
     TRACE("Created decoder %p\n", *ret);
     return S_OK;
 
 failed:
+    if (decoder->allocator)
+        IMFVideoSampleAllocatorEx_Release(decoder->allocator);
+    if (decoder->wg_sample_queue)
+        wg_sample_queue_destroy(decoder->wg_sample_queue);
     if (decoder->output_attributes)
         IMFAttributes_Release(decoder->output_attributes);
     if (decoder->attributes)
         IMFAttributes_Release(decoder->attributes);
+    if (decoder->stream_type)
+        IMFMediaType_Release(decoder->stream_type);
     free(decoder);
     return hr;
 }
