@@ -25,6 +25,8 @@
 #import "cocoa_event.h"
 #import "cocoa_window.h"
 
+#pragma GCC diagnostic ignored "-Wdeclaration-after-statement"
+
 
 static NSString* const WineAppWaitQueryResponseMode = @"WineAppWaitQueryResponseMode";
 
@@ -34,6 +36,12 @@ static NSString* const WineAppWaitQueryResponseMode = @"WineAppWaitQueryResponse
 static NSString* const NSWindowWillStartDraggingNotification = @"NSWindowWillStartDraggingNotification";
 static NSString* const NSWindowDidEndDraggingNotification = @"NSWindowDidEndDraggingNotification";
 
+// Internal distributed notification to handle cooperative app activation in Sonoma.
+static NSString* const WineAppWillActivateNotification = @"WineAppWillActivateNotification";
+static NSString* const WineActivatingAppPIDKey = @"ActivatingAppPID";
+static NSString* const WineActivatingAppPrefixKey = @"ActivatingAppPrefix";
+static NSString* const WineActivatingAppConfigDirKey = @"ActivatingAppConfigDir";
+
 
 int macdrv_err_on;
 
@@ -47,6 +55,24 @@ int macdrv_err_on;
 #endif
 
 
+#if !defined(MAC_OS_VERSION_14_0) || MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_VERSION_14_0
+@interface NSApplication (CooperativeActivationSelectorsForOldSDKs)
+
+    - (void)activate;
+    - (void)yieldActivationToApplication:(NSRunningApplication *)application;
+    - (void)yieldActivationToApplicationWithBundleIdentifier:(NSString *)bundleIdentifier;
+
+@end
+
+@interface NSRunningApplication (CooperativeActivationSelectorsForOldSDKs)
+
+    - (BOOL)activateFromApplication:(NSRunningApplication *)application
+                            options:(NSApplicationActivationOptions)options;
+
+@end
+#endif
+
+
 /***********************************************************************
  *              WineLocalizedString
  *
@@ -54,8 +80,7 @@ int macdrv_err_on;
  */
 static NSString* WineLocalizedString(unsigned int stringID)
 {
-    NSNumber* key = [NSNumber numberWithUnsignedInt:stringID];
-    return [(NSDictionary*)localized_strings objectForKey:key];
+    return ((NSDictionary*)localized_strings)[@(stringID)];
 }
 
 
@@ -111,11 +136,13 @@ static NSString* WineLocalizedString(unsigned int stringID)
     {
         if (self == [WineApplicationController class])
         {
-            NSDictionary* defaults = [NSDictionary dictionaryWithObjectsAndKeys:
-                                      @"", @"NSQuotedKeystrokeBinding",
-                                      @"", @"NSRepeatCountBinding",
-                                      [NSNumber numberWithBool:NO], @"ApplePressAndHoldEnabled",
-                                      nil];
+            NSDictionary<NSString *, id> *defaults =
+            @{
+                @"NSQuotedKeystrokeBinding" : @"",
+                    @"NSRepeatCountBinding" : @"",
+                @"ApplePressAndHoldEnabled" : @NO
+            };
+
             [[NSUserDefaults standardUserDefaults] registerDefaults:defaults];
 
             if ([NSWindow respondsToSelector:@selector(setAllowsAutomaticWindowTabbing:)])
@@ -227,7 +254,7 @@ static NSString* WineLocalizedString(unsigned int stringID)
             [NSApp setActivationPolicy:NSApplicationActivationPolicyRegular];
 
             if (activateIfTransformed)
-                [NSApp activateIgnoringOtherApps:YES];
+                [self tryToActivateIgnoringOtherApps:YES];
 
 #if defined(MAC_OS_X_VERSION_10_9) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_9
             if (!enable_app_nap && [NSProcessInfo instancesRespondToSelector:@selector(beginActivityWithOptions:reason:)])
@@ -309,14 +336,15 @@ static NSString* WineLocalizedString(unsigned int stringID)
         {
             if (processEvents)
             {
-                NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-                NSEvent* event = [NSApp nextEventMatchingMask:NSEventMaskAny
-                                                    untilDate:timeout
-                                                       inMode:NSDefaultRunLoopMode
-                                                      dequeue:YES];
-                if (event)
-                    [NSApp sendEvent:event];
-                [pool release];
+                @autoreleasepool
+                {
+                    NSEvent* event = [NSApp nextEventMatchingMask:NSEventMaskAny
+                                                        untilDate:timeout
+                                                           inMode:NSDefaultRunLoopMode
+                                                          dequeue:YES];
+                    if (event)
+                        [NSApp sendEvent:event];
+                }
             }
             else
                 [[NSRunLoop currentRunLoop] runMode:WineAppWaitQueryResponseMode beforeDate:timeout];
@@ -514,7 +542,7 @@ static NSString* WineLocalizedString(unsigned int stringID)
                 CGRect* rect;
                 NSScreen* screen;
 
-                primaryScreenHeight = NSHeight([[screens objectAtIndex:0] frame]);
+                primaryScreenHeight = NSHeight([screens[0] frame]);
                 primaryScreenHeightValid = TRUE;
 
                 size = count * sizeof(CGRect);
@@ -552,7 +580,7 @@ static NSString* WineLocalizedString(unsigned int stringID)
         // We don't use -primaryScreenHeight here so there's no chance of having
         // out-of-date cached info.  This method is called infrequently enough
         // that getting the screen height each time is not prohibitively expensive.
-        rect->origin.y = NSMaxY([[[NSScreen screens] objectAtIndex:0] frame]) - NSMaxY(*rect);
+        rect->origin.y = NSMaxY([[NSScreen screens][0] frame]) - NSMaxY(*rect);
     }
 
     - (WineWindow*) frontWineWindow
@@ -633,18 +661,24 @@ static NSString* WineLocalizedString(unsigned int stringID)
             {
                 [window setLevel:newLevel];
 
-                // -setLevel: puts the window at the front of its new level.  If
-                // we decreased the level, that's good (it was in front of that
-                // level before, so it should still be now).  But if we increased
-                // the level, the window should be toward the back (but still
-                // ahead of the previous windows we did this to).
                 if (origLevel < newLevel)
                 {
+                    // If we increased the level, the window should be toward the
+                    // back of its new level (but still ahead of the previous
+                    // windows we did this to).
                     if (prev)
                         [window orderWindow:NSWindowAbove relativeTo:[prev windowNumber]];
                     else
                         [window orderBack:nil];
                 }
+                else
+                {
+                    // If we decreased the level, we want the window at the top
+                    // of its new level. -setLevel: is documented to do that on
+                    // its own, but that's buggy on Ventura. Since we're looping
+                    // back-to-front here, -orderFront: will do the right thing.
+                    [window orderFront:nil];
+                }
             }
 
             prev = window;
@@ -804,7 +838,7 @@ static NSString* WineLocalizedString(unsigned int stringID)
         NSNumber* displayIDKey = [NSNumber numberWithUnsignedInt:displayID];
         CGDisplayModeRef originalMode;
 
-        originalMode = (CGDisplayModeRef)[originalDisplayModes objectForKey:displayIDKey];
+        originalMode = (CGDisplayModeRef)originalDisplayModes[displayIDKey];
 
         if (originalMode && [self mode:mode matchesMode:originalMode])
         {
@@ -835,7 +869,7 @@ static NSString* WineLocalizedString(unsigned int stringID)
             CGDisplayModeRef currentMode;
             NSArray* modes;
 
-            currentMode = CGDisplayModeRetain((CGDisplayModeRef)[latentDisplayModes objectForKey:displayIDKey]);
+            currentMode = CGDisplayModeRetain((CGDisplayModeRef)latentDisplayModes[displayIDKey]);
             if (!currentMode)
                 currentMode = CGDisplayCopyDisplayMode(displayID);
             if (!currentMode) // Invalid display ID
@@ -982,11 +1016,11 @@ static NSString* WineLocalizedString(unsigned int stringID)
 
     - (void) setCursor
     {
-        NSDictionary* frame = [cursorFrames objectAtIndex:cursorFrame];
-        CGImageRef cgimage = (CGImageRef)[frame objectForKey:@"image"];
+        NSDictionary* frame = cursorFrames[cursorFrame];
+        CGImageRef cgimage = (CGImageRef)frame[@"image"];
         CGSize size = CGSizeMake(CGImageGetWidth(cgimage), CGImageGetHeight(cgimage));
         NSImage* image = [[NSImage alloc] initWithCGImage:cgimage size:NSSizeFromCGSize(cgsize_mac_from_win(size))];
-        CFDictionaryRef hotSpotDict = (CFDictionaryRef)[frame objectForKey:@"hotSpot"];
+        CFDictionaryRef hotSpotDict = (CFDictionaryRef)frame[@"hotSpot"];
         CGPoint hotSpot;
 
         if (!CGPointMakeWithDictionaryRepresentation(hotSpotDict, &hotSpot))
@@ -1008,15 +1042,15 @@ static NSString* WineLocalizedString(unsigned int stringID)
             cursorFrame = 0;
         [self setCursor];
 
-        frame = [cursorFrames objectAtIndex:cursorFrame];
-        duration = [[frame objectForKey:@"duration"] doubleValue];
+        frame = cursorFrames[cursorFrame];
+        duration = [frame[@"duration"] doubleValue];
         date = [[theTimer fireDate] dateByAddingTimeInterval:duration];
         [cursorTimer setFireDate:date];
     }
 
     - (void) setCursorWithFrames:(NSArray*)frames
     {
-        if (self.cursorFrames == frames)
+        if (self.cursorFrames == frames || [self.cursorFrames isEqualToArray:frames])
             return;
 
         self.cursorFrames = frames;
@@ -1028,8 +1062,8 @@ static NSString* WineLocalizedString(unsigned int stringID)
         {
             if ([frames count] > 1)
             {
-                NSDictionary* frame = [frames objectAtIndex:0];
-                NSTimeInterval duration = [[frame objectForKey:@"duration"] doubleValue];
+                NSDictionary* frame = frames[0];
+                NSTimeInterval duration = [frame[@"duration"] doubleValue];
                 NSDate* date = [NSDate dateWithTimeIntervalSinceNow:duration];
                 self.cursorTimer = [[[NSTimer alloc] initWithFireDate:date
                                                              interval:1000000
@@ -1933,8 +1967,103 @@ static NSString* WineLocalizedString(unsigned int stringID)
                 selector:@selector(enabledKeyboardInputSourcesChanged)
                     name:(NSString*)kTISNotifyEnabledKeyboardInputSourcesChanged
                   object:nil];
+
+        if ([NSApplication instancesRespondToSelector:@selector(yieldActivationToApplication:)])
+        {
+            /* App activation cooperation, starting in macOS 14 Sonoma. */
+            [dnc addObserver:self
+                    selector:@selector(otherWineAppWillActivate:)
+                        name:WineAppWillActivateNotification
+                      object:nil
+          suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately];
+        }
     }
 
+    - (void) otherWineAppWillActivate:(NSNotification *)note
+    {
+        NSProcessInfo *ourProcess;
+        pid_t otherPID;
+        NSString *ourConfigDir, *otherConfigDir, *ourPrefix, *otherPrefix;
+        NSRunningApplication *otherApp;
+
+        /* No point in yielding if we're not the foreground app. */
+        if (![NSApp isActive]) return;
+
+        /* Ignore requests from ourself, dead processes, and other prefixes. */
+        ourProcess = [NSProcessInfo processInfo];
+        otherPID = [note.userInfo[WineActivatingAppPIDKey] integerValue];
+        if (otherPID == ourProcess.processIdentifier) return;
+
+        otherApp = [NSRunningApplication runningApplicationWithProcessIdentifier:otherPID];
+        if (!otherApp) return;
+
+        ourConfigDir = ourProcess.environment[@"WINECONFIGDIR"];
+        otherConfigDir = note.userInfo[WineActivatingAppConfigDirKey];
+        if (ourConfigDir.length && otherConfigDir.length &&
+            ![ourConfigDir isEqualToString:otherConfigDir])
+        {
+            return;
+        }
+
+        ourPrefix = ourProcess.environment[@"WINEPREFIX"];
+        otherPrefix = note.userInfo[WineActivatingAppPrefixKey];
+        if (ourPrefix.length && otherPrefix.length &&
+            ![ourPrefix isEqualToString:otherPrefix])
+        {
+            return;
+        }
+
+        /* There's a race condition here. The requesting app sends out
+           WineAppWillActivateNotification and then activates itself, but since
+           distributed notifications are asynchronous, we may not have yielded
+           in time. So we call activateFromApplication: on the other app here,
+           which will work around that race if it happened. If we didn't hit the
+           race, the activateFromApplication: call will be a no-op. */
+
+        /* We only add this observer if NSApplication responds to the yield
+           methods, so they're safe to call without checking here. */
+        [NSApp yieldActivationToApplication:otherApp];
+        [otherApp activateFromApplication:[NSRunningApplication currentApplication]
+                                  options:0];
+    }
+
+    - (void) tryToActivateIgnoringOtherApps:(BOOL)ignore
+    {
+        NSProcessInfo *processInfo;
+        NSString *configDir, *prefix;
+        NSDictionary *userInfo;
+
+        if ([NSApp isActive]) return;  /* Nothing to do. */
+
+        if (!ignore ||
+            ![NSApplication instancesRespondToSelector:@selector(yieldActivationToApplication:)])
+        {
+            /* Either we don't need to force activation, or the OS is old enough
+               that this is our only option. */
+            [NSApp activateIgnoringOtherApps:ignore];
+            return;
+        }
+
+        /* Ask other Wine apps to yield activation to us. */
+        processInfo = [NSProcessInfo processInfo];
+        configDir = processInfo.environment[@"WINECONFIGDIR"];
+        prefix = processInfo.environment[@"WINEPREFIX"];
+        userInfo = @{
+            WineActivatingAppPIDKey: @(processInfo.processIdentifier),
+            WineActivatingAppPrefixKey: prefix ? prefix : @"",
+            WineActivatingAppConfigDirKey: configDir ? configDir : @""
+        };
+
+        [[NSDistributedNotificationCenter defaultCenter]
+            postNotificationName:WineAppWillActivateNotification
+                          object:nil
+                        userInfo:userInfo
+              deliverImmediately:YES];
+
+        /* This is racy. See the note in otherWineAppWillActivate:. */
+        [NSApp activate];
+     }
+
     - (BOOL) inputSourceIsInputMethod
     {
         if (!inputSourceIsInputMethodValid)
@@ -2014,7 +2143,7 @@ static NSString* WineLocalizedString(unsigned int stringID)
         latentDisplayModes = [[NSMutableDictionary alloc] init];
         for (displayID in modesToRealize)
         {
-            CGDisplayModeRef mode = (CGDisplayModeRef)[modesToRealize objectForKey:displayID];
+            CGDisplayModeRef mode = (CGDisplayModeRef)modesToRealize[displayID];
             [self setMode:mode forDisplay:[displayID unsignedIntValue]];
         }
 
@@ -2156,34 +2285,34 @@ static NSString* WineLocalizedString(unsigned int stringID)
  */
 static void PerformRequest(void *info)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
+@autoreleasepool
+{
     WineApplicationController* controller = [WineApplicationController sharedController];
 
     for (;;)
     {
-        __block dispatch_block_t block;
+        @autoreleasepool
+        {
+            __block dispatch_block_t block;
 
-        dispatch_sync(controller->requestsManipQueue, ^{
-            if ([controller->requests count])
-            {
-                block = (dispatch_block_t)[[controller->requests objectAtIndex:0] retain];
-                [controller->requests removeObjectAtIndex:0];
-            }
-            else
-                block = nil;
-        });
+            dispatch_sync(controller->requestsManipQueue, ^{
+                if ([controller->requests count])
+                {
+                    block = (dispatch_block_t)[controller->requests[0] retain];
+                    [controller->requests removeObjectAtIndex:0];
+                }
+                else
+                    block = nil;
+            });
 
-        if (!block)
-            break;
+            if (!block)
+                break;
 
-        block();
-        [block release];
-
-        [pool release];
-        pool = [[NSAutoreleasePool alloc] init];
+            block();
+            [block release];
+        }
     }
-
-    [pool release];
+}
 }
 
 /***********************************************************************
@@ -2222,13 +2351,12 @@ void LogError(const char* func, NSString* format, ...)
  */
 void LogErrorv(const char* func, NSString* format, va_list args)
 {
-    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
-
+@autoreleasepool
+{
     NSString* message = [[NSString alloc] initWithFormat:format arguments:args];
     fprintf(stderr, "err:%s:%s", func, [message UTF8String]);
     [message release];
-
-    [pool release];
+}
 }
 
 /***********************************************************************
