@@ -187,7 +187,7 @@ CFStringRef copy_system_cursor_name(ICONINFOEXW *info)
     else
     {
         char buf[16];
-        sprintf(buf, "%hu", info->wResID);
+        snprintf(buf, sizeof(buf), "%hu", info->wResID);
         asciiz_to_unicode(p, buf);
     }
 
@@ -659,11 +659,13 @@ void macdrv_DestroyCursorIcon(HCURSOR cursor)
  *
  * Set the cursor clipping rectangle.
  */
-BOOL macdrv_ClipCursor(LPCRECT clip)
+BOOL macdrv_ClipCursor(const RECT *clip, BOOL reset)
 {
     CGRect rect;
 
-    TRACE("%s\n", wine_dbgstr_rect(clip));
+    TRACE("%s %u\n", wine_dbgstr_rect(clip), reset);
+
+    if (reset) return TRUE;
 
     if (clip)
     {
@@ -742,12 +744,12 @@ static BOOL get_icon_info(HICON handle, ICONINFOEXW *ret)
 /***********************************************************************
  *              SetCursor (MACDRV.@)
  */
-void macdrv_SetCursor(HCURSOR cursor)
+void macdrv_SetCursor(HWND hwnd, HCURSOR cursor)
 {
     CFStringRef cursor_name = NULL;
     CFArrayRef cursor_frames = NULL;
 
-    TRACE("%p\n", cursor);
+    TRACE("%p %p\n", hwnd, cursor);
 
     if (cursor)
     {
