@@ -21,6 +21,10 @@
 /* NOTE: If making changes here, consider whether they should be reflected in
  * the other drivers. */
 
+#if 0
+#pragma makedep unix
+#endif
+
 #include "config.h"
 
 #include <stdarg.h>
@@ -29,7 +33,6 @@
 
 #include "macdrv.h"
 #include "wine/debug.h"
-#include "wine/heap.h"
 
 #define VK_NO_PROTOTYPES
 #define WINE_VK_HOST
@@ -53,7 +56,7 @@ struct wine_vk_surface
 {
     macdrv_metal_device device;
     macdrv_metal_view view;
-    VkSurfaceKHR surface; /* native surface */
+    VkSurfaceKHR host_surface; /* host surface */
 };
 
 typedef struct VkMacOSSurfaceCreateInfoMVK
@@ -101,12 +104,12 @@ static inline struct wine_vk_surface *surface_from_handle(VkSurfaceKHR handle)
 
 static void *vulkan_handle;
 
-static BOOL WINAPI wine_vk_init(INIT_ONCE *once, void *param, void **context)
+static void wine_vk_init(void)
 {
     if (!(vulkan_handle = dlopen(SONAME_LIBMOLTENVK, RTLD_NOW)))
     {
         ERR("Failed to load %s\n", SONAME_LIBMOLTENVK);
-        return TRUE;
+        return;
     }
 
 #define LOAD_FUNCPTR(f) if ((p##f = dlsym(vulkan_handle, #f)) == NULL) goto fail;
@@ -130,12 +133,11 @@ static BOOL WINAPI wine_vk_init(INIT_ONCE *once, void *param, void **context)
     LOAD_FUNCPTR(vkQueuePresentKHR)
 #undef LOAD_FUNCPTR
 
-    return TRUE;
+    return;
 
 fail:
     dlclose(vulkan_handle);
     vulkan_handle = NULL;
-    return TRUE;
 }
 
 /* Helper function for converting between win32 and MoltenVK compatible VkInstanceCreateInfo.
@@ -158,7 +160,7 @@ static VkResult wine_vk_instance_convert_create_info(const VkInstanceCreateInfo
 
     if (src->enabledExtensionCount > 0)
     {
-        enabled_extensions = heap_calloc(src->enabledExtensionCount, sizeof(*src->ppEnabledExtensionNames));
+        enabled_extensions = calloc(src->enabledExtensionCount, sizeof(*src->ppEnabledExtensionNames));
         if (!enabled_extensions)
         {
             ERR("Failed to allocate memory for enabled extensions\n");
@@ -192,7 +194,7 @@ static void wine_vk_surface_destroy(VkInstance instance, struct wine_vk_surface
     if (!surface)
         return;
 
-    pvkDestroySurfaceKHR(instance, surface->surface, NULL /* allocator */);
+    pvkDestroySurfaceKHR(instance, surface->host_surface, NULL /* allocator */);
 
     if (surface->view)
         macdrv_view_release_metal_view(surface->view);
@@ -200,7 +202,7 @@ static void wine_vk_surface_destroy(VkInstance instance, struct wine_vk_surface
     if (surface->device)
         macdrv_release_metal_device(surface->device);
 
-    heap_free(surface);
+    free(surface);
 }
 
 static VkResult macdrv_vkCreateInstance(const VkInstanceCreateInfo *create_info,
@@ -226,7 +228,7 @@ static VkResult macdrv_vkCreateInstance(const VkInstanceCreateInfo *create_info,
 
     res = pvkCreateInstance(&create_info_host, NULL /* allocator */, instance);
 
-    heap_free((void *)create_info_host.ppEnabledExtensionNames);
+    free((void *)create_info_host.ppEnabledExtensionNames);
     return res;
 }
 
@@ -241,7 +243,7 @@ static VkResult macdrv_vkCreateSwapchainKHR(VkDevice device,
         FIXME("Support for allocation callbacks not implemented yet\n");
 
     create_info_host = *create_info;
-    create_info_host.surface = surface_from_handle(create_info->surface)->surface;
+    create_info_host.surface = surface_from_handle(create_info->surface)->host_surface;
 
     return pvkCreateSwapchainKHR(device, &create_info_host, NULL /* allocator */,
             swapchain);
@@ -266,7 +268,7 @@ static VkResult macdrv_vkCreateWin32SurfaceKHR(VkInstance instance,
         return VK_ERROR_INCOMPATIBLE_DRIVER;
     }
 
-    mac_surface = heap_alloc_zero(sizeof(*mac_surface));
+    mac_surface = calloc(1, sizeof(*mac_surface));
     if (!mac_surface)
     {
         release_win_data(data);
@@ -299,7 +301,7 @@ static VkResult macdrv_vkCreateWin32SurfaceKHR(VkInstance instance,
         create_info_host.flags = 0; /* reserved */
         create_info_host.pLayer = macdrv_view_get_metal_layer(mac_surface->view);
 
-        res = pvkCreateMetalSurfaceEXT(instance, &create_info_host, NULL /* allocator */, &mac_surface->surface);
+        res = pvkCreateMetalSurfaceEXT(instance, &create_info_host, NULL /* allocator */, &mac_surface->host_surface);
     }
     else
     {
@@ -309,7 +311,7 @@ static VkResult macdrv_vkCreateWin32SurfaceKHR(VkInstance instance,
         create_info_host.flags = 0; /* reserved */
         create_info_host.pView = macdrv_view_get_metal_layer(mac_surface->view);
 
-        res = pvkCreateMacOSSurfaceMVK(instance, &create_info_host, NULL /* allocator */, &mac_surface->surface);
+        res = pvkCreateMacOSSurfaceMVK(instance, &create_info_host, NULL /* allocator */, &mac_surface->host_surface);
     }
     if (res != VK_SUCCESS)
     {
@@ -417,7 +419,7 @@ static VkResult macdrv_vkEnumerateInstanceExtensionProperties(const char *layer_
     return res;
 }
 
-static const char *wine_vk_native_fn_name(const char *name)
+static const char *wine_vk_host_fn_name(const char *name)
 {
     const char *create_surface_name =
         pvkCreateMetalSurfaceEXT ? "vkCreateMetalSurfaceEXT" : "vkCreateMacOSSurfaceMVK";
@@ -425,7 +427,7 @@ static const char *wine_vk_native_fn_name(const char *name)
     if (!strcmp(name, "vkCreateWin32SurfaceKHR"))
         return create_surface_name;
     /* We just need something where non-NULL is returned if the correct extension is enabled.
-     * So since there is no native equivalent of this function check for the create
+     * So since there is no host equivalent of this function check for the create
      * surface function.
      */
     if (!strcmp(name, "vkGetPhysicalDeviceWin32PresentationSupportKHR"))
@@ -440,7 +442,7 @@ static void *macdrv_vkGetDeviceProcAddr(VkDevice device, const char *name)
 
     TRACE("%p, %s\n", device, debugstr_a(name));
 
-    if (!pvkGetDeviceProcAddr(device, wine_vk_native_fn_name(name)))
+    if (!pvkGetDeviceProcAddr(device, wine_vk_host_fn_name(name)))
         return NULL;
 
     if ((proc_addr = macdrv_get_vk_device_proc_addr(name)))
@@ -455,7 +457,7 @@ static void *macdrv_vkGetInstanceProcAddr(VkInstance instance, const char *name)
 
     TRACE("%p, %s\n", instance, debugstr_a(name));
 
-    if (!pvkGetInstanceProcAddr(instance, wine_vk_native_fn_name(name)))
+    if (!pvkGetInstanceProcAddr(instance, wine_vk_host_fn_name(name)))
         return NULL;
 
     if ((proc_addr = macdrv_get_vk_instance_proc_addr(instance, name)))
@@ -472,7 +474,7 @@ static VkResult macdrv_vkGetPhysicalDeviceSurfaceCapabilities2KHR(VkPhysicalDevi
     TRACE("%p, %p, %p\n", phys_dev, surface_info, capabilities);
 
     surface_info_host = *surface_info;
-    surface_info_host.surface = surface_from_handle(surface_info->surface)->surface;
+    surface_info_host.surface = surface_from_handle(surface_info->surface)->host_surface;
 
     return pvkGetPhysicalDeviceSurfaceCapabilities2KHR(phys_dev, &surface_info_host, capabilities);
 }
@@ -484,7 +486,7 @@ static VkResult macdrv_vkGetPhysicalDeviceSurfaceCapabilitiesKHR(VkPhysicalDevic
 
     TRACE("%p, 0x%s, %p\n", phys_dev, wine_dbgstr_longlong(surface), capabilities);
 
-    return pvkGetPhysicalDeviceSurfaceCapabilitiesKHR(phys_dev, mac_surface->surface,
+    return pvkGetPhysicalDeviceSurfaceCapabilitiesKHR(phys_dev, mac_surface->host_surface,
             capabilities);
 }
 
@@ -496,7 +498,7 @@ static VkResult macdrv_vkGetPhysicalDeviceSurfaceFormats2KHR(VkPhysicalDevice ph
     TRACE("%p, %p, %p, %p\n", phys_dev, surface_info, count, formats);
 
     surface_info_host = *surface_info;
-    surface_info_host.surface = surface_from_handle(surface_info->surface)->surface;
+    surface_info_host.surface = surface_from_handle(surface_info->surface)->host_surface;
 
     return pvkGetPhysicalDeviceSurfaceFormats2KHR(phys_dev, &surface_info_host, count, formats);
 }
@@ -508,7 +510,7 @@ static VkResult macdrv_vkGetPhysicalDeviceSurfaceFormatsKHR(VkPhysicalDevice phy
 
     TRACE("%p, 0x%s, %p, %p\n", phys_dev, wine_dbgstr_longlong(surface), count, formats);
 
-    return pvkGetPhysicalDeviceSurfaceFormatsKHR(phys_dev, mac_surface->surface,
+    return pvkGetPhysicalDeviceSurfaceFormatsKHR(phys_dev, mac_surface->host_surface,
             count, formats);
 }
 
@@ -519,7 +521,7 @@ static VkResult macdrv_vkGetPhysicalDeviceSurfacePresentModesKHR(VkPhysicalDevic
 
     TRACE("%p, 0x%s, %p, %p\n", phys_dev, wine_dbgstr_longlong(surface), count, modes);
 
-    return pvkGetPhysicalDeviceSurfacePresentModesKHR(phys_dev, mac_surface->surface, count,
+    return pvkGetPhysicalDeviceSurfacePresentModesKHR(phys_dev, mac_surface->host_surface, count,
             modes);
 }
 
@@ -530,7 +532,7 @@ static VkResult macdrv_vkGetPhysicalDeviceSurfaceSupportKHR(VkPhysicalDevice phy
 
     TRACE("%p, %u, 0x%s, %p\n", phys_dev, index, wine_dbgstr_longlong(surface), supported);
 
-    return pvkGetPhysicalDeviceSurfaceSupportKHR(phys_dev, index, mac_surface->surface,
+    return pvkGetPhysicalDeviceSurfaceSupportKHR(phys_dev, index, mac_surface->host_surface,
             supported);
 }
 
@@ -560,7 +562,7 @@ static VkResult macdrv_vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR *
         static long prev_time, start_time;
         DWORD time;
 
-        time = GetTickCount();
+        time = NtGetTickCount();
         frames++;
         frames_total++;
         if (time - prev_time > 1500)
@@ -578,13 +580,13 @@ static VkResult macdrv_vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR *
     return res;
 }
 
-static VkSurfaceKHR macdrv_wine_get_native_surface(VkSurfaceKHR surface)
+static VkSurfaceKHR macdrv_wine_get_host_surface(VkSurfaceKHR surface)
 {
     struct wine_vk_surface *mac_surface = surface_from_handle(surface);
 
     TRACE("0x%s\n", wine_dbgstr_longlong(surface));
 
-    return mac_surface->surface;
+    return mac_surface->host_surface;
 }
 
 static const struct vulkan_funcs vulkan_funcs =
@@ -610,7 +612,7 @@ static const struct vulkan_funcs vulkan_funcs =
     macdrv_vkGetSwapchainImagesKHR,
     macdrv_vkQueuePresentKHR,
 
-    macdrv_wine_get_native_surface,
+    macdrv_wine_get_host_surface,
 };
 
 static void *macdrv_get_vk_device_proc_addr(const char *name)
@@ -625,7 +627,7 @@ static void *macdrv_get_vk_instance_proc_addr(VkInstance instance, const char *n
 
 static const struct vulkan_funcs *get_vulkan_driver(UINT version)
 {
-    static INIT_ONCE init_once = INIT_ONCE_STATIC_INIT;
+    static pthread_once_t init_once = PTHREAD_ONCE_INIT;
 
     if (version != WINE_VULKAN_DRIVER_VERSION)
     {
@@ -633,7 +635,7 @@ static const struct vulkan_funcs *get_vulkan_driver(UINT version)
         return NULL;
     }
 
-    InitOnceExecuteOnce(&init_once, wine_vk_init, NULL, NULL);
+    pthread_once(&init_once, wine_vk_init);
     if (vulkan_handle)
         return &vulkan_funcs;
 
@@ -650,7 +652,7 @@ static const struct vulkan_funcs *get_vulkan_driver(UINT version)
 
 #endif /* SONAME_LIBMOLTENVK */
 
-const struct vulkan_funcs * CDECL macdrv_wine_get_vulkan_driver(UINT version)
+const struct vulkan_funcs *macdrv_wine_get_vulkan_driver(UINT version)
 {
     return get_vulkan_driver( version );
 }
