@@ -29,7 +29,6 @@
 WINE_DEFAULT_DEBUG_CHANNEL(x11drv);
 
 static struct x11drv_display_device_handler host_handler;
-struct x11drv_display_device_handler desktop_handler;
 static struct x11drv_settings_handler settings_handler;
 
 #define NEXT_DEVMODEW(mode) ((DEVMODEW *)((char *)((mode) + 1) + (mode)->dmDriverExtra))
@@ -37,7 +36,7 @@ static struct x11drv_settings_handler settings_handler;
 struct x11drv_display_depth
 {
     struct list entry;
-    ULONG_PTR display_id;
+    x11drv_settings_id display_id;
     DWORD depth;
 };
 
@@ -66,13 +65,13 @@ void X11DRV_Settings_SetHandler(const struct x11drv_settings_handler *new_handle
  * Default handlers if resolution switching is not enabled
  *
  */
-static BOOL nores_get_id(const WCHAR *device_name, BOOL is_primary, ULONG_PTR *id)
+static BOOL nores_get_id(const WCHAR *device_name, BOOL is_primary, x11drv_settings_id *id)
 {
-    *id = is_primary ? 1 : 0;
+    id->id = is_primary ? 1 : 0;
     return TRUE;
 }
 
-static BOOL nores_get_modes(ULONG_PTR id, DWORD flags, DEVMODEW **new_modes, UINT *mode_count)
+static BOOL nores_get_modes(x11drv_settings_id id, DWORD flags, DEVMODEW **new_modes, UINT *mode_count)
 {
     RECT primary = get_host_primary_monitor_rect();
     DEVMODEW *modes;
@@ -105,7 +104,7 @@ static void nores_free_modes(DEVMODEW *modes)
     free(modes);
 }
 
-static BOOL nores_get_current_mode(ULONG_PTR id, DEVMODEW *mode)
+static BOOL nores_get_current_mode(x11drv_settings_id id, DEVMODEW *mode)
 {
     RECT primary = get_host_primary_monitor_rect();
 
@@ -116,7 +115,7 @@ static BOOL nores_get_current_mode(ULONG_PTR id, DEVMODEW *mode)
     mode->dmPosition.x = 0;
     mode->dmPosition.y = 0;
 
-    if (id != 1)
+    if (id.id != 1)
     {
         FIXME("Non-primary adapters are unsupported.\n");
         mode->dmBitsPerPel = 0;
@@ -133,7 +132,7 @@ static BOOL nores_get_current_mode(ULONG_PTR id, DEVMODEW *mode)
     return TRUE;
 }
 
-static LONG nores_set_current_mode(ULONG_PTR id, const DEVMODEW *mode)
+static LONG nores_set_current_mode(x11drv_settings_id id, const DEVMODEW *mode)
 {
     WARN("NoRes settings handler, ignoring mode change request.\n");
     return DISP_CHANGE_SUCCESSFUL;
@@ -156,49 +155,14 @@ void X11DRV_Settings_Init(void)
     X11DRV_Settings_SetHandler(&nores_handler);
 }
 
-/* Initialize registry display settings when new display devices are added */
-void init_registry_display_settings(void)
-{
-    DEVMODEW dm = {.dmSize = sizeof(dm)};
-    DISPLAY_DEVICEW dd = {sizeof(dd)};
-    UNICODE_STRING device_name;
-    DWORD i = 0;
-    int ret;
-
-    while (!NtUserEnumDisplayDevices( NULL, i++, &dd, 0 ))
-    {
-        RtlInitUnicodeString( &device_name, dd.DeviceName );
-
-        /* Skip if the device already has registry display settings */
-        if (NtUserEnumDisplaySettings( &device_name, ENUM_REGISTRY_SETTINGS, &dm, 0 ))
-            continue;
-
-        if (!NtUserEnumDisplaySettings( &device_name, ENUM_CURRENT_SETTINGS, &dm, 0 ))
-        {
-            ERR("Failed to query current display settings for %s.\n", wine_dbgstr_w(dd.DeviceName));
-            continue;
-        }
-
-        TRACE("Device %s current display mode %ux%u %ubits %uHz at %d,%d.\n",
-              wine_dbgstr_w(dd.DeviceName), (int)dm.dmPelsWidth, (int)dm.dmPelsHeight,
-              (int)dm.dmBitsPerPel, (int)dm.dmDisplayFrequency, (int)dm.dmPosition.x, (int)dm.dmPosition.y);
-
-        ret = NtUserChangeDisplaySettings( &device_name, &dm, NULL,
-                                           CDS_GLOBAL | CDS_NORESET | CDS_UPDATEREGISTRY, NULL );
-        if (ret != DISP_CHANGE_SUCCESSFUL)
-            ERR("Failed to save registry display settings for %s, returned %d.\n",
-                wine_dbgstr_w(dd.DeviceName), ret);
-    }
-}
-
-static void set_display_depth(ULONG_PTR display_id, DWORD depth)
+static void set_display_depth(x11drv_settings_id display_id, DWORD depth)
 {
     struct x11drv_display_depth *display_depth;
 
     pthread_mutex_lock( &settings_mutex );
     LIST_FOR_EACH_ENTRY(display_depth, &x11drv_display_depth_list, struct x11drv_display_depth, entry)
     {
-        if (display_depth->display_id == display_id)
+        if (display_depth->display_id.id == display_id.id)
         {
             display_depth->depth = depth;
             pthread_mutex_unlock( &settings_mutex );
@@ -220,7 +184,7 @@ static void set_display_depth(ULONG_PTR display_id, DWORD depth)
     pthread_mutex_unlock( &settings_mutex );
 }
 
-static DWORD get_display_depth(ULONG_PTR display_id)
+static DWORD get_display_depth(x11drv_settings_id display_id)
 {
     struct x11drv_display_depth *display_depth;
     DWORD depth;
@@ -228,7 +192,7 @@ static DWORD get_display_depth(ULONG_PTR display_id)
     pthread_mutex_lock( &settings_mutex );
     LIST_FOR_EACH_ENTRY(display_depth, &x11drv_display_depth_list, struct x11drv_display_depth, entry)
     {
-        if (display_depth->display_id == display_id)
+        if (display_depth->display_id.id == display_id.id)
         {
             depth = display_depth->depth;
             pthread_mutex_unlock( &settings_mutex );
@@ -241,7 +205,7 @@ static DWORD get_display_depth(ULONG_PTR display_id)
 
 INT X11DRV_GetDisplayDepth(LPCWSTR name, BOOL is_primary)
 {
-    ULONG_PTR id;
+    x11drv_settings_id id;
 
     if (settings_handler.get_id( name, is_primary, &id ))
         return get_display_depth( id );
@@ -256,7 +220,7 @@ INT X11DRV_GetDisplayDepth(LPCWSTR name, BOOL is_primary)
 BOOL X11DRV_GetCurrentDisplaySettings( LPCWSTR name, BOOL is_primary, LPDEVMODEW devmode )
 {
     DEVMODEW mode;
-    ULONG_PTR id;
+    x11drv_settings_id id;
 
     if (!settings_handler.get_id( name, is_primary, &id ) || !settings_handler.get_current_mode( id, &mode ))
     {
@@ -278,9 +242,18 @@ BOOL is_detached_mode(const DEVMODEW *mode)
            mode->dmPelsHeight == 0;
 }
 
+static BOOL is_same_devmode( const DEVMODEW *a, const DEVMODEW *b )
+{
+    return a->dmDisplayOrientation == b->dmDisplayOrientation &&
+           a->dmBitsPerPel == b->dmBitsPerPel &&
+           a->dmPelsWidth == b->dmPelsWidth &&
+           a->dmPelsHeight == b->dmPelsHeight &&
+           a->dmDisplayFrequency == b->dmDisplayFrequency;
+}
+
 /* Get the full display mode with all the necessary fields set.
  * Return NULL on failure. Caller should call free_full_mode() to free the returned mode. */
-static DEVMODEW *get_full_mode(ULONG_PTR id, DEVMODEW *dev_mode)
+static DEVMODEW *get_full_mode(x11drv_settings_id id, DEVMODEW *dev_mode)
 {
     DEVMODEW *modes, *full_mode, *found_mode = NULL;
     UINT mode_count, mode_idx;
@@ -294,19 +267,7 @@ static DEVMODEW *get_full_mode(ULONG_PTR id, DEVMODEW *dev_mode)
     for (mode_idx = 0; mode_idx < mode_count; ++mode_idx)
     {
         found_mode = (DEVMODEW *)((BYTE *)modes + (sizeof(*modes) + modes[0].dmDriverExtra) * mode_idx);
-
-        if (found_mode->dmBitsPerPel != dev_mode->dmBitsPerPel)
-            continue;
-        if (found_mode->dmPelsWidth != dev_mode->dmPelsWidth)
-            continue;
-        if (found_mode->dmPelsHeight != dev_mode->dmPelsHeight)
-            continue;
-        if (found_mode->dmDisplayFrequency != dev_mode->dmDisplayFrequency)
-            continue;
-        if (found_mode->dmDisplayOrientation != dev_mode->dmDisplayOrientation)
-            continue;
-
-        break;
+        if (is_same_devmode( found_mode, dev_mode )) break;
     }
 
     if (!found_mode || mode_idx == mode_count)
@@ -335,7 +296,7 @@ static void free_full_mode(DEVMODEW *mode)
         free(mode);
 }
 
-static LONG apply_display_settings( DEVMODEW *displays, ULONG_PTR *ids, BOOL do_attach )
+static LONG apply_display_settings( DEVMODEW *displays, x11drv_settings_id *ids, BOOL do_attach )
 {
     DEVMODEW *full_mode;
     BOOL attached_mode;
@@ -344,7 +305,7 @@ static LONG apply_display_settings( DEVMODEW *displays, ULONG_PTR *ids, BOOL do_
 
     for (count = 0, mode = displays; mode->dmSize; mode = NEXT_DEVMODEW(mode), count++)
     {
-        ULONG_PTR *id = ids + count;
+        x11drv_settings_id *id = ids + count;
 
         attached_mode = !is_detached_mode(mode);
         if ((attached_mode && !do_attach) || (!attached_mode && do_attach))
@@ -381,7 +342,7 @@ LONG X11DRV_ChangeDisplaySettings( LPDEVMODEW displays, LPCWSTR primary_name, HW
 {
     INT left_most = INT_MAX, top_most = INT_MAX;
     LONG count, ret = DISP_CHANGE_BADPARAM;
-    ULONG_PTR *ids;
+    x11drv_settings_id *ids;
     DEVMODEW *mode;
 
     /* Convert virtual screen coordinates to root coordinates, and find display ids.
@@ -441,7 +402,7 @@ RECT get_host_primary_monitor_rect(void)
     RECT rect = {0};
 
     /* The first monitor is always primary */
-    if (host_handler.get_gpus(&gpus, &gpu_count) && gpu_count &&
+    if (host_handler.get_gpus(&gpus, &gpu_count, FALSE) && gpu_count &&
         host_handler.get_adapters(gpus[0].id, &adapters, &adapter_count) && adapter_count &&
         host_handler.get_monitors(adapters[0].id, &monitors, &monitor_count) && monitor_count)
         rect = monitors[0].rc_monitor;
@@ -452,21 +413,6 @@ RECT get_host_primary_monitor_rect(void)
     return rect;
 }
 
-BOOL get_host_primary_gpu(struct gdi_gpu *gpu)
-{
-    struct gdi_gpu *gpus;
-    INT gpu_count;
-
-    if (host_handler.get_gpus(&gpus, &gpu_count) && gpu_count)
-    {
-        *gpu = gpus[0];
-        host_handler.free_gpus(gpus);
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
 RECT get_work_area(const RECT *monitor_rect)
 {
     Atom type;
@@ -537,10 +483,7 @@ void X11DRV_DisplayDevices_SetHandler(const struct x11drv_display_device_handler
 
 void X11DRV_DisplayDevices_RegisterEventHandlers(void)
 {
-    struct x11drv_display_device_handler *handler = is_virtual_desktop() ? &desktop_handler : &host_handler;
-
-    if (handler->register_event_handlers)
-        handler->register_event_handlers();
+    if (host_handler.register_event_handlers) host_handler.register_event_handlers();
 }
 
 /* Report whether a display device handler supports detecting dynamic device changes */
@@ -551,9 +494,27 @@ BOOL X11DRV_DisplayDevices_SupportEventHandlers(void)
 
 static BOOL force_display_devices_refresh;
 
+static const char *debugstr_devmodew( const DEVMODEW *devmode )
+{
+    char position[32] = {0};
+
+    if (devmode->dmFields & DM_POSITION)
+    {
+        snprintf( position, sizeof(position), " at (%d,%d)",
+                 (int)devmode->dmPosition.x, (int)devmode->dmPosition.y );
+    }
+
+    return wine_dbg_sprintf( "%ux%u %ubits %uHz rotated %u degrees%s",
+                             (unsigned int)devmode->dmPelsWidth,
+                             (unsigned int)devmode->dmPelsHeight,
+                             (unsigned int)devmode->dmBitsPerPel,
+                             (unsigned int)devmode->dmDisplayFrequency,
+                             (unsigned int)devmode->dmDisplayOrientation * 90,
+                             position );
+}
+
 BOOL X11DRV_UpdateDisplayDevices( const struct gdi_device_manager *device_manager, BOOL force, void *param )
 {
-    struct x11drv_display_device_handler *handler;
     struct gdi_adapter *adapters;
     struct gdi_monitor *monitors;
     struct gdi_gpu *gpus;
@@ -564,12 +525,11 @@ BOOL X11DRV_UpdateDisplayDevices( const struct gdi_device_manager *device_manage
 
     if (!force && !force_display_devices_refresh) return TRUE;
     force_display_devices_refresh = FALSE;
-    handler = is_virtual_desktop() ? &desktop_handler : &host_handler;
 
-    TRACE("via %s\n", wine_dbgstr_a(handler->name));
+    TRACE( "via %s\n", debugstr_a(host_handler.name) );
 
     /* Initialize GPUs */
-    if (!handler->get_gpus( &gpus, &gpu_count )) return FALSE;
+    if (!host_handler.get_gpus( &gpus, &gpu_count, TRUE )) return FALSE;
     TRACE("GPU count: %d\n", gpu_count);
 
     for (gpu = 0; gpu < gpu_count; gpu++)
@@ -577,42 +537,59 @@ BOOL X11DRV_UpdateDisplayDevices( const struct gdi_device_manager *device_manage
         device_manager->add_gpu( &gpus[gpu], param );
 
         /* Initialize adapters */
-        if (!handler->get_adapters(gpus[gpu].id, &adapters, &adapter_count)) break;
+        if (!host_handler.get_adapters( gpus[gpu].id, &adapters, &adapter_count )) break;
         TRACE("GPU: %#lx %s, adapter count: %d\n", gpus[gpu].id, wine_dbgstr_w(gpus[gpu].name), adapter_count);
 
         for (adapter = 0; adapter < adapter_count; adapter++)
         {
+            DEVMODEW current_mode = {.dmSize = sizeof(current_mode)};
+            WCHAR devname[32];
+            char buffer[32];
+            x11drv_settings_id settings_id;
+            BOOL is_primary = adapters[adapter].state_flags & DISPLAY_DEVICE_PRIMARY_DEVICE;
+
             device_manager->add_adapter( &adapters[adapter], param );
 
-            if (!handler->get_monitors(adapters[adapter].id, &monitors, &monitor_count)) break;
+            if (!host_handler.get_monitors( adapters[adapter].id, &monitors, &monitor_count )) break;
             TRACE("adapter: %#lx, monitor count: %d\n", adapters[adapter].id, monitor_count);
 
             /* Initialize monitors */
             for (monitor = 0; monitor < monitor_count; monitor++)
-            {
-                TRACE("monitor: %#x %s\n", monitor, wine_dbgstr_w(monitors[monitor].name));
                 device_manager->add_monitor( &monitors[monitor], param );
-            }
 
-            handler->free_monitors(monitors, monitor_count);
+            host_handler.free_monitors( monitors, monitor_count );
 
-            if (!settings_handler.get_modes( adapters[adapter].id, EDS_ROTATEDMODE, &modes, &mode_count ))
+            /* Get the settings handler id for the adapter */
+            snprintf( buffer, sizeof(buffer), "\\\\.\\DISPLAY%d", adapter + 1 );
+            asciiz_to_unicode( devname, buffer );
+            if (!settings_handler.get_id( devname, is_primary, &settings_id )) break;
+
+            settings_handler.get_current_mode( settings_id, &current_mode );
+            if (!settings_handler.get_modes( settings_id, EDS_ROTATEDMODE, &modes, &mode_count ))
                 continue;
 
             for (mode = modes; mode_count; mode_count--)
             {
-                TRACE( "mode: %p\n", mode );
-                device_manager->add_mode( mode, param );
+                if (is_same_devmode( mode, &current_mode ))
+                {
+                    TRACE( "current mode: %s\n", debugstr_devmodew( &current_mode ) );
+                    device_manager->add_mode( &current_mode, TRUE, param );
+                }
+                else
+                {
+                    TRACE( "mode: %s\n", debugstr_devmodew( mode ) );
+                    device_manager->add_mode( mode, FALSE, param );
+                }
                 mode = (DEVMODEW *)((char *)mode + sizeof(*modes) + modes[0].dmDriverExtra);
             }
 
             settings_handler.free_modes( modes );
         }
 
-        handler->free_adapters(adapters);
+        host_handler.free_adapters( adapters );
     }
 
-    handler->free_gpus(gpus);
+    host_handler.free_gpus( gpus );
     return TRUE;
 }
 
