@@ -29,7 +29,7 @@
 
 #include "winineti.h"
 
-extern HMODULE WININET_hModule DECLSPEC_HIDDEN;
+extern HMODULE WININET_hModule;
 
 typedef struct {
     WCHAR *name;
@@ -52,15 +52,15 @@ typedef struct {
     struct list conn_pool;
 } server_t;
 
-void server_addref(server_t*) DECLSPEC_HIDDEN;
-void server_release(server_t*) DECLSPEC_HIDDEN;
+void server_addref(server_t*);
+void server_release(server_t*);
 
 typedef enum {
     COLLECT_TIMEOUT,
     COLLECT_CONNECTIONS,
     COLLECT_CLEANUP
 } collect_type_t;
-BOOL collect_connections(collect_type_t) DECLSPEC_HIDDEN;
+BOOL collect_connections(collect_type_t);
 
 /* used for netconnection.c stuff */
 typedef struct
@@ -85,8 +85,8 @@ typedef struct
     struct list pool_entry;
 } netconn_t;
 
-BOOL is_valid_netconn(netconn_t *) DECLSPEC_HIDDEN;
-void close_netconn(netconn_t *) DECLSPEC_HIDDEN;
+BOOL is_valid_netconn(netconn_t *);
+void close_netconn(netconn_t *);
 
 static inline WCHAR *strndupW(const WCHAR *str, UINT max_len)
 {
@@ -225,6 +225,7 @@ typedef struct {
     void (*CloseConnection)(object_header_t*);
     DWORD (*QueryOption)(object_header_t*,DWORD,void*,DWORD*,BOOL);
     DWORD (*SetOption)(object_header_t*,DWORD,void*,DWORD);
+    DWORD (*SetFilePointer)(object_header_t*,LONG,DWORD);
     DWORD (*ReadFile)(object_header_t*,void*,DWORD,DWORD*,DWORD,DWORD_PTR);
     DWORD (*WriteFile)(object_header_t*,const void*,DWORD,DWORD*);
     DWORD (*QueryDataAvailable)(object_header_t*,DWORD*,DWORD,DWORD_PTR);
@@ -332,6 +333,7 @@ typedef struct
 
     FILETIME last_modified;
     HANDLE hCacheFile;
+    ULONGLONG cache_size;  /* size of cached data */
     req_file_t *req_file;
     FILETIME expires;
     struct HttpAuthInfo *authInfo;
@@ -339,6 +341,7 @@ typedef struct
 
     CRITICAL_SECTION read_section;  /* section to protect the following fields */
     ULONGLONG contentLength;  /* total number of bytes to be read */
+    ULONGLONG content_pos;    /* content read position */
     BOOL  read_gzip;      /* are we reading in gzip mode? */
     DWORD read_pos;       /* current read position in read_buf */
     DWORD read_size;      /* valid data size in read_buf */
@@ -357,61 +360,61 @@ struct task_header_t
     object_header_t *hdr;
 };
 
-void *alloc_async_task(object_header_t*,async_task_proc_t,size_t) DECLSPEC_HIDDEN;
+void *alloc_async_task(object_header_t*,async_task_proc_t,size_t);
 
-void *alloc_object(object_header_t*,const object_vtbl_t*,size_t) DECLSPEC_HIDDEN;
-object_header_t *get_handle_object( HINTERNET hinternet ) DECLSPEC_HIDDEN;
-object_header_t *WININET_AddRef( object_header_t *info ) DECLSPEC_HIDDEN;
-BOOL WININET_Release( object_header_t *info ) DECLSPEC_HIDDEN;
+void *alloc_object(object_header_t*,const object_vtbl_t*,size_t);
+object_header_t *get_handle_object( HINTERNET hinternet );
+object_header_t *WININET_AddRef( object_header_t *info );
+BOOL WININET_Release( object_header_t *info );
 
-DWORD INET_QueryOption(object_header_t*,DWORD,void*,DWORD*,BOOL) DECLSPEC_HIDDEN;
-DWORD INET_SetOption(object_header_t*,DWORD,void*,DWORD) DECLSPEC_HIDDEN;
+DWORD INET_QueryOption(object_header_t*,DWORD,void*,DWORD*,BOOL);
+DWORD INET_SetOption(object_header_t*,DWORD,void*,DWORD);
 
-time_t ConvertTimeString(LPCWSTR asctime) DECLSPEC_HIDDEN;
+time_t ConvertTimeString(LPCWSTR asctime);
 
 HINTERNET FTP_Connect(appinfo_t *hIC, LPCWSTR lpszServerName,
 	INTERNET_PORT nServerPort, LPCWSTR lpszUserName,
 	LPCWSTR lpszPassword, DWORD dwFlags, DWORD_PTR dwContext,
-	DWORD dwInternalFlags) DECLSPEC_HIDDEN;
+	DWORD dwInternalFlags);
 
 DWORD HTTP_Connect(appinfo_t*,LPCWSTR,
         INTERNET_PORT nServerPort, LPCWSTR lpszUserName,
         LPCWSTR lpszPassword, DWORD dwFlags, DWORD_PTR dwContext,
-        DWORD dwInternalFlags, HINTERNET*) DECLSPEC_HIDDEN;
+        DWORD dwInternalFlags, HINTERNET*);
 
-BOOL GetAddress(const WCHAR*,INTERNET_PORT,SOCKADDR*,int*,char*) DECLSPEC_HIDDEN;
+BOOL GetAddress(const WCHAR*,INTERNET_PORT,SOCKADDR*,int*,char*);
 
-DWORD get_cookie_header(const WCHAR*,const WCHAR*,WCHAR**) DECLSPEC_HIDDEN;
-DWORD set_cookie(substr_t,substr_t,substr_t,substr_t,DWORD) DECLSPEC_HIDDEN;
+DWORD get_cookie_header(const WCHAR*,const WCHAR*,WCHAR**);
+DWORD set_cookie(substr_t,substr_t,substr_t,substr_t,DWORD);
 
-void INTERNET_SetLastError(DWORD dwError) DECLSPEC_HIDDEN;
-DWORD INTERNET_GetLastError(void) DECLSPEC_HIDDEN;
-DWORD INTERNET_AsyncCall(task_header_t*) DECLSPEC_HIDDEN;
-LPSTR INTERNET_GetResponseBuffer(void) DECLSPEC_HIDDEN;
+void INTERNET_SetLastError(DWORD dwError);
+DWORD INTERNET_GetLastError(void);
+DWORD INTERNET_AsyncCall(task_header_t*);
+LPSTR INTERNET_GetResponseBuffer(void);
 
 VOID INTERNET_SendCallback(object_header_t *hdr, DWORD_PTR dwContext,
                            DWORD dwInternetStatus, LPVOID lpvStatusInfo,
-                           DWORD dwStatusInfoLength) DECLSPEC_HIDDEN;
-WCHAR *INTERNET_FindProxyForProtocol(LPCWSTR szProxy, LPCWSTR proto) DECLSPEC_HIDDEN;
+                           DWORD dwStatusInfoLength);
+WCHAR *INTERNET_FindProxyForProtocol(LPCWSTR szProxy, LPCWSTR proto);
 
-DWORD create_netconn(server_t*,DWORD,BOOL,DWORD,netconn_t**) DECLSPEC_HIDDEN;
-void free_netconn(netconn_t*) DECLSPEC_HIDDEN;
-void NETCON_unload(void) DECLSPEC_HIDDEN;
-DWORD NETCON_secure_connect(netconn_t*,server_t*) DECLSPEC_HIDDEN;
+DWORD create_netconn(server_t*,DWORD,BOOL,DWORD,netconn_t**);
+void free_netconn(netconn_t*);
+void NETCON_unload(void);
+DWORD NETCON_secure_connect(netconn_t*,server_t*);
 DWORD NETCON_send(netconn_t *connection, const void *msg, size_t len, int flags,
-		int *sent /* out */) DECLSPEC_HIDDEN;
-DWORD NETCON_recv(netconn_t*,void*,size_t,BOOL,int*) DECLSPEC_HIDDEN;
-BOOL NETCON_is_alive(netconn_t*) DECLSPEC_HIDDEN;
-LPCVOID NETCON_GetCert(netconn_t *connection) DECLSPEC_HIDDEN;
-int NETCON_GetCipherStrength(netconn_t*) DECLSPEC_HIDDEN;
-DWORD NETCON_set_timeout(netconn_t *connection, BOOL send, DWORD value) DECLSPEC_HIDDEN;
-int sock_send(int fd, const void *msg, size_t len, int flags) DECLSPEC_HIDDEN;
-int sock_recv(int fd, void *msg, size_t len, int flags) DECLSPEC_HIDDEN;
+		int *sent /* out */);
+DWORD NETCON_recv(netconn_t*,void*,size_t,BOOL,int*);
+BOOL NETCON_is_alive(netconn_t*);
+LPCVOID NETCON_GetCert(netconn_t *connection);
+int NETCON_GetCipherStrength(netconn_t*);
+DWORD NETCON_set_timeout(netconn_t *connection, BOOL send, DWORD value);
+int sock_send(int fd, const void *msg, size_t len, int flags);
+int sock_recv(int fd, void *msg, size_t len, int flags);
 
-server_t *get_server(substr_t,INTERNET_PORT,BOOL,BOOL) DECLSPEC_HIDDEN;
+server_t *get_server(substr_t,INTERNET_PORT,BOOL,BOOL);
 
-DWORD create_req_file(const WCHAR*,req_file_t**) DECLSPEC_HIDDEN;
-void req_file_release(req_file_t*) DECLSPEC_HIDDEN;
+DWORD create_req_file(const WCHAR*,req_file_t**);
+void req_file_release(req_file_t*);
 
 static inline req_file_t *req_file_addref(req_file_t *req_file)
 {
@@ -419,12 +422,12 @@ static inline req_file_t *req_file_addref(req_file_t *req_file)
     return req_file;
 }
 
-BOOL init_urlcache(void) DECLSPEC_HIDDEN;
-void free_urlcache(void) DECLSPEC_HIDDEN;
-void free_cookie(void) DECLSPEC_HIDDEN;
-void free_authorization_cache(void) DECLSPEC_HIDDEN;
+BOOL init_urlcache(void);
+void free_urlcache(void);
+void free_cookie(void);
+void free_authorization_cache(void);
 
-void init_winsock(void) DECLSPEC_HIDDEN;
+void init_winsock(void);
 
 #define MAX_REPLY_LEN   0x1000
 
