@@ -98,7 +98,7 @@ static char* wave_generate_la(WAVEFORMATEX* wfx, double duration, DWORD* size)
 
     nb_samples=(int)(duration*wfx->nSamplesPerSec);
     *size=nb_samples*wfx->nBlockAlign;
-    b=buf=HeapAlloc(GetProcessHeap(), 0, *size);
+    b=buf=malloc(*size);
     for (i=0;i<nb_samples;i++) {
         double y=sin(440.0*2*PI*i/wfx->nSamplesPerSec);
         if (wfx->wBitsPerSample==8) {
@@ -157,7 +157,7 @@ static char* wave_generate_silence(WAVEFORMATEX* wfx, double duration, DWORD* si
 
     nb_samples=(int)(duration*wfx->nSamplesPerSec);
     *size=nb_samples*wfx->nBlockAlign;
-    b=buf=HeapAlloc(GetProcessHeap(), 0, *size);
+    b=buf=malloc(*size);
     for (i=0;i<nb_samples;i++) {
         if (wfx->wBitsPerSample==8) {
             for (j = 0; j < wfx->nChannels; j++)
@@ -671,7 +671,7 @@ static void wave_out_test_deviceOut(int device, double duration, int headers, in
        pwfx->nSamplesPerSec, pwfx->wBitsPerSample,
        pwfx->nChannels, nSamplesPerSec, wBitsPerSample, nChannels);
 
-    frags = HeapAlloc(GetProcessHeap(), 0, headers * sizeof(WAVEHDR));
+    frags = malloc(headers * sizeof(WAVEHDR));
 
     if (sine)
         buffer=wave_generate_la(pwfx,duration / (loops + 1),&length);
@@ -687,7 +687,7 @@ static void wave_out_test_deviceOut(int device, double duration, int headers, in
        "waveOutGetVolume(%s): rc=%s\n",dev_name(device),wave_out_error(rc));
 
     /* waveOutGetDevCaps allows an open handle instead of a device id */
-    rc=waveOutGetDevCapsW(HandleToUlong(wout),&capsW,sizeof(capsW));
+    rc=waveOutGetDevCapsW((UINT_PTR)wout,&capsW,sizeof(capsW));
     ok(rc==MMSYSERR_NOERROR,
        "waveOutGetDevCapsW(%s): MMSYSERR_NOERROR "
        "expected, got %s\n",dev_name(device),wave_out_error(rc));
@@ -855,14 +855,14 @@ static void wave_out_test_deviceOut(int device, double duration, int headers, in
     ok(rc==WAIT_TIMEOUT, "Notification from %s rc=%x\n",
        wave_open_flags(flags|WAVE_FORMAT_QUERY),rc);
 
-    HeapFree(GetProcessHeap(), 0, buffer);
+    free(buffer);
 EXIT:
     if ((flags & CALLBACK_TYPEMASK) == CALLBACK_THREAD) {
         PostThreadMessageW(thread_id, WM_APP, 0, 0);
         WaitForSingleObject(hevent,10000);
     }
     CloseHandle(hevent);
-    HeapFree(GetProcessHeap(), 0, frags);
+    free(frags);
 }
 
 static void wave_out_test_device(UINT_PTR device)
@@ -948,22 +948,21 @@ static void wave_out_test_device(UINT_PTR device)
        "waveOutMessage(%s): failed to get interface size, rc=%s\n",
        dev_name(device),wave_out_error(rc));
     if (rc==MMSYSERR_NOERROR) {
-        nameW = HeapAlloc(GetProcessHeap(), 0, size);
+        nameW = malloc(size);
         rc=waveOutMessage((HWAVEOUT)device, DRV_QUERYDEVICEINTERFACE,
                           (DWORD_PTR)nameW, size);
         ok(rc==MMSYSERR_NOERROR,"waveOutMessage(%s): failed to get interface "
            "name, rc=%s\n",dev_name(device),wave_out_error(rc));
         ok(lstrlenW(nameW)+1==size/sizeof(WCHAR),"got an incorrect size %ld\n",size);
         if (rc==MMSYSERR_NOERROR) {
-            nameA = HeapAlloc(GetProcessHeap(), 0, size/sizeof(WCHAR));
+            nameA = malloc(size/sizeof(WCHAR));
             WideCharToMultiByte(CP_ACP, 0, nameW, size/sizeof(WCHAR), nameA,
                                 size/sizeof(WCHAR), NULL, NULL);
         }
-        HeapFree(GetProcessHeap(), 0, nameW);
+        free(nameW);
     }
     else if (rc==MMSYSERR_NOTSUPPORTED) {
-        nameA=HeapAlloc(GetProcessHeap(), 0, sizeof("not supported"));
-        strcpy(nameA, "not supported");
+        nameA=strdup("not supported");
     }
 
     rc=waveOutGetDevCapsA(device,&capsA,sizeof(capsA));
@@ -972,7 +971,7 @@ static void wave_out_test_device(UINT_PTR device)
        dev_name(device),wave_out_error(rc));
     if (rc!=MMSYSERR_NOERROR)
     {
-        HeapFree(GetProcessHeap(), 0, nameA);
+        free(nameA);
         return;
     }
 
@@ -982,7 +981,7 @@ static void wave_out_test_device(UINT_PTR device)
     trace("     channels=%d formats=%05lx support=%04lx\n",
           capsA.wChannels,capsA.dwFormats,capsA.dwSupport);
     trace("     %s\n",wave_out_caps(capsA.dwSupport));
-    HeapFree(GetProcessHeap(), 0, nameA);
+    free(nameA);
 
     if (winetest_interactive && (device != WAVE_MAPPER))
     {
@@ -1587,8 +1586,7 @@ static void test_fragmentsize(void)
     memset(hdr, 0, sizeof(hdr));
     hdr[0].dwBufferLength = (fmt.nSamplesPerSec * fmt.nBlockAlign / 4) + 1;
     hdr[1].dwBufferLength = hdr[0].dwBufferLength - 2;
-    hdr[1].lpData = hdr[0].lpData =
-        HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, hdr[0].dwBufferLength);
+    hdr[1].lpData = hdr[0].lpData = calloc(1, hdr[0].dwBufferLength);
 
     rc = waveOutPrepareHeader(wout, &hdr[0], sizeof(hdr[0]));
     ok(rc == MMSYSERR_NOERROR, "waveOutPrepareHeader failed: %s\n", wave_out_error(rc));
@@ -1622,7 +1620,7 @@ static void test_fragmentsize(void)
     rc = waveOutClose(wout);
     ok(rc == MMSYSERR_NOERROR, "waveOutClose failed: %s\n", wave_out_error(rc));
 
-    HeapFree(GetProcessHeap(), 0, hdr[0].lpData);
+    free(hdr[0].lpData);
     CloseHandle(hevent);
 }
 
@@ -1695,8 +1693,7 @@ static void test_reentrant_callback(void)
     memset(hdr, 0, sizeof(hdr));
     hdr[0].dwBufferLength = (fmt.nSamplesPerSec * fmt.nBlockAlign / 10);
     hdr[1].dwBufferLength = hdr[0].dwBufferLength;
-    hdr[1].lpData = hdr[0].lpData =
-        HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, hdr[0].dwBufferLength);
+    hdr[1].lpData = hdr[0].lpData = calloc(1, hdr[0].dwBufferLength);
 
     rc = waveOutPrepareHeader(wout, &hdr[0], sizeof(hdr[0]));
     ok(rc == MMSYSERR_NOERROR, "waveOutPrepareHeader failed: %s\n", wave_out_error(rc));
@@ -1723,7 +1720,7 @@ static void test_reentrant_callback(void)
     rc = waveOutClose(wout);
     ok(rc == MMSYSERR_NOERROR, "waveOutClose failed: %s\n", wave_out_error(rc));
 
-    HeapFree(GetProcessHeap(), 0, hdr[0].lpData);
+    free(hdr[0].lpData);
     CloseHandle(hevent);
 }
 
@@ -1770,7 +1767,7 @@ static void create_wav_file(char *temp_file)
     ok(written == length, "mmioWrite failed, got %ld\n", written);
     rc = mmioAscend(h, &chunk, 0);
     ok(rc == MMSYSERR_NOERROR, "mmioAscend failed, got %d\n", rc);
-    HeapFree(GetProcessHeap(), 0, buffer);
+    free(buffer);
 
     rc = mmioAscend(h, &riff_chunk, 0);
     ok(rc == MMSYSERR_NOERROR, "mmioAscend failed, got %d\n", rc);
