@@ -330,12 +330,16 @@ NTSTATUS WINAPI wow64_NtQuerySystemInformation( UINT *args )
     case SystemCodeIntegrityInformation:  /* SYSTEM_CODEINTEGRITY_INFORMATION */
     case SystemKernelDebuggerInformationEx:  /* SYSTEM_KERNEL_DEBUGGER_INFORMATION_EX */
     case SystemCpuSetInformation:  /* SYSTEM_CPU_SET_INFORMATION */
+    case SystemProcessorBrandString:  /* char[] */
+    case SystemProcessorFeaturesInformation:  /* SYSTEM_PROCESSOR_FEATURES_INFORMATION */
     case SystemWineVersionInformation:  /* char[] */
         return NtQuerySystemInformation( class, ptr, len, retlen );
 
     case SystemCpuInformation:  /* SYSTEM_CPU_INFORMATION */
     case SystemEmulationProcessorInformation:  /* SYSTEM_CPU_INFORMATION */
-        return NtQuerySystemInformation( SystemEmulationProcessorInformation, ptr, len, retlen );
+        status = NtQuerySystemInformation( SystemEmulationProcessorInformation, ptr, len, retlen );
+        if (!status && pBTCpuUpdateProcessorInformation) pBTCpuUpdateProcessorInformation( ptr );
+        return status;
 
     case SystemBasicInformation:  /* SYSTEM_BASIC_INFORMATION */
     case SystemEmulationBasicInformation:  /* SYSTEM_BASIC_INFORMATION */
@@ -389,7 +393,7 @@ NTSTATUS WINAPI wow64_NtQuerySystemInformation( UINT *args )
                     info32->Modules[i].InitOrderIndex    = info->Modules[i].InitOrderIndex;
                     info32->Modules[i].LoadCount         = info->Modules[i].LoadCount;
                     info32->Modules[i].NameOffset        = info->Modules[i].NameOffset;
-                    strcpy( (char *)info->Modules[i].Name, (char *)info32->Modules[i].Name );
+                    strcpy( (char *)info32->Modules[i].Name, (char *)info->Modules[i].Name );
                 }
             }
         }
@@ -653,7 +657,7 @@ NTSTATUS WINAPI wow64_NtQuerySystemInformationEx( UINT *args )
     }
 
     case SystemCpuSetInformation:  /* SYSTEM_CPU_SET_INFORMATION */
-    case SystemSupportedProcessorArchitectures:  /* ULONG */
+    case SystemSupportedProcessorArchitectures:  /* SYSTEM_SUPPORTED_PROCESSOR_ARCHITECTURES_INFORMATION */
         return NtQuerySystemInformationEx( class, &handle, sizeof(handle), ptr, len, retlen );
 
     default:
