@@ -81,8 +81,8 @@ struct xinput_controller
         char *feature_report_buf;
 
         BYTE haptics_report;
-        BYTE haptics_rumble_index;
-        BYTE haptics_buzz_index;
+        HIDP_VALUE_CAPS haptics_rumble_caps;
+        HIDP_VALUE_CAPS haptics_buzz_caps;
     } hid;
 };
 
@@ -121,11 +121,9 @@ static struct xinput_controller controllers[XUSER_MAX_COUNT] =
 
 static HMODULE xinput_instance;
 static HANDLE start_event;
-static HANDLE stop_event;
-static HANDLE done_event;
 static HANDLE update_event;
 
-static BOOL find_opened_device(SP_DEVICE_INTERFACE_DETAIL_DATA_W *detail, int *free_slot)
+static BOOL find_opened_device(const WCHAR *device_path, int *free_slot)
 {
     int i;
 
@@ -133,7 +131,7 @@ static BOOL find_opened_device(SP_DEVICE_INTERFACE_DETAIL_DATA_W *detail, int *f
     for (i = XUSER_MAX_COUNT; i > 0; i--)
     {
         if (!controllers[i - 1].device) *free_slot = i - 1;
-        else if (!wcscmp(detail->DevicePath, controllers[i - 1].device_path)) return TRUE;
+        else if (!wcsicmp(device_path, controllers[i - 1].device_path)) return TRUE;
     }
     return FALSE;
 }
@@ -151,16 +149,74 @@ static void check_value_caps(struct xinput_controller *controller, USHORT usage,
     }
 }
 
+static void check_waveform_caps(struct xinput_controller *controller, HANDLE device, PHIDP_PREPARSED_DATA preparsed,
+                                HIDP_LINK_COLLECTION_NODE *collections, HIDP_VALUE_CAPS *caps)
+{
+    USHORT count, report_len = controller->hid.caps.FeatureReportByteLength;
+    char *report_buf = controller->hid.feature_report_buf;
+    ULONG parent = caps->LinkCollection, waveform = 0;
+    HIDP_VALUE_CAPS value_caps;
+    USAGE_AND_PAGE phy_usages;
+    NTSTATUS status;
+
+    while (collections[parent].LinkUsagePage != HID_USAGE_PAGE_HAPTICS ||
+           collections[parent].LinkUsage != HID_USAGE_HAPTICS_SIMPLE_CONTROLLER)
+        if (!(parent = collections[parent].Parent)) break;
+
+    if (collections[parent].LinkUsagePage != HID_USAGE_PAGE_HAPTICS ||
+        collections[parent].LinkUsage != HID_USAGE_HAPTICS_SIMPLE_CONTROLLER)
+    {
+        WARN("Failed to find haptics simple controller collection\n");
+        return;
+    }
+    phy_usages.UsagePage = collections[collections[parent].Parent].LinkUsagePage;
+    phy_usages.Usage = collections[collections[parent].Parent].LinkUsage;
+
+    status = HidP_InitializeReportForID(HidP_Feature, caps->ReportID, preparsed, report_buf, report_len);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_InitializeReportForID returned %#lx\n", status);
+    if (!HidD_GetFeature(device, report_buf, report_len))
+    {
+        WARN("Failed to get waveform list report, error %lu\n", GetLastError());
+        return;
+    }
+
+    status = HidP_GetUsageValue(HidP_Feature, caps->UsagePage, caps->LinkCollection, caps->NotRange.Usage,
+                                &waveform, preparsed, report_buf, report_len);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue returned %#lx\n", status);
+
+    count = 1;
+    status = HidP_GetSpecificValueCaps(HidP_Output, HID_USAGE_PAGE_HAPTICS, parent, HID_USAGE_HAPTICS_INTENSITY,
+                                       &value_caps, &count, preparsed);
+    if (status != HIDP_STATUS_SUCCESS || !count) WARN("Failed to get waveform intensity caps, status %#lx\n", status);
+    else if (phy_usages.UsagePage == HID_USAGE_PAGE_GENERIC && phy_usages.Usage == HID_USAGE_GENERIC_Z)
+        TRACE( "Ignoring left rumble caps\n" );
+    else if (phy_usages.UsagePage == HID_USAGE_PAGE_GENERIC && phy_usages.Usage == HID_USAGE_GENERIC_RZ)
+        TRACE( "Ignoring right rumble caps\n" );
+    else if (waveform == HID_USAGE_HAPTICS_WAVEFORM_RUMBLE)
+    {
+        TRACE("Found rumble caps, report %u collection %u\n", value_caps.ReportID, value_caps.LinkCollection);
+        controller->hid.haptics_report = value_caps.ReportID;
+        controller->hid.haptics_rumble_caps = value_caps;
+    }
+    else if (waveform == HID_USAGE_HAPTICS_WAVEFORM_BUZZ)
+    {
+        TRACE("Found buzz caps, report %u collection %u\n", value_caps.ReportID, value_caps.LinkCollection);
+        controller->hid.haptics_report = value_caps.ReportID;
+        controller->hid.haptics_buzz_caps = value_caps;
+    }
+    else FIXME("Unsupported waveform type %#lx\n", waveform);
+}
+
 static BOOL controller_check_caps(struct xinput_controller *controller, HANDLE device, PHIDP_PREPARSED_DATA preparsed)
 {
-    ULONG collections_count = 0, report_len = controller->hid.caps.FeatureReportByteLength;
-    char *report_buf = controller->hid.feature_report_buf;
+    USHORT caps_count = 0, waveform_caps_count = 0;
     XINPUT_CAPABILITIES *caps = &controller->caps;
-    HIDP_VALUE_CAPS *value_caps, waveform_cap;
-    int i, u, waveform_list, button_count = 0;
     HIDP_LINK_COLLECTION_NODE *collections;
+    HIDP_VALUE_CAPS waveform_caps[8];
     HIDP_BUTTON_CAPS *button_caps;
-    USHORT caps_count = 0;
+    ULONG collections_count = 0;
+    HIDP_VALUE_CAPS *value_caps;
+    int i, u, button_count = 0;
     NTSTATUS status;
 
     /* Count buttons */
@@ -168,7 +224,7 @@ static BOOL controller_check_caps(struct xinput_controller *controller, HANDLE d
 
     if (!(button_caps = malloc(sizeof(*button_caps) * controller->hid.caps.NumberInputButtonCaps))) return FALSE;
     status = HidP_GetButtonCaps(HidP_Input, button_caps, &controller->hid.caps.NumberInputButtonCaps, preparsed);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetButtonCaps returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetButtonCaps returned %#lx\n", status);
     else for (i = 0; i < controller->hid.caps.NumberInputButtonCaps; i++)
     {
         if (button_caps[i].UsagePage != HID_USAGE_PAGE_BUTTON)
@@ -185,7 +241,7 @@ static BOOL controller_check_caps(struct xinput_controller *controller, HANDLE d
 
     if (!(value_caps = malloc(sizeof(*value_caps) * controller->hid.caps.NumberInputValueCaps))) return FALSE;
     status = HidP_GetValueCaps(HidP_Input, value_caps, &controller->hid.caps.NumberInputValueCaps, preparsed);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetValueCaps returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetValueCaps returned %#lx\n", status);
     else for (i = 0; i < controller->hid.caps.NumberInputValueCaps; i++)
     {
         HIDP_VALUE_CAPS *caps = value_caps + i;
@@ -214,63 +270,27 @@ static BOOL controller_check_caps(struct xinput_controller *controller, HANDLE d
     collections_count = controller->hid.caps.NumberLinkCollectionNodes;
     if (!(collections = malloc(sizeof(*collections) * controller->hid.caps.NumberLinkCollectionNodes))) return FALSE;
     status = HidP_GetLinkCollectionNodes(collections, &collections_count, preparsed);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetLinkCollectionNodes returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetLinkCollectionNodes returned %#lx\n", status);
     else for (i = 0; i < collections_count; ++i)
     {
         if (collections[i].LinkUsagePage != HID_USAGE_PAGE_HAPTICS) continue;
-        if (collections[i].LinkUsage == HID_USAGE_HAPTICS_WAVEFORM_LIST) break;
+        if (collections[i].LinkUsage == HID_USAGE_HAPTICS_WAVEFORM_LIST)
+        {
+            caps_count = ARRAY_SIZE(waveform_caps) - waveform_caps_count;
+            value_caps = waveform_caps + waveform_caps_count;
+            status = HidP_GetSpecificValueCaps(HidP_Feature, HID_USAGE_PAGE_ORDINAL, i, 0, value_caps, &caps_count, preparsed);
+            if (status == HIDP_STATUS_SUCCESS) waveform_caps_count += caps_count;
+        }
     }
+    for (i = 0; i < waveform_caps_count; ++i) check_waveform_caps(controller, device, preparsed, collections, waveform_caps + i);
     free(collections);
-    if (status != HIDP_STATUS_SUCCESS || i == collections_count)
-    {
-        WARN("could not find haptics waveform list collection\n");
-        return TRUE;
-    }
-    waveform_list = i;
 
-    caps_count = 1;
-    status = HidP_GetSpecificValueCaps(HidP_Feature, HID_USAGE_PAGE_ORDINAL, waveform_list, 3, &waveform_cap, &caps_count, preparsed);
-    if (status != HIDP_STATUS_SUCCESS || !caps_count)
-    {
-        WARN("could not find haptics waveform list report id, status %#x\n", status);
-        return TRUE;
-    }
-
-    status = HidP_InitializeReportForID(HidP_Feature, waveform_cap.ReportID, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_InitializeReportForID returned %#x\n", status);
-    if (!HidD_GetFeature(device, report_buf, report_len))
-    {
-        WARN("failed to get waveform list report, error %u\n", GetLastError());
-        return TRUE;
-    }
-
-    controller->hid.haptics_buzz_index = 0;
-    controller->hid.haptics_rumble_index = 0;
-    for (i = 3; status == HIDP_STATUS_SUCCESS; ++i)
-    {
-        ULONG waveform = 0;
-        status = HidP_GetUsageValue(HidP_Feature, HID_USAGE_PAGE_ORDINAL, waveform_list,
-                                    i, &waveform, preparsed, report_buf, report_len);
-        if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue returned %#x\n", status);
-        else if (waveform == HID_USAGE_HAPTICS_WAVEFORM_BUZZ) controller->hid.haptics_buzz_index = i;
-        else if (waveform == HID_USAGE_HAPTICS_WAVEFORM_RUMBLE) controller->hid.haptics_rumble_index = i;
-    }
-
-    if (!controller->hid.haptics_buzz_index) WARN("haptics buzz not supported\n");
-    if (!controller->hid.haptics_rumble_index) WARN("haptics rumble not supported\n");
-    if (!controller->hid.haptics_rumble_index && !controller->hid.haptics_buzz_index) return TRUE;
-
-    caps_count = 1;
-    status = HidP_GetSpecificValueCaps(HidP_Output, HID_USAGE_PAGE_HAPTICS, 0, HID_USAGE_HAPTICS_MANUAL_TRIGGER,
-                                       &waveform_cap, &caps_count, preparsed);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetSpecificValueCaps MANUAL_TRIGGER returned %#x\n", status);
-    else if (!caps_count) WARN("haptics manual trigger not supported\n");
-    else
+    if (controller->hid.haptics_rumble_caps.UsagePage ||
+        controller->hid.haptics_buzz_caps.UsagePage)
     {
         caps->Flags |= XINPUT_CAPS_FFB_SUPPORTED;
         caps->Vibration.wLeftMotorSpeed = 255;
         caps->Vibration.wRightMotorSpeed = 255;
-        controller->hid.haptics_report = waveform_cap.ReportID;
     }
 
     return TRUE;
@@ -281,45 +301,38 @@ static DWORD HID_set_state(struct xinput_controller *controller, XINPUT_VIBRATIO
     ULONG report_len = controller->hid.caps.OutputReportByteLength;
     PHIDP_PREPARSED_DATA preparsed = controller->hid.preparsed;
     char *report_buf = controller->hid.output_report_buf;
-    BYTE report_id = controller->hid.haptics_report;
+    BOOL ret, update_rumble, update_buzz;
+    USHORT collection;
     NTSTATUS status;
+    BYTE report_id;
 
     if (!(controller->caps.Flags & XINPUT_CAPS_FFB_SUPPORTED)) return ERROR_SUCCESS;
 
+    update_rumble = (controller->vibration.wLeftMotorSpeed != state->wLeftMotorSpeed);
     controller->vibration.wLeftMotorSpeed = state->wLeftMotorSpeed;
+    update_buzz = (controller->vibration.wRightMotorSpeed != state->wRightMotorSpeed);
     controller->vibration.wRightMotorSpeed = state->wRightMotorSpeed;
 
     if (!controller->enabled) return ERROR_SUCCESS;
+    if (!update_rumble && !update_buzz) return ERROR_SUCCESS;
 
-    /* send haptics rumble report (left motor) */
+    report_id = controller->hid.haptics_report;
     status = HidP_InitializeReportForID(HidP_Output, report_id, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_InitializeReportForID returned %#x\n", status);
-    status = HidP_SetUsageValue(HidP_Output, HID_USAGE_PAGE_HAPTICS, 0, HID_USAGE_HAPTICS_INTENSITY,
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_InitializeReportForID returned %#lx\n", status);
+
+    collection = controller->hid.haptics_rumble_caps.LinkCollection;
+    status = HidP_SetUsageValue(HidP_Output, HID_USAGE_PAGE_HAPTICS, collection, HID_USAGE_HAPTICS_INTENSITY,
                                 state->wLeftMotorSpeed, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_SetUsageValue INTENSITY returned %#x\n", status);
-    status = HidP_SetUsageValue(HidP_Output, HID_USAGE_PAGE_HAPTICS, 0, HID_USAGE_HAPTICS_MANUAL_TRIGGER,
-                                controller->hid.haptics_rumble_index, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_SetUsageValue MANUAL_TRIGGER returned %#x\n", status);
-    if (!HidD_SetOutputReport(controller->device, report_buf, report_len))
-    {
-        WARN("HidD_SetOutputReport failed with error %u\n", GetLastError());
-        return GetLastError();
-    }
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_SetUsageValue INTENSITY returned %#lx\n", status);
 
-    /* send haptics buzz report (right motor) */
-    status = HidP_InitializeReportForID(HidP_Output, report_id, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_InitializeReportForID returned %#x\n", status);
-    status = HidP_SetUsageValue(HidP_Output, HID_USAGE_PAGE_HAPTICS, 0, HID_USAGE_HAPTICS_INTENSITY,
+    collection = controller->hid.haptics_buzz_caps.LinkCollection;
+    status = HidP_SetUsageValue(HidP_Output, HID_USAGE_PAGE_HAPTICS, collection, HID_USAGE_HAPTICS_INTENSITY,
                                 state->wRightMotorSpeed, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_SetUsageValue INTENSITY returned %#x\n", status);
-    status = HidP_SetUsageValue(HidP_Output, HID_USAGE_PAGE_HAPTICS, 0, HID_USAGE_HAPTICS_MANUAL_TRIGGER,
-                                controller->hid.haptics_buzz_index, preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_SetUsageValue MANUAL_TRIGGER returned %#x\n", status);
-    if (!HidD_SetOutputReport(controller->device, report_buf, report_len))
-    {
-        WARN("HidD_SetOutputReport failed with error %u\n", GetLastError());
-        return GetLastError();
-    }
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_SetUsageValue INTENSITY returned %#lx\n", status);
+
+    ret = HidD_SetOutputReport(controller->device, report_buf, report_len);
+    if (!ret) WARN("HidD_SetOutputReport failed with error %lu\n", GetLastError());
+    return 0;
 
     return ERROR_SUCCESS;
 }
@@ -358,7 +371,7 @@ static void controller_disable(struct xinput_controller *controller)
 }
 
 static BOOL controller_init(struct xinput_controller *controller, PHIDP_PREPARSED_DATA preparsed,
-                            HIDP_CAPS *caps, HANDLE device, WCHAR *device_path)
+                            HIDP_CAPS *caps, HANDLE device, const WCHAR *device_path)
 {
     HANDLE event = NULL;
 
@@ -417,7 +430,7 @@ static void get_registry_keys(HKEY *defkey, HKEY *appkey)
     }
 }
 
-static BOOL device_is_overriden(HANDLE device)
+static BOOL device_is_overridden(HANDLE device)
 {
     WCHAR name[MAX_PATH], buffer[MAX_PATH];
     DWORD size = sizeof(buffer);
@@ -440,19 +453,60 @@ static BOOL device_is_overriden(HANDLE device)
     return disable;
 }
 
+static BOOL try_add_device(const WCHAR *device_path)
+{
+    SP_DEVICE_INTERFACE_DATA iface = {sizeof(iface)};
+    PHIDP_PREPARSED_DATA preparsed;
+    HIDP_CAPS caps;
+    NTSTATUS status;
+    HANDLE device;
+    int i;
+
+    if (find_opened_device(device_path, &i)) return TRUE; /* already opened */
+    if (i == XUSER_MAX_COUNT) return FALSE; /* no more slots */
+
+    device = CreateFileW(device_path, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
+                         NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING, NULL);
+    if (device == INVALID_HANDLE_VALUE) return TRUE;
+
+    preparsed = NULL;
+    if (!HidD_GetPreparsedData(device, &preparsed))
+        WARN("ignoring HID device, HidD_GetPreparsedData failed with error %lu\n", GetLastError());
+    else if ((status = HidP_GetCaps(preparsed, &caps)) != HIDP_STATUS_SUCCESS)
+        WARN("ignoring HID device, HidP_GetCaps returned %#lx\n", status);
+    else if (caps.UsagePage != HID_USAGE_PAGE_GENERIC)
+        WARN("ignoring HID device, unsupported usage page %04x\n", caps.UsagePage);
+    else if (caps.Usage != HID_USAGE_GENERIC_GAMEPAD && caps.Usage != HID_USAGE_GENERIC_JOYSTICK &&
+             caps.Usage != HID_USAGE_GENERIC_MULTI_AXIS_CONTROLLER)
+        WARN("ignoring HID device, unsupported usage %04x:%04x\n", caps.UsagePage, caps.Usage);
+    else if (device_is_overridden(device))
+        WARN("ignoring HID device, overridden for dinput\n");
+    else if (!controller_init(&controllers[i], preparsed, &caps, device, device_path))
+        WARN("ignoring HID device, failed to initialize\n");
+    else
+        return TRUE;
+
+    CloseHandle(device);
+    HidD_FreePreparsedData(preparsed);
+    return TRUE;
+}
+
+static void try_remove_device(const WCHAR *device_path)
+{
+    int i;
+
+    if (find_opened_device(device_path, &i))
+        controller_destroy(&controllers[i], TRUE);
+}
+
 static void update_controller_list(void)
 {
     char buffer[sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA_W) + MAX_PATH * sizeof(WCHAR)];
     SP_DEVICE_INTERFACE_DETAIL_DATA_W *detail = (SP_DEVICE_INTERFACE_DETAIL_DATA_W *)buffer;
     SP_DEVICE_INTERFACE_DATA iface = {sizeof(iface)};
-    PHIDP_PREPARSED_DATA preparsed;
-    HIDP_CAPS caps;
-    NTSTATUS status;
     HDEVINFO set;
-    HANDLE device;
     DWORD idx;
     GUID guid;
-    int i;
 
     guid = GUID_DEVINTERFACE_WINEXINPUT;
 
@@ -464,34 +518,8 @@ static void update_controller_list(void)
     {
         if (!SetupDiGetDeviceInterfaceDetailW(set, &iface, detail, sizeof(buffer), NULL, NULL))
             continue;
-
-        if (find_opened_device(detail, &i)) continue; /* already opened */
-        if (i == XUSER_MAX_COUNT) break; /* no more slots */
-
-        device = CreateFileW(detail->DevicePath, GENERIC_READ | GENERIC_WRITE,
-                             FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
-                             FILE_FLAG_OVERLAPPED | FILE_FLAG_NO_BUFFERING, NULL);
-        if (device == INVALID_HANDLE_VALUE) continue;
-
-        preparsed = NULL;
-        if (!HidD_GetPreparsedData(device, &preparsed))
-            WARN("ignoring HID device, HidD_GetPreparsedData failed with error %u\n", GetLastError());
-        else if ((status = HidP_GetCaps(preparsed, &caps)) != HIDP_STATUS_SUCCESS)
-            WARN("ignoring HID device, HidP_GetCaps returned %#x\n", status);
-        else if (caps.UsagePage != HID_USAGE_PAGE_GENERIC)
-            WARN("ignoring HID device, unsupported usage page %04x\n", caps.UsagePage);
-        else if (caps.Usage != HID_USAGE_GENERIC_GAMEPAD && caps.Usage != HID_USAGE_GENERIC_JOYSTICK &&
-                 caps.Usage != HID_USAGE_GENERIC_MULTI_AXIS_CONTROLLER)
-            WARN("ignoring HID device, unsupported usage %04x:%04x\n", caps.UsagePage, caps.Usage);
-        else if (device_is_overriden(device))
-            WARN("ignoring HID device, overriden for dinput\n");
-        else if (!controller_init(&controllers[i], preparsed, &caps, device, detail->DevicePath))
-            WARN("ignoring HID device, failed to initialize\n");
-        else
-            continue;
-
-        CloseHandle(device);
-        HidD_FreePreparsedData(preparsed);
+        if (!try_add_device(detail->DevicePath))
+            break;
     }
 
     SetupDiDestroyDeviceInfoList(set);
@@ -517,21 +545,6 @@ static void controller_destroy(struct xinput_controller *controller, BOOL alread
     LeaveCriticalSection(&controller->crit);
 }
 
-static void stop_update_thread(void)
-{
-    int i;
-
-    SetEvent(stop_event);
-    WaitForSingleObject(done_event, INFINITE);
-
-    CloseHandle(start_event);
-    CloseHandle(stop_event);
-    CloseHandle(done_event);
-    CloseHandle(update_event);
-
-    for (i = 0; i < XUSER_MAX_COUNT; i++) controller_destroy(&controllers[i], FALSE);
-}
-
 static LONG sign_extend(ULONG value, const HIDP_VALUE_CAPS *caps)
 {
     UINT sign = 1 << (caps->BitSize - 1);
@@ -561,13 +574,13 @@ static void read_controller_state(struct xinput_controller *controller)
     {
         if (GetLastError() == ERROR_OPERATION_ABORTED) return;
         if (GetLastError() == ERROR_ACCESS_DENIED || GetLastError() == ERROR_INVALID_HANDLE) controller_destroy(controller, TRUE);
-        else ERR("Failed to read input report, GetOverlappedResult failed with error %u\n", GetLastError());
+        else ERR("Failed to read input report, GetOverlappedResult failed with error %lu\n", GetLastError());
         return;
     }
 
     button_length = ARRAY_SIZE(buttons);
     status = HidP_GetUsages(HidP_Input, HID_USAGE_PAGE_BUTTON, 0, buttons, &button_length, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsages HID_USAGE_PAGE_BUTTON returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsages HID_USAGE_PAGE_BUTTON returned %#lx\n", status);
 
     state.Gamepad.wButtons = 0;
     for (i = 0; i < button_length; i++)
@@ -589,7 +602,7 @@ static void read_controller_state(struct xinput_controller *controller)
     }
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_HATSWITCH, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_HATSWITCH returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_HATSWITCH returned %#lx\n", status);
     else switch (value)
     {
     /* 8 1 2
@@ -607,27 +620,27 @@ static void read_controller_state(struct xinput_controller *controller)
     }
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_X, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_X returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_X returned %#lx\n", status);
     else state.Gamepad.sThumbLX = scale_value(value, &controller->hid.lx_caps, -32768, 32767);
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_Y, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_Y returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_Y returned %#lx\n", status);
     else state.Gamepad.sThumbLY = -scale_value(value, &controller->hid.ly_caps, -32768, 32767) - 1;
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_RX, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_RX returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_RX returned %#lx\n", status);
     else state.Gamepad.sThumbRX = scale_value(value, &controller->hid.rx_caps, -32768, 32767);
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_RY, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_RY returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_RY returned %#lx\n", status);
     else state.Gamepad.sThumbRY = -scale_value(value, &controller->hid.ry_caps, -32768, 32767) - 1;
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_RZ, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_RZ returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_RZ returned %#lx\n", status);
     else state.Gamepad.bRightTrigger = scale_value(value, &controller->hid.rt_caps, 0, 255);
 
     status = HidP_GetUsageValue(HidP_Input, HID_USAGE_PAGE_GENERIC, 0, HID_USAGE_GENERIC_Z, &value, controller->hid.preparsed, report_buf, report_len);
-    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_Z returned %#x\n", status);
+    if (status != HIDP_STATUS_SUCCESS) WARN("HidP_GetUsageValue HID_USAGE_PAGE_GENERIC / HID_USAGE_GENERIC_Z returned %#lx\n", status);
     else state.Gamepad.bLeftTrigger = scale_value(value, &controller->hid.lt_caps, 0, 255);
 
     EnterCriticalSection(&controller->crit);
@@ -645,15 +658,21 @@ static void read_controller_state(struct xinput_controller *controller)
 
 static LRESULT CALLBACK xinput_devnotify_wndproc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
 {
-    if (msg == WM_DEVICECHANGE && wparam == DBT_DEVICEARRIVAL) update_controller_list();
+    if (msg == WM_DEVICECHANGE)
+    {
+        DEV_BROADCAST_DEVICEINTERFACE_W *iface = (DEV_BROADCAST_DEVICEINTERFACE_W *)lparam;
+        if (wparam == DBT_DEVICEARRIVAL) try_add_device(iface->dbcc_name);
+        if (wparam == DBT_DEVICEREMOVECOMPLETE) try_remove_device(iface->dbcc_name);
+    }
+
     return DefWindowProcW(hwnd, msg, wparam, lparam);
 }
 
 static DWORD WINAPI hid_update_thread_proc(void *param)
 {
-    struct xinput_controller *devices[XUSER_MAX_COUNT + 2];
-    HANDLE events[XUSER_MAX_COUNT + 2];
-    DWORD i, count = 2, ret = WAIT_TIMEOUT;
+    struct xinput_controller *devices[XUSER_MAX_COUNT + 1];
+    HANDLE events[XUSER_MAX_COUNT + 1];
+    DWORD i, count = 1, ret = WAIT_TIMEOUT;
     DEV_BROADCAST_DEVICEINTERFACE_W filter =
     {
         .dbcc_size = sizeof(DEV_BROADCAST_DEVICEINTERFACE_W),
@@ -671,6 +690,8 @@ static DWORD WINAPI hid_update_thread_proc(void *param)
     HWND hwnd;
     MSG msg;
 
+    SetThreadDescription(GetCurrentThread(), L"wine_xinput_hid_update");
+
     RegisterClassExW(&cls);
     hwnd = CreateWindowExW(0, cls.lpszClassName, NULL, 0, 0, 0, 0, 0,
                            HWND_MESSAGE, NULL, NULL, NULL);
@@ -683,7 +704,7 @@ static DWORD WINAPI hid_update_thread_proc(void *param)
     {
         if (ret == count) while (PeekMessageW(&msg, hwnd, 0, 0, PM_REMOVE)) DispatchMessageW(&msg);
         if (ret == WAIT_TIMEOUT) update_controller_list();
-        if (ret < count - 2) read_controller_state(devices[ret]);
+        if (ret < count - 1) read_controller_state(devices[ret]);
 
         count = 0;
         for (i = 0; i < XUSER_MAX_COUNT; ++i)
@@ -699,38 +720,35 @@ static DWORD WINAPI hid_update_thread_proc(void *param)
             LeaveCriticalSection(&controllers[i].crit);
         }
         events[count++] = update_event;
-        events[count++] = stop_event;
     }
-    while ((ret = MsgWaitForMultipleObjectsEx(count, events, 2000, QS_ALLINPUT, MWMO_ALERTABLE)) < count - 1 ||
-            ret == count || ret == WAIT_TIMEOUT);
+    while ((ret = MsgWaitForMultipleObjectsEx(count, events, 2000, QS_ALLINPUT, MWMO_ALERTABLE)) <= count ||
+            ret == WAIT_TIMEOUT);
+
+    ERR("wait failed in the update thread, ret %lu, error %lu\n", ret, GetLastError());
 
     UnregisterDeviceNotification(notif);
     DestroyWindow(hwnd);
     UnregisterClassW(cls.lpszClassName, xinput_instance);
 
-    if (ret != count - 1) ERR("update thread exited unexpectedly, ret %u\n", ret);
-    SetEvent(done_event);
-    return ret;
+    FreeLibraryAndExitThread(xinput_instance, ret);
 }
 
 static BOOL WINAPI start_update_thread_once( INIT_ONCE *once, void *param, void **context )
 {
     HANDLE thread;
+    HMODULE module;
+
+    if (!GetModuleHandleExW(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, (void*)hid_update_thread_proc, &module))
+        WARN("Failed to increase module's reference count, error: %lu\n", GetLastError());
 
     start_event = CreateEventA(NULL, FALSE, FALSE, NULL);
-    if (!start_event) ERR("failed to create start event, error %u\n", GetLastError());
-
-    stop_event = CreateEventA(NULL, FALSE, FALSE, NULL);
-    if (!stop_event) ERR("failed to create stop event, error %u\n", GetLastError());
-
-    done_event = CreateEventA(NULL, FALSE, FALSE, NULL);
-    if (!done_event) ERR("failed to create done event, error %u\n", GetLastError());
+    if (!start_event) ERR("failed to create start event, error %lu\n", GetLastError());
 
     update_event = CreateEventA(NULL, FALSE, FALSE, NULL);
-    if (!update_event) ERR("failed to create update event, error %u\n", GetLastError());
+    if (!update_event) ERR("failed to create update event, error %lu\n", GetLastError());
 
     thread = CreateThread(NULL, 0, hid_update_thread_proc, NULL, 0, NULL);
-    if (!thread) ERR("failed to create update thread, error %u\n", GetLastError());
+    if (!thread) ERR("failed to create update thread, error %lu\n", GetLastError());
     CloseHandle(thread);
 
     WaitForSingleObject(start_event, INFINITE);
@@ -765,16 +783,14 @@ static void controller_unlock(struct xinput_controller *controller)
 
 BOOL WINAPI DllMain(HINSTANCE inst, DWORD reason, LPVOID reserved)
 {
+    TRACE("inst %p, reason %lu, reserved %p.\n", inst, reason, reserved);
+
     switch (reason)
     {
     case DLL_PROCESS_ATTACH:
         xinput_instance = inst;
         DisableThreadLibraryCalls(inst);
         break;
-    case DLL_PROCESS_DETACH:
-        if (reserved) break;
-        stop_update_thread();
-        break;
     }
     return TRUE;
 }
@@ -783,7 +799,7 @@ void WINAPI DECLSPEC_HOTPATCH XInputEnable(BOOL enable)
 {
     int index;
 
-    TRACE("(enable %d)\n", enable);
+    TRACE("enable %d.\n", enable);
 
     /* Setting to false will stop messages from XInputSetState being sent
     to the controllers. Setting to true will send the last vibration
@@ -804,7 +820,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH XInputSetState(DWORD index, XINPUT_VIBRATION *vib
 {
     DWORD ret;
 
-    TRACE("(index %u, vibration %p)\n", index, vibration);
+    TRACE("index %lu, vibration %p.\n", index, vibration);
 
     start_update_thread();
 
@@ -839,7 +855,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH XInputGetState(DWORD index, XINPUT_STATE *state)
 {
     DWORD ret;
 
-    TRACE("(index %u, state %p)!\n", index, state);
+    TRACE("index %lu, state %p.\n", index, state);
 
     ret = xinput_get_state(index, state);
     if (ret != ERROR_SUCCESS) return ret;
@@ -852,7 +868,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH XInputGetState(DWORD index, XINPUT_STATE *state)
 
 DWORD WINAPI DECLSPEC_HOTPATCH XInputGetStateEx(DWORD index, XINPUT_STATE *state)
 {
-    TRACE("(index %u, state %p)!\n", index, state);
+    TRACE("index %lu, state %p.\n", index, state);
 
     return xinput_get_state(index, state);
 }
@@ -1052,7 +1068,7 @@ done:
 
 DWORD WINAPI DECLSPEC_HOTPATCH XInputGetKeystroke(DWORD index, DWORD reserved, PXINPUT_KEYSTROKE keystroke)
 {
-    TRACE("(index %u, reserved %u, keystroke %p)\n", index, reserved, keystroke);
+    TRACE("index %lu, reserved %lu, keystroke %p.\n", index, reserved, keystroke);
 
     if (index >= XUSER_MAX_COUNT && index != XUSER_INDEX_ANY) return ERROR_BAD_ARGUMENTS;
 
@@ -1070,7 +1086,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH XInputGetKeystroke(DWORD index, DWORD reserved, P
 
 DWORD WINAPI DECLSPEC_HOTPATCH XInputGetCapabilities(DWORD index, DWORD flags, XINPUT_CAPABILITIES *capabilities)
 {
-    TRACE("(index %u, flags 0x%x, capabilities %p)\n", index, flags, capabilities);
+    TRACE("index %lu, flags %#lx, capabilities %p.\n", index, flags, capabilities);
 
     start_update_thread();
 
@@ -1093,7 +1109,8 @@ DWORD WINAPI DECLSPEC_HOTPATCH XInputGetCapabilities(DWORD index, DWORD flags, X
 
 DWORD WINAPI DECLSPEC_HOTPATCH XInputGetDSoundAudioDeviceGuids(DWORD index, GUID *render_guid, GUID *capture_guid)
 {
-    FIXME("(index %u, render guid %p, capture guid %p) Stub!\n", index, render_guid, capture_guid);
+    FIXME("index %lu, render_guid %s, capture_guid %s stub!\n", index, debugstr_guid(render_guid),
+          debugstr_guid(capture_guid));
 
     if (index >= XUSER_MAX_COUNT) return ERROR_BAD_ARGUMENTS;
     if (!controllers[index].device) return ERROR_DEVICE_NOT_CONNECTED;
@@ -1105,7 +1122,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH XInputGetBatteryInformation(DWORD index, BYTE typ
 {
     static int once;
 
-    if (!once++) FIXME("(index %u, type %u, battery %p) Stub!\n", index, type, battery);
+    if (!once++) FIXME("index %lu, type %u, battery %p.\n", index, type, battery);
 
     if (index >= XUSER_MAX_COUNT) return ERROR_BAD_ARGUMENTS;
     if (!controllers[index].device) return ERROR_DEVICE_NOT_CONNECTED;
