@@ -139,8 +139,8 @@ RPCRTAPI void      WINAPI CStdStubBuffer_DebugServerRelease( IRpcStubBuffer *Thi
 RPCRTAPI ULONG     WINAPI NdrCStdStubBuffer_Release( IRpcStubBuffer *This, IPSFactoryBuffer *pPSF );
 RPCRTAPI ULONG     WINAPI NdrCStdStubBuffer2_Release(IRpcStubBuffer *This, IPSFactoryBuffer *pPSF);
 
-ULONG STDMETHODCALLTYPE CStdStubBuffer_Release( IRpcStubBuffer *This ) DECLSPEC_HIDDEN;
-ULONG STDMETHODCALLTYPE CStdStubBuffer2_Release(IRpcStubBuffer *This) DECLSPEC_HIDDEN;
+ULONG STDMETHODCALLTYPE CStdStubBuffer_Release( IRpcStubBuffer *This );
+ULONG STDMETHODCALLTYPE CStdStubBuffer2_Release(IRpcStubBuffer *This);
 
 #define CStdStubBuffer_METHODS \
   CStdStubBuffer_QueryInterface, \
@@ -191,8 +191,8 @@ RPCRTAPI HRESULT RPC_ENTRY
 RPCRTAPI HRESULT RPC_ENTRY
   NdrDllUnregisterProxy( HMODULE hDll, const ProxyFileInfo **pProxyFileList, const CLSID *pclsid );
 
-HRESULT __cdecl __wine_register_resources(void) DECLSPEC_HIDDEN;
-HRESULT __cdecl __wine_unregister_resources(void) DECLSPEC_HIDDEN;
+HRESULT __cdecl __wine_register_resources(void);
+HRESULT __cdecl __wine_unregister_resources(void);
 
 #define CSTDSTUBBUFFERRELEASE(pFactory) \
 ULONG WINAPI CStdStubBuffer_Release(IRpcStubBuffer *This) \
@@ -224,10 +224,10 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
 
 /* macros used in dlldata.c files */
 #define EXTERN_PROXY_FILE(proxy) \
-    EXTERN_C const ProxyFileInfo proxy##_ProxyFileInfo DECLSPEC_HIDDEN;
+    EXTERN_C const ProxyFileInfo proxy##_ProxyFileInfo;
 
 #define PROXYFILE_LIST_START \
-    const ProxyFileInfo * aProxyFileList[] DECLSPEC_HIDDEN = \
+    const ProxyFileInfo * aProxyFileList[] = \
     {
 
 #define REFERENCE_PROXY_FILE(proxy) \
@@ -242,11 +242,10 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
 /* define PROXY_CLSID_IS to specify the CLSID data of the PSFactoryBuffer */
 /* define neither to use the GUID of the first interface */
 #ifdef PROXY_CLSID
-# define CLSID_PSFACTORYBUFFER extern CLSID PROXY_CLSID DECLSPEC_HIDDEN;
+# define CLSID_PSFACTORYBUFFER extern CLSID PROXY_CLSID;
 #else
 # ifdef PROXY_CLSID_IS
-#  define CLSID_PSFACTORYBUFFER const CLSID CLSID_PSFactoryBuffer DECLSPEC_HIDDEN; \
-    const CLSID CLSID_PSFactoryBuffer = PROXY_CLSID_IS;
+#  define CLSID_PSFACTORYBUFFER const CLSID CLSID_PSFactoryBuffer = PROXY_CLSID_IS;
 #  define PROXY_CLSID CLSID_PSFactoryBuffer
 # else
 #  define CLSID_PSFACTORYBUFFER
@@ -287,7 +286,7 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
 
 #define DLLDATA_GETPROXYDLLINFO(pfl, rclsid) \
     void RPC_ENTRY GetProxyDllInfo(const ProxyFileInfo ***ppProxyFileInfo, \
-                                   const CLSID **ppClsid) DECLSPEC_HIDDEN; \
+                                   const CLSID **ppClsid); \
     void RPC_ENTRY GetProxyDllInfo(const ProxyFileInfo ***ppProxyFileInfo, \
                                    const CLSID **ppClsid) \
     { \
@@ -296,7 +295,7 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
     }
 
 #define DLLGETCLASSOBJECTROUTINE(pfl, factory_clsid, factory) \
-    HRESULT WINAPI DLLGETCLASSOBJECT_ENTRY(REFCLSID rclsid, REFIID riid, void **ppv) DECLSPEC_HIDDEN; \
+    HRESULT WINAPI DLLGETCLASSOBJECT_ENTRY(REFCLSID rclsid, REFIID riid, void **ppv); \
     HRESULT WINAPI DLLGETCLASSOBJECT_ENTRY(REFCLSID rclsid, REFIID riid, \
                                            void **ppv) \
     { \
@@ -305,16 +304,16 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
     }
 
 #define DLLCANUNLOADNOW(factory) \
-    HRESULT WINAPI DLLCANUNLOADNOW_ENTRY(void) DECLSPEC_HIDDEN; \
+    HRESULT WINAPI DLLCANUNLOADNOW_ENTRY(void); \
     HRESULT WINAPI DLLCANUNLOADNOW_ENTRY(void) \
     { \
         return NdrDllCanUnloadNow((factory)); \
     }
 
 #define REGISTER_PROXY_DLL_ROUTINES(pfl, factory_clsid) \
-    HINSTANCE hProxyDll DECLSPEC_HIDDEN = NULL; \
+    HINSTANCE hProxyDll = NULL; \
     \
-    BOOL WINAPI DLLMAIN_ENTRY(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) DECLSPEC_HIDDEN; \
+    BOOL WINAPI DLLMAIN_ENTRY(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved); \
     BOOL WINAPI DLLMAIN_ENTRY(HINSTANCE hinstDLL, DWORD fdwReason, \
                               LPVOID lpvReserved) \
     { \
@@ -326,13 +325,13 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
         return TRUE; \
     } \
     \
-    HRESULT WINAPI DLLREGISTERSERVER_ENTRY(void) DECLSPEC_HIDDEN; \
+    HRESULT WINAPI DLLREGISTERSERVER_ENTRY(void); \
     HRESULT WINAPI DLLREGISTERSERVER_ENTRY(void) \
     { \
         WINE_DO_REGISTER_DLL( (pfl), (factory_clsid) ); \
     } \
     \
-    HRESULT WINAPI DLLUNREGISTERSERVER_ENTRY(void) DECLSPEC_HIDDEN; \
+    HRESULT WINAPI DLLUNREGISTERSERVER_ENTRY(void); \
     HRESULT WINAPI DLLUNREGISTERSERVER_ENTRY(void) \
     { \
         WINE_DO_UNREGISTER_DLL( (pfl), (factory_clsid) ); \
@@ -347,7 +346,7 @@ ULONG WINAPI CStdStubBuffer2_Release(IRpcStubBuffer *This) \
 
 #define DLLDATA_ROUTINES(pfl, factory_clsid) \
     CLSID_PSFACTORYBUFFER \
-    CStdPSFactoryBuffer DECLSPEC_HIDDEN gPFactory = { NULL, 0, NULL, 0 }; \
+    CStdPSFactoryBuffer gPFactory = { NULL, 0, NULL, 0 }; \
     DLLDATA_GETPROXYDLLINFO(pfl, factory_clsid) \
     DLLGETCLASSOBJECTROUTINE(pfl, factory_clsid, &gPFactory) \
     DLLCANUNLOADNOW(&gPFactory) \
