@@ -43,6 +43,8 @@
 #define IOCTL_CONDRV_BEEP                  CTL_CODE(FILE_DEVICE_CONSOLE, 20, METHOD_BUFFERED, FILE_ANY_ACCESS)
 #define IOCTL_CONDRV_FLUSH                 CTL_CODE(FILE_DEVICE_CONSOLE, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)
 #define IOCTL_CONDRV_GET_WINDOW            CTL_CODE(FILE_DEVICE_CONSOLE, 22, METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_CONDRV_GET_PROCESS_LIST      CTL_CODE(FILE_DEVICE_CONSOLE, 23, METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_CONDRV_READ_CONSOLE_CONTROL  CTL_CODE(FILE_DEVICE_CONSOLE, 24, METHOD_BUFFERED, FILE_READ_ACCESS)
 
 /* console output ioctls */
 #define IOCTL_CONDRV_WRITE_CONSOLE         CTL_CODE(FILE_DEVICE_CONSOLE, 30, METHOD_BUFFERED, FILE_WRITE_ACCESS)
@@ -155,6 +157,14 @@ struct condrv_output_info_params
 #define SET_CONSOLE_OUTPUT_INFO_DISPLAY_WINDOW  0x0010
 #define SET_CONSOLE_OUTPUT_INFO_MAX_SIZE        0x0020
 #define SET_CONSOLE_OUTPUT_INFO_POPUP_ATTR      0x0040
+#define SET_CONSOLE_OUTPUT_INFO_FONT            0x0080
+
+/* IOCTL_CONDRV_GET_TITLE params */
+struct condrv_title_params
+{
+    unsigned int title_len;
+    WCHAR buffer[1];
+};
 
 /* IOCTL_CONDRV_FILL_OUTPUT params */
 struct condrv_fill_output_params
@@ -185,7 +195,9 @@ struct condrv_ctrl_event
 };
 
 /* Wine specific values for console inheritance (params->ConsoleHandle) */
-#define CONSOLE_HANDLE_ALLOC  ((HANDLE)1)
-#define CONSOLE_HANDLE_SHELL  ((HANDLE)2)
+#define CONSOLE_HANDLE_ALLOC            LongToHandle(-1)
+#define CONSOLE_HANDLE_ALLOC_NO_WINDOW  LongToHandle(-2)
+#define CONSOLE_HANDLE_SHELL            LongToHandle(-3)
+#define CONSOLE_HANDLE_SHELL_NO_WINDOW  LongToHandle(-4)
 
 #endif /* _INC_CONDRV */
