@@ -8,52 +8,19 @@
 #ifndef __XML_EXPORTS_H__
 #define __XML_EXPORTS_H__
 
-#if defined(_WIN32) || defined(__CYGWIN__)
 /** DOC_DISABLE */
-
-#include <windef.h>
-
-#ifdef LIBXML_STATIC
-  #define XMLPUBLIC
-#elif defined(IN_LIBXML)
-  #define XMLPUBLIC __declspec(dllexport)
-#else
-  #define XMLPUBLIC __declspec(dllimport)
-#endif
-
-#if defined(LIBXML_FASTCALL)
-  #define XMLCALL __fastcall
-#else
-  #define XMLCALL __cdecl
-#endif
-#define XMLCDECL WINAPIV
-
-/** DOC_ENABLE */
+#if defined(_WIN32) || defined(__CYGWIN__)
+  #ifdef LIBXML_STATIC
+    #define XMLPUBLIC
+  #elif defined(IN_LIBXML)
+    #define XMLPUBLIC __declspec(dllexport)
+  #else
+    #define XMLPUBLIC __declspec(dllimport)
+  #endif
 #else /* not Windows */
-
-/**
- * XMLPUBLIC:
- *
- * Macro which declares a public symbol
- */
-#define XMLPUBLIC
-
-/**
- * XMLCALL:
- *
- * Macro which declares the calling convention for exported functions
- */
-#define XMLCALL
-
-/**
- * XMLCDECL:
- *
- * Macro which declares the calling convention for exported functions that
- * use '...'.
- */
-#define XMLCDECL
-
+  #define XMLPUBLIC
 #endif /* platform switch */
+/** DOC_ENABLE */
 
 /*
  * XMLPUBFUN:
@@ -69,10 +36,14 @@
  */
 #define XMLPUBVAR XMLPUBLIC extern
 
+/** DOC_DISABLE */
 /* Compatibility */
+#define XMLCALL
+#define XMLCDECL WINAPIV
 #if !defined(LIBXML_DLL_IMPORT)
 #define LIBXML_DLL_IMPORT XMLPUBVAR
 #endif
+/** DOC_ENABLE */
 
 #endif /* __XML_EXPORTS_H__ */
 
