@@ -17,7 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define NONAMELESSUNION
 #include <stdlib.h>
 
 #include "conhost.h"
@@ -1549,7 +1548,7 @@ static BOOL select_font( struct dialog_info *di )
     args[0] = di->config.cell_width;
     args[1] = di->config.cell_height;
     FormatMessageW( FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
-                    fmt, 0, 0, buf, ARRAY_SIZE(buf), (__ms_va_list*)args );
+                    fmt, 0, 0, buf, ARRAY_SIZE(buf), (va_list *)args );
 
     SendDlgItemMessageW( di->dialog, IDC_FNT_FONT_INFO, WM_SETTEXT, 0, (LPARAM)buf );
     return TRUE;
@@ -1924,15 +1923,15 @@ static BOOL config_dialog( struct console *console, BOOL current )
     psp.hInstance = wndclass.hInstance;
     psp.lParam = (LPARAM)&di;
 
-    psp.u.pszTemplate = MAKEINTRESOURCEW(IDD_OPTION);
+    psp.pszTemplate = MAKEINTRESOURCEW(IDD_OPTION);
     psp.pfnDlgProc = option_dialog_proc;
     pages[0] = CreatePropertySheetPageW( &psp );
 
-    psp.u.pszTemplate = MAKEINTRESOURCEW(IDD_FONT);
+    psp.pszTemplate = MAKEINTRESOURCEW(IDD_FONT);
     psp.pfnDlgProc = font_dialog_proc;
     pages[1] = CreatePropertySheetPageW( &psp );
 
-    psp.u.pszTemplate = MAKEINTRESOURCEW(IDD_CONFIG);
+    psp.pszTemplate = MAKEINTRESOURCEW(IDD_CONFIG);
     psp.pfnDlgProc = config_dialog_proc;
     pages[2] = CreatePropertySheetPageW( &psp );
 
@@ -1947,7 +1946,7 @@ static BOOL config_dialog( struct console *console, BOOL current )
     header.pszCaption = buff;
     header.nPages     = 3;
     header.hwndParent = console->win;
-    header.u3.phpage  = pages;
+    header.phpage     = pages;
     header.dwFlags    = PSH_NOAPPLYNOW;
     if (PropertySheetW( &header ) < 1)
         return TRUE;
