@@ -81,33 +81,6 @@ VOID SetFileNameAndEncoding(LPCWSTR szFileName, ENCODING enc)
     Globals.encFile = enc;
 }
 
-/******************************************************************************
- *      get_dpi
- *
- * Get the dpi from registry HKCC\Software\Fonts\LogPixels.
- */
-DWORD get_dpi(void)
-{
-    static const WCHAR dpi_key_name[] = {'S','o','f','t','w','a','r','e','\\','F','o','n','t','s','\0'};
-    static const WCHAR dpi_value_name[] = {'L','o','g','P','i','x','e','l','s','\0'};
-    DWORD dpi = 96;
-    HKEY hkey;
-
-    if (RegOpenKeyW(HKEY_CURRENT_CONFIG, dpi_key_name, &hkey) == ERROR_SUCCESS)
-    {
-        DWORD type, size, new_dpi;
-
-        size = sizeof(new_dpi);
-        if(RegQueryValueExW(hkey, dpi_value_name, NULL, &type, (LPBYTE)&new_dpi, &size) == ERROR_SUCCESS)
-        {
-            if(type == REG_DWORD && new_dpi != 0)
-                dpi = new_dpi;
-        }
-        RegCloseKey(hkey);
-    }
-    return dpi;
-}
-
 void UpdateStatusBar(void)
 {
     int currentLine;
@@ -210,7 +183,7 @@ static VOID NOTEPAD_SaveSettingToRegistry(void)
 #undef SET_NOTEPAD_REG
 
         /* Store the current value as 10 * twips */
-        data = MulDiv(abs(Globals.lfFont.lfHeight), 720 , get_dpi());
+        data = MulDiv(abs(Globals.lfFont.lfHeight), 720, GetDpiForWindow(Globals.hMainWnd));
         RegSetValueExW(hkey, value_iPointSize, 0, REG_DWORD, (LPBYTE)&data, sizeof(DWORD));
 
         RegSetValueExW(hkey, value_lfFaceName, 0, REG_SZ, (LPBYTE)&Globals.lfFont.lfFaceName,
@@ -307,9 +280,9 @@ static VOID NOTEPAD_LoadSettingFromRegistry(void)
 
         size = sizeof(DWORD);
         if(RegQueryValueExW(hkey, value_iPointSize, 0, &type, (LPBYTE)&point_size, &size) == ERROR_SUCCESS)
-            if(type == REG_DWORD)
+            if(type == REG_DWORD && point_size)
                 /* The value is stored as 10 * twips */
-                Globals.lfFont.lfHeight = -MulDiv(abs(point_size), get_dpi(), 720);
+                Globals.lfFont.lfHeight = -MulDiv(abs(point_size), GetDpiForWindow(GetDesktopWindow()), 720);
 
         size = sizeof(Globals.lfFont.lfFaceName);
         if(RegQueryValueExW(hkey, value_lfFaceName, 0, &type, (LPBYTE)&data_helper, &size) == ERROR_SUCCESS)
