@@ -20,11 +20,9 @@
  */
 
 #define WIN32_LEAN_AND_MEAN
-#define NONAMELESSUNION
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
 
 #define COBJMACROS
