@@ -32,7 +32,7 @@ static BOOL be_arm64_get_addr(HANDLE hThread, const dbg_ctx_t *ctx,
     case be_cpu_addr_stack:
         return be_cpu_build_addr(hThread, ctx, addr, 0, ctx->ctx.Sp);
     case be_cpu_addr_frame:
-        return be_cpu_build_addr(hThread, ctx, addr, 0, ctx->ctx.u.s.Fp);
+        return be_cpu_build_addr(hThread, ctx, addr, 0, ctx->ctx.Fp);
         break;
     }
     return FALSE;
@@ -81,19 +81,19 @@ static void be_arm64_print_context(HANDLE hThread, const dbg_ctx_t *ctx, int all
             buf[i] = '-';
 
     dbg_printf(" Pc:%016I64x Sp:%016I64x Lr:%016I64x Cpsr:%08lx(%s)\n",
-               ctx->ctx.Pc, ctx->ctx.Sp, ctx->ctx.u.s.Lr, ctx->ctx.Cpsr, buf);
+               ctx->ctx.Pc, ctx->ctx.Sp, ctx->ctx.Lr, ctx->ctx.Cpsr, buf);
     dbg_printf(" x0: %016I64x x1: %016I64x x2: %016I64x x3: %016I64x x4: %016I64x\n",
-               ctx->ctx.u.s.X0, ctx->ctx.u.s.X1, ctx->ctx.u.s.X2, ctx->ctx.u.s.X3, ctx->ctx.u.s.X4);
+               ctx->ctx.X0, ctx->ctx.X1, ctx->ctx.X2, ctx->ctx.X3, ctx->ctx.X4);
     dbg_printf(" x5: %016I64x x6: %016I64x x7: %016I64x x8: %016I64x x9: %016I64x\n",
-               ctx->ctx.u.s.X5, ctx->ctx.u.s.X6, ctx->ctx.u.s.X7, ctx->ctx.u.s.X8, ctx->ctx.u.s.X9);
+               ctx->ctx.X5, ctx->ctx.X6, ctx->ctx.X7, ctx->ctx.X8, ctx->ctx.X9);
     dbg_printf(" x10:%016I64x x11:%016I64x x12:%016I64x x13:%016I64x x14:%016I64x\n",
-               ctx->ctx.u.s.X10, ctx->ctx.u.s.X11, ctx->ctx.u.s.X12, ctx->ctx.u.s.X13, ctx->ctx.u.s.X14);
+               ctx->ctx.X10, ctx->ctx.X11, ctx->ctx.X12, ctx->ctx.X13, ctx->ctx.X14);
     dbg_printf(" x15:%016I64x ip0:%016I64x ip1:%016I64x x18:%016I64x x19:%016I64x\n",
-               ctx->ctx.u.s.X15, ctx->ctx.u.s.X16, ctx->ctx.u.s.X17, ctx->ctx.u.s.X18, ctx->ctx.u.s.X19);
+               ctx->ctx.X15, ctx->ctx.X16, ctx->ctx.X17, ctx->ctx.X18, ctx->ctx.X19);
     dbg_printf(" x20:%016I64x x21:%016I64x x22:%016I64x x23:%016I64x x24:%016I64x\n",
-               ctx->ctx.u.s.X20, ctx->ctx.u.s.X21, ctx->ctx.u.s.X22, ctx->ctx.u.s.X23, ctx->ctx.u.s.X24);
+               ctx->ctx.X20, ctx->ctx.X21, ctx->ctx.X22, ctx->ctx.X23, ctx->ctx.X24);
     dbg_printf(" x25:%016I64x x26:%016I64x x27:%016I64x x28:%016I64x Fp:%016I64x\n",
-               ctx->ctx.u.s.X25, ctx->ctx.u.s.X26, ctx->ctx.u.s.X27, ctx->ctx.u.s.X28, ctx->ctx.u.s.Fp);
+               ctx->ctx.X25, ctx->ctx.X26, ctx->ctx.X27, ctx->ctx.X28, ctx->ctx.Fp);
 
     if (all_regs) dbg_printf( "Floating point ARM64 dump not implemented\n" );
 }
@@ -104,41 +104,41 @@ static void be_arm64_print_segment_info(HANDLE hThread, const dbg_ctx_t *ctx)
 
 static struct dbg_internal_var be_arm64_ctx[] =
 {
-    {CV_ARM64_PSTATE,     "cpsr",   (void*)FIELD_OFFSET(CONTEXT, Cpsr),    dbg_itype_unsigned_int32},
-    {CV_ARM64_X0 +  0,    "x0",     (void*)FIELD_OFFSET(CONTEXT, u.s.X0),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  1,    "x1",     (void*)FIELD_OFFSET(CONTEXT, u.s.X1),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  2,    "x2",     (void*)FIELD_OFFSET(CONTEXT, u.s.X2),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  3,    "x3",     (void*)FIELD_OFFSET(CONTEXT, u.s.X3),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  4,    "x4",     (void*)FIELD_OFFSET(CONTEXT, u.s.X4),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  5,    "x5",     (void*)FIELD_OFFSET(CONTEXT, u.s.X5),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  6,    "x6",     (void*)FIELD_OFFSET(CONTEXT, u.s.X6),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  7,    "x7",     (void*)FIELD_OFFSET(CONTEXT, u.s.X7),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  8,    "x8",     (void*)FIELD_OFFSET(CONTEXT, u.s.X8),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  9,    "x9",     (void*)FIELD_OFFSET(CONTEXT, u.s.X9),  dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  10,   "x10",    (void*)FIELD_OFFSET(CONTEXT, u.s.X10), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  11,   "x11",    (void*)FIELD_OFFSET(CONTEXT, u.s.X11), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  12,   "x12",    (void*)FIELD_OFFSET(CONTEXT, u.s.X12), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  13,   "x13",    (void*)FIELD_OFFSET(CONTEXT, u.s.X13), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  14,   "x14",    (void*)FIELD_OFFSET(CONTEXT, u.s.X14), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  15,   "x15",    (void*)FIELD_OFFSET(CONTEXT, u.s.X15), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  16,   "x16",    (void*)FIELD_OFFSET(CONTEXT, u.s.X16), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  17,   "x17",    (void*)FIELD_OFFSET(CONTEXT, u.s.X17), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  18,   "x18",    (void*)FIELD_OFFSET(CONTEXT, u.s.X18), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  19,   "x19",    (void*)FIELD_OFFSET(CONTEXT, u.s.X19), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  20,   "x20",    (void*)FIELD_OFFSET(CONTEXT, u.s.X20), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  21,   "x21",    (void*)FIELD_OFFSET(CONTEXT, u.s.X21), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  22,   "x22",    (void*)FIELD_OFFSET(CONTEXT, u.s.X22), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  23,   "x23",    (void*)FIELD_OFFSET(CONTEXT, u.s.X23), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  24,   "x24",    (void*)FIELD_OFFSET(CONTEXT, u.s.X24), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  25,   "x25",    (void*)FIELD_OFFSET(CONTEXT, u.s.X25), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  26,   "x26",    (void*)FIELD_OFFSET(CONTEXT, u.s.X26), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  27,   "x27",    (void*)FIELD_OFFSET(CONTEXT, u.s.X27), dbg_itype_unsigned_int64},
-    {CV_ARM64_X0 +  28,   "x28",    (void*)FIELD_OFFSET(CONTEXT, u.s.X28), dbg_itype_unsigned_int64},
-    {CV_ARM64_FP,         "fp",     (void*)FIELD_OFFSET(CONTEXT, u.s.Fp),  dbg_itype_unsigned_int64},
-    {CV_ARM64_LR,         "lr",     (void*)FIELD_OFFSET(CONTEXT, u.s.Lr),  dbg_itype_unsigned_int64},
-    {CV_ARM64_SP,         "sp",     (void*)FIELD_OFFSET(CONTEXT, Sp),      dbg_itype_unsigned_int64},
-    {CV_ARM64_PC,         "pc",     (void*)FIELD_OFFSET(CONTEXT, Pc),      dbg_itype_unsigned_int64},
-    {0,                   NULL,     0,                                     dbg_itype_none}
+    {CV_ARM64_PSTATE,     "cpsr",   (void*)FIELD_OFFSET(CONTEXT, Cpsr), dbg_itype_unsigned_int32},
+    {CV_ARM64_X0 +  0,    "x0",     (void*)FIELD_OFFSET(CONTEXT, X0),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  1,    "x1",     (void*)FIELD_OFFSET(CONTEXT, X1),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  2,    "x2",     (void*)FIELD_OFFSET(CONTEXT, X2),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  3,    "x3",     (void*)FIELD_OFFSET(CONTEXT, X3),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  4,    "x4",     (void*)FIELD_OFFSET(CONTEXT, X4),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  5,    "x5",     (void*)FIELD_OFFSET(CONTEXT, X5),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  6,    "x6",     (void*)FIELD_OFFSET(CONTEXT, X6),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  7,    "x7",     (void*)FIELD_OFFSET(CONTEXT, X7),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  8,    "x8",     (void*)FIELD_OFFSET(CONTEXT, X8),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  9,    "x9",     (void*)FIELD_OFFSET(CONTEXT, X9),   dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  10,   "x10",    (void*)FIELD_OFFSET(CONTEXT, X10),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  11,   "x11",    (void*)FIELD_OFFSET(CONTEXT, X11),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  12,   "x12",    (void*)FIELD_OFFSET(CONTEXT, X12),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  13,   "x13",    (void*)FIELD_OFFSET(CONTEXT, X13),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  14,   "x14",    (void*)FIELD_OFFSET(CONTEXT, X14),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  15,   "x15",    (void*)FIELD_OFFSET(CONTEXT, X15),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  16,   "x16",    (void*)FIELD_OFFSET(CONTEXT, X16),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  17,   "x17",    (void*)FIELD_OFFSET(CONTEXT, X17),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  18,   "x18",    (void*)FIELD_OFFSET(CONTEXT, X18),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  19,   "x19",    (void*)FIELD_OFFSET(CONTEXT, X19),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  20,   "x20",    (void*)FIELD_OFFSET(CONTEXT, X20),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  21,   "x21",    (void*)FIELD_OFFSET(CONTEXT, X21),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  22,   "x22",    (void*)FIELD_OFFSET(CONTEXT, X22),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  23,   "x23",    (void*)FIELD_OFFSET(CONTEXT, X23),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  24,   "x24",    (void*)FIELD_OFFSET(CONTEXT, X24),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  25,   "x25",    (void*)FIELD_OFFSET(CONTEXT, X25),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  26,   "x26",    (void*)FIELD_OFFSET(CONTEXT, X26),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  27,   "x27",    (void*)FIELD_OFFSET(CONTEXT, X27),  dbg_itype_unsigned_int64},
+    {CV_ARM64_X0 +  28,   "x28",    (void*)FIELD_OFFSET(CONTEXT, X28),  dbg_itype_unsigned_int64},
+    {CV_ARM64_FP,         "fp",     (void*)FIELD_OFFSET(CONTEXT, Fp),   dbg_itype_unsigned_int64},
+    {CV_ARM64_LR,         "lr",     (void*)FIELD_OFFSET(CONTEXT, Lr),   dbg_itype_unsigned_int64},
+    {CV_ARM64_SP,         "sp",     (void*)FIELD_OFFSET(CONTEXT, Sp),   dbg_itype_unsigned_int64},
+    {CV_ARM64_PC,         "pc",     (void*)FIELD_OFFSET(CONTEXT, Pc),   dbg_itype_unsigned_int64},
+    {0,                   NULL,     0,                                  dbg_itype_none}
 };
 
 static BOOL be_arm64_is_step_over_insn(const void* insn)
@@ -247,37 +247,37 @@ static BOOL be_arm64_set_context(HANDLE thread, const dbg_ctx_t *ctx)
 #define REG(f,n,t,r)  {f, n, t, FIELD_OFFSET(CONTEXT, r), sizeof(((CONTEXT*)NULL)->r)}
 
 static struct gdb_register be_arm64_gdb_register_map[] = {
-    REG("core", "x0",   NULL,         u.s.X0),
-    REG(NULL,   "x1",   NULL,         u.s.X1),
-    REG(NULL,   "x2",   NULL,         u.s.X2),
-    REG(NULL,   "x3",   NULL,         u.s.X3),
-    REG(NULL,   "x4",   NULL,         u.s.X4),
-    REG(NULL,   "x5",   NULL,         u.s.X5),
-    REG(NULL,   "x6",   NULL,         u.s.X6),
-    REG(NULL,   "x7",   NULL,         u.s.X7),
-    REG(NULL,   "x8",   NULL,         u.s.X8),
-    REG(NULL,   "x9",   NULL,         u.s.X9),
-    REG(NULL,   "x10",  NULL,         u.s.X10),
-    REG(NULL,   "x11",  NULL,         u.s.X11),
-    REG(NULL,   "x12",  NULL,         u.s.X12),
-    REG(NULL,   "x13",  NULL,         u.s.X13),
-    REG(NULL,   "x14",  NULL,         u.s.X14),
-    REG(NULL,   "x15",  NULL,         u.s.X15),
-    REG(NULL,   "x16",  NULL,         u.s.X16),
-    REG(NULL,   "x17",  NULL,         u.s.X17),
-    REG(NULL,   "x18",  NULL,         u.s.X18),
-    REG(NULL,   "x19",  NULL,         u.s.X19),
-    REG(NULL,   "x20",  NULL,         u.s.X20),
-    REG(NULL,   "x21",  NULL,         u.s.X21),
-    REG(NULL,   "x22",  NULL,         u.s.X22),
-    REG(NULL,   "x23",  NULL,         u.s.X23),
-    REG(NULL,   "x24",  NULL,         u.s.X24),
-    REG(NULL,   "x25",  NULL,         u.s.X25),
-    REG(NULL,   "x26",  NULL,         u.s.X26),
-    REG(NULL,   "x27",  NULL,         u.s.X27),
-    REG(NULL,   "x28",  NULL,         u.s.X28),
-    REG(NULL,   "x29",  NULL,         u.s.Fp),
-    REG(NULL,   "x30",  NULL,         u.s.Lr),
+    REG("core", "x0",   NULL,         X0),
+    REG(NULL,   "x1",   NULL,         X1),
+    REG(NULL,   "x2",   NULL,         X2),
+    REG(NULL,   "x3",   NULL,         X3),
+    REG(NULL,   "x4",   NULL,         X4),
+    REG(NULL,   "x5",   NULL,         X5),
+    REG(NULL,   "x6",   NULL,         X6),
+    REG(NULL,   "x7",   NULL,         X7),
+    REG(NULL,   "x8",   NULL,         X8),
+    REG(NULL,   "x9",   NULL,         X9),
+    REG(NULL,   "x10",  NULL,         X10),
+    REG(NULL,   "x11",  NULL,         X11),
+    REG(NULL,   "x12",  NULL,         X12),
+    REG(NULL,   "x13",  NULL,         X13),
+    REG(NULL,   "x14",  NULL,         X14),
+    REG(NULL,   "x15",  NULL,         X15),
+    REG(NULL,   "x16",  NULL,         X16),
+    REG(NULL,   "x17",  NULL,         X17),
+    REG(NULL,   "x18",  NULL,         X18),
+    REG(NULL,   "x19",  NULL,         X19),
+    REG(NULL,   "x20",  NULL,         X20),
+    REG(NULL,   "x21",  NULL,         X21),
+    REG(NULL,   "x22",  NULL,         X22),
+    REG(NULL,   "x23",  NULL,         X23),
+    REG(NULL,   "x24",  NULL,         X24),
+    REG(NULL,   "x25",  NULL,         X25),
+    REG(NULL,   "x26",  NULL,         X26),
+    REG(NULL,   "x27",  NULL,         X27),
+    REG(NULL,   "x28",  NULL,         X28),
+    REG(NULL,   "x29",  NULL,         Fp),
+    REG(NULL,   "x30",  NULL,         Lr),
     REG(NULL,   "sp",   "data_ptr",   Sp),
     REG(NULL,   "pc",   "code_ptr",   Pc),
     REG(NULL,   "cpsr", "cpsr_flags", Cpsr),
