@@ -122,17 +122,18 @@ static BOOL     source_locate_file(const char* srcfile, char* path)
         strcpy(path, srcfile);
         found = TRUE;
     }
-    else if (dbg_curr_process->search_path)
+    else
     {
         const char* spath;
-
+        const char* sp = dbg_curr_process->search_path;
+        if (!sp) sp = ".";
         spath = srcfile;
         while (!found)
         {
             while (*spath && *spath != '/' && *spath != '\\') spath++;
             if (!*spath) break;
             spath++;
-            found = SearchPathA(dbg_curr_process->search_path, spath, NULL, MAX_PATH, path, NULL);
+            found = SearchPathA(sp, spath, NULL, MAX_PATH, path, NULL);
         }
     }
     return found;
