@@ -21,9 +21,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
-#include "ntstatus.h"
-#define WIN32_NO_STATUS
 
 #include <assert.h>
 #include <fcntl.h>
@@ -33,16 +30,15 @@
 #include <stdlib.h>
 #include <sys/time.h>
 #include <sys/types.h>
-
-#ifdef HAVE_SYS_IOCTL_H
+#include <unistd.h>
 #include <sys/ioctl.h>
-#endif
-#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
-#endif
 #ifdef HAVE_SYS_FILIO_H
 #include <sys/filio.h>
 #endif
+
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winternl.h"
 
@@ -73,8 +69,8 @@ static void mailslot_destroy( struct object * );
 static const struct object_ops mailslot_ops =
 {
     sizeof(struct mailslot),   /* size */
+    &file_type,                /* type */
     mailslot_dump,             /* dump */
-    file_get_type,             /* get_type */
     add_queue,                 /* add_queue */
     remove_queue,              /* remove_queue */
     default_fd_signaled,       /* signaled */
@@ -90,7 +86,7 @@ static const struct object_ops mailslot_ops =
     default_unlink_name,       /* unlink_name */
     mailslot_open_file,        /* open_file */
     no_kernel_obj_list,        /* get_kernel_obj_list */
-    fd_close_handle,           /* close_handle */
+    no_close_handle,           /* close_handle */
     mailslot_destroy           /* destroy */
 };
 
@@ -108,6 +104,7 @@ static const struct fd_ops mailslot_fd_ops =
     default_fd_get_file_info,   /* get_file_info */
     no_fd_get_volume_info,      /* get_volume_info */
     default_fd_ioctl,           /* ioctl */
+    default_fd_cancel_async,    /* cancel_async */
     mailslot_queue_async,       /* queue_async */
     default_fd_reselect_async   /* reselect_async */
 };
@@ -131,8 +128,8 @@ static void mail_writer_destroy( struct object *obj);
 static const struct object_ops mail_writer_ops =
 {
     sizeof(struct mail_writer), /* size */
+    &file_type,                 /* type */
     mail_writer_dump,           /* dump */
-    file_get_type,              /* get_type */
     no_add_queue,               /* add_queue */
     NULL,                       /* remove_queue */
     NULL,                       /* signaled */
@@ -148,7 +145,7 @@ static const struct object_ops mail_writer_ops =
     NULL,                       /* unlink_name */
     no_open_file,               /* open_file */
     no_kernel_obj_list,         /* get_kernel_obj_list */
-    fd_close_handle,            /* close_handle */
+    no_close_handle,            /* close_handle */
     mail_writer_destroy         /* destroy */
 };
 
@@ -165,6 +162,7 @@ static const struct fd_ops mail_writer_fd_ops =
     default_fd_get_file_info,    /* get_file_info */
     no_fd_get_volume_info,       /* get_volume_info */
     default_fd_ioctl,            /* ioctl */
+    default_fd_cancel_async,     /* cancel_async */
     default_fd_queue_async,      /* queue_async */
     default_fd_reselect_async    /* reselect_async */
 };
@@ -184,7 +182,6 @@ struct mailslot_device_file
 };
 
 static void mailslot_device_dump( struct object *obj, int verbose );
-static struct object_type *mailslot_device_get_type( struct object *obj );
 static struct object *mailslot_device_lookup_name( struct object *obj, struct unicode_str *name,
                                                    unsigned int attr, struct object *root );
 static struct object *mailslot_device_open_file( struct object *obj, unsigned int access,
@@ -194,15 +191,15 @@ static void mailslot_device_destroy( struct object *obj );
 static const struct object_ops mailslot_device_ops =
 {
     sizeof(struct mailslot_device), /* size */
+    &device_type,                   /* type */
     mailslot_device_dump,           /* dump */
-    mailslot_device_get_type,       /* get_type */
     no_add_queue,                   /* add_queue */
     NULL,                           /* remove_queue */
     NULL,                           /* signaled */
     no_satisfied,                   /* satisfied */
     no_signal,                      /* signal */
     no_get_fd,                      /* get_fd */
-    no_map_access,                  /* map_access */
+    default_map_access,             /* map_access */
     default_get_sd,                 /* get_sd */
     default_set_sd,                 /* set_sd */
     default_get_full_name,          /* get_full_name */
@@ -224,15 +221,15 @@ static enum server_fd_type mailslot_device_file_get_fd_type( struct fd *fd );
 static const struct object_ops mailslot_device_file_ops =
 {
     sizeof(struct mailslot_device_file),    /* size */
+    &file_type,                             /* type */
     mailslot_device_file_dump,              /* dump */
-    file_get_type,                          /* get_type */
     add_queue,                              /* add_queue */
     remove_queue,                           /* remove_queue */
     default_fd_signaled,                    /* signaled */
     no_satisfied,                           /* satisfied */
     no_signal,                              /* signal */
     mailslot_device_file_get_fd,            /* get_fd */
-    default_fd_map_access,                  /* map_access */
+    default_map_access,                     /* map_access */
     default_get_sd,                         /* get_sd */
     default_set_sd,                         /* set_sd */
     mailslot_device_file_get_full_name,     /* get_full_name */
@@ -241,7 +238,7 @@ static const struct object_ops mailslot_device_file_ops =
     NULL,                                   /* unlink_name */
     no_open_file,                           /* open_file */
     no_kernel_obj_list,                     /* get_kernel_obj_list */
-    fd_close_handle,                        /* close_handle */
+    no_close_handle,                        /* close_handle */
     mailslot_device_file_destroy            /* destroy */
 };
 
@@ -256,6 +253,7 @@ static const struct fd_ops mailslot_device_fd_ops =
     default_fd_get_file_info,           /* get_file_info */
     no_fd_get_volume_info,              /* get_volume_info */
     default_fd_ioctl,                   /* ioctl */
+    default_fd_cancel_async,            /* cancel_async */
     default_fd_queue_async,             /* queue_async */
     default_fd_reselect_async           /* reselect_async */
 };
@@ -298,9 +296,7 @@ static struct fd *mailslot_get_fd( struct object *obj )
 static unsigned int mailslot_map_access( struct object *obj, unsigned int access )
 {
     /* mailslots can only be read */
-    if (access & GENERIC_READ)    access |= FILE_GENERIC_READ;
-    if (access & GENERIC_ALL)     access |= FILE_GENERIC_READ;
-    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL);
+    return default_map_access( obj, access ) & FILE_GENERIC_READ;
 }
 
 static int mailslot_link_name( struct object *obj, struct object_name *name, struct object *parent )
@@ -387,13 +383,6 @@ static void mailslot_device_dump( struct object *obj, int verbose )
     fputs( "Mailslot device\n", stderr );
 }
 
-static struct object_type *mailslot_device_get_type( struct object *obj )
-{
-    static const WCHAR name[] = {'D','e','v','i','c','e'};
-    static const struct unicode_str str = { name, sizeof(name) };
-    return get_object_type( &str );
-}
-
 static struct object *mailslot_device_lookup_name( struct object *obj, struct unicode_str *name,
                                                    unsigned int attr, struct object *root )
 {
@@ -546,9 +535,7 @@ static struct fd *mail_writer_get_fd( struct object *obj )
 static unsigned int mail_writer_map_access( struct object *obj, unsigned int access )
 {
     /* mailslot writers can only get write access */
-    if (access & GENERIC_WRITE)   access |= FILE_GENERIC_WRITE;
-    if (access & GENERIC_ALL)     access |= FILE_GENERIC_WRITE;
-    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL);
+    return default_map_access( obj, access ) & FILE_GENERIC_WRITE;
 }
 
 static struct mailslot *get_mailslot_obj( struct process *process, obj_handle_t handle,
