@@ -23,7 +23,6 @@
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
-#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -912,11 +911,7 @@ int main(int argc, char **argv)
     font_off = (fontdir_off + fontdir_len + 15) & ~0x0f;
 
     atexit( cleanup );
-    signal( SIGTERM, exit_on_signal );
-    signal( SIGINT, exit_on_signal );
-#ifdef SIGHUP
-    signal( SIGHUP, exit_on_signal );
-#endif
+    init_signals( exit_on_signal );
 
     if (!output_name)  /* build a default output name */
         output_name = strmake( "%s%s", get_basename_noext( input_file ),
