@@ -28,8 +28,8 @@
 #include <string.h>
 #include <stdarg.h>
 
-#include "wine/list.h"
 #include "build.h"
+#include "wine/list.h"
 
 /* standard C functions that are also exported from ntdll */
 static const char *stdc_names[] =
@@ -576,7 +576,6 @@ static void check_undefined_exports( DLLSPEC *spec )
         ORDDEF *odp = &spec->entry_points[i];
         if (odp->type == TYPE_STUB || odp->type == TYPE_ABS || odp->type == TYPE_VARIABLE) continue;
         if (odp->flags & FLAG_FORWARD) continue;
-        if (odp->flags & FLAG_SYSCALL) continue;
         if (find_name( odp->link_name, undef_symbols ))
         {
             switch(odp->type)
@@ -594,7 +593,6 @@ static void check_undefined_exports( DLLSPEC *spec )
                             spec->src_name, odp->lineno, odp->link_name );
                 break;
             default:
-                if (!strcmp( odp->link_name, "__wine_syscall_dispatcher" )) break;
                 error( "%s:%d: external symbol '%s' is not a function\n",
                        spec->src_name, odp->lineno, odp->link_name );
                 break;
@@ -618,7 +616,6 @@ static char *create_undef_symbols_file( DLLSPEC *spec )
         ORDDEF *odp = &spec->entry_points[i];
         if (odp->type == TYPE_STUB || odp->type == TYPE_ABS || odp->type == TYPE_VARIABLE) continue;
         if (odp->flags & FLAG_FORWARD) continue;
-        if (odp->flags & FLAG_SYSCALL) continue;
         output( "\t%s %s\n", get_asm_ptr_keyword(), asm_name( get_link_name( odp )));
     }
     for (j = 0; j < extra_ld_symbols.count; j++)
@@ -627,7 +624,7 @@ static char *create_undef_symbols_file( DLLSPEC *spec )
     output_gnu_stack_note();
     fclose( output_file );
 
-    obj_file = get_temp_file_name( output_file_name, ".o" );
+    obj_file = make_temp_file( output_file_name, ".o" );
     assemble_file( as_file, obj_file );
     return obj_file;
 }
@@ -640,7 +637,7 @@ static const char *ldcombine_files( DLLSPEC *spec, struct strarray files )
     struct strarray args = get_ld_command();
 
     undef_file = create_undef_symbols_file( spec );
-    ld_tmp_file = get_temp_file_name( output_file_name, ".o" );
+    ld_tmp_file = make_temp_file( output_file_name, ".o" );
 
     strarray_add( &args, "-r" );
     strarray_add( &args, "-o" );
@@ -752,24 +749,16 @@ int is_undefined( const char *name )
 void output_get_pc_thunk(void)
 {
     assert( target.cpu == CPU_i386 );
-    output( "\n\t.text\n" );
-    output( "\t.align %d\n", get_alignment(4) );
-    output( "\t%s\n", func_declaration("__wine_spec_get_pc_thunk_eax") );
-    output( "%s:\n", asm_name("__wine_spec_get_pc_thunk_eax") );
-    output_cfi( ".cfi_startproc" );
+    output_function_header( "__wine_spec_get_pc_thunk_eax", 0 );
     output( "\tmovl (%%esp),%%eax\n" );
     output( "\tret\n" );
-    output_cfi( ".cfi_endproc" );
     output_function_size( "__wine_spec_get_pc_thunk_eax" );
 }
 
 /* output a single import thunk */
 static void output_import_thunk( const char *name, const char *table, int pos )
 {
-    output( "\n\t.align %d\n", get_alignment(4) );
-    output( "\t%s\n", func_declaration(name) );
-    output( "%s\n", asm_globl(name) );
-    output_cfi( ".cfi_startproc" );
+    output_function_header( name, 1 );
 
     switch (target.cpu)
     {
@@ -810,8 +799,10 @@ static void output_import_thunk( const char *name, const char *table, int pos )
         output( "\tldr x16, [x16, #%u]\n", pos & 0x7fff );
         output( "\tbr x16\n" );
         break;
+    case CPU_ARM64EC:
+        assert( 0 );
+        break;
     }
-    output_cfi( ".cfi_endproc" );
     output_function_size( name );
 }
 
@@ -839,7 +830,7 @@ static void output_immediate_imports(void)
 
     output( "\n/* import table */\n" );
     output( "\n\t.data\n" );
-    output( "\t.align %d\n", get_alignment(4) );
+    output( "\t.balign 4\n" );
     output( ".L__wine_spec_imports:\n" );
 
     /* list of dlls */
@@ -860,7 +851,7 @@ static void output_immediate_imports(void)
     output( "\t.long 0\n" );     /* Name */
     output( "\t.long 0\n" );     /* FirstThunk */
 
-    output( "\n\t.align %d\n", get_alignment(get_ptr_size()) );
+    output( "\n\t.balign %u\n", get_ptr_size() );
     /* output the names twice, once for OriginalFirstThunk and once for FirstThunk */
     for (i = 0; i < 2; i++)
     {
@@ -889,7 +880,7 @@ static void output_immediate_imports(void)
         {
             struct import_func *func = &import->imports[j];
             if (!func->name) continue;
-            output( "\t.align %d\n", get_alignment(2) );
+            output( "\t.balign 2\n" );
             output( ".L__wine_spec_import_data_%s_%s:\n", import->c_name, func->name );
             output( "\t.short %d\n", func->hint );
             output( "\t%s \"%s\"\n", get_asm_string_keyword(), func->name );
@@ -914,7 +905,7 @@ static void output_immediate_import_thunks(void)
 
     output( "\n/* immediate import thunks */\n\n" );
     output( "\t.text\n" );
-    output( "\t.align %d\n", get_alignment(8) );
+    output( "\t.balign 8\n" );
     output( "%s:\n", asm_name(import_thunks));
 
     pos = 0;
@@ -941,7 +932,7 @@ static void output_delayed_imports( const DLLSPEC *spec )
 
     output( "\n/* delayed imports */\n\n" );
     output( "\t.data\n" );
-    output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+    output( "\t.balign %u\n", get_ptr_size() );
     output( ".L__wine_spec_delay_imports:\n" );
 
     /* list of dlls */
@@ -1007,7 +998,7 @@ static void output_delayed_imports( const DLLSPEC *spec )
         {
             struct import_func *func = &import->imports[j];
             if (!func->name) continue;
-            output( "\t.align %d\n", get_alignment(2) );
+            output( "\t.balign 2\n" );
             output( ".L__wine_delay_data_%s_%s:\n", import->c_name, func->name );
             output( "\t.short %d\n", func->hint );
             output( "\t%s \"%s\"\n", get_asm_string_keyword(), func->name );
@@ -1027,16 +1018,14 @@ static void output_delayed_import_thunks( const DLLSPEC *spec )
 
     output( "\n/* delayed import thunks */\n\n" );
     output( "\t.text\n" );
-    output( "\t.align %d\n", get_alignment(8) );
+    output( "\t.balign 8\n" );
     output( "%s:\n", asm_name(delayed_import_loaders));
 
     pos = iat_pos = 0;
     LIST_FOR_EACH_ENTRY( import, &dll_delayed, struct import, entry )
     {
         char *module_func = strmake( "__wine_delay_load_asm_%s", import->c_name );
-        output( "\t.align %d\n", get_alignment(4) );
-        output( "\t%s\n", func_declaration(module_func) );
-        output( "%s:\n", asm_name(module_func) );
+        output_function_header( module_func, 0 );
         output_cfi( ".cfi_startproc" );
         switch (target.cpu)
         {
@@ -1132,6 +1121,9 @@ static void output_delayed_import_thunks( const DLLSPEC *spec )
             output( "\tldp x29, x30, [sp],#80\n" );
             output( "\tbr x16\n" );
             break;
+        case CPU_ARM64EC:
+            assert( 0 );
+            break;
         }
         output_cfi( ".cfi_endproc" );
         output_function_size( module_func );
@@ -1144,7 +1136,6 @@ static void output_delayed_import_thunks( const DLLSPEC *spec )
 
             if (thumb_mode) output( "\t.thumb_func\n" );
             output( "__wine_delay_imp_%s_%s:\n", import->c_name, name );
-            output_cfi( ".cfi_startproc" );
             switch (target.cpu)
             {
             case CPU_i386:
@@ -1182,15 +1173,17 @@ static void output_delayed_import_thunks( const DLLSPEC *spec )
                 if (iat_pos) output( "\tadd x16, x16, #%u\n", iat_pos );
                 output( "\tb %s\n", asm_name(module_func) );
                 break;
+            case CPU_ARM64EC:
+                assert( 0 );
+                break;
             }
-            output_cfi( ".cfi_endproc" );
             iat_pos += get_ptr_size();
         }
         pos += 8 * 4;  /* IMAGE_DELAYLOAD_DESCRIPTOR is 8 DWORDs */
     }
     output_function_size( delayed_import_loaders );
 
-    output( "\n\t.align %d\n", get_alignment(get_ptr_size()) );
+    output( "\n\t.balign %u\n", get_ptr_size() );
     output( "%s:\n", asm_name(delayed_import_thunks));
     pos = 0;
     LIST_FOR_EACH_ENTRY( import, &dll_delayed, struct import, entry )
@@ -1223,13 +1216,13 @@ static void output_external_link_imports( DLLSPEC *spec )
 
     output( "\n/* external link thunks */\n\n" );
     output( "\t.data\n" );
-    output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+    output( "\t.balign %u\n", get_ptr_size() );
     output( ".L__wine_spec_external_links:\n" );
     for (i = 0; i < ext_link_imports.count; i++)
         output( "\t%s %s\n", get_asm_ptr_keyword(), asm_name(ext_link_imports.str[i]) );
 
     output( "\n\t.text\n" );
-    output( "\t.align %d\n", get_alignment(get_ptr_size()) );
+    output( "\t.balign %u\n", get_ptr_size() );
     output( "%s:\n", asm_name("__wine_spec_external_link_thunks") );
 
     for (i = pos = 0; i < ext_link_imports.count; i++)
@@ -1255,7 +1248,6 @@ void output_stubs( DLLSPEC *spec )
     if (!has_stubs( spec )) return;
 
     output( "\n/* stub functions */\n\n" );
-    output( "\t.text\n" );
 
     for (i = 0; i < spec->nb_entry_points; i++)
     {
@@ -1264,14 +1256,12 @@ void output_stubs( DLLSPEC *spec )
 
         name = get_stub_name( odp, spec );
         exp_name = odp->name ? odp->name : odp->export_name;
-        output( "\t.align %d\n", get_alignment(4) );
-        output( "\t%s\n", func_declaration(name) );
-        output( "%s:\n", asm_name(name) );
-        output_cfi( ".cfi_startproc" );
+        output_function_header( name, 0 );
 
         switch (target.cpu)
         {
         case CPU_i386:
+            output_cfi( ".cfi_startproc" );
             /* flesh out the stub a bit to make safedisc happy */
             output(" \tnop\n" );
             output(" \tnop\n" );
@@ -1309,16 +1299,23 @@ void output_stubs( DLLSPEC *spec )
                 output( "\tmovl $.L__wine_spec_file_name,(%%esp)\n" );
             }
             output( "\tcall %s\n", asm_name("__wine_spec_unimplemented_stub") );
+            output_cfi( ".cfi_endproc" );
             break;
         case CPU_x86_64:
+            output_cfi( ".cfi_startproc" );
+            output_seh( ".seh_proc %s", asm_name(name) );
             output( "\tsubq $0x28,%%rsp\n" );
-            output_cfi( ".cfi_adjust_cfa_offset 8" );
+            output_cfi( ".cfi_adjust_cfa_offset 0x28" );
+            output_seh( ".seh_stackalloc 0x28" );
+            output_seh( ".seh_endprologue" );
             output( "\tleaq .L__wine_spec_file_name(%%rip),%%rcx\n" );
             if (exp_name)
                 output( "leaq .L%s_string(%%rip),%%rdx\n", name );
             else
                 output( "\tmovq $%d,%%rdx\n", odp->ordinal );
             output( "\tcall %s\n", asm_name("__wine_spec_unimplemented_stub") );
+            output_cfi( ".cfi_endproc" );
+            output_seh( ".seh_endproc" );
             break;
         case CPU_ARM:
             if (UsePIC)
@@ -1346,6 +1343,9 @@ void output_stubs( DLLSPEC *spec )
             }
             break;
         case CPU_ARM64:
+        case CPU_ARM64EC:
+            output_seh( ".seh_proc %s", arm64_name(name) );
+            output_seh( ".seh_endprologue" );
             output( "\tadrp x0, %s\n", arm64_page(".L__wine_spec_file_name") );
             output( "\tadd x0, x0, #%s\n", arm64_pageoff(".L__wine_spec_file_name") );
             if (exp_name)
@@ -1357,12 +1357,10 @@ void output_stubs( DLLSPEC *spec )
             }
             else
                 output( "\tmov x1, %u\n", odp->ordinal );
-            output( "\tbl %s\n", asm_name("__wine_spec_unimplemented_stub") );
+            output( "\tb %s\n", arm64_name("__wine_spec_unimplemented_stub") );
+            output_seh( ".seh_endproc" );
             break;
-        default:
-            assert(0);
         }
-        output_cfi( ".cfi_endproc" );
         output_function_size( name );
     }
 
@@ -1383,158 +1381,6 @@ void output_stubs( DLLSPEC *spec )
     }
 }
 
-static int cmp_link_name( const void *e1, const void *e2 )
-{
-    const ORDDEF *odp1 = *(const ORDDEF * const *)e1;
-    const ORDDEF *odp2 = *(const ORDDEF * const *)e2;
-
-    return strcmp( odp1->link_name, odp2->link_name );
-}
-
-
-/* output the functions for system calls */
-void output_syscalls( DLLSPEC *spec )
-{
-    int i, count;
-    ORDDEF **syscalls = NULL;
-
-    for (i = count = 0; i < spec->nb_entry_points; i++)
-    {
-        ORDDEF *odp = &spec->entry_points[i];
-        if (!(odp->flags & FLAG_SYSCALL)) continue;
-        if (!syscalls) syscalls = xmalloc( (spec->nb_entry_points - i) * sizeof(*syscalls) );
-        syscalls[count++] = odp;
-    }
-    if (!count) return;
-    count = sort_func_list( syscalls, count, cmp_link_name );
-
-    output( "\n/* system calls */\n\n" );
-    output( "\t.text\n" );
-
-    for (i = 0; i < count; i++)
-    {
-        ORDDEF *odp = syscalls[i];
-        const char *name = get_link_name(odp);
-        unsigned int id = (spec->syscall_table << 12) + i;
-
-        output( "\t.align %d\n", get_alignment(16) );
-        output( "\t%s\n", func_declaration(name) );
-        output( "%s\n", asm_globl(name) );
-        output_cfi( ".cfi_startproc" );
-        switch (target.cpu)
-        {
-        case CPU_i386:
-            if (UsePIC)
-            {
-                output( "\tcall %s\n", asm_name("__wine_spec_get_pc_thunk_eax") );
-                output( "1:\tmovl %s-1b(%%eax),%%edx\n", asm_name("__wine_syscall_dispatcher") );
-                output( "\tmovl $%u,%%eax\n", id );
-                needs_get_pc_thunk = 1;
-            }
-            else
-            {
-                output( "\tmovl $%u,%%eax\n", id );
-                output( "\tmovl $%s,%%edx\n", asm_name("__wine_syscall") );
-            }
-            output( "\tcall *%%edx\n" );
-            output( "\tret $%u\n", odp->type == TYPE_STDCALL ? get_args_size( odp ) : 0 );
-            break;
-        case CPU_x86_64:
-            /* Chromium depends on syscall thunks having the same form as on
-             * Windows. For 64-bit systems the only viable form we can emulate is
-             * having an int $0x2e fallback. Since actually using an interrupt is
-             * expensive, and since for some reason Chromium doesn't actually
-             * validate that instruction, we can just put a jmp there instead. */
-            output( "\t.byte 0x4c,0x8b,0xd1\n" ); /* movq %rcx,%r10 */
-            output( "\t.byte 0xb8\n" );           /* movl $i,%eax */
-            output( "\t.long %u\n", id );
-            output( "\t.byte 0xf6,0x04,0x25,0x08,0x03,0xfe,0x7f,0x01\n" ); /* testb $1,0x7ffe0308 */
-            output( "\t.byte 0x75,0x03\n" );      /* jne 1f */
-            output( "\t.byte 0x0f,0x05\n" );      /* syscall */
-            output( "\t.byte 0xc3\n" );           /* ret */
-            output( "\tjmp 1f\n" );
-            output( "\t.byte 0xc3\n" );           /* ret */
-            if (is_pe())
-            {
-                output( "1:\t.byte 0xff,0x14,0x25\n" ); /* 1: callq *(0x7ffe1000) */
-                output( "\t.long 0x7ffe1000\n" );
-            }
-            else
-            {
-                output( "\tnop\n" );
-                output( "1:\tcallq *%s(%%rip)\n", asm_name("__wine_syscall_dispatcher") );
-            }
-            output( "\tret\n" );
-            break;
-        case CPU_ARM:
-            output( "\tpush {r0-r3}\n" );
-            output( "\tmovw ip, #%u\n", id );
-            output( "\tmov r3, lr\n" );
-            output( "\tbl %s\n", asm_name("__wine_syscall") );
-            output( "\tbx lr\n" );
-            break;
-        case CPU_ARM64:
-            output( "\tmov x8, #%u\n", id );
-            output( "\tmov x9, x30\n" );
-            output( "\tbl %s\n", asm_name("__wine_syscall" ));
-            output( "\tret\n" );
-            break;
-        default:
-            assert(0);
-        }
-        output_cfi( ".cfi_endproc" );
-        output_function_size( name );
-    }
-
-    switch (target.cpu)
-    {
-    case CPU_i386:
-        if (UsePIC) break;
-        output( "\t.align %d\n", get_alignment(16) );
-        output( "\t%s\n", func_declaration("__wine_syscall") );
-        output( "%s:\n", asm_name("__wine_syscall") );
-        output( "\tjmp *(%s)\n", asm_name("__wine_syscall_dispatcher") );
-        output_function_size( "__wine_syscall" );
-        break;
-    case CPU_ARM:
-        output( "\t.align %d\n", get_alignment(16) );
-        output( "\t%s\n", func_declaration("__wine_syscall") );
-        output( "%s:\n", asm_name("__wine_syscall") );
-        if (UsePIC)
-        {
-            output( "\tldr r0, 2f\n");
-            output( "1:\tadd r0, pc\n" );
-        }
-        else
-        {
-            output( "\tmovw r0, :lower16:%s\n", asm_name("__wine_syscall_dispatcher") );
-            output( "\tmovt r0, :upper16:%s\n", asm_name("__wine_syscall_dispatcher") );
-        }
-        output( "\tldr r0, [r0]\n");
-        output( "\tbx r0\n");
-        if (UsePIC) output( "2:\t.long %s-1b-%u\n", asm_name("__wine_syscall_dispatcher"), thumb_mode ? 4 : 8 );
-        output_function_size( "__wine_syscall" );
-        break;
-    case CPU_ARM64:
-        output( "\t.align %d\n", get_alignment(16) );
-        output( "\t%s\n", func_declaration("__wine_syscall") );
-        output( "%s:\n", asm_name("__wine_syscall") );
-        output( "\tadrp x16, %s\n", arm64_page( asm_name("__wine_syscall_dispatcher") ) );
-        output( "\tldr x16, [x16, #%s]\n", arm64_pageoff( asm_name("__wine_syscall_dispatcher") ) );
-        output( "\tbr x16\n");
-        output_function_size( "__wine_syscall" );
-    default:
-        break;
-    }
-    output( "\t.data\n" );
-    output( "\t.align %d\n", get_alignment( get_ptr_size() ) );
-    output( "%s\n", asm_globl("__wine_syscall_dispatcher") );
-    output( "\t%s 0\n", get_asm_ptr_keyword() );
-    output( "\t.short %u\n", count );
-    for (i = 0; i < count; i++) output( "\t.byte %u\n", get_args_size( syscalls[i] ));
-}
-
-
 /* output the import and delayed import tables of a Win32 module */
 void output_imports( DLLSPEC *spec )
 {
@@ -1566,12 +1412,26 @@ static void assemble_files( const char *prefix )
 
     for (i = 0; i < as_files.count; i++)
     {
-        char *obj = get_temp_file_name( prefix, ".o" );
+        char *obj = make_temp_file( prefix, ".o" );
         assemble_file( as_files.str[i], obj );
         as_files.str[i] = obj;
     }
 }
 
+static const char *get_target_machine(void)
+{
+    static const char *machine_names[] =
+    {
+        [CPU_i386]    = "x86",
+        [CPU_x86_64]  = "x64",
+        [CPU_ARM]     = "arm",
+        [CPU_ARM64]   = "arm64",
+        [CPU_ARM64EC] = "arm64ec",
+    };
+
+    return machine_names[target.cpu];
+}
+
 /* build a library from the current asm files and any additional object files in argv */
 void output_static_lib( const char *output_name, struct strarray files, int create )
 {
@@ -1587,6 +1447,7 @@ void output_static_lib( const char *output_name, struct strarray files, int crea
     {
         args = find_link_tool();
         strarray_add( &args, "/lib" );
+        strarray_add( &args, strmake( "-machine:%s", get_target_machine() ));
         strarray_add( &args, strmake( "-out:%s", output_name ));
     }
     strarray_addall( &args, as_files );
@@ -1639,6 +1500,10 @@ static void build_dlltool_import_lib( const char *lib_name, DLLSPEC *spec, struc
             strarray_add( &args, "-m" );
             strarray_add( &args, "arm64" );
             break;
+        case CPU_ARM64EC:
+            strarray_add( &args, "-m" );
+            strarray_add( &args, "arm64ec" );
+            break;
         default:
             break;
     }
@@ -1669,15 +1534,12 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
 
     if (is_delay)
     {
-        output( "\n\t.text\n" );
-        output( "\t.align %d\n", get_alignment( get_ptr_size() ));
-        output( "%s\n", asm_globl( delay_load ) );
-        output( "\t%s\n", func_declaration( delay_load ) );
+        output_function_header( delay_load, 1 );
 
-        output_cfi( ".cfi_startproc" );
         switch (target.cpu)
         {
         case CPU_i386:
+            output_cfi( ".cfi_startproc" );
             output( "\tpushl %%ecx\n" );
             output_cfi( ".cfi_adjust_cfa_offset 4" );
             output( "\tpushl %%edx\n" );
@@ -1692,13 +1554,13 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
             output( "\tpopl %%ecx\n" );
             output_cfi( ".cfi_adjust_cfa_offset -4" );
             output( "\tjmp *%%eax\n" );
+            output_cfi( ".cfi_endproc" );
             break;
         case CPU_x86_64:
-            output_cfi( ".seh_proc %s", asm_name( delay_load ) );
+            output_seh( ".seh_proc %s", asm_name( delay_load ) );
             output( "\tsubq $0x48, %%rsp\n" );
-            output_cfi( ".cfi_adjust_cfa_offset 0x48" );
-            output_cfi( ".seh_stackalloc 0x48" );
-            output_cfi( ".seh_endprologue" );
+            output_seh( ".seh_stackalloc 0x48" );
+            output_seh( ".seh_endprologue" );
             output( "\tmovq %%rcx, 0x40(%%rsp)\n" );
             output( "\tmovq %%rdx, 0x38(%%rsp)\n" );
             output( "\tmovq %%r8, 0x30(%%rsp)\n" );
@@ -1711,9 +1573,8 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
             output( "\tmovq 0x38(%%rsp), %%rdx\n" );
             output( "\tmovq 0x40(%%rsp), %%rcx\n" );
             output( "\taddq $0x48, %%rsp\n" );
-            output_cfi( ".cfi_adjust_cfa_offset -0x48" );
             output( "\tjmp *%%rax\n" );
-            output_cfi( ".seh_endproc" );
+            output_seh( ".seh_endproc" );
             break;
         case CPU_ARM:
             output( "\tpush {r0-r3, FP, LR}\n" );
@@ -1727,8 +1588,12 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
             output( "1:\t.long %s\n", asm_name( import_desc ) );
             break;
         case CPU_ARM64:
+            output_seh( ".seh_proc %s", asm_name( delay_load ) );
             output( "\tstp x29, x30, [sp, #-80]!\n" );
+            output_seh( ".seh_save_fplr_x 80" );
             output( "\tmov x29, sp\n" );
+            output_seh( ".seh_set_fp" );
+            output_seh( ".seh_endprologue" );
             output( "\tstp x0, x1, [sp, #16]\n" );
             output( "\tstp x2, x3, [sp, #32]\n" );
             output( "\tstp x4, x5, [sp, #48]\n" );
@@ -1744,9 +1609,12 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
             output( "\tldp x6, x7, [sp, #64]\n" );
             output( "\tldp x29, x30, [sp], #80\n" );
             output( "\tbr x16\n" );
+            output_seh( ".seh_endproc" );
+            break;
+        case CPU_ARM64EC:
+            assert( 0 );
             break;
         }
-        output_cfi( ".cfi_endproc" );
         output_function_size( delay_load );
         output_gnu_stack_note();
 
@@ -1839,11 +1707,7 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
                                 asm_name( abi_name ) );
 
             new_output_as_file();
-
-            output( "\n\t.text\n" );
-            output( "\t.align %d\n", get_alignment( get_ptr_size() ) );
-            output( "%s\n", asm_globl( abi_name ) );
-            output( "\t%s\n", func_declaration( abi_name ) );
+            output_function_header( abi_name, 1 );
 
             switch (target.cpu)
             {
@@ -1893,6 +1757,9 @@ static void build_windows_import_lib( const char *lib_name, DLLSPEC *spec, struc
                     output( "\tb %s\n", asm_name( delay_load ) );
                 }
                 break;
+            case CPU_ARM64EC:
+                assert( 0 );
+                break;
             }
 
             output( "\n\t.section .idata$4\n" );
@@ -1968,10 +1835,7 @@ static void build_unix_import_lib( DLLSPEC *spec, struct strarray files )
         case TYPE_STDCALL:
             prefix = (!odp->name || (odp->flags & FLAG_ORDINAL)) ? import_ord_prefix : import_func_prefix;
             new_output_as_file();
-            output( "\t.text\n" );
-            output( "\n\t.align %d\n", get_alignment( get_ptr_size() ));
-            output( "\t%s\n", func_declaration( name ) );
-            output( "%s\n", asm_globl( name ) );
+            output_function_header( name, 1 );
             output( "\t%s %s%s$%u$%s\n", get_asm_ptr_keyword(),
                     asm_name( prefix ), dll_name, odp->ordinal, name );
             output_function_size( name );
