@@ -23,7 +23,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -339,9 +338,7 @@ static void output_call16_function( ORDDEF *odp )
 
     name = strmake( ".L__wine_spec_call16_%s", get_relay_name(odp) );
 
-    output( "\t.align %d\n", get_alignment(4) );
-    output( "\t%s\n", func_declaration(name) );
-    output( "%s:\n", name );
+    output_function_header( name, 0 );
     output_cfi( ".cfi_startproc" );
     output( "\tpushl %%ebp\n" );
     output_cfi( ".cfi_adjust_cfa_offset 4" );
@@ -546,7 +543,7 @@ static void output_module16( DLLSPEC *spec )
 
     output( "\n/* module data */\n\n" );
     output( "\t.data\n" );
-    output( "\t.align %d\n", get_alignment(16) );
+    output( "\t.balign 16\n" );
     output( ".L__wine_spec_dos_header:\n" );
     output( "\t.short 0x5a4d\n" );                                         /* e_magic */
     output( "\t.short 0\n" );                                              /* e_cblp */
@@ -570,7 +567,7 @@ static void output_module16( DLLSPEC *spec )
     output( "\t.long .L__wine_spec_ne_header-.L__wine_spec_dos_header\n" );/* e_lfanew */
 
     output( "\t%s \"%s\"\n", get_asm_string_keyword(), fakedll_signature );
-    output( "\t.align %d\n", get_alignment(16) );
+    output( "\t.balign 16\n" );
     output( ".L__wine_spec_ne_header:\n" );
     output( "\t.short 0x454e\n" );                                         /* ne_magic */
     output( "\t.byte 0\n" );                                               /* ne_ver */
@@ -629,7 +626,7 @@ static void output_module16( DLLSPEC *spec )
 
     /* resident names table */
 
-    output( "\n\t.align %d\n", get_alignment(2) );
+    output( "\n\t.balign 2\n" );
     output( ".L__wine_spec_ne_restab:\n" );
     output_resident_name( spec->dll_name, 0 );
     for (i = 1; i <= spec->limit; i++)
@@ -643,7 +640,7 @@ static void output_module16( DLLSPEC *spec )
 
     /* imported names table */
 
-    output( "\n\t.align %d\n", get_alignment(2) );
+    output( "\n\t.balign 2\n" );
     output( ".L__wine_spec_ne_modtab:\n" );
     output( ".L__wine_spec_ne_imptab:\n" );
     output( "\t.byte 0,0\n" );
@@ -656,7 +653,7 @@ static void output_module16( DLLSPEC *spec )
 
     /* code segment */
 
-    output( "\n\t.align %d\n", get_alignment(2) );
+    output( "\n\t.balign 2\n" );
     output( ".L__wine_spec_code_segment:\n" );
 
     for ( i = 0; i < nb_funcs; i++ )
@@ -830,7 +827,8 @@ void output_fake_module16( DLLSPEC *spec )
     const unsigned int lfanew = (0x40 + sizeof(fakedll_signature) + 15) & ~15;
     const unsigned int segtab = lfanew + 0x40;
 
-    unsigned int i, rsrctab, restab, namelen, modtab, imptab, enttab, cbenttab, codeseg, dataseg, rsrcdata;
+    unsigned int i, rsrctab, restab, namelen, modtab, imptab, enttab, cbenttab, codeseg, dataseg, rsrcdata, rsrc_size = 0;
+    void *rsrc_ptr = NULL;
 
     init_output_buffer();
 
@@ -844,6 +842,10 @@ void output_fake_module16( DLLSPEC *spec )
         restab += output_buffer_pos;
         free( output_buffer );
         init_output_buffer();
+        output_bin_res16_data( spec );
+        rsrc_ptr = output_buffer;
+        rsrc_size = output_buffer_pos;
+        init_output_buffer();
     }
 
     namelen  = strlen( spec->dll_name );
@@ -875,7 +877,7 @@ void output_fake_module16( DLLSPEC *spec )
     put_dword( 0 );
     put_word( 0 );            /* e_oemid */
     put_word( 0 );            /* e_oeminfo */
-    put_dword( 0 );           /* e_res2 */
+    put_dword( rsrcdata + rsrc_size ); /* e_res2 */
     put_dword( 0 );
     put_dword( 0 );
     put_dword( 0 );
@@ -954,7 +956,5 @@ void output_fake_module16( DLLSPEC *spec )
     put_data( data_segment, sizeof(data_segment) );
 
     /* resource data */
-    output_bin_res16_data( spec );
-
-    flush_output_buffer();
+    put_data( rsrc_ptr, rsrc_size );
 }
